<?php

/**
 * This product Copyright 2010 metaVentis GmbH.	For detailed notice,
 * see the "NOTICE" file with this distribution.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

try
{
	include_once ('../../conf.inc.php');

	// start session
	if ( empty($ESRENDER_SESSION_NAME) ) {
		error_log('ESRENDER_SESSION_NAME not set in conf/system.conf.php');
		$ESRENDER_SESSION_NAME = 'ESSID';
	}

	session_name($ESRENDER_SESSION_NAME);
	if ( ! session_start() )
	{
		throw new Exception('Could not start session.');
	}

	$esrenderSessionId = session_id();
	if ( ! $esrenderSessionId )
	{
		throw new Exception('Could not get current session_id().');
	}

	// init LOGGER
	require_once(dirname(__FILE__).'/../../func/extern/apache-log4php-2.0.0-incubating/src/main/php/Logger.php');
	Logger::configure(dirname(__FILE__).'/../../conf/de.metaventis.esrender.log4php.properties');
	$Logger = Logger::getLogger('de.metaventis.esrender.index');

	$Logger->info('Starting up.');
	$Logger->debug($_SERVER['REQUEST_URI']);

	################################################################################

	// init PROXY
	$ProxyConfig = '../../conf/proxy.conf.php';
	if ( file_exists($ProxyConfig) )
	{
		include_once ($ProxyConfig);
	}

	require_once (MC_LIB_PATH.'ESApp.php');
	require_once (MC_LIB_PATH.'EsApplications.php');
	require_once (MC_LIB_PATH.'EsApplication.php');
	require_once (MC_LIB_PATH.'ESRepository.php');

	require_once (MC_LIB_PATH.'ESModule.php');
	require_once (MC_LIB_PATH.'ESObject.php');

	unset($CFG);

	// init PLUGINS
	$Plugins = array();
	$PluginConfig = '../../conf/plugins.conf.php';
	if ( file_exists($PluginConfig) )
	{
		require_once($PluginConfig);
	}

	/*
	 * as this is the first plugin-loop we'll try to set the plugin's
	 * logger here to assure each plugin has logging-capability.
	 */
	foreach($Plugins as $name => $Plugin)
	{
		$Plugin->setDefaultLogger($Logger);
	}

	// init DATABASE
	require_once(dirname(__FILE__).'/../../func/classes.new/DB/DBMC.php');
	$Database = DBMC::connect(DSN, false);

	// init APPLICATION
	$RenderApplication = new ESRender_Application($Database);
	$RenderApplication->setLogger($Logger);

	// init translate
	global $Translate;
	$Translate = new Phools_Translate_Array();

    // LANGUAGE
    $LanguageCode = mc_Request::fetch('language', 'CHAR', 'de');
    switch( strtolower($LanguageCode) )
    {
        case 'en':
            $Locale = new Phools_Locale_Default('en', 'EN', '.', '`');
            require_once (dirname(__FILE__).'/../../locale/esmain/EN/lang.common.php');
            break;
        case 'zh':
            $Locale = new Phools_Locale_Default('zh', 'ZH', ',', '.');
            require_once (dirname(__FILE__).'/../../locale/esmain/ZH/lang.common.php');
            break;
        case 'de':
        default:
            $Locale = new Phools_Locale_Default('de', 'DE', ',', '.');
            require_once (dirname(__FILE__).'/../../locale/esmain/DE/lang.common.php');
    }
	// init templating
	$TemplateDirectory = dirname(__FILE__).'/../../theme';
	$Template = new Phools_Template_Script($TemplateDirectory);
	$Template
		->setTheme('default')
		->setLocale($Locale)
		->setTranslate($Translate);

	// REPOSITORY-ID
	$req_data['rep_id'] = mc_Request::fetch('rep_id','CHAR');
	if ( ! $req_data['rep_id'] )
	{
		$Logger->error('Missing request-param "rep_id".');
		throw new ESRender_Exception_MissingRequestParam('rep_id');
	}

	$Validator = new ESRender_Validator_ApplicationId();
	if ( ! $Validator->validate($req_data['rep_id']) )
	{
		$Logger->error('Invalid request-param "rep_id".');
		throw new ESRender_Exception_InvalidRequestParam('rep_id');
	}

	// APPLICATION-ID
	$req_data['app_id'] = mc_Request::fetch('app_id','CHAR');
	if ( ! $req_data['app_id'] )
	{
		$Logger->info('Using repository-id as requested application-id, because request parameter "app-id" was empty.');
		$req_data['app_id'] = $req_data['rep_id'];
	}

	$Validator = new ESRender_Validator_ApplicationId();
	if ( ! $Validator->validate($req_data['app_id']) )
	{
		$Logger->error('Invalid request-param "app_id".');
		throw new ESRender_Exception_InvalidRequestParam('app_id');
	}

	// OBJECT-ID
	$req_data['obj_id'] = mc_Request::fetch('obj_id','CHAR');
	if ( ! $req_data['obj_id'] )
	{
		$Logger->error('Missing request-param "obj_id".');
		throw new ESRender_Exception_MissingRequestParam('obj_id');
	}

	$Validator = new ESRender_Validator_ObjectId();
	if ( ! $Validator->validate($req_data['obj_id']) )
	{
		$Logger->error('Invalid request-param "obj_id".');
		throw new ESRender_Exception_InvalidRequestParam('obj_id');
	}

	// SESSION-ID
	$req_data['session'] = mc_Request::fetch('session','CHAR');
	if ( ! $req_data['session'] )
	{
		$Logger->error('Missing request-param "session".');
		throw new ESRender_Exception_MissingRequestParam('session');
	}

	$Validator = new ESRender_Validator_SessionId();
	if ( ! $Validator->validate($req_data['session']) )
	{
		$Logger->error('Invalid request-param "session".');
		throw new ESRender_Exception_InvalidRequestParam('session');
	}

	// RESOURCE-ID (optional)
	$req_data['resource_id'] = mc_Request::fetch('resource_id','CHAR');
	if ( ! empty($req_data['resource_id']) )
	{
		$Validator = new ESRender_Validator_ResourceId();
		if ( ! $Validator->validate($req_data['resource_id']) )
		{
			$Logger->error('Invalid request-param "resource_id".');
			throw new ESRender_Exception_InvalidRequestParam('resource_id');
		}
	}

	// COURSE-ID (optional)
	$req_data['course_id'] = mc_Request::fetch('course_id','CHAR');
	if ( ! empty($req_data['course_id']) )
	{
		$Validator = new ESRender_Validator_CourseId('/^[a-z0-9-]+$/ui');
		if ( ! $Validator->validate($req_data['course_id']) )
		{
			$Logger->error('Invalid request-param "course_id".');
			throw new ESRender_Exception_InvalidRequestParam('course_id');
		}
	}

	// USERNAME (required for repository, optional for LMS)
	$user_name = '';

	$req_data['username'] = mc_Request::fetch('u','CHAR');
	if ( ! empty($req_data['username']) )
	{
		$Validator = new ESRender_Validator_Username();
		if ( ! $Validator->validate($req_data['username']) )
		{
			$Logger->error('Invalid request-param "u".');
			$Logger->debug('Given username "'.$req_data['username'].'"');
			throw new ESRender_Exception_InvalidRequestParam('u');
		}
	}

	// VERSION (optional)
	$req_data['version'] = mc_Request::fetch('version','CHAR');
	if ( $req_data['version'] )
	{
		$Validator = new ESRender_Validator_Version();
		if ( ! $Validator->validate($req_data['version']) )
		{
			$Logger->error('Invalid request-param "version".');
			$Logger->debug('Given version "'.$req_data['version'].'"');

			throw new ESRender_Exception_InvalidRequestParam('version');
		}
	}

	// THEME
	$Theme = mc_Request::fetch('theme','CHAR');
	if ( $Theme )
	{
		$Validator = new ESRender_Validator_Theme();
		if ( ! $Validator->validate($Theme) )
		{
			throw new ESRender_Exception_InvalidRequestParam('theme');
		}

		$Template->setTheme($Theme);
	}

	// DISPLAY MODE
	$display_kind = mc_Request::fetch('display', 'CHAR', $RenderApplication->getDefaultDisplayMode());
	if ( $display_kind )
	{
		$Validator = new ESRender_Validator_DisplayMode();
		if ( ! $Validator->validate($display_kind) )
		{
			throw new ESRender_Exception_InvalidRequestParam('display');
		}
	}

	// WIDTH
	$width = mc_Request::fetch('width', 'INT', $RenderApplication->getDefaultWidth());
	if ( $width )
	{/*
		$Validator = new Phools_Validator_LesserThan(1024);
		if ( ! $Validator->validate($width) )
		{
			throw new ESRender_Exception_InvalidRequestParam('width');
		}

		$Validator = new Phools_Validator_GreaterThan(16);
		if ( ! $Validator->validate($width) )
		{
			throw new ESRender_Exception_InvalidRequestParam('width');
		}
*/
		$req_data['width'] = $width;
	}

	// HEIGHT
	$height = mc_Request::fetch('height', 'INT', $RenderApplication->getDefaultHeight());
	if ( $height )
	{/*
		$Validator = new Phools_Validator_LesserThan(1024);
		if ( ! $Validator->validate($height) )
		{
			throw new ESRender_Exception_InvalidRequestParam('height');
		}

		$Validator = new Phools_Validator_GreaterThan(16);
		if ( ! $Validator->validate($height) )
		{
			throw new ESRender_Exception_InvalidRequestParam('height');
		}
*/
		$req_data['height'] = $height;
	}

	$CurrentDirectoryName = basename(dirname(__FILE__));
	$LanguageFile = CC_LOCALE_PATH.$CurrentDirectoryName.'/DE/'.CC_LOCALE_FILE;
	if ( ! include($LanguageFile) )
	{
		$Logger->warn('Error including language-file "'.$LanguageFile.'".');
	}

	$application = new ESApp();
	$application->getApp($CurrentDirectoryName);

	$Logger->debug('Initialized application.');

	$hc = $application->getHomeConf();
	if ( ! $hc )
	{
		$Logger->error('Error loading home-configuration.');

		throw new ESRender_Exception_HomeConfigNotLoaded();
	}

	$Logger->debug('Successfully loaded home-configuration.');

	// load repository-config
	foreach($Plugins as $name => $Plugin)
	{
		$Logger->debug('Running plugin "'.$name.'"::preLoadRepository()');
		$Plugin->preLoadRepository(
			$req_data['rep_id'],
			$req_data['app_id'],
			$req_data['obj_id'],
			$req_data['course_id'],
			$req_data['resource_id'],
			$user_name);
	}

	$remote_rep = $application->getAppByID($req_data['rep_id']);
	if ( ! $remote_rep )
	{
		$Logger->error('Error loading application by requested repository-id "'.$req_data['rep_id'].'".');

		throw new ESRender_Exception_AppConfigNotLoaded($req_data['rep_id']);
	}

	$Logger->debug('Successfully loaded repository by id "'.$req_data['rep_id'].'".');

	foreach($Plugins as $name => $Plugin)
	{
		$Logger->debug('Running plugin "'.$name.'"::postLoadRepository()');
		$Plugin->postLoadRepository(
			$remote_rep,
			$req_data['app_id'],
			$req_data['obj_id'],
			$req_data['course_id'],
			$req_data['resource_id'],
			$user_name);
	}

	// load application-config
	foreach($Plugins as $name => $Plugin)
	{
		$Logger->debug('Running plugin "'.$name.'"::preLoadApplication()');
		$Plugin->preLoadApplication(
			$remote_rep,
			$req_data['app_id'],
			$req_data['obj_id'],
			$req_data['course_id'],
			$req_data['resource_id'],
			$user_name);
	}

	$remote_app = $application->getAppByID($req_data['app_id']);
	if ( ! $remote_app )
	{
		$Logger->error('Error loading application by requested application-id "'.$req_data['app_id'].'".');

		throw new ESRender_Exception_AppConfigNotLoaded($req_data['app_id']);
	}

	$Logger->debug('Successfully loaded application by id "'.$req_data['app_id'].'".');

	/*
	 * Repositories must supply an username
	 */
	if ( 'REPOSITORY' == strtoupper($remote_app->prop_array['type']) )
	{
		if ( empty($req_data['username']) )
		{
			throw new ESRender_Exception_MissingRequestParam('username');
		}

		// Handler erzeugen
		$handler = mcrypt_module_open('blowfish', '', 'cbc', '');

		$secretKey = ES_KEY;
		$iv= ES_IV;
		mcrypt_generic_init($handler, $secretKey, $iv);

		$decrypted = mdecrypt_generic($handler, base64_decode($req_data['username']));
		mcrypt_generic_deinit($handler);
		$user_name = trim($decrypted);

		mcrypt_module_close($handler);
	}

	foreach($Plugins as $name => $Plugin)
	{
		$Logger->debug('Running plugin "'.$name.'"::postLoadApplication()');
		$Plugin->postLoadApplication(
			$remote_rep,
			$remote_app,
			$req_data['obj_id'],
			$req_data['course_id'],
			$req_data['resource_id'],
			$user_name);
	}

	// Session Check
	foreach($Plugins as $name => $Plugin)
	{
		$Logger->debug('Running plugin "'.$name.'"::preCheckTicket()');
		$Plugin->preCheckTicket(
			$remote_rep,
			$remote_app,
			$req_data['obj_id'],
			$req_data['course_id'],
			$req_data['resource_id'],
			$user_name);
	}

	$SoapClientParams = array();
	if ( defined('USE_HTTP_PROXY') && USE_HTTP_PROXY )
	{
		$SoapClientParams = array(
			'proxy_host' => HTTP_PROXY_HOST,
			'proxy_port' => HTTP_PROXY_PORT,
			'proxy_login' => HTTP_PROXY_USER,
			'proxy_password' => HTTP_PROXY_PASS,
		);
	}

	$client = new SoapClient(
		$remote_app->prop_array['authenticationwebservice_wsdl'],
		$SoapClientParams);

	try
	{
		$user_ticket = $req_data['session'];

		$params = array(
			"username" => $user_name,
			"ticket" => $user_ticket,
		);

		$ret = $client->checkTicket($params);
	}
	catch (Exception $e)
	{
		$Logger->error('Error calling RemoteApplicationWebService::checkTicket() using username "'.$user_name.'" and ticket "'.$user_ticket.'".');
		$Logger->error($e);

		throw new ESRender_Exception_NetworkError('An error occured during checkTicket().');
	}

	// return early on invalid session
	if ( $ret->checkTicketReturn != true )
	{
		$Logger->warn('Ticket "'.$user_ticket.'" invalid.');

		throw new ESRender_Exception_Unauthorized('The supplied ticket could not be validated.');
	}

	$Logger->info('Successfully checked ticket "'.$user_ticket.'".');

	foreach($Plugins as $name => $Plugin)
	{
		$Logger->debug('Running plugin "'.$name.'"::postCheckTicket()');
		$Plugin->postCheckTicket(
			$remote_rep,
			$remote_app,
			$req_data['obj_id'],
			$req_data['course_id'],
			$req_data['resource_id'],
			$user_name);
	}

	// user daten holen
	foreach($Plugins as $name => $Plugin)
	{
		$Logger->debug('Running plugin "'.$name.'"::preRetrieveUserData()');
		$Plugin->preRetrieveUserData(
			$remote_rep,
			$remote_app,
			$req_data['obj_id'],
			$req_data['course_id'],
			$req_data['resource_id'],
			$user_name);
	}

	try
	{
		if ( 'REPOSITORY' == strtoupper($remote_app->prop_array['type']) )
		{
			$user_ticket = session_id();
		}
		else
		{
			$user_ticket = $req_data['session'];
		}

		$params = array(
			"applicationId" => $hc->prop_array['appid'],
			"username" => $user_name,
			"email" => $user_name,
			"ticket" => $user_ticket,
			"createUser" => false,
		);

		$user_data = $client->authenticateByApp($params);

		// dev hack
		if ( 'admin' == $user_data->authenticateByAppReturn->username  )
		{
			$user_data->authenticateByAppReturn->email = 'admin';
		}

		if(empty($user_data->authenticateByAppReturn->userid))
		{
			$user_data->authenticateByAppReturn->userid = 0;
		}

		$user_id = $user_data->authenticateByAppReturn->userid;
		$user_name = $user_data->authenticateByAppReturn->username;
		$user_givenname = $user_data->authenticateByAppReturn->givenname;
		$user_surname = $user_data->authenticateByAppReturn->surname;
		$user_email = $user_data->authenticateByAppReturn->email;
	}
	catch (Exception $exception)
	{
		$Logger->error('Error while calling RemoteApplicationWebService::authenticateByApp() using applicationId "'.$hc->prop_array['appid'].'", username "'.$user_name.'", email "'.$user_email.'", ticket "'.$user_ticket.'" and createUser "false".');
		$Logger->error($exception);

		$_errorMessage='unbekannter Fehler';

		if (isset($exception->detail->{"de.cc.ws.authentication.AuthenticationException"}->message))
		{
			if ($exception->detail->{"de.cc.ws.authentication.AuthenticationException"}->message == 'APPLICATIONACCESS_NOT_ACTIVATED_BY_USER') {
				$_errorMessage = 'Die Application Renderservice muss für Sie freigeschaltet werden. <br>Bitte schauen Sie in Ihre Mailbox und klicken Sie auf den Bestätigunglink. ';
				$_access = false;
			}

			else if ($exception->detail->{"de.cc.ws.authentication.AuthenticationException"}->message == 'MISSING_PARAM') {
				$_errorMessage = 'Parameter fehlt ';
				$_access = false;
			}
		}
		else {
			$_errorMessage = " unbekannter Fehler ";
		}

		throw $exception;
	}

	$Logger->info('Successfully fetched user-data.');

	foreach($Plugins as $name => $Plugin)
	{
		$Logger->debug('Running plugin "'.$name.'"::postRetrieveUserData()');
		$Plugin->postRetrieveUserData(
			$remote_rep,
			$remote_app,
			$req_data['obj_id'],
			$req_data['course_id'],
			$req_data['resource_id'],
			$user_name);
	}

	// permission check
	foreach($Plugins as $name => $Plugin)
	{
		$Logger->debug('Running plugin "'.$name.'"::preCheckPermission()');
		$Plugin->preCheckPermission(
			$remote_rep,
			$remote_app,
			$req_data['obj_id'],
			$req_data['course_id'],
			$req_data['resource_id'],
			$user_name);
	}

	$rep = new ESRepository();

	$_access = true;
	try
	{
		// alf session für user holen
		if ( empty($remote_rep->prop_array['username']) )
		{
			throw new ESRender_Exception_ConfigParamInvalidOrMissing($req_data['rep_id'], 'username');
		}

		if ( empty($remote_rep->prop_array['password']) )
		{
			throw new ESRender_Exception_ConfigParamInvalidOrMissing($req_data['rep_id'], 'password');
		}

		$Logger->debug('Calling RepositoryWebService::GetTicketByUser().');
		if ( empty($hc->prop_array['appid']) )
		{
			throw new Exception('Missing "appid" in homeApplication-config.');
		}

		$params = array(
			"applicationId" => $hc->prop_array['appid'],
			"username" => $user_name,
			"email" => $user_email,
			"ticket" => session_id(),
			"createUser" => false);

		if ( empty($remote_rep->prop_array['authenticationwebservice_wsdl']) )
		{
			throw new Exception('Missing home-conf-param "authenticationwebservice_wsdl".');
		}

		$alfservice =  new SoapClient($remote_rep->prop_array['authenticationwebservice_wsdl'], $SoapClientParams);
		$alfReturn = $alfservice->authenticateByApp($params);
		$user_ticket = $alfReturn->authenticateByAppReturn->ticket;

		if ( empty($remote_rep->prop_array['alfresco_webservice_url']) )
		{
			throw new Exception('Missing remote-repository-config "alfresco_webservice_url".');
		}

		$alf_user = new AlfrescoRepository($remote_rep->prop_array['alfresco_webservice_url'], $SoapClientParams, $user_ticket);
		$session_user = $alf_user->createSession($user_ticket);

		require_once('Alfresco/Service/SpacesStore.php');
		$spacesStore_user = new SpacesStore($session_user);

		$Logger->debug('Calling CCAlfWrapper::hasPermission().');
		if ( empty($remote_rep->prop_array['authenticationwebservice_wsdl']) )
		{
			throw new Exception('Missing config-entry "authenticationwebservice_wsdl".');
		}

		$acl_client = new AlfrescoWebService(
			$remote_rep->prop_array['authenticationwebservice'] . '/../NativeAlfrescoWrapper?wsdl',
			$SoapClientParams,
			$user_ticket);

		$store['scheme'] = $spacesStore_user->getScheme();
		$store['address'] = $spacesStore_user->getAddress();

		$perm = $acl_client->hasPermissions(array(
			'nodeId' => $req_data['obj_id'],
			'permissions' => 'Read',
			'userId' => $user_name));
	}
	catch (Exception $exception)
	{
		$Logger->error('Error while calling RepositoryWebService::GetTicketByUser() using username "'.$user_name.'" and email "'.$user_email.'".');
		$Logger->error($exception);

		if (isSet($exception->detail->faultData->message))
		{
			$pos = strpos($exception->detail->faultData->message, 'Access Denied.');
			if ($pos === false)
			{
				throw new ESRender_Exception_Unauthorized('Access denied.');
			}
		}

		throw new ESRender_Exception_NetworkError($exception->getMessage());
	}

	$Logger->info('Successfully checked permissions.');

	foreach($Plugins as $name => $Plugin)
	{
		$Logger->debug('Running plugin "'.$name.'"::postCheckPermission()');
		$Plugin->postCheckPermission(
			$remote_rep,
			$remote_app,
			$req_data['obj_id'],
			$req_data['course_id'],
			$req_data['resource_id'],
			$user_name);
	}

	// get ticket for further interaction with the repository
	$rep_ticket = $rep->getTicket($req_data['rep_id']);
	$Logger->info('Successfully retrieved ticket.');

	// check usage
	if ( 'REPOSITORY' != $remote_app->prop_array['type'] )
	{
		// non-repositories MUST supply usage-info
		foreach($Plugins as $name => $Plugin)
		{
			$Logger->debug('Running plugin "'.$name.'"::postCheckPermission()');
			$Plugin->preCheckUsage(
				$remote_rep,
				$remote_app,
				$req_data['obj_id'],
				$req_data['course_id'],
				$req_data['resource_id'],
				$user_name);
		}

		$data['repositoryTicket'] = $rep_ticket;
		//$data['repositoryUsername'] = $user_name;
		$data['repositoryUsername'] = $remote_rep->prop_array['username'];
		$data['repId'] = $req_data['rep_id'];
		$data['lmsId'] = $req_data['app_id'];
		$data['courseId'] = $req_data['course_id'];
		$data['parentNodeId'] = $req_data['obj_id'];
		$data['appUser'] = $user_name;
		$data['resourceId'] = $req_data['resource_id'];

		$ret = $rep->checkUsage($data, $req_data['rep_id']);
		if ( ! $ret )
		{
			// LMS is required to set usage
			throw new ESRender_Exception_UsageError('No usage-information retrieved.');
		}

		$Logger->info('Successfully retrieved usage-informations.');

		$dummy_courseId = $ret->getUsageReturn->courseId;
		$dummy_lmsId = $ret->getUsageReturn->lmsId;
		$dummy_resourceId = $ret->getUsageReturn->resourceId;

		$l_DOMDocument = new DOMDocument();
		$UsageReturn = $ret->getUsageReturn->usageXmlParams;
		if ( ! $l_DOMDocument->loadXML( $UsageReturn ) )
		{
			throw new Exception('Error loading usageXmlParams.');
		}

		if ( ! empty($ret->getUsageReturn->usageVersion) )
		{
			$req_data['version'] = $ret->getUsageReturn->usageVersion;
		}

		$l_nv = $l_DOMDocument->getElementsByTagName("version");
		if ( 0 < $l_nv->length )
		{
			// usage-data contains specific version-information, so we'll adhere to that
			if ( ! empty($l_nv->item(0)->nodeValue) )
			{
				$req_data['version'] = $l_nv->item(0)->nodeValue;
			}
		}

		$l_np = $l_DOMDocument->getElementsByTagName("forcepopup");
		if ( 0 < $l_np->length )
		{
			$forcepopup = $l_np->item(0)->nodeValue;
		}
		else {
			$forcepopup = 0;
		}

		$l_nd = $l_DOMDocument->getElementsByTagName("download");
		if ( 0 < $l_nd->length )
		{
			$download = $l_nd->item(0)->nodeValue;
		}
		else {
			$download = 1;
		}

/*
		if ($forcepopup == 1)
		{
			$display_kind = ESRender_Application_Interface::DISPLAY_MODE_WINDOW;
		}
		else if ($download == 1)
		{
			$display_kind = ESRender_Application_Interface::DISPLAY_MODE_DOWNLOAD;
		}
*/

		foreach($Plugins as $name => $Plugin)
		{
			$Logger->debug('Running plugin "'.$name.'"::preRetrieveObjectProperties()');
			$Plugin->postCheckUsage(
				$remote_rep,
				$remote_app,
				$ret->getUsageReturn,
				$req_data['obj_id'],
				$req_data['course_id'],
				$req_data['resource_id'],
				$user_name);
		}
	}
	else
	{
		$Logger->info('No usage-informations retrieved.');

		$dummy_courseId = 0;
		$dummy_lmsId = 0;
	}

	foreach($Plugins as $name => $Plugin)
	{
		$Logger->debug('Running plugin "'.$name.'"::preRetrieveObjectProperties()');
		$Plugin->preRetrieveObjectProperties(
			$remote_rep,
			$remote_app,
			$req_data['obj_id'],
			$req_data['course_id'],
			$req_data['resource_id'],
			$user_name);
	}

	// check permissions
	if ( empty($remote_rep->prop_array['alfresco_webservice_url']) )
	{
		throw new Exception('Missing config-entry "alfresco_webservice_url".');
	}

	$alf = new AlfrescoRepository($remote_rep->prop_array['alfresco_webservice_url']);

	$session = $alf->createSession($rep_ticket);
	if ( ! $session )
	{
		$Logger->error('Error creating session for ticket "'.$rep_ticket.'".');

		throw new ESRender_Exception_Unauthorized('Error creating session.');
	}

	$Logger->info('Successfully created ticket for session for ticket "'.$rep_ticket.'".');

	// test permissions
	if ( ! ( $dummy_courseId == $req_data['course_id'] && $dummy_lmsId == $req_data['app_id'] or $_access ) )
	{
		$Logger->error('Not authorized.');

		throw new ESRender_Exception_Unauthorized('Access denied.');
	}

	$Logger->info('Successfully checked permissions.');

	// fetch node
	//
	// if usage does not specify a particular object-version to use, switch to the
	// current object-version from repository
	$spacesStore = new SpacesStore($session);
	$contentNode = $session->getNode($spacesStore, $req_data['obj_id']);
	if ( ! $contentNode )
	{
		$Logger->error('Error fetching node for object-id "'.$req_data['obj_id'].'".');

		$contentNode = null;
		throw new ESRender_Exception_NetworkError('Error fetching node for object "'.$req_data['obj_id'].'".');
	}

	$Logger->info('Successfully fetched node.');


//@todo handle this in plugin or ...
if($contentNode -> properties['{http://www.campuscontent.de/model/1.0}remoterepositorytype'] == 'YOUTUBE'
    || $contentNode -> properties['{http://www.campuscontent.de/model/1.0}replicationsource'] == 'oai:dmglib.org')
    $req_data['version'] = false;

	if ( $req_data['version'])
	{
		$Logger->info('Using required object-version "'.$req_data['version'].'".');

		if ( empty($remote_rep->prop_array['alfresco_webservice_url']) )
		{
			throw new Exception('Missing config-entry "alfresco_webservice_url".');
		}

		// fetch history
		$authoringServiceClient = WebServiceFactory::getAuthoringService($remote_rep->prop_array['alfresco_webservice_url'], $rep_ticket);
		$versionHistory = $authoringServiceClient->getVersionHistory(array(
			'node' => array(
				'store' => array(
					'scheme' => 'workspace',
					'address' => 'SpacesStore',
				),
				'uuid' => $req_data['obj_id'],
			)));

		if ( empty($versionHistory->getVersionHistoryReturn->versions) )
		{
			throw new Exception('No version-history available.');
		}

		// alfresco delivers a single version-history as standalone-object, NOT as list having a single element. screw them.
		if (is_object($versionHistory->getVersionHistoryReturn->versions) )
		{
			$Logger->debug('Got single entry for version-history.');

			$version = $versionHistory->getVersionHistoryReturn->versions;

			$store = new Store($session, $version->id->store->address, $version->id->store->scheme);
			$contentNode = $session->getNode($store, $version->id->uuid);
			if ( ! $contentNode )
			{
				$contentNode = null;
				throw new Exception('Error retrieving archived node.');
			}

			if ( empty($contentNode->properties['{http://www.alfresco.org/model/content/1.0}versionLabel']) )
			{
				$contentNode = null;
				throw new Exception('Empty version-label.');
			}
		}
		else if(is_array($versionHistory -> getVersionHistoryReturn->versions))
		{
			$Logger->debug('Got multiple entries in version-history. Will cycle.');

            foreach($versionHistory -> getVersionHistoryReturn -> versions as $version) {                
                if(null === $version) {
                    $contentNode = null;
                    throw new Exception('Requested version not found.');
                }
                
                //if latest version requested use first element in version array
                if($req_data['version'] === '1') {
                    $req_data['version'] = $version -> label;
                }

                if($req_data['version'] === $version -> label) {
                    $store = new Store($session, $version->id->store->address, $version -> id -> store -> scheme);
                    $contentNode = $session -> getNode($store, $version -> id -> uuid);
                }               
            }

            if(!$contentNode) {
                $contentNode = null;
                throw new Exception('Error retrieving archived node.');
            }
		}
		else
		{
			throw new Exception('Unhandled format for version-history.');
		}
	}
	else
	{
		$Logger->info('No certain version requested, trying current.');

		// versionLabel is required later on but not requested, so pre-fill by most current object-version.
		if ( $contentNode->properties['{http://www.campuscontent.de/model/lom/1.0}version'] )
		{
			$req_data['version'] = $contentNode->properties['{http://www.campuscontent.de/model/lom/1.0}version'];
			$Logger->info('Using version "'.$req_data['version'].'" from property "{http://www.campuscontent.de/model/lom/1.0}version".');
		}
		else {
			$req_data['version'] = '';
			$Logger->info('Continuing having NO version-information.');
		}
	}

	$ESObject = new ESObject($req_data['obj_id'], $req_data['version']);
	$ESObject->setAlfrescoNode($contentNode);

	if ( ! $ESObject->setDataByNode($contentNode) )
	{
		$Logger->error('Error importing Alfresco\'s property-node.');

		throw new ESRender_Exception_Unauthorized('Could not import object-properties.');
	}

	foreach($Plugins as $name => $Plugin)
	{
		$Logger->debug('Running plugin "'.$name.'"::postRetrieveObjectProperties()');
		$Plugin->postRetrieveObjectProperties(
			$remote_rep,
			$remote_app,
			$contentNode,
			$req_data['course_id'],
			$req_data['resource_id'],
			$user_name);
	}

	// set partial object data
	$ObjectData = array(
		'ESOBJECT_REP_ID' => $req_data['rep_id'],
		'ESOBJECT_LMS_ID' => $req_data['app_id'],
		'ESOBJECT_COURSE_ID' => $req_data['course_id'],
		'ESOBJECT_RESOURCE_ID' => $req_data['resource_id'],
		'ESOBJECT_VERSION' => $req_data['version'],
	);

	if ( ! $ESObject->setData($ObjectData) )
	{
		$Logger->error('Error setting instance-data.');

		throw new ESRender_Exception_Unauthorized('Could not set instance-data.');
	}

	$Logger->info('Successfully initialized instance.');


	// stop session to allow flawless module-operation
	session_write_close();
    $sessionSavePath = session_save_path();
	// find appropriate module
	$ESObject->setModule();

	$moduleName = $ESObject->ESModule->getName();
	if (empty($moduleName))
	{
		//.oO no display modul for this file
		$Logger->error('No module found');
		$Logger->debug('Object mime-type: "'.$ESObject->getMimeType().'", resource-type: "'.$ESObject->getResourceType().'", resource-version: "'.$ESObject->getResourceVersion().'".');

		throw new Exception('Could not load module to render object.');
	}

	//.oO include the appropriate display module
	$Logger->debug('Attempting to use module "'.$moduleName.'".');

	$moduleFile = realpath( dirname(__FILE__) . '/../../modules/'.$moduleName.'/mod_'.$moduleName.'.php' );
	$Logger->debug('Module-file: "'.$moduleFile.'"');
	if ( ! require_once($moduleFile) ) {
		$Logger->error('Error including module-file.');

		throw new Exception('Error loading module.');
	}

	$Logger->debug('Successfully included module-file "'.$moduleFile.'".');

	$_mod_class = 'mod_'.$moduleName;
	$Module = new $_mod_class($moduleName, $RenderApplication, $ESObject, $Logger, $Template);

	$Logger->info('Loaded module "'.$moduleName.'".');

	$data['givenname'] = $user_givenname;
	$data['surname'] = $user_surname;
	$data['lmsId'] = $req_data['app_id'];
	$data['courseId'] = $req_data['course_id'];


    $instanceParams = array(
        'rep_id' => $req_data['rep_id'],
        'app_id' => $req_data['app_id'],
        'course_id' => $req_data['course_id'],
        'object_id' => $req_data['obj_id'],
        'resource_id' => $req_data['resource_id'],
        'user_id' => $user_id,
        'user_name' => $user_name,
        'user_email' => $user_email,
        'user_givenname' => $user_givenname,
        'user_surname' => $user_surname,
        'version' => $req_data['version'],
        'width' => $req_data['width'],
        'height' => $req_data['height'],
    );

	// create new object instance if not existent
	if((!$Module ->instanceExists($ESObject, $instanceParams, $req_data['version']) && !$Module -> instanceLocked($ESObject, $instanceParams, $req_data['version']))) {

        //esure that instance is not created several times
	    $Module -> instanceLock($instanceParams['rep_id'], $instanceParams['object_id'], $req_data['version']);
        
		foreach($Plugins as $name => $Plugin)
		{
			$Logger->debug('Running plugin "'.$name.'"::preInstanciateObject()');
			$Plugin->preInstanciateObject();
		}

		try
		{
			$Logger->info('Instance does not yet exists. Attempting to create new object-instance.');


			if ( ! $Module->createInstance($remote_app, array(
				'rep_id' => $req_data['rep_id'],
				'app_id' => $req_data['app_id'],
				'course_id' => $req_data['course_id'],
				'object_id' => $req_data['obj_id'],
				'resource_id' => $req_data['resource_id'],
				'user_id' => $user_id,
				'user_name' => $user_name,
				'user_email' => $user_email,
				'user_givenname' => $user_givenname,
				'user_surname' => $user_surname,
				'version' => $req_data['version'],
				'session' => $req_data['session'],
				'width' => $req_data['width'],
				'height' => $req_data['height'],
			)))
			{
				$Logger->error('Error creating new object-instance. Attempting to remove created object.');

				if ( ! $ESObject->deleteFromDb() )
				{
					$Logger->error('Error removing object-instance "'.$ESObject->getObjectID().'".');
				}

				$Logger->info('Successfully removed created object.');

				throw new Exception('Error creating instance.');
			}

			if ( ! $ESObject->setData2Db() )
			{
				$Logger->error('Error storing object-data in database.');

				throw new Exception('Error storing instance-data.');
			}            
		}
		catch (Exception $e)
		{
			$Logger->error('Error while creating new object-instance.');
			$Logger->error($e);

			$ESObject->deleteFromDb();

			throw new Exception('Error creating instance.');
		}

		foreach($Plugins as $name => $Plugin)
		{
			$Logger->debug('Running plugin "'.$name.'"::postInstanciateObject()');
			$Plugin->postInstanciateObject();
		}
        
        // if all went right unlock instance so that it can be processed
        $Module -> instanceUnlock($ESObject, $instanceParams, $req_data['version']);
        
	}

	$Logger->info('Successfully fetched instance.');

	// start session to store esrender-data.
	$moduleSessionName = session_name();
	$moduleSessionId = session_id();
    $moduleSessionSavePath = session_save_path();
	session_write_close();

    session_save_path($sessionSavePath);

	session_name($ESRENDER_SESSION_NAME);
	session_id($esrenderSessionId);
	session_start();

	$Logger->info('Preparing render-session.');

	// prepare module render data
	$_SESSION['esrender'] = array(
		'file_name' => $ESObject->getFilename(),
		'mod_name'	=> $moduleName,
	// relative path to DOC_ROOT, e.g. '/esrender/modules/doc/files/'
		'mod_path'	=> MC_PATH.$ESObject->ESModule->getTmpFilepath().DIRECTORY_SEPARATOR,
	// absolute path e.g. '/srv/www/htdocs/esrender/modules/doc/files/'
		'mod_root'	=> MC_ROOT_PATH.$ESObject->ESModule->getTmpFilepath().DIRECTORY_SEPARATOR,
	// absolute path, e.g. '/srv/www/docs/'.$mod_name.'/'
		'src_root'	=> CC_RENDER_PATH . DIRECTORY_SEPARATOR . $moduleName . DIRECTORY_SEPARATOR,
		'TOU'	 => $Module->getTimesOfUsage(), // times of usage (0:forbidden, -1:unlimited)
		'check' => parse_url($ESObject->getPathfile(), PHP_URL_PATH),
		'display_kind' => $display_kind,
	);

	$cookie_path	= '/';
	$cookie_expire = 0;
	$cookie_domain = MC_HOST;

	$Logger->debug('Setting esrender-cookie for host "'.MC_HOST.'".');
	if( ! setcookie($ESRENDER_SESSION_NAME, session_id(), $cookie_expire, $cookie_path, $cookie_domain) )
	{
		$Logger->error('Error setting esrender-cookie named "'.$ESRENDER_SESSION_NAME.'" = "'.session_id().'" for host "'.MC_HOST.'".');
	}
	// re-start module-session to finish processing
	session_write_close();

    session_save_path($moduleSessionSavePath);
	session_name($moduleSessionName);
	session_id($moduleSessionId);
	session_start();

    if(ENABLE_TRACK_OBJECT) {
    	// START OBJECT TRACKING
    	$TrackingId = $RenderApplication->trackObject(
    		$remote_rep->prop_array['appid'],
    		$remote_app->prop_array['appid'],
    		$req_data['obj_id'],
    		$ESObject->getFilename(),
    		$req_data['version'],
    		$ESObject->ESModule->getModuleId(),
    		$ESObject->ESModule->getName(),
    		$user_id,
    		$user_name,
    		$req_data['course_id']);
    	if ( ! $TrackingId )
    	{
    		$Logger->error('Error tracking object.');
    		throw new Exception('Error tracking object.');
    	}
    
    	$Logger->info('Tracking return ID "'.$TrackingId.'".');
    	// END OBJECT TRACKING
    }

	foreach($Plugins as $name => $Plugin)
	{
		$Logger->debug('Running plugin "'.$name.'"::preProcessObject()');
		$Plugin->preProcessObject();
	}

	$Logger->info('Processing render-object.');
	if ( ! $Module->process($display_kind, $remote_app, array(
		'rep_id' => $req_data['rep_id'],
		'app_id' => $req_data['app_id'],
		'course_id' => $req_data['course_id'],
		'object_id' => $req_data['obj_id'],
		'resource_id' => $req_data['resource_id'],
		'user_id' => $user_id,
		'user_name' => $user_name,
		'user_email' => $user_email,
		'user_givenname' => $user_givenname,
		'user_surname' => $user_surname,
		'tracking_id' => $TrackingId,
		'session' => $req_data['session'],
		'width' => $req_data['width'],
		'height' => $req_data['height'],
		'callback' => mc_Request::fetch('callback','CHAR'),
		'user_name_encr' => $req_data['username'],
		'sessionName' => $ESRENDER_SESSION_NAME,
		'sessionId' => $esrenderSessionId
	), $Module -> instanceLocked($ESObject, $instanceParams, $req_data['version']) ) )
	{
		$Logger->error('Error processing object "'.$data['parentNodeId'].'".');

		throw new Exception('Error processing object.');
	}

	foreach($Plugins as $name => $Plugin)
	{
		$Logger->debug('Running plugin "'.$name.'"::postProcessObject()');
		$Plugin->postProcessObject();
	}

	$Logger->info('Shutting down.');

	exit(0);
}
catch(ESRender_Exception_MissingRequestParam $exception)
{
	$Logger->error('Missing parameter "'.$exception->getParamName().'"');
	$Logger->error($exception);

	header('HTTP/1.0 400 Bad Request');

	$Message = new Phools_Message_Default('Mising parameter ":name".',
		array(new Phools_Message_Param_String(':name', $exception->getParamName())));

	echo $Template->render('/error/missing_request_param', array(
		'error' => $Message->localize($Locale, $Translate),
	));
}
catch(ESRender_Exception_InvalidRequestParam $exception)
{
	$Logger->error('Invalid parameter "'.$exception->getParamName().'"');
	$Logger->error($exception);

	header('HTTP/1.0 400 Bad Request');

	$Message = new Phools_Message_Default('Invalid parameter ":name".',
		array(new Phools_Message_Param_String(':name', $exception->getParamName())));

	echo $Template->render('/error/invalid_request_param', array(
		'error' => $Message->localize($Locale, $Translate),
	));
}
catch(ESRender_Exception_HomeConfigNotLoaded $exception)
{
	$Logger->error('Error loading home-configuration.');
	$Logger->error($exception);

	header('HTTP/1.0 500 Internal Server Error');

	$Message = new Phools_Message_Default('Error loading configuration.');

	echo $Template->render('/error/load_home_config', array(
		'error' => $Message->localize($Locale, $Translate),
	));
}
catch(ESRender_Exception_AppConfigNotLoaded $exception)
{
	$Logger->error('Error loading config for application "'.$exception->getAppId().'".');
	$Logger->error($exception);

	header('HTTP/1.0 500 Internal Server Error');

	$Message = new Phools_Message_Default('Error loading config for application ":app_id".',
		array(new Phools_Message_Param_String(':app_id', $exception->getAppId())));

	echo $Template->render('/error/load_app_config', array(
		'error' => $Message->localize($Locale, $Translate),
	));
}
catch(ESRender_Exception_NetworkError $exception)
{
	$Logger->error('A network error occured.');
	$Logger->error($exception);

	header('HTTP/1.0 500 Internal Server Error');

	$Message = new Phools_Message_Default('A network error occured.');

	echo $Template->render('/error/network_error', array(
		'error' => $Message->localize($Locale, $Translate),
	));
}
catch(ESRender_Exception_Unauthorized $exception)
{
	$Logger->error('You\'re not authorized to access this resource.');
	$Logger->error($exception);

	header('HTTP/1.0 401 Unauthorized');

	$Message = new Phools_Message_Default('You\'re not authorized to access this resource.');

	echo $Template->render('/error/unauthorized', array(
		'error' => $Message->localize($Locale, $Translate),
	));
}
catch(ESRender_Exception_ConfigParamInvalidOrMissing $exception)
{
	$Logger->error('Missing or wrong config parameter "'.$exception->getParam().'".');
	$Logger->error($exception);

	header('HTTP/1.0 500 Internal Server Error');

	$Message = new Phools_Message_Default('The config param ":param" for app ":app" is invalid or missing. Please contact your system-administrator.');

	echo $Template->render('/error/config_param_invalid', array(
		'error' => $Message->localize($Locale, $Translate),
	));
}
catch(ESRender_Exception_UsageError $exception){
    $Logger->error($exception->getMessage());
    $Logger->debug($exception);

    header('HTTP/1.0 500 Internal Server Error');

    $Message = new Phools_Message_Default($exception->getMessage());

    echo $Template->render('/error/default', array(
        'error' => $Message->localize($Locale, $Translate),
    ));
}
catch(Exception $exception)
{
	$Logger->error('An internal server error occured.');
	$Logger->debug($exception);

	header('HTTP/1.0 500 Internal Server Error');

	$Message = new Phools_Message_Default('An internal server error occured.');

	echo $Template->render('/error/default', array(
		'error' => $Message->localize($Locale, $Translate),
	));
}

$Logger->info('Shutting down.');

exit(255);
