<?php

/**
 * Base-class for all modules handling objects, which contain binary data,
 * e.g. video/audio/images.
 *
 *
 */
abstract class ESRender_Module_ContentNode_Abstract
extends ESRender_Module_Base
{

	/**
	 *
	 * @return string
	 */
	protected function getCacheFileName()
	{
		$Logger = $this->getLogger();

		$Filename = CC_RENDER_PATH . DIRECTORY_SEPARATOR;
		$Filename .= $this->getName() . DIRECTORY_SEPARATOR;
		$Filename .= $this->_ESOBJECT->getEsobjectFilePath() . DIRECTORY_SEPARATOR;
		$Filename .= $this->_ESOBJECT->getObjectID() . $this->_ESOBJECT->getObjectVersion();

		return str_replace('\\','/',$Filename);
	}

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_Base::createInstance()
	 */
	public function createInstance(
		EsApplication $RemoteApplication,
		array $requestData)
	{
		$Logger = $this->getLogger();

		$upath = date('Y/m/d/H/i/s');

		// update esobject member data
		$this->_ESOBJECT->setFilePath($upath);
		$this->_ESOBJECT->setSubUri($upath);

		$this->filename = $this->_ESOBJECT->getObjectIdVersion();

		// real path
		$this->render_path = CC_RENDER_PATH . DIRECTORY_SEPARATOR
			. $this->_ESOBJECT->ESModule->getName()
			.DIRECTORY_SEPARATOR
			.$upath;

		if ( ! file_exists($this->render_path) )
		{
			if ( ! mkdir($this->render_path, 0777, true) )
			{
				$Logger->error('Error creating path "'.$this->render_path.'".');
				return false;
			}

			if ( ! chmod($this->render_path, 0777) )
			{
				$Logger->error('Error changing permissions on "'.$this->render_path.'".');
				return false;
			}
		}

		if ( empty($this->_ESOBJECT->AlfrescoNode->properties['{http://www.alfresco.org/model/content/1.0}content']) )
		{
			$Logger->error('No content in alfresco-node.');
			return false;
		}

		$cacheFile = $this->getCacheFileName();
		$Logger->info('Storing content in file "'.$cacheFile.'".');

		$content = $this->_ESOBJECT->AlfrescoNode->properties['{http://www.alfresco.org/model/content/1.0}content'];
		$content->readContentToFile($cacheFile);

		return true;
	}

	protected function renderUrl(array $requestData)
	{
		$url = MC_SCHEME . '://' . MC_HOST . '/' . trim(MC_PATH, '/') . '/application/esmain?';

		if ( ! $requestData['object_id'] )
		{
			throw new ESRender_Exception_MissingRequestParam('obj_id');
		}

		$url .= 'obj_id=' . urlencode($requestData['object_id']);

		if ( ! $requestData['app_id'] )
		{
			throw new ESRender_Exception_MissingRequestParam('app_id');
		}

		$url .= '&app_id=' . urlencode($requestData['app_id']);

		if ( ! $requestData['rep_id'] )
		{
			throw new ESRender_Exception_MissingRequestParam('rep_id');
		}

		$url .= '&rep_id=' . urlencode($requestData['rep_id']);

		if ( ! $requestData['session'] )
		{
			throw new ESRender_Exception_MissingRequestParam('session');
		}

		$url .= '&session=' . urlencode($requestData['session']);

		if ( $requestData['course_id'] )
		{
			$url .= '&course_id=' . urlencode($requestData['course_id']);
		}

		if ( $requestData['resource_id'] )
		{
			$url .= '&resource_id=' . urlencode($requestData['resource_id']);
		}

		return $url;
	}

	/**
	 *
	 * @param EsApplication $RemoteApplication
	 * @param array $requestData
	 */
	protected function download(
		EsApplication $RemoteApplication,
		array $requestData)
	{
		$Logger = $this->getLogger();
        $url = $this->_ESOBJECT -> getPathfile();
		$Logger->debug('Redirecting to location: "' . $url . '"');

		header('HTTP/1.1 303 See other');
		header('Location: ' . $url);

        exit(0);
	}

	/**
	 *
	 * @param EsApplication $RemoteApplication
	 * @param array $requestData
	 */
	protected function display(
		EsApplication $RemoteApplication,
		array $requestData)
	{
		$Logger = $this->getLogger();

		$url = $this->_ESOBJECT->getPath() . '?' . session_name() . '=' . session_id();
		$Logger->debug('Redirecting to location: "' . $url . '"');

		header('HTTP/1.1 303 See other');
        header('Location: ' . $url);

		return true;
	}

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_Base::inline()
	 */
	protected function inline(
		EsApplication $RemoteApplication,
		array $requestData)
	{
		$Logger = $this->getLogger();

		$snippet = $this->getTemplate()->render('/module/default/inline', array(
			'title' => $this->_ESOBJECT->getTitle(),
			'url' => $this->renderUrl($requestData),
		));
		$Logger->debug('ESRender_Module_Base::inline Snippet "' . $snippet . '"');

 echo $snippet;

		return true;
	}

}
