<?php

/**
 *
 *
 *
 */
abstract class ESRender_Module_PhpVideoToolkit_Abstract
extends ESRender_Module_ContentNode_Abstract {

    /**
     * Format constants
     *
     * @var string
     */
    const FORMAT_IPAD = 'FORMAT_IPAD';
    const FORMAT_FLASH = 'FORMAT_FLASH';

    /**
     *
     * @param string $format
     *
     * @throws Exception
     *
     * @return string
     */
    abstract protected function getOutputFilename($format = self::FORMAT_FLASH);

    /**
     *
     * @return string from this class' format-constants.
     */
    protected function formatByUserAgent() {
        $format = self::FORMAT_FLASH;

        // return early
        if (empty($_SERVER['HTTP_USER_AGENT'])) {
            return $format;
        }

        // iPhone
        if (stripos($_SERVER['HTTP_USER_AGENT'], 'iPhone')) {
            $format = self::FORMAT_IPAD;
        }

        // iPad
        if (stripos($_SERVER['HTTP_USER_AGENT'], 'iPad')) {
            $format = self::FORMAT_IPAD;
        }

        // Android
        if (stripos($_SERVER['HTTP_USER_AGENT'], 'Android')) {
            $format = self::FORMAT_IPAD;
        }

        return $format;
    }



    /**
     * (non-PHPdoc)
     * @see ESRender_Module_ContentNode_Abstract::createInstance()
     */
    final public function createInstance(EsApplication $RemoteApplication, array $requestData) {

        if (!parent::createInstance($RemoteApplication, $requestData)) {
            return false;
        }

        return true;
    }

    /**
     * Checking for HTTP-header "User-Agent" allows conversion to a supported
     * audio/video-format "on-the-fly".
     *
     * (non-PHPdoc)
     * @see ESRender_Module_Base::process()
     */
    public function process($p_kind, EsApplication $RemoteApplication, array $requestData, $objectLocked = false) {
       
        global $CC_RENDER_PATH;
        if ($objectLocked) {
            return parent::process(ESRender_Application_Interface::DISPLAY_MODE_LOCKED, $RemoteApplication, $requestData);
        }
        
        if ($p_kind == ESRender_Application_Interface::DISPLAY_MODE_DOWNLOAD) {
            return parent::process($p_kind, $RemoteApplication, $requestData);
        }
        
        $formatUA = $this -> formatByUserAgent();
        //handle this differently for further formats
        $formats = array($formatUA);
        ($formatUA == self::FORMAT_FLASH) ? $formats[] = self::FORMAT_IPAD : $formats[] = self::FORMAT_FLASH;

        foreach ($formats as $format) {
            $output_filename = $this -> getOutputFilename($format);
            if (!file_exists($output_filename)) {
                if (!$this -> _ESOBJECT -> inConversionQueue($format)) {
                    $this -> _ESOBJECT -> addToConversionQueue($format, DIRECTORY_SEPARATOR, $this -> getCacheFileName(), $this -> getOutputFilename($format), $CC_RENDER_PATH, $this -> _ESOBJECT -> getMimeType());
                        //$logfile = $this -> getLogfile();
                        $logfile = dirname(__FILE__) . '/../../../../../error/conversion/'.$this -> _ESOBJECT -> getObjectID() . '_' . $this->_ESOBJECT->getObjectVersion() . '.log';
                        exec("php " . dirname(__FILE__) . "/Converter.php >> " . $logfile . " 2>&1 &");
                }
                //show lock screen (progress bar) but not in display mode 'window'
                if ($formatUA == $format && $p_kind != ESRender_Application_Interface::DISPLAY_MODE_WINDOW)
                    $p_kind = ESRender_Application_Interface::DISPLAY_MODE_LOCKED;
            } 
        }
        return parent::process($p_kind, $RemoteApplication, $requestData);
    }

    protected function getLogfile() {
        $filename = dirname(__FILE__) . '/../../../../../error/conversion.log';
        if (filesize($filename) > 2097152) { // > 2MB
            $filename2 = "$filename.".date('Y_m_d_H_i_s');
            rename($filename, $filename2);

        }
        return $filename;
    }

    /**
     * (non-PHPdoc)
     * @see ESRender_Module_Base::getTimesOfUsage()
     */
    public function getTimesOfUsage() {
        return PHP_INT_MAX;
    }

}
