<?php
/**
 * Copyright (C) 2000  Author
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



include_once(LIB_NEW_FULLPATH.'Plattform.php');

/**
	* handles  php and database sessions
	*
	* @author steffen gross / matthias hupfer

	*/
class SessionManager
	extends Plattform
{

	var $session_path;

	var $session_prefix;

	var $session_timeout;

	var $sessions_from_php;


	/**
		*
		*/
	function SessionManager()
	{
		$this->Plattform();

		// read session save path from possible sources
		if ($this->session_path = ini_get('session.save_path')) {}
		else if ($this->session_path = $_ENV['TMP'])  {}
		else if ($this->session_path = $_ENV['TEMP']) {}
		else
		{
			mc_Debug::error($this->session_path, 'session save path not found');
		}

		$this->session_timeout   = time() - ini_get('session.gc_maxlifetime');
		$this->session_prefix    = 'sess_';
		$this->sessions_from_php = array();

		return true;
	} // end constructor



	/**
		*
		*/
	function isSessionInDatabase($p_sid)
	{
		if (empty($p_sid))
		{
			die(mc_Debug::error('empty parameter! cannot check for id in database without a specific id'));
		}

		$sel = 'select SESSION_ID from SESSION where SESSION_SID = '.$this->DB->quote($p_sid, 'text');

		$res = $this->DB->getOne($sel);
		if (Plattform::isError($res))
		{
			return false;
		}

		if (empty($res))
		{
			return false;
		}

		return true;
	} // end function isSessionInDatabase



	/**
		*
		*/
	function readPhpSessions()
	{
		$this->sessions_from_php = array();

		$this->readSessionDirectory($this->session_path);

		return true;
	} // end function readPhpSessions



	/**
		*
		*/
	function readSessionDirectory($p_dirname)
	{
		if ($handle = opendir($p_dirname))
		{
				$l_len = strlen($this->session_prefix);

		    while (false !== ($file = readdir($handle)))
		    {

					if ($file[0] == '.')
					{
						continue;
		    	}
/*
					else if (is_dir($file))
					{
						$this->readSessionDirectory($p_dirname.'/'.$file);
					}
*/
					else if (substr($file, 0, $l_len) == $this->session_prefix)
		    	{
/*
		    		if (filemtime($file) < $this->session_timeout)
		    		{
		    			// session timed out
						}
*/
						$this->sessions_from_php[] = substr($file, $l_len);
					}
		    }
		    closedir($handle);
		}
		else
		{
			mc_Debug::error($handle, 'cannot open session path');
			return false;
		}

		return true;
	} // end function readSessionDirectory




	/**
		*
		*/
	function sessionExists($p_sid)
	{
		if ($this->sessionExistsInPhp($p_sid))
		{
			return true;
		}

		if ($this->isSessionInDatabase($p_sid))
		{
			return true;
		}

		return false;
	} // end function sessionExists




	/**
		*
		*/
	function sessionExistsInPhp($p_sid)
	{
		if (array_search($p_sid, $this->sessions_from_php) === false)
		{
			return false;
		}

		return true;
	} // end function sessionExistsInPhp



	/**
		*
		*/
	function generateSessionId()
	{
		$l_tmp = array(
			'SERVER' => $_SERVER['SERVER_NAME'],
			'REMOTE' => $_SERVER['REMOTE_ADDR'],
			'TIME'   => microtime(),
			'RAND'   => rand(0, 1000000000),
		);
		$l_id = md5(serialize($l_tmp));

		return $l_id;
	} // end function generateSessionId



	/**
		*
		*/
	function getNewSessionId()
	{
		$this->readPhpSessions();

		while ($this->sessionExists($l_sid = $this->generateSessionId()));

		return $l_sid;
	} // end function getNewSessionId



	/**
		*
		*/
	function registerSession($p_sid)
	{
		if (empty($p_sid))
		{
			mc_Debug::error('empty parameter! cannot register a session without a specific id');
			return false;
		}

		$l_uid = $_SESSION['USER_IDENT'];

		$select = 'insert into SESSION ('
			.' SESSION_SID, SESSION_APP_IDENT, SESSION_LASTACTION, SESSION_USER_IDENT, SESSION_CURRENT_COURSE_ID, INSDAT, UPDDAT, STATE'
			.' ) values ('
			.' ' . $this->DB->quote($p_sid, 'text')
			.', ' . $this->DB->quote($_SESSION['APP_IDENT'], 'integer')
			.', ' . $this->DB->quote(time(), 'integer')
			.', ' . $this->DB->quote($_SESSION['USER_IDENT'], 'integer')
			.', ' . $this->DB->quote(intval($_SESSION['COURSE_ID']), 'integer')
			.', now(), now(), '.$this->DB->quote('Y', 'text')
			.' )'
		;
		$res = $this->DB->query($select);
		if (Plattform::isError($res))
		{
			return false;
		}

		include_once(MC_LIB_PATH.'Extfunc.php');
		$l_Extfunc = new Extfunc();
		$l_Extfunc->triggerByName('session_create', $p_sid);

		return true;
	} // end function registerSession




	/**
		*
		*/
	function writeLastActionTime($p_sid)
	{
		if (empty($p_sid))
		{
			mc_Debug::error('empty parameter! cannot write session time without a specific id');
			return false;
		}

		if ($this->isSessionInDatabase($p_sid) == false)
		{
			return false;
		}

		$select = 'update SESSION'
			.' set'
		.'  SESSION_LASTACTION = '.$this->DB->quote(time(), 'text')
			.', SESSION_CURRENT_COURSE_ID = '.$this->DB->quote(intval($_SESSION['COURSE_ID']), 'integer')
			.' where'
			.'     SESSION_SID = '.$this->DB->quote($p_sid, 'text')
		;
		$res = $this->DB->query($select);
		if (Plattform::isError($res))
		{
			return false;
		}

		return true;
	} // end function writeLastActionTime



	/**
		* deletes a given session from the session table
		*/
	function unregisterSession($p_sid, $p_kill_session_data = false)
	{
		if (empty($p_sid))
		{
			mc_Debug::error('empty parameter! cannot delete session without a specific id');
			return false;
		}

		include_once(MC_LIB_PATH.'Extfunc.php');
		$l_Extfunc = new Extfunc();
		$l_Extfunc->triggerByName('session_destroy', $p_sid);

		$del = 'delete from SESSION'
			.' where'
			.' SESSION_SID = '.$this->DB->quote($p_sid, 'text')
		;
		$res = $this->DB->query($del);
		if (Plattform::isError($res))
		{
		  return false;
		}
		session_destroy();

		if ($p_kill_session_data)
		{
			$_SESSION = array();
		}

		include_once(MC_LIB_PATH.'Login.php');
		$l_Login = new Login();
		$l_Login->killDeadDSLocks();

		return true;
	} // end function unregisterSession




	/**
		* deletes outdated sessions from session table
		*/
	function cleanupSessionTable()
	{
		$this->readPhpSessions();

		$sel = 'select SESSION_ID'
			.' from SESSION'
		;
		$db_sid_list = $this->DB->getCol($sel);
		if (Plattform::isError($db_sid_list))
		{
			return false;
		}

		$counter = 0;

		foreach($db_sid_list as $sid)
		{
			if ($this->sessionExistsInPhp($sid) == false)
			{
				$sel = 'delete from SESSION'
					.' where SESSION_SID = ' . $this->DB->quote($sid, 'text')
				;
				$db_sid_list = $this->DB->query($sel);
				if (Plattform::isError($db_sid_list))
				{
					return false;
				}
			}
		}

	} // end function cleanupSessionTable



} // end class SessionManager

?>