<?php
/*
* $McLicense$
*
* $Id$
*
*/


require_once(MCP_LIB_PATH . "DBStruct.php");
require_once(MCP_LIB_PATH . "File.php");
require_once(MCP_LIB_PATH . "Stack.php");
require_once(MCP_LIB_PATH . "Check.php");
require_once(MCP_LIB_PATH . "MpsParser.php");
require_once(MCP_LIB_PATH . "Theme.php");


class mc_Batch
{
	var $U;
	var $DB;

	var $LASTCHANGE;
	var $PFINFO;

	var $OS;
	var $CMD_PREFIX;
	var $CMD_QUOTE;

	var $MYSQL_BASEDIR="";
	var $MYSQL_DB="";
	var $MYSQL_CMD_USER="";
	var $MYSQL_MSG_USER="";
	var $MYSQL_CMD_PASS="";
	var $MYSQL_MSG_PASS="";

//	var $CMD_MYSQLDUMP="";
	var $CMD_MYSQL="";

	var $ERROR   = null;
	var $ERRINFO = array();

	var $File;
	var $DBStruct;
	var $Stack;
	var $Version;
	var $Config;

	var $detail_onclick;

	var $uch_info;

	var $header;
	var $prefix;
	var $footer;

	// for BC
	var $Conf;
	var $PVersion;

	/**
	 * constructor
	 */
	public function __construct(mc_U $U, $overwrite = false)
	{
		$this->U = $U;

		$this->init($overwrite);

		if ( $this->errorExists() )
		{
			$this->displayErrors();
			exit(0);
		}

	}



	public function init($overwrite = false)
	{
		$this->setError(MCP_ERR_UNKNOWN);

		$this->DB       = &$this->U->DB;
		$this->Version  = &$this->U->Version;
		$this->Config   = &$this->U->Config;
		$this->DBStruct = &$this->U->DBStruct;

		// for BC
		$this->Conf = &$this->Config;
		$this->PVersion = &$this->Version;

		$this->initHtml();

		$this->Log = $this->U->loadLog();

		$this->checkOS();

		if ( ! $this->initDbSettings() )
		{
			$this->setError(MCP_ERR_ON_DB_INIT);
			return;
		}

		$this->Stack = new Stack();

		$check = Check::checkModInfoPath($this->Config);
		if ($check !== true)
		{
			$this->setError(MCP_ERR_ON_MODINFO_CHECK);
			$this->ERRINFO = $check;
			return;
		}

		if ( ! $this->isTestEnabled() )
		{
			if (Check::checkPhpLibs($this->Config) )
			{
				SysMsg::showInfo('<br>'.patch_label_lib_check_ok.'<br><br>');
			}
			else
			{
				$this->setError(MCP_ERR_ON_LIB_CHECK);
				return;
			}
		}

		$this->matching_revision = Check::checkHeadRevision($this->Version);
		if ( ! empty($this->matching_revision) )
		{
			$this->setError(MCP_ERR_ALREADY_PATCHED);
			return;
		}


/*
 * TODO for mods:
 * match failes due to constuctor of version class using 
 * $last = $U->Config->getLastUpdate(); 
 * instead of matching against list of mods
 * 
 * probably needs an own batch-class?  
 */
		if ( ! $this->Version->matchesRequirements() )
		{
			$this->setError(MCP_ERR_REVISION_MISMATCH);
			$this->PFINFO = $this->getInfo(false);
			return;
		}

		$this->LASTCHANGE = $this->Version->getLastChange($this->Config);

		if ( ! $this->initFileHandling($overwrite) )
		{
			$this->setError(MCP_ERR_ON_FILE_INIT);
			return;
		}

		$this->detail_onclick = "switchBlock('%s', '".patch_label_show_details."', '".patch_label_hide_details."')";

		$this->setError(MCP_ERR_NONE);
	}



	/**
	 * isTestEnabled
	 */
	public function isTestEnabled() {
		return $this->Config->isTestEnabled();
	}



	/**
	 * initHtml
	 */
	private function initHtml()
	{
		$this->head  = file_get_contents(MCP_LAY_PATH.'head.htm');
		$this->head .= '
			<h2 style="text-align:center;">{headline}</h2>
			<span class="version">{version}</span>
			<span id="intro">{intro}</span>'
		;

		$replace = array(
			'{headline}' => patch_headline,
			'{intro}'    => patch_intro,
			'{version}'  => '',
			'{MC_CHAR_SET}' => MC_CHAR_SET,
			'{MC_ROOT_URI}' => MC_ROOT_URI,
			'{MCP_URI}'        => MCP_URI,
			'{MCP_IMG_URI}'    => MCP_IMG_URI,
			'{MCP_STYLE_URI}'  => MCP_STYLE_URI,
			'{MCP_SCRIPT_URI}' => MCP_SCRIPT_URI,
		);

		$this->head = $this->replace($this->head, $replace);

		$this->footer = file_get_contents(MCP_LAY_PATH.'footer.htm');

		return true;
	}



	/**
	 * displayForm
	 */
	public function displayForm ()
	{
		// Note: for switches look in section [UPDATE] of update/conf.ini
		$confMsg = $this->checkSwitches(true);

		$formSid = $this->U->Session->getFormSid();
		$updInfo = $this->getInfo(true);
		$updForm = $this->Config->getForm();
#		$modForm = $this->Config->getModForm();

		$replace = array(
			'{MC_ROOT_URI}' => MC_ROOT_URI,
			'{MCP_URI}'     => MCP_URI,
			'{MCP_IMG_URI}' => MCP_IMG_URI,
			'{MCP_FSID}' => $formSid,
			'{upd_info}' => $updInfo,
			'{upd_form}' => $updForm,
			'{conf_msg}' => $confMsg,
#			'{mod_form}' => $modForm,
			'{label_03}' => patch_label_03,
			'{label_05}' => patch_label_05,
			'{label_06}' => patch_label_06,
			'{label_11}' => patch_label_11,
			'{label_submit}' => patch_submit,
		);

		$bodyTmpl = MCP_LAY_PATH.'form.htm';

		echo $this->replace($this->head, $replace);
		echo $this->replace(file_get_contents($bodyTmpl), $replace);
		echo $this->replace($this->footer, $replace);

		return;
	}



	/**
	 * getMatchingRevisionDesc
	 */
	public function getMatchingRevisionDesc()
	{
		return $this->matching_revision['VERSION_DESC'];
	}



	/**
	 * setError
	 */
	private function setError($err) 
	{
		$this->ERROR = $err;
	}

	/**
	 * getError
	 */
	public function getError() 
	{
		return $this->ERROR;
	}

	/**
	 * isError
	 */
	public function isError($err) 
	{
		if ( $this->getError() == $err ) {
			return true;
		}
		else {
			return false;
		}
	}

	/**
	 * errorExists
	 */
	public function errorExists() {
		if ( $this->getError() == MCP_ERR_NONE ) {
			return false;
		}
		else {
			return true;
		}
	}



	/**
	 * displayError
	 */
	public function displayErrors()
	{
		echo $this->head;

		$error = $this->getError();

		switch($error)
		{
			case MCP_ERR_ALREADY_PATCHED :
				SysMsg::showInfo('release '.$this->Version->getCurrentName().' is already installed.');
				break;

			case MCP_ERR_REVISION_MISMATCH :
				SysMsg::showWarning(sprintf(patch_error_01, $this->Version->getRequirementsInfo(), ''));
				echo $this->PFINFO;
				break;

			case MCP_ERR_ON_DB_INIT :
				SysMsg::showError('error on database initialization');
				break;

			case MCP_ERR_ON_FILE_INIT :
				SysMsg::showError('error on file system initialization');
				break;

			case MCP_ERR_ON_LIB_CHECK :
				SysMsg::showError('<br>'.patch_label_lib_check_fail.'<br><br>');
				break;

			case MCP_ERR_ON_MODINFO_CHECK :
				SysMsg::showError('<br>'.sprintf(patch_label_modinfo_check_fail, $this->ERRINFO).'<br><br>');
				break;

			case MCP_ERR_UNKNOWN :
				SysMsg::showError('unknown error');
				break;

			case MCP_ERR_NONE :
			case '' :
				break;

			default :
				throw new Exception("ERROR STATE '{$error}' IS UNKNOWN !");
		}
	}



	/**
	 * checkOS
	 */
	public function checkOS()
	{
		// checking operating system
		$this->OS = PHP_OS;
		$this->CMD_QUOTE  = '';
		$this->CMD_PREFIX = '';
		if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN')
		{
			$this->CMD_QUOTE  = "\"";
			$this->CMD_PREFIX = "cmd /C ";
		}

		return true;
	}



	/**
	 * initDbSettings
	 */
	public function initDbSettings()
	{

		$this->MYSQL_DB = MC_DBNAME;
		$this->MYSQL_CMD_USER = MC_DBUSER;
		$this->MYSQL_MSG_USER = str_repeat("*", strlen(MC_DBUSER));

		if (empty($pwd) == false)
		{
			$this->MYSQL_CMD_PASS = "--password=".escapeshellcmd(MC_DBPASS);
			$this->MYSQL_MSG_PASS = "--password=".str_repeat("*", strlen(MC_DBPASS));
		}

		$MCP_MYSQL_BASEDIR = $this->Config->getDbServerBasedir();
		if ($MCP_MYSQL_BASEDIR === null)
		{
			// fetching mysql basedir
			$sel = 'show variables like "basedir"';
			$res = $this->DB->getRow($sel);

			if (isSet($res[1]))
			{
				$this->MYSQL_BASEDIR = $res[1];
			}

			if (empty($this->MYSQL_BASEDIR))
			{
		    SysMsg::showError(patch_err_no_basedir);
		    return false;
			}
		}
		else
		{
			$this->MYSQL_BASEDIR = $MCP_MYSQL_BASEDIR;
		}

		$MCP_MYSQL_PARAM = $this->Config->getDbServerParams();

		if ($MCP_MYSQL_PARAM === null)
		{
			$this->MYSQL_PARAM = '';
		}
		else
		{
			$this->MYSQL_PARAM = $MCP_MYSQL_PARAM.' ';
		}

		$this->CMD_MYSQL = $this->CMD_PREFIX.$this->CMD_QUOTE.$this->MYSQL_BASEDIR."bin/mysql ".$this->MYSQL_PARAM.$this->CMD_QUOTE;

		return true;
	}



	/**
	 * initFileHandling
	 */
	public function initFileHandling($overwrite)
	{
		$this->File = new File($overwrite);

		if ($this->File->IS_FILE_WRITABLE_ERROR == TRUE)
		{
			SysMsg::showError(patch_err_file_access);
			$this->STATE = MCP_ERR_ON_FILE_INIT;
		}

		/*if ( ! is_writable(MC_ROOT_PATH.'extern/Templates/ext_room_pages') )
		{
			SysMsg::showError(patch_err_extp_access);
			$this->STATE = MCP_ERR_ON_FILE_INIT;
		}*/

		return true;
	}



	/**
	 * backupTables
	 */
	private function backupTables ($updateDirname)
	{
		SysMsg::showMsg("running database backup. please hang on...");

		$arrTables = $this->Config->getBakTables($updateDirname);
		$backupPath = $this->Config->getBakPath();

		$sqlBackupPath = $backupPath . '/_database';
		if ( ! is_dir($sqlBackupPath) )
		{
			if ( ! mkdir($sqlBackupPath) )
			{
				throw new Exception("unable to create path '{$sqlBackupPath}'");
			}
			chmod($sqlBackupPath, 0700);
		}

		if (empty($arrTables) )
		{
			$arrTables = $this->DBStruct->getAllTableNamesAsArray();
		}

		foreach ($arrTables as $tableName)
		{
			$this->DBStruct->backupTableDefinition($tableName, $sqlBackupPath);
			$this->DBStruct->backupTableContent($tableName, $sqlBackupPath);
		}

		return true;
	}



	/**
	 * processBatch
	 */
	private function processBatch($nodeList, $updateDirname)
	{
		$this->Log->write("\n<hr />\n");

		if ( ! is_array($nodeList) )
		{
			throw new Exception("first parameter is not an array!");
		}

		$revisionPath = $this->Config->getUpdatePath($updateDirname);

		$fileList = array();
		$dirHandle = opendir($revisionPath);
		if ( ! $dirHandle )
		{
			throw new Exception("unable to open directory '{$revisionPath}'");
		}

		$checkFileName = $this->Config->getCheckFile();
		$actionsFileName = $this->Config->getActionFile();
		$descriptionFileName = $this->Config->getDescFile();

		while ( ($file = readdir($dirHandle)) !== false)
		{
			switch (true)
			{
				case ($file == ".")  : // local directory
				case ($file == "..") : // parent directory
				case ($file == $checkFileName) :
				case ($file == $actionsFileName) :
				case ($file == $descriptionFileName) :
				case (is_dir($revisionPath . $file)) :
					// skip directories
					continue 2; // switch, while

				default : break;
			}

			$fileList[$file] = true;
		}
		closedir($dirHandle);

		$checkFile = MCP_PATH . $checkFileName;
		if ( file_exists($checkFile) )
		{
			require_once($checkFile);
			if ( ! function_exists('mc_app_check') )
			{
				throw new Exception("function 'mc_app_check' does not exist!");
			}

			if ( ! mc_app_check() )
			{
				SysMsg::showError("application check failed!");
				return false;
			}
		}


		foreach ($nodeList as $idx => $fileName)
		{
			ob_start();
			set_time_limit(1800);

			$filePath = $revisionPath . DIRECTORY_SEPARATOR . $fileName;
			$fileSub  = $updateDirname . DIRECTORY_SEPARATOR . $fileName;

			if ( ! file_exists($filePath) )
			{
				$errMsg = "'$filePath' not found! (".__FILE__.":".__LINE__.")";
				SysMsg::showError($errMsg);
				continue;
			}

			if ( ! is_file($filePath) )
			{
				$errMsg = "'$filePath' is not a file! (".__FILE__.":".__LINE__.")";
				SysMsg::showError($errMsg);
				continue;
			}

			unset($fileList[$fileName]);

			$extension = pathinfo($fileName, PATHINFO_EXTENSION);

			switch ($extension)
			{
				case "sql" :
					$example = MCP_PATH . '#update.template/mps.example.mps';
					SysMsg::showError('execution of sql files is forbidden!');
					SysMsg::showTask('You need to use *.mps syntax instead!');
					SysMsg::showMsg('Please look at <b>'.$example.'</b> for information');
					die();

				case "php" :
					if ( is_numeric($idx) )
					{
						// simple file
						$this->executePhpFile($fileSub);
						continue;
					}
					// non-numeric index should indicate a view or room import

					// check for room import (ROOM_TYPE.ROOM_TYPE_UID)
					if (substr($idx, 0, 4) == 'http')
					{
						$roomtype = $this->DB->getOne(
							 'select ROOM_TYPE_IDENT from ROOM_TYPE'
							.' where ROOM_TYPE_UID = "'.$idx.'"'
							.' and ROOM_TYPE.STATE in ("Y", "S")'
						);

						if ( ! empty($roomtype) )
						{
							SysMsg::showInfo('import of room type '.$idx.' {SKIPPED} (room type already exists).');
							break;
						}
					}
					// check for view import
					else
					{
						$view = $this->DB->getOne(
							'select VIEW_IDENT from VIEW where VIEW_NAME = "'.$idx.'"'
						);

						if ( ! empty($view) )
						{
							SysMsg::showInfo('import of view '.$idx.' {SKIPPED} (view already exists).');
							break;
						}
					}
					$this->executePhpFile($fileSub);
					break;

				case "mps" :
					// *.mps = MetacoonPatchScript
					$this->executeMpsFile($fileSub);
					break;

				case "lay" :
//					$this->parseMcLayout($l_file_name);
					throw new Exception("method Batch::parseMcLayout is not implemented!");

				default :
					throw new Exception("extension of file '{$fileName}' is unknown to updater!");

			} // end switch ($l_fileinfo['extension'])

			$l_patch_log = ob_get_contents();
			ob_end_flush();
			flush();
			$this->Log->write($l_patch_log);

		} // end foreach ($l_dirlist)

		clearstatcache();

		if (count($fileList) > 0)
		{
			$msg  = "there are unused files inside directory $revisionPath! ";
			$msg .= "please check :<br>".implode('<br>', array_keys($fileList));
			SysMsg::showTask($msg);
			$this->Log->write("\n".$msg."\n");
		}

		$this->Log->write("\n<hr />\n");

		return true;
	}



	/**
	 * executePhpFile
	 */
	function executePhpFile($fileName)
	{
		$fileSubPath  = MCP_DIR . DIRECTORY_SEPARATOR . $fileName;
		$fileFullpath = MCP_PATH . $fileName;

		if ($this->isTestEnabled() )
		{
			SysMsg::showMsg("would include('[...]" . $fileSubPath . "');");
			return true;
		}

		$U        =  $this->U;
		$DB       = &$this->DB;
		$DBStruct = &$this->DBStruct;
		$Config   = &$this->Config;
		$File     = &$this->File;
		$Stack    = &$this->Stack;

		// @DEPRECATED (BC only)
		$l_DBStruct = &$DBStruct;
		$l_File  = &$File;
		$l_Stack = &$Stack;
		$l_Conf  = &$Config;
		$Conf    = &$Config;

		$date = '['.date('H:i:s').']';
		$domId = 'file_' . md5($fileName);
		$onclick = sprintf($this->detail_onclick, $domId);

		echo '
<div class="fileblock">';

		SysMsg::showAction("$date ".patch_msg_exec_php." <b>$fileSubPath</b>");

		echo '
	<a name="ankh_'.$domId.'_start"></a>
	<a class="toggle_block" href="#ankh_'.$domId.'" id="'.$domId.'_link" onclick="'.$onclick.'">'.patch_label_hide_details.'</a>
	<div id="'.$domId.'">';

		include_once($fileFullpath);

		echo '
		<br /><a class="toggle_block" href="#ankh_'.$domId.'_start">'.patch_label_section_top.'</a>
	</div>
</div>';

		return true;
	} // end function executePhpFile($p_file_name)



	/**
	 * executeMpsFile
	 */
	function executeMpsFile($fileName)
	{
		$fileSubPath  = MCP_DIR . DIRECTORY_SEPARATOR . $fileName;
		$fileFullpath = MCP_PATH . $fileName;

		if ($this->isTestEnabled() )
		{
			SysMsg::showMsg('would load and parse file "[...]'.$fileSubPath.'"');
			return true;
		}

		$date = '['.date('H:i:s').']';
		$domId = 'mps_'.md5($fileName);
		$onclick = sprintf($this->detail_onclick, $domId);

		echo '
<div class="mpsblock">';

		SysMsg::showAction("$date ".patch_msg_exec_mps." <b>$fileSubPath</b>");

		echo '
	<a name="ankh_'.$domId.'_start"></a>
	<a class="toggle_block" href="#ankh_'.$domId.'" id="'.$domId.'_link" onclick="'.$onclick.'">'.patch_label_hide_details.'</a>
	<div id="'.$domId.'">';

		$MpsParser = new MpsParser($fileFullpath);
		$state = $MpsParser->execute();

		echo '
		<br /><a class="toggle_block" href="#ankh_'.$domId.'_start">'.patch_label_section_top.'</a>
	</div>
</div>';

		return $state;
	} // end function executeMpsFile($p_file_name)



	/**
	 * updateVersion
	 */
	function updateVersion($updateDirname)
	{
		if ($this->isTestEnabled() )
		{
			return true;
		}

		if (empty($this->Version) )
		{
			throw new Exception('version object is empty');
		}

		$source = $this->Config->getUpdateInfoPath($updateDirname);
		$dest = MCP_FILE_DEST . MCP_INFO_SRC_DIR . DIRECTORY_SEPARATOR . $updateDirname . DIRECTORY_SEPARATOR;

		if ($this->File->rcopy($source, $dest) != true)
		{
			// copy contents of update direactory (default: modifications/*)
			SysMsg::showWarning("information about changes is not available");
			return false;
		}

		if ( ! $this->Version->writeVersion( $this->Config) )
		{
			SysMsg::showError("version table update {FAILED}");
			return false;
		}

		$updateName = $this->Config->getCurrentDetailsParam('name');
		$updateDesc = $updateName.' ('.$this->Config->getUpdateDestinedName().')';

		SysMsg::showInfo(sprintf(patch_label_14, $updateDesc));

		return true;
	} // end method updateVersion



	/**
	 * execute
	 */
	function execute($cmd)
	{
		$out   = array();
		$error = false;

		exec($cmd, $out, $error); //or $alert=ob_get_contents();

		if ($error || $out)
		{
			if ($error)
			{
				SysMsg::showError('cmd line error code : '.$error);
			}
			if ($out)
			{
				SysMsg::showError(implode('<BR />', $out));
			}
			return false;
		}

		return true;
	} // end method execute



	/**
	 * updateFiles
	 */
	function updateFiles($updateDirname)
	{
		$arrMovedFiles = $this->Config->getMovedFiles($updateDirname);
		if ($arrMovedFiles)
		{
			ob_start();

			echo '
<div class="fileblock">';

			// move files/directories (with backup)
			SysMsg::showAction(patch_msg_file_move."</b>");
			$domId = str_replace('.', '', "move_path_{$updateDirname}");

			echo '
	<a class="toggle_block" href="#ankh_'.$domId.'" id="'.$domId.'_link"'
	.' onclick="'.sprintf($this->detail_onclick, $domId).'">'.patch_label_hide_details.'</a>
	<div id="'.$domId.'">';

			$this->File->move($arrMovedFiles);
			clearstatcache();

			echo '
	</div>
	<a href="ankh_'.$domId.'"></a>
</div>';

			$this->Log->write(ob_get_contents());
			ob_end_flush();
			flush();
		}


		ob_start();
		echo '
<div class="fileblock">';

		// update files (with backup)
		SysMsg::showAction(patch_msg_file_update."</b>");
		$domId = str_replace('.', '', "update_files_{$updateDirname}");

		echo '
	<a class="toggle_block" href="#ankh_'.$domId.'" id="'.$domId.'_link"'
	.' onclick="'.sprintf($this->detail_onclick, $domId).'">'.patch_label_hide_details.'</a>
	<div id="'.$domId.'">';

		$updatePath = $this->getFileSource($updateDirname);
		if ($updatePath)
		{
			$bakPath = $this->Config->getBakPath();
			$this->File->update($updatePath, MC_BASE_DIR, $bakPath);
		}

		$Theme = new Theme($updateDirname);
		if ($Theme->directoryExists() )
		{
			$Theme->updateThemeFiles();
		}

		clearstatcache();

		echo '
		<a class="toggle_block" href="#ankh_'.$domId.'" onclick="'.sprintf($this->detail_onclick, $domId).'">'.patch_label_hide_details.'</a>
	</div>
	<a href="ankh_'.$domId.'"></a>
</div>';

		$this->Log->write(ob_get_contents());
		ob_end_flush();
		flush();


		$arrDeletedFiles = $this->Config->getDeletedFiles($updateDirname);
		if ($arrDeletedFiles)
		{
			ob_start();
			echo '
<div class="fileblock">';

			// delete files/directories (with backup)
			SysMsg::showAction(patch_msg_file_delete."</b>");
			$domId = str_replace('.', '', "delete_path_{$updateDirname}");

			echo '
	<a class="toggle_block" href="#ankh_'.$domId.'" id="'.$domId.'_link"'
	.' onclick="'.sprintf($this->detail_onclick, $domId).'">'.patch_label_hide_details.'</a>
	<div id="'.$domId.'">';

			$this->File->delete($arrDeletedFiles);
			clearstatcache();

			echo '
	</div>
	<a href="ankh_'.$domId.'"></a>
</div>';

			$this->Log->write(ob_get_contents());
			ob_end_flush();
			flush();
		}

		$this->uch_info = $this->File->getAffectedCustomFileChanges();
	}



	/**
	 *
	 */
	private function getFileSource($updateDirname)
	{
		$updatePath = $this->Config->getUpdatePath($updateDirname);

		$srcPath = $updatePath . MCP_FILE_SRC_DIR;

		if ( ! @is_dir($srcPath) )
		{
			SysMsg::showInfo(patch_msg_file_update_skip);
			return false;
		}

		if ( ! @is_readable($srcPath) )
		{
			throw new Exception("directory {$srcPath} exists but is NOT readable!");
		}

		return $srcPath;
	}



	/**
	 * showFileState
	 */
	private function showFileState()
	{
		ob_start();

		if ( $this->isTestEnabled() ) {
//			SysMsg::showInfo(patch_label_???); // nothing done, just testing
			$msg = patch_test_label_01."<p />\n".$this->uch_info;
		}
		else {
			SysMsg::showInfo(patch_label_01, false, true); // all done, no-header, show-big
			$msg = $this->uch_info."\n".patch_label_07."<p />\n".patch_label_08.$this->Config->getBakPath();
		}

		echo '<hr />';
		SysMsg::showMsg($msg);
		echo '<hr />';

		SysMsg::showMsg(patch_label_logfile.': <a target="_new" class="link" href="'.$this->Log->getLogFullUri().'">'.$this->Log->getLogFullUri().'</a>');

		if ( ! mc_Session::isAdmin() )
		{
			echo "<p>\n".'<a href="'.MC_ROOT_URI.'" target="_blank"><button>'.patch_button_back.'</button></a>'."\n</p>\n";
		}

		$this->Log->write(ob_get_contents());
		ob_end_flush();
		flush();

		return;
	}



	/**
	 * getInfo
	 */
	function getInfo($p_match_state)
	{
		$V = &$this->Version;
		$requiredName = $V->getReqVersion()    .'.R'.$V->getReqRevision()    .'.B'.$V->getReqBuild();
		$currentName  = $V->getCurrentVersion().'.R'.$V->getCurrentRevision().'.B'.$V->getCurrentBuild();
		$destinedName = $V->getVersion()       .'.R'.$V->getRevision()       .'.B'.$V->getBuild();

		$arrCheck = array();
		if (version_compare($currentName, $requiredName, "<") )
		{
			$arrCheck['current'] = $currentName;
			$arrCheck['required'] = $requiredName;
			$arrCheck['destined'] = $destinedName;
		}
		else if (version_compare($currentName, $destinedName, ">") )
		{
			$arrCheck['required'] = $requiredName;
			$arrCheck['destined'] = $destinedName;
			$arrCheck['current']  = $currentName;
		}
		else
		{
			$arrCheck['required'] = $requiredName;
			$arrCheck['current']  = $currentName;
			$arrCheck['destined'] = $destinedName;
		}

		$info = '
<table class="update_info" cellpadding="2" cellspacing="2">
	<tr>
		<td colspan="2"></td>
		<td valign="middle">Version</td>
		<td valign="middle">Revision</td>
		<td valign="middle">Build</td>
	</tr>';

		foreach($arrCheck as $l_key => $l_val)
		{
			switch ($l_key)
			{
				case 'required' :
					$info .= '
	<tr>
		<td valign="middle">required version</td>
		<td valign="middle">:</td>
		<td valign="middle" align="right">'.$V->getReqVersion().'</td>
		<td valign="middle" align="right">'.$V->getReqRevision().'</td>
		<td valign="middle" align="right">'.$V->getReqBuild().'</td>
	</tr>';
					break;

				case 'current' :
					$info .= '
	<tr style="color:'.($p_match_state ? 'green' : 'red').';">
		<td valign="middle">installed version</td>
		<td valign="middle">:</td>
		<td valign="middle" align="right">'.$V->getCurrentVersion().'</td>
		<td valign="middle" align="right">'.$V->getCurrentRevision().'</td>
		<td valign="middle" align="right">'.$V->getCurrentBuild().'</td>
	</tr>';
					break;

				case 'destined' :
					$info .= '
	<tr>
		<td valign="middle">will be updated to</td>
		<td valign="middle">:</td>
		<td valign="middle" align="right">'.$V->getVersion().'</td>
		<td valign="middle" align="right">'.$V->getRevision().'</td>
		<td valign="middle" align="right">'.$V->getBuild().'</td>
	</tr>';
					break;

				default :
					break;
			} // end switch
		} // end foreach

		$info .= '
</table>';

		return $info;
	} //  end function getInfo



	/**
	 * process
	 */
	function process()
	{
		global $U;

		echo $this->head;

		$this->Log->write($this->head);
		$info = '
<table>
	<tr><td>Patch</td><td>:</td><td>'.$this->U->Version->getModName().'</td></tr>
	<tr><td>Page </td><td>:</td><td>'.MC_URL.'</td></tr>
</table>
<hr />';
		$this->Log->write($info);

		$currentVersion  = $this->Version->getCurrentVersion();
		$currentRevision = $this->Version->getCurrentRevision();
		$currentBuild    = $this->Version->getCurrentBuild();
		$currentName = "{$currentVersion}.R{$currentRevision}.B{$currentBuild}";

		$arrBatch = $this->Config->getScriptList();

		$this->checkSwitches();

		$this->File->createBackupPath();

		foreach ($arrBatch as $updateDirname => $arrScript)
		{
			$destinedName = $this->Config->getUpdateDestinedName($updateDirname);
			if (version_compare($destinedName, $currentName, "<="))
			{
				$msg = "update to version <b>$destinedName</b> {SKIPPED} (already installed)";
				SysMsg::showInfo($msg);
				$this->Log->write($msg);
				continue;
			}

			$requiredEnvironmentVersion = $this->Config->getUpdateRequiredEnvironmentVersion($updateDirname);
			if (MCP_ENVIRONMENT_VERSION != $requiredEnvironmentVersion)
			{
				SysMsg::showError("required environment version {$requiredEnvironmentVersion} of update '{$updateDirname}' does not match the current environment version ".MCP_ENVIRONMENT_VERSION);
				SysMsg::showTask("please check the requirements and bundle settings");
				exit(0);
			}

			$bundle = $this->Config->getUpdateBundle($updateDirname);
			if (empty($lastBundle) )
			{
				$lastBundle = $bundle;
			}
			else if ($bundle != $lastBundle)
			{
				$msg = "update to version <b>$destinedName</b> {SKIPPED} (belongs to other bundle)";
				SysMsg::showInfo($msg);
				$this->Log->write($msg);
				continue;
			}

			$verification_src = MCP_PATH.$updateDirname.DIRECTORY_SEPARATOR.'_verification.php';
			if (file_exists($verification_src) )
			{
				SysMsg::showMsg("Checking for application issues that may break the update process. Hang on...");
				$state = include_once($verification_src);
				if ($state == FALSE)
				{
					SysMsg::showError("Update runs only if all application issues are solved (see list above). Sorry.");
					exit(0);
				}
				else
				{
					SysMsg::showInfo("Check done. All fine, proceeding update...");
				}
			}

			$this->Config->setCurrentUpdate($updateDirname);

			$this->createBakPath( $this->Config );

			SysMsg::showAction("<h3>Processing updates for version <b>$destinedName</b></h3>hang on ...");

			$this->setError(MCP_ERR_NONE);

			$this->DBStruct->refreshStructures();

			// backup database tables
			if ($this->isDbBackupEnabled() )
			{
				$this->backupTables($updateDirname);
			}

			// update files (with backup)
			if ($this->isFileUpdateEnabled() )
			{
				$this->updateFiles($updateDirname);
			}

            $state = null;
            
			// execute additional php-scripts and sql-script
			if ($this->isBatchExecutionEnabled() )
			{
				$state = $this->processBatch($arrScript, $updateDirname);
			}

			if ($state === FALSE || $this->errorExists() )
			{
				SysMsg::showError('Error detected. Update halted.');
				SysMsg::showTask('Please check the log messages above for more information, fix the problem and run update again.');
				exit(0);
			}

			// show state of revision/build update
			if ($this->isRegisterVersionEnabled())
			{
				$this->updateVersion($updateDirname);
			}

			SysMsg::showAction("<p>... Done. (updated to version <b>$destinedName</b></p>");
		} // end foreach update

		if ($this->isRegisterVersionEnabled() )
		{
			// show final update state
			$this->showFileState();
		}

		$chkUpdFile = 'checkupdate.php';
		$chkUpdPath = MC_BASE_DIR . 'admin' . DIRECTORY_SEPARATOR . $chkUpdFile;
		if ( $this->isTestEnabled() && file_exists($chkUpdPath) ) {
			$link = MC_URL . 'admin/' . $chkUpdFile . '?' . MC_SESSION_NAME_INT . '=' . session_id();
			echo '<span><a href="'.$link.'" target="_parent">[return to update checker]</a></span>';
		}

		$this->Log->write('</body></html>');

		echo $this->footer;

		return true;
	} // end method process



	/**
	 * checkSwitches
	 */
	function checkSwitches($returnOutput = false)
	{
		if ($returnOutput) {
			ob_start();
		}

		$msg_suffix = '<br><nobr>(for presets look at file "update/conf.ini", section [UPDATE], option "%s")</nobr>';

		if ( ! $this->isDbBackupEnabled() ) {
			$msg  = '<b>Database Backup will NOT be created!</b> Check Settings!';
			$msg .= sprintf($msg_suffix, 'backup_database');
			SysMsg::showWarning($msg);
		}

		if ( ! $this->isFileUpdateEnabled() ) {
			$msg  = '<b>Filesystem will NOT be updated!</b>';
			$msg .= sprintf($msg_suffix, 'replace_files');
			SysMsg::showWarning($msg);
		}

		if ( ! $this->isBatchExecutionEnabled() ) {
			$msg  = '<b>Mod-scripts will NOT be processed!</b>';
			$msg .= sprintf($msg_suffix, 'run_actions');
			SysMsg::showWarning($msg);
		}

		if ( ! $this->isRegisterVersionEnabled() ) {
			$msg  = '<b>The new version/revision/build will NOT be registered!</b> Check Settings!';
			$msg .= sprintf($msg_suffix, 'register_version');
			SysMsg::showWarning($msg);
		}

		if ($returnOutput) {
			$output = ob_get_clean();
			return $output;
		}

		return;
	} // end method checkSwitches



	/**
	 * getModForm
	 */
	function getModForm()
	{
		throw new Exception("mod feature not implemented yet!");
/*

		if (empty($this->Config->_MODS) ) {
			return '';
		}

		$str = '';

		foreach ($this->Config->_MODS as $modRevision => $modBuilds)
		{
			foreach ($modBuilds as $modBuild => $mod)
			{
//				dump($mod);
				SysMsg::showTask('mod "'.$mod['DETAILS']['product'].'" is registered but will not be processed! (feature is not implemented yet)');
//				$str .= '';
		}}

		return $str;
*/
	} // end method getModForm



	/**
	 * createBakPath
	 */
	function createBakPath($Config)
	{
		$bakPath = $Config->getBakPath();

		if ( @mkdir($bakPath, 0777, true) ) {
			return true;
		}

		return false;
	} // end method createBakPath


	/**
	 * isDbBackupEnabled
	 */
	function isDbBackupEnabled()
	{
		return $this->Config->isEnabled('backup_database');
	} // end method isDbBackupEnabled


	/**
	 * isFileUpdateEnabled
	 */
	function isFileUpdateEnabled()
	{
		return $this->Config->isEnabled('replace_files');
	} // end method isFileUpdateEnabled


	/**
	 * isBatchExecutionEnabled
	 */
	function isBatchExecutionEnabled()
	{
		return $this->Config->isEnabled('run_actions');
	} // end method isBatchExecutionEnabled


	/**
	 * isRegisterVersionEnabled
	 */
	function isRegisterVersionEnabled()
	{
		return $this->Config->isEnabled('register_version');
	} // end method isRegisterVersionEnabled


	/**
	 * replace
	 */
	private function replace($mixed, $replace, $pattern=null)
	{
		if ($pattern === null) {
			$pattern = array_keys($replace);
		}

		if ( is_array($mixed) ) {
			foreach($mixed as $idx => $value) {
				$value = $this->replace($value, $replace, $pattern);
				$mixed[$idx] = $value;
			}
			return $mixed;
		}

		$mixed = str_replace($pattern, $replace, $mixed);

		return $mixed;
	} // end method replace


} //  end class mc_Batch

