<?php
/**
 * @package core
 */
/*
* $McLicense$
*
* $Id$
*
*/



/**
* contains all methods for configuration
*
* @author	Steffen Gross <gross@metacoon.net>
* @package core
* @subpackage classes.new
*/
class mc_Config
{
	private $DB;

	private $arrInstalledLanguages;

	private $sysconfig;

	private $testState = true;


	/**
	 * Constructor
	 */
	function __construct($DB)
	{
		$this->DB = $DB;

		$this->arrInstalledLanguages  = array();
		$this->sysconfig = array();

		if (file_exists(MC_BASE_DIR . 'conf/extlib.conf.php') )
		{
			// php 5.x
			$this->load();
		}
		else
		{
			// PHP 4.x
			// DEFAULT_LANG was numeric under PHP 4!
			if ( empty($GLOBALS['DEFAULT_LANG']) ) 
			{
				$defLang = 1;
			}
			else 
			{
				$defLang = intval($GLOBALS['DEFAULT_LANG']);
			}
			$this->sysconfig['DEFAULT_LANG_ID'] = $defLang;
			$this->sysconfig['DEFAULT_LANG'] = ($defLang == 1) ? 'DE' : 'EN';

			if ( empty($GLOBALS['USE_EMAIL_REGISTRATION']) ) 
			{
				$this->sysconfig['USE_EMAIL_REGISTRATION'] = 0;
			}
			else 
			{
				$this->sysconfig['USE_EMAIL_REGISTRATION'] = 1;
			}
			$this->sysconfig['STRIP_TAGS_EXCEPTIONS'] = '<div><p><span><br><em><strong><code><samp><kbd><var><cite><dfn><abbr><acronym><q><font><h1><h2><h3><h4><h5><h6><b><i><u><s><big><small><strike><sup><sub><tt><a><ol><ul><li><pre><hr><img><table><tr><td>';
			$this->sysconfig['DEFAULT_TIMEZONE'] = 'Europe/Berlin';
			$this->sysconfig['SESSION_MAX_LIFETIME'] = ini_get('session.gc.maxlifetime');
			$this->sysconfig['SESSION_MAX_USER_MULTIPLE'] = 0;
		}

		if (empty($this->sysconfig) )
		{
			//throw new Exception('no system config parameters found');
		}

		return true;
	}



	/**
	 *
	 */
	public function getLangMap()
	{
		$res = $this->DB->query('select LANGUAGE_ID, LANGUAGE_SHORT from LANGUAGE where STATE = "Y"');

		$this->arrInstalledLanguages = array();

		while ($row = $res->fetchRow())
		{
			$langId    = intval($row[0]);
			$langShort = trim($row[1]);
			$this->arrInstalledLanguages[$langId]    = $langShort;
			$this->arrInstalledLanguages[$langShort] = $langId;
		}
		$res->free();

		if (empty($this->arrInstalledLanguages) )
		{
			throw new Exception('no active language packs found');
		}

		return $this->arrInstalledLanguages;
	}



	/**
	 *
	 */
	private function load()
	{
		/*$query = 'select MC_CONFIG_NAME, MC_CONFIG_TYPE, MC_CONFIG_VALUE'
			.' from MC_CONFIG'
			.' where STATE = "Y"'
		;
		$res = $this->DB->query($query);

		$this->sysconfig = array();

		while ($row = $res->fetchRow() )
		{
			switch($row[1])
			{
				case 'INT':
					$this->sysconfig[$row[0]] = intval($row[2]);
					break;

				case 'BOOL':
					$this->sysconfig[$row[0]] = (($row[2]) ? true : false);
					break;

				case 'ARRAY':
					$this->sysconfig[$row[0]] = explode(',', ($row[2]));
					break;

				default:
					$this->sysconfig[$row[0]] = $row[2];
					break;
			}
		}
		$res->free();

		return true;*/
	}



	/**
	 *
	 */
	public function get($paramName)
	{
		if(!array_key_exists($paramName, $this->sysconfig))
        throw new Exception('index ' . $paramName . ' does not exist in sysconfig');
        return $this->sysconfig[$paramName];
	}



	/**
	 *
	 */
	public function getAll()
	{
		return $this->sysconfig;
	}



	/**
	 *
	 */
	public function exists($paramName)
	{
		return (isset($this->sysconfig[$paramName]) ? false : true);
	}



	/**
	 *
	 */
	public function notExists($paramName)
	{
		return (isset($this->sysconfig[$paramName]) ? true : false);
	}



	/**
	 *
	 */
	public function isEnabled($paramName)
	{
		return (empty($this->sysconfig[$paramName]) ? false : true);
	}



	/**
	 *
	 */
	public function isDisabled($paramName)
	{
		return (empty($this->sysconfig[$paramName]) ? true : false);
	}



	/**
	 *
	 */
	public function isTestEnabled()
	{
		return $this->testState;
	}

	/**
	 *
	 */
	public function enableTest()
	{
		$this->testState = true;
	}

	/**
	 *
	 */
	public function disableTest()
	{
		$this->testState = false;
	}



}

