<?php
/*
* $McLicense$
*
* $Id$
*
*/


require_once("MDB2.php"); // from PEAR

define('DBMC_FETCHMODE_DEFAULT', MDB2_FETCHMODE_DEFAULT);
define('DBMC_FETCHMODE_ORDERED', MDB2_FETCHMODE_ORDERED);
define('DBMC_FETCHMODE_ASSOC',   MDB2_FETCHMODE_ASSOC);
define('DBMC_FETCHMODE_OBJECT',  MDB2_FETCHMODE_OBJECT);
define('DBMC_FETCHMODE_FLIPPED', MDB2_FETCHMODE_FLIPPED);



/**
 * wrapper for PEAR DB class
 *
 * @author	Steffen Gross <gross@metacoon.net>
 * @package 	classes.new
 */
class DBMC
{
	/**
	 * enable / disable database logging
	 */
	private $log = false;

	/**
	 * database connection
	 */
	private $DB;

	/**
	 * constructor
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 * @package 	admin
	 */
	private function __construct($DB)
	{
		if (mc_Debug::isError($DB))
		{
			exit(0);
		}

		$this->DB = $DB;

		$time = date("Y-m-d_H\hi\ms\s");

		$logSrc = dirname(dirname(dirname(__FILE__))) . "/_LOG/db_full_{$time}.log";
		$logSrc = str_replace('/', DIRECTORY_SEPARATOR, $logSrc);

		$this->log = fopen($logSrc, 'a');
		if ( ! $this->log)
		{
			throw new Exception("unable to write database log");
		}
	}


	/**
	 * log
	 *
	 * logs text into logfile
	 */
	function log($logText="finished")
	{
		if ( ! $this->log)
		{
			return;
		}

		$time = date("r");
		fwrite($this->log, "[{$time}] {$logText}\n");
	}


	/**
	 *
	 */
	public static final function &connect($DSN=null)
	{
		if (empty($DSN) ) {
			$DSN = DSN;
		}

		$MDB2 = MDB2::connect($DSN, false);
		if ( empty($MDB2->connection) ) {
			throw new mc_Exception( $MDB2, 'DB connection failed' );
		}

		$DBMC = new DBMC($MDB2);

		DBMC::configureConnection($DBMC);

		return $DBMC;
	}


	/**
	 *
	 */
	public static final function configureConnection($dbObj)
	{
		global $SQL_MODE;

		$dbObj->setOption('portability', MDB2_PORTABILITY_ALL ^ MDB2_PORTABILITY_EMPTY_TO_NULL ^ MDB2_PORTABILITY_FIX_CASE);
		$dbObj->query('set NAMES utf8');

		if(empty($SQL_MODE))
		{
			return true;
		}

		$dbObj->query("set sql_mode = '{$SQL_MODE}'");

		return true;
	}


	/**
	 *
	 */
	function setOption($name, $value)
	{
		$result = $this->DB->setOption($name, $value);
		$this->checkResult($result);
		return $result;
	}


	/**
	 *
	 */
	function isError(&$object)
	{
		return MDB2::isError($object);
	}


	/**
	 *
	 */
	function checkResult(&$object)
	{
		if (MDB2::isError($object) )
		{
			throw new DbQueryException( $object, $object->getUserInfo() );
		}
	}



	/**
	 *
	 */
	public function &insert($table, $arrFields, $quote_values=TRUE, $quote_identifiers=FALSE)
	{
		$arrIdentifier = array();
		$arrValues = array();

		if ($quote_values == FALSE && $quote_identifiers == FALSE)
		{
			$strIdentifier = implode(', ', array_keys($arrFields) );
			$strValues = implode(', ', $arrFields);
		}
		else if ($quote_identifiers == FALSE)
		{
			foreach ($arrFields as $identifier => $value)
			{
				$arrValues[] = $this->quote($value);
			}
			$strIdentifier = implode(', ', array_keys($arrFields) );
			$strValues = implode(', ', $arrValues);
		}
		else if ($quote_values == FALSE)
		{
			foreach ($arrFields as $identifier => $value)
			{
				$arrIdentifier[] = $this->quoteIdentifier($identifier);
			}
			$strIdentifier = implode(', ', $arrIdentifier);
			$strValues = implode(', ', $arrFields);
		}
		else // both are TRUE
		{
			foreach ($arrFields as $identifier => $value)
			{
				$arrIdentifier[] = $this->quoteIdentifier($identifier);
				$arrValues[] = $this->quote($value);
			}
			$strIdentifier = implode(', ', $arrIdentifier);
			$strValues = implode(', ', $arrValues);
		}

		$query = "insert into {$table} ({$strIdentifier}) values ({$strValues})";

		return $this->query($query);
	}



	/**
	 *
	 * from MDB2::mixed &query( string $query, [mixed $types = null], [mixed $result_class = true], [mixed $result_wrap_class = false])
	 */
	function &query($query, $types = null, $result_class = true, $result_wrap_class = false)
	{
		$this->log($query);
		$result = $this->DB->query($query, $types, $result_class, $result_wrap_class);
		$this->checkResult($result);
		$this->log();

		return $result;
	}


	/**
	 * method is deprecated (from deprecated package Pear::DB)
	 */
	function getOne($query)
	{
		$this->log($query);
		$result = $this->DB->queryOne($query);
		$this->checkResult($result);
		$this->log();

		return $result;
	}


	/**
	 *
	 * from MDB2::mixed queryOne( string $query, [string $type = null], [int $colnum = 0])
	 */
	function queryOne($query, $type = null, $column = 0)
	{
		$this->log($query);
		$result = $this->DB->queryOne($query, $type, $column);
		$this->checkResult($result);
		$this->log();

		return $result;
	}


	/**
	 * method is deprecated (from deprecated package Pear::DB)
	 */
	function getCol($query)
	{
		$this->log($query);
		$result = $this->DB->queryCol($query);
		$this->checkResult($result);
		$this->log();

		return $result;
	}


	/**
	 *
	 * from MDB2::mixed queryCol( string $query, [string $type = null], [int $colnum = 0])
	 */
	function queryCol($query, $type = null, $column = 0)
	{
		$this->log($query);
		$result = $this->DB->queryCol($query, $type, $column);
		$this->checkResult($result);
		$this->log();

		return $result;
	}


	/**
	 * method is deprecated (from deprecated package Pear::DB)
	 */
	function getRow($query, $fetchmode = DBMC_FETCHMODE_DEFAULT)
	{
		$this->log($query);
		$result = $this->DB->queryRow($query, null, $fetchmode);
		$this->checkResult($result);
		$this->log();

		return $result;
	}


	/**
	 *
	 * from MDB2::mixed queryRow( string $query, [array $types = null], [int $fetchmode = MDB2_FETCHMODE_DEFAULT])
	 */
	function queryRow($query, $types = null, $fetchmode = DBMC_FETCHMODE_DEFAULT)
	{
		$this->log($query);
		$result = $this->DB->queryRow($query, $types, $fetchmode);
		$this->checkResult($result);
		$this->log();

		return $result;
	}


	/**
	 * method is deprecated (from deprecated package Pear::DB)
	 */
	function getAll($query, $fetchmode = DBMC_FETCHMODE_DEFAULT)
	{
		$this->log($query);
		$result = $this->DB->queryAll($query, null, $fetchmode);
		$this->checkResult($result);
		$this->log();

		return $result;
	}


	/**
	 *
	 * from MDB2::mixed queryAll( string $query, [array $types = null], [int $fetchmode = MDB2_FETCHMODE_DEFAULT], [bool $rekey = false], [bool $force_array = false], [bool $group = false])
	 */
	function queryAll($query, $types = null, $fetchmode = DBMC_FETCHMODE_DEFAULT, $rekey = false, $force_array = false, $group = false)
	{
		$this->log($query);
		$result = $this->DB->queryAll($query, $types, $fetchmode, $rekey, $force_array, $group);
		$this->checkResult($result);
		$this->log();

		return $result;
	}



	/**
	 *
	 */
	function getLastInsertId()
	{
		$result = $this->DB->lastInsertID();
		$this->checkResult($result);

		return ( intval($result) );
	}



	/**
	 *
	 * from MDB2::mixed &prepare( string $query, [mixed $types = null], [mixed $result_types = null], [mixed $lobs = array()])
	 */
	function &prepare($query)
	{
		$this->log($query);
		$result = $this->DB->prepare($query);
		$this->checkResult($result);
		$this->log();

		return $result;
	}



	/**
	 *
	 */
	function execute($prepare, $values)
	{
		$this->log("{$values} => prepared query {$prepare}");
		$result = $prepare->execute($values);
		$this->checkResult($result);
		$this->log();

		return $result;
	}



	/**
	 * from MDB2::string quote( string $value, [string $type = null], [bool $quote = true], [bool $escape_wildcards = false])
	 */
	function quote($value, $type = null, $quote = true, $escape_wildcards = false)
	{
		return $this->DB->quote($value, $type, $quote, $escape_wildcards);
	}



	/**
	 * from MDB2::string quoteIdentifier($str, $check_option = false)
	 */
	public function quoteIdentifier($identifier, $checkOption = false)
	{
		return $this->DB->quoteIdentifier($identifier, $checkOption);
	}



	/**
	 * Returns the number of rows affected
	 *
	 * @param resource $result
	 * @return mixed MDB2 Error Object or the number of rows affected
	 * @access private
	 */
	public function getAffectedRows($result)
	{
		$affectedRows = $this->DB->_affectedRows(null, $result);
		if ( ! is_numeric($affectedRows) )
		{
			throw new Exception("DB::_affectedRows did not return a numeric value! ({$affectedRows})");
		}

		return $affectedRows;
	}



	/**
	 * fetch last query from DB_result
	 */
	function getQuery($result)
	{
		$class_name = get_class($result);
		switch($class_name)
		{
			case 'MDB2_BufferedResult_mysql' :
				return $result->db->last_query;

			default :
				break;
		}

		return $result;
	}



}

