<?php
/*
* $McLicense$
*
* $Id$
*
*/


class DBStruct
{
	var $DB;

	var $ALL_TABLE_STRUCTURES;

	var $RESERVED_WORDS;

	var $ALL_TABLE_INDEXES;
	var $NEW_TABLE_INDEXES;

	var $OPTIMIZED_TABLES;

	private $backup_path;

	function __construct()
	{
		global $U;

		$this->DB = &$U->DB;

		$this->RESERVED_WORDS = array('RELEASE');

		$this->fetchAllTableStructures();

		global $_ALL_TABLE_INDEXES;
		global $_NEW_TABLE_INDEXES;

		$this->ALL_TABLE_INDEXES = &$_ALL_TABLE_INDEXES;
		$this->NEW_TABLE_INDEXES = &$_NEW_TABLE_INDEXES;

		$this->fetchAllTableIndexes();

		if (empty($_NEW_TABLE_INDEXES))
		{
			$_NEW_TABLE_INDEXES = array();
		}

		$this->OPTIMIZED_TABLES = array();

		return true;
	}


	// alias for refreshStructures()
	function refresh() 
	{
		return $this->refreshStructures();
	}

/*
	private function setBackupPath($path)
	{
		$this->backup_path = $path;
	}

	private function getBackupPath()
	{
		if (empty($this->backup_path) )
		{
			throw new Exception("In order to change database structures you need to specify a backup path (DBStruct::setBackupPath)");
		}

		return $this->backup_path;
	}
*/
	function refreshStructures()
	{
		$this->ALL_TABLE_STRUCTURES = array();
		if ( !$this->fetchAllTableStructures() )
		{
			throw new Exception(__CLASS__.'::fetchAllTableStructures failed!');
		}

		$this->ALL_TABLE_INDEXES = array();
		$this->fetchAllTableIndexes();

		return true;
	}



	function fetchAllTableIndexes($l_show_primary = false)
	{
		$this->ALL_TABLE_INDEXES = $this->getTableIndexes(null, $l_show_primary);
		if ($this->ALL_TABLE_INDEXES === false)
		{
			throw new Exception(__CLASS__.'::getTableIndexes failed!');
		}

		return true;
	}



	function getTableIndexes($p_table_list = null, $l_show_primary = false)
	{
		$l_index_list = array();

		if ($p_table_list === null)
		{
			$l_table_list = $this->getAllTableNamesAsArray();
		}
		else
		{
			$l_table_list = array_intersect($p_table_list, $this->getAllTableNamesAsArray());

			$l_arr_diff = array_diff($p_table_list, $l_table_list);
			if (sizeof($l_arr_diff) > 0)
			{
				SysMsg::showWarning('DBStruct::getTableIndexes => table(s) not found: '.implode(', ', $l_arr_diff));
			}

		}

		if (sizeof($l_table_list) == 0)
		{
			throw new Exception('empty table list: no tables defined for listing indexes');
		}

		foreach ($l_table_list as $l_table_name)
		{

			$l_res = $this->DB->query("show index from ".$l_table_name);

			while ($l_row = $l_res->fetchRow(DBMC_FETCHMODE_ASSOC))
			{
				if ( !$l_show_primary == false && strtoupper($l_row['Key_name']) == 'PRIMARY')
				{
					continue;
				}

				if (empty($l_row['Sub_part']))
				{
					$l_sub_part = '';
				}
				else
				{
					$l_sub_part = '('.$l_row['Sub_part'].')';
				}

				if ( isSet($l_index_list[$l_row['Table']][$l_row['Key_name']]) )
				{
					$l_index_list[$l_row['Table']][$l_row['Key_name']]['Sequence'] .= ','.$l_row['Column_name'].$l_sub_part;
				}
				else
				{
					$l_index_list[$l_row['Table']][$l_row['Key_name']]['Sequence'] = $l_row['Column_name'].$l_sub_part;
				}
//				$l_index_list[$l_row['Table']][$l_row['Key_name']]['Details'][$l_row['Seq_in_index']] = $l_row;
			}

			$l_res->free();

			if (empty($l_index_list[$l_table_name]))
			{
				continue;
			}

			foreach($l_index_list[$l_table_name] as $l_keyname => $keydesc)
			{
				if ($keydesc['Sequence'] == $l_table_name.'_ID')
				{
					unset($l_index_list[$l_table_name][$l_keyname]);
					break;
				}
			}

			if (empty($l_index_list[$l_table_name]))
			{
				unset($l_index_list[$l_table_name]);
			}

		} // end foreach

		return $l_index_list;
	}



	function fetchAllTableStructures()
	{
		$all_tab_names = $this->DB->getCol("show tables");

		$this->ALL_TABLE_STRUCTURES = array();
		foreach ($all_tab_names as $tab_name)
		{
			$this->ALL_TABLE_STRUCTURES[$tab_name] = array(); // preset, otherwise check of this::tableExists in this::fetchTableStructure will fail !
			$this->fetchTableStructure($tab_name);
		} // end foreach

		return true;
	}



	function fetchTableStructure($p_table_name)
	{
		if ($this->tableExists($p_table_name) == false)
		{
			return false;
		}

		$described_table = $this->DB->query("describe ".$p_table_name);

		$cols = array();
		$desc = array();
		while($row = $described_table->fetchRow(DBMC_FETCHMODE_ASSOC))
		{
			$cols[] = $row['Field'];
			$desc[] = $row;
		} // end while

		$this->ALL_TABLE_STRUCTURES[$p_table_name] = array(
			"ColNameList" => $cols,
			"ColDescList" => $desc,
		);

		return true;
	}



	function getFieldNameByIndex($p_table_name, $l_index)
	{
		if ($this->tableExists($p_table_name) == false)
		{
			return false;
		}

		$l_index = intval($l_index);
		if ( !isset($this->ALL_TABLE_STRUCTURES[$p_table_name]['ColNameList'][$l_index]) )
		{
			return false;
		}

		return $this->ALL_TABLE_STRUCTURES[$p_table_name]['ColNameList'][$l_index];
	}



	function getAllTableNamesAsArray()
	{
		return array_keys($this->ALL_TABLE_STRUCTURES);
	}



	function tableExists(&$tableName = "")
	{
		if ( ! is_string($tableName) ) {
			return false;
		}

		if ( empty($tableName) ) {
			return false;
		}

		if ( isset($this->ALL_TABLE_STRUCTURES[$tableName]) ) {
			return true;
		}

		$lowerTableName = strtolower($tableName);
		if ( isset($this->ALL_TABLE_STRUCTURES[$lowerTableName]) ) {
			$tableName = $lowerTableName;
			return true;
		}

		return false;
	}



	function getTableStructure($p_tablename = "")
	{
		if ( !is_string($p_tablename) ) { return false; }
		if (empty($p_tablename)) { return false; }
		if ($this->tableExists($p_tablename))
		{
			return $this->ALL_TABLE_STRUCTURES[$p_tablename];
		}
		return false;
	}



	function getFieldIndexFromTable($p_tablename = "", $p_fieldname = "")
	{
		if ( !is_string($p_tablename) ) { return false; }
		if ( !is_string($p_fieldname) ) { return false; }
		if (empty($p_tablename)) { return false; }
		if (empty($p_fieldname)) { return false; }
		if ($this->tableExists($p_tablename))
		{
			$p_fieldname = str_replace('`','',$p_fieldname);
			$index = array_search($p_fieldname, $this->ALL_TABLE_STRUCTURES[$p_tablename]["ColNameList"]);
			return $index;
		}
		return false;
	}



	function fieldExistsInTable($p_tablename = "", $p_fieldname = "")
	{
		if ($this->getFieldIndexFromTable($p_tablename, $p_fieldname) === false)
		{
			return false;
		}
		return true;
	}



	function getFieldDescription($p_tablename, $p_fieldname)
	{
		$index = $this->getFieldIndexFromTable($p_tablename, $p_fieldname);
		if ($index === false)
		{
			return false;
		}

		return $this->ALL_TABLE_STRUCTURES[$p_tablename]["ColDescList"][$index];
	}



	// @DEPRECATED
	function fieldHasDescription($p_tablename, $p_fieldname, &$p_new_fielddesc)
	{
		return $this->fieldDescriptionExists($p_tablename, $p_fieldname, $p_new_fielddesc);
	}


	function fieldDescriptionExists($p_tablename, $p_fieldname, &$p_new_fielddesc)
	{
		if (is_array($p_new_fielddesc) == false)
		{
			$p_new_fielddesc = array(
				'Type' => $p_new_fielddesc,
			);
		}

		if (empty($p_new_fielddesc['Field']) )
		{
			$p_new_fielddesc['Field'] = $p_fieldname;
		}

		$current_desc = $this->getFieldDescription($p_tablename, $p_fieldname);
		if ($current_desc == false)
		{
			return false;
		}

		$l_full_match = true;
		$l_arr_new_key = array_keys($p_new_fielddesc);
		foreach ($current_desc as $attr_name => $attr_value)
		{
			if ( ! in_array($attr_name, $l_arr_new_key) )
			{
				$p_new_fielddesc[$attr_name] = $attr_value;
				continue;
			}

			if ($attr_value === null || $p_new_fielddesc[$attr_name] === null)
			{
				// type check
				if ($attr_value === $p_new_fielddesc[$attr_name])
				{
					continue;
				}
			}
			else if (strtolower($attr_value) == strtolower($p_new_fielddesc[$attr_name]))
			{
				continue;
			}

			$l_full_match = false;
		}

		return $l_full_match;
	}



	// @DEPRECATED
	function fieldHasPositionInTable($p_tablename, $p_fieldname, $p_position="")
	{
		$this->fieldPositionInTableMatches($p_tablename, $p_fieldname, $p_position);
	}



	function fieldPositionInTableMatches($p_tablename, $p_fieldname, $p_position="")
	{
		$p_position = trim($p_position);
		if (empty($p_position)) { return false; }

		$index = $this->getFieldIndexFromTable($p_tablename, $p_fieldname);
		if ($index === false) { return false; }

		if (strtolower($p_position) == "first")
		{
			if ($index === 0) { return true; }

			return false;
		}

		$p_position = explode(" ", $p_position);
		if (sizeof($p_position) != 2) { return false; }
		if (strtolower($p_position[0]) != "after") { return false; }

		$prev_index = $this->getFieldIndexFromTable($p_tablename, $p_position[1]); // get index from (the expected) previous table field
		if ($prev_index === false) { return false; }
		if ($prev_index != ($index - 1)) { return false; }

		return true;
	}



	function optimize($tableName, $forceOptimize = false)
	{
		if ( ! empty($this->OPTIMIZED_TABLES[$tableName]) && ! $forceOptimize)
		{
			SysMsg::showMsg("optimizing table '{$tableName}' {SKIPPED} (already optimized).");
			return true;
		}

		$optimize = "optimize table ".$tableName;

		$timerName = "optimize_{$tableName}";
		startTimer($timerName);

		$res = $this->DB->query($optimize);
		$this->OPTIMIZED_TABLES[$tableName] = true;

		$time = getTimer($timerName, true);

		SysMsg::showInfo("table <b>{$tableName}</b> optimized in <b>{$time}</b> seconds.");

		return true;
	}



	function optimizeTable($p_tablename, $p_force_optimize = false)
	{
		return $this->optimize($p_tablename, $p_force_optimize);
	}



	function getIndexCountFromTable($p_table_name)
	{
		if (empty($p_table_name))
		{
			throw new Exception($p_table_name, 'parameter $p_table_name is empty.');
		}

		if (isSet($this->ALL_TABLE_INDEXES[$p_table_name]))
		{
			return sizeof($this->ALL_TABLE_INDEXES[$p_table_name]);
		}

		return false;
	}



	function indexNameExistsOnTable($p_table_name, $p_index_name)
	{
		if (isset($this->ALL_TABLE_INDEXES[$p_table_name]))
		{
			if (isset($this->ALL_TABLE_INDEXES[$p_table_name][$p_index_name]))
			{
				return true;
			}
		}

		if (isset($this->NEW_TABLE_INDEXES[$p_table_name]))
		{
			if (isset($this->NEW_TABLE_INDEXES[$p_table_name][$p_index_name]))
			{
				return true;
			}
		}

		return false;
	}



	function getIndexNameBySequence($p_table_name, $p_sequence)
	{
		if (is_array($p_sequence))
		{
			$p_sequence = implode(',', $p_sequence);
		}

		if ( empty($p_sequence) )
		{
			throw new Exception('parameter $p_sequence is empty.');
		}

		$p_sequence = trim($p_sequence);
		$p_sequence = str_replace(' ', '', $p_sequence);

		$l_table_name = trim($p_table_name);
		if (empty($this->ALL_TABLE_INDEXES[$l_table_name]) || is_array($this->ALL_TABLE_INDEXES[$l_table_name]) == false)
		{
			// table has no indices at all
			return false;
		}

		foreach ($this->ALL_TABLE_INDEXES[$l_table_name] as $l_index_name => $l_index)
		{
			if (empty($l_index['Sequence']))
			{
				continue;
			}

			if ($p_sequence == $l_index['Sequence'])
			{
				return $l_index_name;
			}
		}

		return false;
	}



	function getIndexNamesFromTable($p_table_name)
	{
		if ($this->getIndexCountFromTable($p_table_name) > 0)
		{
			return array_keys($this->ALL_TABLE_INDEXES[$p_table_name]);
		}

		return false;
	}



	function createIndexOnTable($p_table_name, $p_sequence)
	{
		if (is_array($p_sequence) ) {
			$p_sequence = implode(',', $p_sequence);
		}
		else {
			$p_sequence = str_replace(' ', '', $p_sequence);
		}

		if ( empty($p_sequence) ) {
			throw new Exception('parameter $p_sequence is empty.');
		}

		$p_sequence = trim($p_sequence);

		$l_index_name = $this->getIndexNameBySequence($p_table_name, $p_sequence);

		if ( $l_index_name != false)
		{
			SysMsg::showMsg('creating index '.$l_index_name.' on table "'.$p_table_name.'" {SKIPPED} (sequence "'.$p_sequence.'" already exists). optimize table...');
			$this->optimizeTable($p_table_name);
			return $l_index_name;
		}

		$l_index_name = $this->createIndexNameForTable($p_table_name);

#		$this->backupTableDefinition($p_table_name);

		$l_sel = 'create index '.$l_index_name.' on '.$p_table_name.' ('.$p_sequence.')';

		$timerName = "create_index_{$l_index_name}";
		startTimer($timerName);

		$l_res = $this->DB->query($l_sel);

		$time = getTimer($timerName, true);

		if ($time < 1)
		{
			$time = "<b>{$time}</b>";
		}
		else if ($time < 10)
		{
			$time = "<b style=\"color:#ffa500;\">{$time}</b>";
		}
		else
		{
			$time = "<b style=\"color:red;\">{$time}</b>";
		}

		SysMsg::showInfo('creating index '.$l_index_name.' on table <b>'.$p_table_name.'</b> {SUCCEEDED} (sequence "'.$p_sequence.'" created in <b>'.$time.'</b> seconds).');
		$this->ALL_TABLE_INDEXES[$p_table_name][$l_index_name]['Sequence'] = $p_sequence;
		$this->NEW_TABLE_INDEXES[$p_table_name][$l_index_name] = true;

		return $l_index_name;
	}



	function createIndexNameForTable($p_table_name)
	{
		$i = 1;

		while ($this->indexNameExistsOnTable($p_table_name, $p_table_name.'_IDX'.$i) == true)
		{
			$i++;
		}

		return $p_table_name.'_IDX'.$i;
	}



	function createTable($tableName, $desc)
	{
		if ($this->tableExists($tableName) )
		{
			SysMsg::showInfo("creating table '{$tableName}' : {SKIPPED} (table already exists)");
			return true;
		}

		$res = $this->DB->query('create table `'.$tableName.'` '.$desc);
		SysMsg::showInfo("creating table '{$tableName}' : {SUCCEED}");

		return true;
	}



	function dropTable($mixed) 
	{
		return $this->dropTables($mixed);
	}



	function dropTables($mixed)
	{
		$arrAllTableNames = $this->getAllTableNamesAsArray();

		if ( empty($mixed) )
		{
			throw new Exception('parameter is empty');
		}

		if ( is_array($mixed) ) {
			$arrTables = array_unique($mixed);
		}
		else {
			$arrTables = array($mixed);
		}

		foreach($arrTables as $tableName)
		{
			// disable TAB-registration entry of table if exists
			// use like for ignore-case
			$select = 'update TAB set STATE = "D" where TAB_NAME like "'.$tableName.'"';
			$this->DB->query($select);

			if ( ! in_array($tableName, $arrAllTableNames) ) 
			{
				SysMsg::showInfo("table $tableName was not dropped (table does not exists). operation {SKIPPED}");
				continue;
			}

#			$this->backupTableDefinition($tableName);
#			$this->backupTableContent($tableName);

			$this->DB->query("drop table $tableName");
			SysMsg::showInfo("dropping table $tableName : {SUCCEED}");
		}

		return true;
	}



	/**
	 * deprecated, synonym for dropIndexOnTable
	 */
	function dropIndexFromTable($p_table_name, $p_index_name, $p_force_drop = false)
	{
		return $this->dropIndexOnTable($p_table_name, $p_index_name, $p_force_drop);
	}



	function dropIndexOnTable($p_table_name, $p_index_name, $p_force_drop = false)
	{
		if ( $p_index_name == 'PRIMARY' )
		{
			return true;
		}

		if ( !$this->tableExists($p_table_name) )
		{
			SysMsg::showWarning('DBStruct::dropIndexOnTable => table not found');
			return false;
		}

		if ($this->indexNameExistsOnTable($p_table_name, $p_index_name) == false)
		{
			SysMsg::showWarning('DBStruct::dropIndexOnTable => index '.$p_index_name.' not found on table '.$p_index_name);
			return false;
		}

		$l_is_original_index = false;
		if (empty($this->NEW_TABLE_INDEXES[$p_table_name]))
		{
			$l_is_original_index = true;
		}
		else if (empty($this->NEW_TABLE_INDEXES[$p_table_name][$p_index_name]))
		{
			$l_is_original_index = true;
		}

		if (empty($p_force_drop) && $l_is_original_index)
		{
			SysMsg::showInfo('dropping index '.$p_index_name.' from table '.$p_index_name.' : {SKIPPED} (keeping original index)');
			return true;
		}

#		$this->backupTableDefinition($p_table_name);

		$l_upd = 'alter table '.$p_table_name.' drop index '.$p_index_name;
		$l_res = $this->DB->query($l_upd);

		unset($this->ALL_TABLE_INDEXES[$p_table_name][$p_index_name]);

		if (empty($this->ALL_TABLE_INDEXES[$p_table_name]))
		{
			unset($this->ALL_TABLE_INDEXES[$p_table_name]);
		}

		$l_msg = 'dropping index '.$p_index_name.' from table '.$p_table_name.' : {SUCCEED}';
		if ($l_is_original_index)
		{
			$l_msg .= ' (forced drop)';
		}
		else
		{
			unset($this->NEW_TABLE_INDEXES[$p_table_name][$p_index_name]);
		}

		if (isset($this->NEW_TABLE_INDEXES[$p_table_name]) && count($this->NEW_TABLE_INDEXES[$p_table_name]) == 0)
		{
			unset($this->NEW_TABLE_INDEXES[$p_table_name]);
		}

		SysMsg::showInfo($l_msg);

		return true;
	}



	function dropAllIndicesOnTable($p_table_name)
	{
		if (empty($this->ALL_TABLE_INDEXES[$p_table_name]))
		{
			return true;
		}

		foreach($this->ALL_TABLE_INDEXES[$p_table_name] as $l_index_name => $l_sequence)
		{
			$this->dropIndexOnTable($p_table_name, $l_index_name, true);
		}

		return true;
	}



	function dropIndicesByPattern($p_search_string)
	{
		return $this->rewriteIndicesByPattern($p_search_string, false);
	}



	function rewriteIndicesByPattern($p_search_string, $p_rewrite = true)
	{
		foreach($this->ALL_TABLE_INDEXES as $l_table_name => $l_indices)
		{
#			$this->backupTableDefinition($l_table_name);

			foreach($l_indices as $l_index_name => $l_sequence)
			{
				$l_arr_sequence = explode(',', $l_sequence['Sequence']);
				$l_keep_index = true;
				foreach($l_arr_sequence as $l_idx => $l_col)
				{
					if (strpos($l_col, $p_search_string) === false)
					{
						continue;
					}

					unset($l_arr_sequence[$l_idx]);
					$l_keep_index = false;
				}

				if ($l_keep_index)
				{
					continue;
				}

				$this->DB->query('alter table '.$l_table_name.' drop index '.$l_index_name);

				if (count($l_arr_sequence) == 0 || $p_rewrite == false)
				{
					SysMsg::showInfo('drop index '.$l_index_name.' from table '.$l_table_name.' (removing columns containing "'.$p_search_string.'") : {SUCCEED}');
					continue;
				}

				$this->DB->query('create index '.$l_index_name.' on '.$l_table_name.'('.implode(',', $l_arr_sequence).')');
				SysMsg::showInfo('rewrite index '.$l_index_name.' on table '.$l_table_name.' (removing columns containing "'.$p_search_string.'") : {SUCCEED}');
			}
		}

		return true;
	}



	function rewriteAllTableIndexes($p_index_array)
	{
		if (is_array($p_index_array) == false)
		{
			throw new Exception('parameter $p_index_array is not an array');
		}

		if (sizeof($p_index_array) == 0)
		{
			throw new Exception('array $p_index_array is empty');
		}

		while (list($l_table_name, $l_sequence_list) = each($p_index_array))
		{
			$l_index_names = $this->getIndexNamesFromTable($l_table_name);

			foreach ($l_sequence_list as $l_sequence)
			{
				$l_res = $this->createIndexOnTable($l_table_name, $l_sequence);
				if ($l_res && isset($l_index_names[$l_res]))
				{
					unset($l_index_names[$l_res]);
				}
			}

			if ( !is_array($l_index_names) || sizeof($l_index_names) == 0)
			{
				continue;
			}

			foreach ($l_index_names as $l_index_name)
			{
				$this->dropIndexFromTable($l_table_name, $l_index_name);
			}

		}

		return $this->refreshStructures();
	}



	function addCol($tabname, $colname, $p_description)
	{
		if ($this->fieldExistsInTable($tabname, $colname))
		{
			SysMsg::showInfo('adding column '.$tabname.'.'.$colname.' : {SKIPPED} (column does already exist).');
		}
		else
		{
#			$this->backupTableDefinition($tabname);
			$ret = $this->DB->query('alter table '.$tabname.' add column '.$colname.' '.$p_description);
			SysMsg::showInfo('ADD column '.$tabname.'.'.$colname.' : {SUCCEED}');
		}

		return true;
	}



	function dropCol($tabname, $colname)
	{
		if ($this->fieldExistsInTable($tabname, $colname))
		{
#			$this->backupTableDefinition($tabname);
#			$this->backupTableContent($tabname);

			$del = 'alter table '.$tabname.' drop column '.$colname;
			$ret = $this->DB->query($del);

			SysMsg::showInfo('DROP column '.$tabname.'.'.$colname.' : {SUCCEED}');
			return true;
		}

		SysMsg::showInfo('DROP column '.$tabname.'.'.$colname.' : {SKIPPED} (column is already missing).');

		return true;
	}



	function changeCol($tabname, $current_colname, $p_new_fielddesc, $p_position = null)
	{
		return $this->change($tabname, $current_colname, $p_new_fielddesc, $p_position);
	}



	function change($tabname, $current_colname, $p_new_fielddesc, $p_position = null)
	{
//    $p_new_fielddesc = array {
//      ["Field"]   => string(12) ACCESSLOG_ID
//      ["Type"]    => string(7) int(11)|etc.
//      ["Null"]    => string(0) NO|YES
//      ["Key"]     => string(3) PRI|MUL|UNI
//      ["Default"] => NULL|<string>|<int>|CURRENT_TIMESTAMP
//      ["Extra"]   => string(14) auto_increment
//    }

		if ( ! is_array($p_new_fielddesc) )
		{
			$p_new_fielddesc = array(
				'Type' => $p_new_fielddesc,
			);
		}

		if (empty($p_new_fielddesc['Field']) )
		{
			$p_new_fielddesc['Field'] = $current_colname;
		}

		$p_position = trim($p_position);

		// original field name check
		if ( ! $this->fieldExistsInTable($tabname, $current_colname) )
		{
			// original name not found - does the given description request a name change?
			if ($current_colname == $p_new_fielddesc['Field'])
			{
				// no name change requested. throw error
				SysMsg::showError("changing field ".$tabname.' . '.$current_colname." {FAILED} (field not found).");
				return false;
			}
			else
			{
				// name change requested - check for requested name
				if ($this->fieldExistsInTable($tabname, $p_new_fielddesc['Field']) == false)
				{
					// requested name not found, throw error
					SysMsg::showError("changing field ".$tabname.' . '.$current_colname." {FAILED} (neither the fields current name nor its descripted name does match).");
					return false;
				}

				// requested name already exists. set requested name as original name
				SysMsg::showMsg("field ".$current_colname.' has been already renamed to '.$p_new_fielddesc['Field'].". checking for field description...");
				$current_colname = $p_new_fielddesc['Field'];
			}
		}

		$l_is_matching = false;
		// parameter $p_new_fielddesc is returned by reference 
		// it contains missing indexes filled with current values
		$l_is_full_match = $this->fieldHasDescription($tabname, $current_colname, $p_new_fielddesc);
		if (isset($p_new_fielddesc['Key']) && $p_new_fielddesc['Key'] == 'PRI')
		{
			// fix: unique fields cause an error when set to unique again 
			unset($p_new_fielddesc['Key']);
		}


		if ($l_is_full_match)
		{
			$l_field_index = $this->getFieldIndexFromTable($tabname, $current_colname);
			if (empty($p_position) )
			{
				$l_is_matching = true;
			}
			else if (strtolower($p_position) == 'first' && $l_field_index === 0)
			{
				$l_is_matching = true;
			}
			else if ($l_field_index > 0)
			{
				if ($this->getFieldNameByIndex($tabname, ($l_field_index - 1)) == $p_position)
				{
					$l_is_matching = true;
				}
			}
		}

		if ($l_is_matching == true)
		{
			SysMsg::showInfo("change column ".$tabname.' . '.$current_colname." : {SKIPPED} (column name and description does match already)");
			@ob_flush();
			return true;
		}

		if (empty($p_new_fielddesc['Field']) )
		{
			$msg = "change column ".$tabname.' . '.$current_colname." : {FAILED} (index 'Field' of field description is empty)";
			SysMsg::showError($msg);
			throw new Exception($msg);
		}

#			$this->backupTableDefinition($tabname);
#			$this->backupTableContent($tabname);

		if (in_array($current_colname, $this->RESERVED_WORDS) )
		{
			$l_old_colname = '`'.$current_colname.'`';
		}
		else
		{
			$l_old_colname = $current_colname;
		}

		if (in_array($p_new_fielddesc['Field'], $this->RESERVED_WORDS) )
		{
			$l_new_colname = ' `'.$p_new_fielddesc['Field'].'`';
		}
		else
		{
			$l_new_colname = ' '.$p_new_fielddesc['Field'];
		}

		$sql = 'alter table '.$tabname.' change column '.$l_old_colname.$l_new_colname;

		if ( ! empty($p_new_fielddesc['Type']) )
		{
			$sql .= ' '.$p_new_fielddesc['Type'];
		}

		if ( ! empty($p_new_fielddesc['Null']) && $p_new_fielddesc['Null'] == 'NO')
		{
			$sql .= ' NOT NULL';
		}
		else
		{
			$sql .= ' NULL';
		}

		if ( ! empty($p_new_fielddesc['Key']) )
		{
			if ($p_new_fielddesc['Key'] == 'PRI')
			{
				$sql .= ' primary';
			}
			else if ($p_new_fielddesc['Key'] == 'UNI')
			{
				$sql .= ' unique';
			}
		}

		if (isset($p_new_fielddesc['Default']) )
		{
			$default = $p_new_fielddesc['Default'];

			if (strtoupper($default) == 'CURRENT_TIMESTAMP')
			{
				$sql .= ' default CURRENT_TIMESTAMP';
			}
			else if (strval($default) == 'null' || $default === null)
			{
				$sql .= ' default NULL';
			}
			else if (is_string($default))
			{
				$sql .= ' default "'.mysql_real_escape_string($default).'"';
			}
			else
			{
				// integer, float etc.
				$sql .= ' default '.$default;
			}
		}

		if (isset($p_new_fielddesc['Extra']) )
		{
			if (strtolower($p_new_fielddesc['Extra']) == 'auto_increment')
			{
				$sql .= ' auto_increment';
			}
			else
			{
				$sql .= ' '.$p_new_fielddesc['Extra'];
			}
		}

		if (strtolower($p_position) == 'first')
		{
			$sql .= ' first';
		}
		else if ($this->fieldExistsInTable($tabname, $p_position) )
		{
			$sql .= ' after '.$p_position;
		}

		$this->DB->query($sql);

		if (isset($p_new_fielddesc['Key']) )
		{
			if (empty($p_new_fielddesc['Key']) )
			{
				$current_desc = $this->getFieldDescription($tabname, $current_colname);
				if ( ! empty($current_desc['Key']) && $current_desc['Key'] == 'PRI')
				{
					$drop_pri = $this->DB->query('alter table '.$tabname.' drop primary key');
				}
			}
		}

		$this->fetchTableStructure($tabname);
		SysMsg::showInfo("change column ".$tabname.' . '.$current_colname." : {SUCCEED}");

		@ob_flush();

		return true;
	}



	public function backupTableDefinition($tabname, $backupPath)
	{
		set_time_limit(300);
		$tabname = trim($tabname);
		if (empty($tabname) )
		{
			SysMsg::showError("unable to backup table: table name is empty");
			return false;
		}

		if ( ! $this->tableExists($tabname) )
		{
			SysMsg::showError("unable to backup table '{$tabname}': table does not exist");
			return false;
		}

		$destDDL = str_replace('//', '/', $backupPath . "/{$tabname}.ddl");
		if ( file_exists($destDDL) )
		{
			SysMsg::showInfo("ddl backup '{$destDDL}' already exists");
			return $destDDL;
		}

		$quotedTabname = $this->DB->quoteIdentifier($tabname);

		// get and write create table statements
		$arrCreateSql = $this->DB->getRow("SHOW CREATE TABLE {$quotedTabname}");
		$createSql = $arrCreateSql[1];

		// stripping "create table `<table name>`" from statement
		$pos = strpos($createSql, '(');
		$createSql = substr($createSql, $pos);

		// remove autoincrement values from create statement
		$createSql = preg_replace('/auto_increment=[0-9]*\s*/i', '', $createSql);

		// replace with statement using the original name
		$createSql = "CREATE TABLE {$quotedTabname} {$createSql}";

		$h = fopen($destDDL, 'w+');
		if ( ! $h )
		{
			throw new Exception("unable to open file '{$destDDL}'!");
		}

		fwrite($h, $createSql);
		fclose($h);

		SysMsg::showInfo("ddl backup '{$destDDL}' created.");

		return $destDDL;
	}



	public function backupTableContent($tabname, $backupPath)
	{
		set_time_limit(300);
		$tabname = trim($tabname);
		if (empty($tabname) )
		{
			SysMsg::showError("unable to backup table: table name is empty");
			return false;
		}

		if ( ! $this->tableExists($tabname) )
		{
			SysMsg::showError("unable to backup table '{$tabname}': table does not exist");
			return false;
		}

		$destSQL = str_replace('//', '/', $backupPath . "/{$tabname}.sql");
		if ( file_exists($destSQL) )
		{
			SysMsg::showInfo("content backup '{$destSQL}' already exists");
			return $destSQL;
		}

		$quotedTabname = $this->DB->quoteIdentifier($tabname);
		$result = $this->DB->query("select * from {$quotedTabname} order by 1");

		// check for table content => skip dump if table is empty
		if ($result->numRows() == 0)
		{
			SysMsg::showInfo("table '{$tabname}' is empty. data backup {SKIPPED}.");
			return null;
		}

		$h = fopen($destSQL, 'w+');
		if ( ! $h )
		{
			throw new Exception("unable to open file '{$destSQL}'!");
		}

		// fetch first row for generating list of fields
		$row = $result->fetchRow(DBMC_FETCHMODE_ASSOC);
		unset($row['LASTCHANGE']);

		$arrFields = array();
		foreach ($row as $field => $value)
		{
			array_push($arrFields, $this->DB->quoteIdentifier($field) );
		}
		$strFields = implode(',	', $arrFields);

		do
		{
			unset($row['LASTCHANGE']);

			// prepare values
			$arrValues = array();
			foreach ($row as $value)
			{
				array_push($arrValues, $this->DB->quote($value) );
			}
			$strValues = implode(',	', $arrValues);

			// write into export file
			$insert = "INSERT DELAYED INTO {$quotedTabname}	(	{$strFields}	)	VALUES	(	{$strValues}	);\n";
			fwrite($h, $insert);

			// read next row
			set_time_limit(60);
			$row = $result->fetchRow(DBMC_FETCHMODE_ASSOC);

		} while ($row);

		$result->free();
		fclose($h);

		SysMsg::showInfo("data backup '{$destSQL}' created.");

		return $destSQL;
	}





/*
	function enableUniqueColumn($tabname, $colname)
	{
		return $this->setColumnUniqueKey($tabname, $colname, true);
	} // end of method enableUniqueColumn



	function disableUniqueColumn($tabname, $colname)
	{
		return $this->setColumnUniqueKey($tabname, $colname, false);
	} // end of method disableUniqueColumn



	function setColumnUniqueKey($tabname, $colname, $unique_state)
	{
		if ( !$this->fieldExistsInTable($tabname, $colname) )
		{
			SysMsg::showError("creating unique key on column ".$tabname.' . '.$colname." {FAILED} : field does not exist!");
			return false;
		}

		$l_fielddesc = $this->getFieldDescription($tabname, $colname);
//    array(6) {
//      ["Field"]   => string(12) ACCESSLOG_ID
//      ["Type"]    => string(7) int(11)|etc.
//      ["Null"]    => string(0) NO|YES
//      ["Key"]     => string(3) PRI|MUL|UNI
//      ["Default"] => NULL|<string>|<int>|CURRENT_TIMESTAMP
//      ["Extra"]   => string(14) auto_increment
//    }

		if ($l_fielddesc['Key'] == 'PRI')
		{
			SysMsg::showError("creating unique key on column ".$tabname.' . '.$colname." {FAILED} : column is primary key!");
			return true;
		}

		if ($unique_state)
		{
			// make unique key
			if ($l_fielddesc['Key'] == 'UNI')
			{
				SysMsg::showInfo("creating unique key on column ".$tabname.' . '.$colname." {SKIPPED} : column is already unique");
				return true;
			}
		}
		else
		{
			// remove unique key
			if ($l_fielddesc['Key'] != 'UNI')
			{
				SysMsg::showInfo("removing unique key from column ".$tabname.' . '.$colname." {SKIPPED} : column is not unique (already)");
				return true;
			}
		}

		if ( !$unique_state )
		{
			// drop index from unique column before removing unique key
			$res = $this->DB->query('alter table '.$tabname.' drop index '.$colname);
		}

		if (in_array($colname, $this->RESERVED_WORDS))
		{
			$colname = '`'.$colname.'`';
		}

		$sql  = 'alter table '.$tabname.' change column '.$colname.' '.$colname.' '.$l_fielddesc['Type'];
		$sql .= (($l_fielddesc['Null'] == 'NO') ? ' NOT NULL' : ' null');
		$sql .= ($unique_state ? ' unique' : '');

		if (strtoupper($l_fielddesc['Default']) == 'CURRENT_TIMESTAMP')
		{
			$sql .= ' default CURRENT_TIMESTAMP';
		}
		else if (is_string($l_fielddesc['Default']))
		{
			$sql .= ' default "'.$l_fielddesc['Default'].'"';
		}
		else
		{
			$sql .= ' default '.$l_fielddesc['Default'];
		}

		$sql .= (empty($l_fielddesc['Extra']) ? '' : ' '.$l_fielddesc['Extra']);

		$res = $this->DB->query($sql);

		return true;
	} // end of method setColumnUniqueKey
*/


}


//EOF