<?php
/*
* $McLicense$
*
* $Id$
*
*/

include_once(MCP_LIB_PATH."RessourceFactory.class.php");

/**
*
*
* @author	Steffen Gross <gross@metacoon.net>
* @package 	classes.new
*/
class DsAccess
{

	/**
	 * RessourceFactory
	 */
	public $RessourceFactory;


	/**
	 * caches group details from method this::getGroupDetails
	 */
	private $wgroup_details;


	/**
	 * caches room member groups this::getMemberGroupByRoom
	 */
	private $member_groups;


	/**
	 * caches private groups this::getPrivateGroupByUser
	 */
	private $private_groups;



	/**
	 * Constructor
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 * @package 	admin
	 */
	public function __construct()
	{
		global $U;

		$this->DB = &$U->DB;

		$this->RessourceFactory = new RessourceFactory();

    // preset caches
		$this->wgroup_details = array();
		$this->member_groups  = array();
		$this->private_groups = array();

		return true;
	} // Constructor end



	/**
	 * registerDs
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	private function registerDs($p_tab_name, $p_res_id, $p_fk_id, $p_view, $p_owner, $p_room_id, $p_flags = 0)
	{
		$l_res_id = intval($p_res_id);
		$l_fk_id  = intval($p_fk_id);
		$l_link_src_count = 0;

		if (empty($p_owner))
		{
			$p_owner = 1; // modification for update framework
		}

		$l_res = $this->DB->query(
			 'insert into DS ('
			.'  DS_RES_ID, DS_FK_ID'
			.', LINK_SRC_COUNT, LINK_DEST_COUNT'
			.', COURSE_ID, FLAGS'
			.', OWNER, INSDAT'
			.', VIEW'
			.', RELSTATE, RELUSER, RELDATE, RELLANG'
			.', STATE'
			.' ) values ('
			.'  '.$l_res_id.', '.$l_fk_id
			.', '.$l_link_src_count.', 0'
			.', '.intval($p_room_id).', '.intval($p_flags)
			.', '.intval($p_owner).', "'.time().'"'
			.', "'.$p_view.'"'
			.', "N", null, null, null'
			.', "Y"'
			.' )'
		);

		$l_ds_id = intval($this->DB->getLastInsertId());

		// write ds_id back into original component record
		$l_res = $this->DB->query('update '.$p_tab_name.' set DS_ID = '.$l_ds_id.' where '.$p_tab_name.'_ID = '.$l_fk_id);

		// self-link : sets relation from new dataset id to access records with same id
		$this->createSelfDsLink($l_ds_id);

		// insert owner access
		$l_owner_group = $this->getPrivateGroupByUser($p_owner);

		$this->insert($l_ds_id, $this->getGroupDetails($l_owner_group, MC_DS_GROUP), $p_owner, $p_room_id, (MC_FLAG_SYSTEM + MC_FLAG_HIDDEN));

		return $l_ds_id;
	} // end method registerDs



	/**
	 * getDsId
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function getDsId($p_ressource, $p_fk_id)
	{
		$l_Ressource = $this->RessourceFactory->create($p_ressource);
		if (empty($l_Ressource))
		{
			return false;
		}

		$l_res_id = $l_Ressource->getId();
		$l_fk_id = intval($p_fk_id);

		$l_ds_id = $this->DB->getOne(
			 'select DS_ID from DS'
			.' where '
			.'     DS_RES_ID = '.$l_res_id
			.' and DS_FK_ID = '.$l_fk_id
			.' and STATE = "Y"'
		);

		if (empty($l_ds_id))
		{
			return false;
		}

		return $l_ds_id;
	} // end method getDsId



	/**
	 * createDsLinkByRessource
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function createDsLinkByRessource($p_s_res_id, $p_s_fk_id, $p_d_res_id, $p_d_fk_id, $p_link_id)
	{
		$l_s_ds_id = $this->getDsId(intval($p_s_res_id), intval($p_s_fk_id));
		$l_d_ds_id = $this->getDsId(intval($p_d_res_id), intval($p_d_fk_id));

		return $this->createDsLinkById($l_s_ds_id, $l_d_ds_id, $p_link_id);
	} // end method createDsLinkByRessource



	/**
	 * createSelfDsLink
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	private function createSelfDsLink($p_ds_id)
	{
		return $this->createDsLinkById($p_ds_id, $p_ds_id, 0);
	} // end method createSelfDsLink



	/**
	 * createDsLinkById
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function createDsLinkById($p_s_ds_id, $p_d_ds_id, $p_link_id = 0)
	{
		$p_s_ds_id = intval($p_s_ds_id);
		$p_d_ds_id = intval($p_d_ds_id);
		$p_link_id = intval($p_link_id);

		$l_res = $this->DB->getOne(
			 'select DS_LINK_ID'
			.' from DS_LINK'
			.' where'
			.'     DS_LINK_SRC_DS_ID  = '.$p_s_ds_id
			.' and DS_LINK_DEST_DS_ID = '.$p_d_ds_id
			.' and DS_LINK_LINK_ID    = '.$p_link_id
			.' and STATE = "Y"'
		);

		if ($l_res)
		{
			return true;
		}

		$l_res = $this->DB->query(
			 'insert into DS_LINK ('
			.'  DS_LINK_SRC_DS_ID'
			.', DS_LINK_DEST_DS_ID'
			.', DS_LINK_LINK_ID'
			.', OWNER, INSDAT'
			.', STATE'
			.' ) values ('
			.'  '.$p_s_ds_id
			.', '.$p_d_ds_id
			.', '.$p_link_id
			.', '.mc_Session::getUserId().', "'.time().'"'
			.', "Y"'
			.' )'
		);

		return true;
	} // end method createDsLinkById



	/**
	 * deleteDs
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function deleteDs($l_ds_id)
	{
		if (mc_Debug::isError($this->DB->query(
			'delete from DS where DS_ID = '.intval($l_ds_id)
		))) { return false; }

		$this->deleteDsLink($l_ds_id);
		$this->deleteDsAccess($l_ds_id);

		return true;
	} // end method deleteDs



	/**
	 * deleteDsAccess
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function deleteDsAccess($l_ds_id)
	{
		return (mc_Debug::isError($this->DB->query(
			'delete from DS_ACCESS where DS_ACCESS_DS_ID = '.intval($l_ds_id)
		)));
	} // end method deleteDsAccess



	/**
	 * deleteDsLink
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function deleteDsLink($l_ds_id)
	{
		return (mc_Debug::isError($this->DB->query(
			'delete from DS_LINK where DS_LINK_SRC_DS_ID = '.intval($l_ds_id).' or DS_LINK_DEST_DS_ID = '.intval($l_ds_id)
		)));
	} // end method deleteDsLink


	/**
	 * disableDsLink
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	private function disableDsLink($p_link_id)
	{
		$l_res = $this->DB->query(
			 'update DS_LINK set STATE = "N" where DS_LINK_LINK_ID = '.intval($p_link_id)
		);

		return true;
	} // end method disableDsLink



	/**
	 * registerByRessource
	 *
	 * alias for createByRessource
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function registerByRessource($p_ressource, $p_fk_id, $p_view, $p_owner, $p_room_id, $p_flags = 0, $p_mgr_access = null)
	{
		$this->createByRessource($p_ressource, $p_fk_id, $p_view, $p_owner, $p_room_id, $p_flags, $p_mgr_access);
	} // end method registerByRessource



	/**
	 * createByRessource
	 *
	 * IMPORTANT NOTE on "creating access from an external session" :
	 * you NEED TO use the parameter arr_access !
	 * you may set owner and room_id as you like.
	 * known issue from external sessions : the owner of the related self-DS_LINK is set to -1
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function createByRessource($p_ressource, $p_fk_id, $p_view, $p_owner, $p_room_id, $p_flags = 0, $p_mgr_access = null)
	{
		if (is_array($p_ressource))
		{
			$p_tab_name = trim($p_ressource[0]);
			$l_res_id   = intval($p_ressource[1]);
		}
		else if (is_numeric($p_ressource))
		{
			$p_tab_name = $this->DB->getOne('select TAB_NAME  from TAB where TAB_IDENT =  '.$p_ressource.'  and STATE = "Y"');
			$l_res_id   = intval($p_ressource);
		}
		else
		{
			$p_tab_name = trim($p_ressource);
			$l_res_id   = $this->DB->getOne('select TAB_IDENT from TAB where TAB_NAME  = "'.$p_tab_name.'" and STATE = "Y"');
		}

		$l_ds_id = $this->registerDs($p_tab_name, $l_res_id, $p_fk_id, $p_view, $p_owner, $p_room_id, $p_flags);
		if (empty($l_ds_id))
		{
			mc_Debug::error($l_ds_id, 'data record for res_id/fk_id = '.$l_res_id.'/'.$p_fk_id.' is not registered in table DS');
			return false;
		}

		// for BC
		if ($p_mgr_access === null)
		{
			// fetch access from orginal data record
			$l_res = $this->DB->getRow(
				 'select'
				.'  WGROUP'
				.', ACCESS'
				.' from '.$p_tab_name
				.' where'
				.'  '.$p_tab_name.'_ID = '.intval($p_fk_id)
			);

		  $p_mgr_access = $this->parseOldAccess($p_room_id, $l_res[0], $l_res[1]);

		} // end if ($p_arr_access === null)

		if ($this->write($l_ds_id, $p_mgr_access, $p_owner, $p_room_id, false))
		{
			return $l_ds_id;
		}

		return false;
	} // end method createByRessource




	/**
	 * parseOldAccess
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function parseOldAccess($p_room_id, $p_wgroup, $p_access, $p_room_member_group = null)
	{
		$p_wgroup = intval($p_wgroup);
		$p_access = intval($p_access);

	  $p_arr_access = array();

		if ($p_access > MC_DS_GROUP)
		{
			if ($p_room_member_group === null)
			{
				$p_room_id = intval($p_room_id);
				$p_arr_access[$this->getMemberGroupByRoom($p_room_id)] = ($p_access & MC_DS_OTHER) / 8;
			}
			else
			{
				$p_arr_access[$p_room_member_group] = ($p_access & MC_DS_OTHER) / 8;
			}
		}

		if ($p_access & MC_DS_GROUP && $p_wgroup > 0)
		{
			$p_arr_access[$p_wgroup] = $p_access & MC_DS_GROUP;
		}

		return $p_arr_access;
	} // end method parseOldAccess




	/**
	 * write
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function writeByRessource($p_ressource, $p_fk_id, $p_mgr_access, $p_is_update = true)
	{
		$l_Ressource = $this->RessourceFactory->create($p_ressource);
		if (empty($l_Ressource))
		{
			return false;
		}

		$l_res_id = $l_Ressource->getId();
		$l_fk_id = intval($p_fk_id);

		$l_ds_id = $this->getDsId($l_res_id, $l_fk_id);

		if (empty($l_ds_id))
		{
			mc_Debug::error($l_ds_id, 'data record for res_id/fk_id = '.$l_res_id.'/'.$l_fk_id.' is not registered in table DS');
			return false;
		}

		$p_tab_name = $l_Ressource->getName();

		$l_data = $this->DB->getRow(
			 'select'
			.'  OWNER'
			.', COURSE_ID'
			.' from '.$p_tab_name
			.' where'
			.'  '.$p_tab_name.'_ID = '.intval($p_fk_id)
		);

		if (empty($l_data))
		{
			return false;
		}

		return $this->write($l_ds_id, $p_mgr_access, intval($l_data[0]), intval($l_data[1]), $p_is_update);
	} // end method deleteAccessLink




	/**
	 * write
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function write($p_ds_id, $p_mgr_access, $p_owner, $p_room_id, $p_skip_dynamic = false, $p_script_id = 0)
	{
		$l_ds_id = intval($p_ds_id);

		$l_mgr_previous = $this->getMgrAccessByDsId($l_ds_id, false);

		if (isset($p_mgr_access['static']))
		{
			$this->writeStatic($p_ds_id, $l_mgr_previous['static'], $p_mgr_access['static'], $p_owner, $p_room_id, $p_script_id);
			unset($p_mgr_access['static']);
		}
		unset($l_mgr_previous['static']);

		if ($p_skip_dynamic)
		{
			return true;
		}

		$this->writeDynamic($p_ds_id, $l_mgr_previous, $p_mgr_access, $p_owner, $p_room_id, $p_script_id);

		return true;
	}



	/**
	 * getMgrAccessByDsId
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function getMgrAccessByDsId($p_ds_id, $p_ignore_disabled_entries = true, $p_show_owner = false)
	{
		$l_res = $this->DB->query(
			 'select'
			.'  DS_ACCESS_WGROUP_IDENT'
			.', DS_ACCESS_ACCESS'
			.', DS_ACCESS.FLAGS'
			.' from'
			.'  DS_ACCESS, WGROUP'
			.' where'
			.'     DS_ACCESS_DS_ID = '.intval($p_ds_id)
			.($p_show_owner
				? ' and (DS_ACCESS_WGROUP_TYPE = "U" or not (DS_ACCESS.FLAGS & '.MC_FLAG_HIDDEN.') )'
				: ' and not (DS_ACCESS.FLAGS & '.MC_FLAG_HIDDEN.')'
			)
			.($p_ignore_disabled_entries
				? ' and DS_ACCESS.STATE = "Y"'
				: ''
			)
			.' and WGROUP_ID = DS_ACCESS_WGROUP_IDENT'
		);

		$l_mgr_access = array();
		$l_mgr_access['static'] = array();
		while ($l_row = $l_res->fetchRow())
		{
			if (intval($l_row[2]) & MC_FLAG_SYSTEM)
			{
				$l_mgr_access['static'][intval($l_row[0])] = intval($l_row[1]);
			}
			else
			{
				$l_mgr_access[intval($l_row[0])] = intval($l_row[1]);
			}
		} // end of while

		return $l_mgr_access;
	} // end function getMgrAccessByDsId



	/**
	 * writeDynamic
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	private function writeDynamic($p_ds_id, $p_arr_previous, $p_arr_new, $p_owner, $p_room_id, $p_script_id = 0)
	{
		$p_arr_new = $this->packArrAccessByGroup($p_arr_new);

		foreach($p_arr_new as $l_wgroup_ident => $l_access)
		{
			if (empty($l_access) || empty($l_wgroup_ident))
			{
				// invalid entry
				continue;
			} // end if

			if (isset($p_arr_previous[$l_wgroup_ident]))
			{
				// group already has an entry => rewrite
				$this->updateDynamic($p_ds_id, $this->getGroupDetails($l_wgroup_ident, $l_access), $p_owner);

				// remove entry from list of previous permissions (all remaining groups will be killed)
				unset($p_arr_previous[$l_wgroup_ident]);

				continue;
			} // end if

			// group hasn't an entry yet
			$this->insert($p_ds_id, $this->getGroupDetails($l_wgroup_ident, $l_access), $p_owner, $p_room_id, 0, $p_script_id);
		} // end foreach

		// setting state of remaining permissions to 'N'
		foreach($p_arr_previous as $l_wgroup_ident => $l_access)
		{
			$this->deleteDynamicAccessByDsGroup($p_ds_id, $l_wgroup_ident);
		} // end foreach

		return true;
	} // end method writeDynamic



	/**
	 * writeStatic
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	private function writeStatic($p_ds_id, $p_arr_previous, $p_arr_new, $p_owner, $p_room_id, $p_script_id = 0)
	{
		foreach($p_arr_new as $l_wgroup_ident => $l_access)
		{
			if (empty($l_access) || empty($l_wgroup_ident))
			{
				// invalid entry
				continue;
			} // end if

			if (isset($p_arr_previous[$l_wgroup_ident]))
			{
				$l_access = $p_arr_previous[$l_wgroup_ident] | $l_access;
				// group already has an entry => rewrite
				$this->updateStatic($p_ds_id, $this->getGroupDetails($l_wgroup_ident, $l_access), $p_owner);

				continue;
			} // end if

			// group hasn't an entry yet
			$this->insert($p_ds_id, $this->getGroupDetails($l_wgroup_ident, $l_access), $p_owner, $p_room_id, MC_FLAG_SYSTEM, $p_script_id);
		} // end foreach

		return true;
	} // end method writeStatic



	/**
	 * updateDynamic
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	private function updateDynamic($p_ds_id, $p_access, $p_upduser)
	{
		if (empty($p_access))
		{
			mc_Debug::error($p_access, 'access data is empty');
			return false;
		}

		$query = 'update DS_ACCESS'
			.' set'
			.'  DS_ACCESS_WGROUP_TYPE = "'.trim($p_access['wgroup_type']).'"'
			.', DS_ACCESS_WGROUP_COURSE_ID = '.intval($p_access['wgroup.room_id'])
			.', DS_ACCESS_ACCESS = '.intval($p_access['wgroup.access'])
			//.', DATE_FROM = '.intval($p_access['from'])
			//.', DATE_TILL = '.intval($p_access['till'])
			.', UPDUSER = '.intval($p_upduser)
			.', UPDDAT  = '.time()
			.', STATE = "Y"'
			.' where'
			.'     DS_ACCESS_DS_ID = '.intval($p_ds_id)
			.' and DS_ACCESS_WGROUP_IDENT = '.intval($p_access['wgroup_ident'])
			.' and not (FLAGS & '.MC_FLAG_SYSTEM.')'
		;

		$this->DB->query($query);

		return true;
	}



	/**
	 * updateStatic
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	private function updateStatic($p_ds_id, $p_access, $p_upduser)
	{
		if (empty($p_access))
		{
			mc_Debug::error($p_access, 'access data is empty');
			return false;
		}

		$query = 'update DS_ACCESS'
			.' set'
			.'  DS_ACCESS_WGROUP_TYPE = "'.trim($p_access['wgroup_type']).'"'
			.', DS_ACCESS_WGROUP_COURSE_ID = '.intval($p_access['wgroup.room_id'])
			.', DS_ACCESS_ACCESS = '.intval($p_access['wgroup.access'])
			//.', DATE_FROM = '.intval($p_access['from'])
			//.', DATE_TILL = '.intval($p_access['till'])
			.', UPDUSER = '.intval($p_upduser)
			.', UPDDAT  = '.time()
			.', STATE = "Y"'
			.' where'
			.'     DS_ACCESS_DS_ID = '.intval($p_ds_id)
			.' and DS_ACCESS_WGROUP_IDENT = '.intval($p_access['wgroup_ident'])
			.' and (FLAGS & '.MC_FLAG_SYSTEM.')'
		;

		$this->DB->query($query);

		return true;
	} // end method updateStatic



	/**
	 * deleteDynamicAccessByDsGroup
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	private function deleteDynamicAccessByDsGroup($dsId, $groupId)
	{
		$query = 'delete from DS_ACCESS'
			.' where DS_ACCESS_DS_ID = '.intval($dsId)
			.'   and DS_ACCESS_WGROUP_IDENT = '.intval($groupId)
			.'   and not (FLAGS & '.MC_FLAG_SYSTEM.')'
		;
		$this->DB->query($query);

		return true;
	} // end method deleteDynamicAccessByDsGroup



	/**
	 * insertDsGroupAccess
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function insertDsGroupAccess($p_ds_id, $p_wgroup_ident, $p_access, $p_owner, $p_room_id, $p_flags = 0)
	{
		$this->insert($p_ds_id, $this->getGroupDetails($p_wgroup_ident, $p_access), $p_owner, $p_room_id, $p_flags);
	} // end method insertDsGroupAccess



	/**
	 * insert
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	private function insert($p_ds_id, $p_access, $p_owner, $p_room_id, $p_flags = 0)
	{
		if (empty($p_access))
		{
			mc_Debug::error($p_access, 'access data is empty');
			return false;
		}
		$query = 'insert into DS_ACCESS ('
			.'  DS_ACCESS_DS_ID'
			.', DS_ACCESS_WGROUP_IDENT'
			.', DS_ACCESS_WGROUP_TYPE'
			.', DS_ACCESS_WGROUP_COURSE_ID'
			.', DS_ACCESS_ACCESS'
//			.', DATE_FROM'
//			.', DATE_TILL'
			.', FLAGS'
			.', COURSE_ID'
			.', OWNER'
			.', INSDAT'
			.', STATE'
			.' ) values ('
			.'  '.intval($p_ds_id)
			.', '.intval($p_access['wgroup_ident'])
			.', "'.trim($p_access['wgroup_type']).'"'
			.', '.intval($p_access['wgroup.room_id'])
			.', '.intval($p_access['wgroup.access'])
//			.', '.intval($p_access['from'])
//			.', '.intval($p_access['till'])
			.', '.intval($p_flags)
			.', '.intval($p_room_id)
			.', '.intval($p_owner)
			.', "'.time().'"'
			.', "Y"'
			.' )'
		;

		$this->DB->query($query);

		return true;
	}



	/**
	 * getDsIdByRessource
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function getDsIdByRessource($p_ressource, $p_fk_id)
	{
		return $this->getDsId($p_ressource, $p_fk_id);
	}



	/**
	 * getDsIdByView
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function getDsIdByView($p_view, $p_fk_id)
	{
		$p_ressource = $this->DB->getOne(
			 'select VIEW_TAB_IDENT from VIEW'
			.' where '
			.'     VIEW_NAME = '.$this->DB->quote($p_view)
			.' and STATE = "Y"'
		);

		return $this->getDsId($p_ressource, $p_fk_id);
	}



	/**
	 * disableByRessource
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function disableByRessource($p_ressource, $p_fk_id, $p_wgroup_ident = null)
	{
		$l_ds_id = $this->getDsIdByRessource($p_ressource, $p_fk_id);

		if (empty($l_ds_id))
		{
			mc_Debug::error($l_ds_id, 'data record for res_id/fk_id = '.$p_ressource.'/'.$p_fk_id.' is not registered in table DS');
			return false;
		}

		return $this->disable($l_ds_id, $p_wgroup_ident);
	}



	/**
	 * disable
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function disable($p_ds_id, $p_wgroup_ident = null)
	{
		$l_del = 'update DS_ACCESS'
			.' set'
			.'  UPDDAT = "'.time().'"'
			.', UPDUSER = '.mc_Session::getUserId()
			.', STATE = "N"'
			.' where'
			.'     DS_ACCESS_DS_ID = '.intval($p_ds_id)
			.' and STATE = "Y"'
		;
		if ( !empty($p_wgroup_ident) )
		{
			$l_del .= ' and DS_ACCESS_WGROUP_IDENT = '.intval($p_wgroup_ident);
		}
		$l_res = $this->DB->query($l_del);

		return true;
	}



	/**
	 * getMemberGroupByRoom
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function getMemberGroupByRoom($roomId)
	{
		if ($roomId == mc_Session::getRoomId())
		{
			return mc_Session::getRoomGroup();
		}

		$roomId = intval($roomId);

		if (empty($this->member_groups[$roomId]))
		{
			$select = 'select WGROUP_ID'
  			.' from WGROUP'
  			." where WGROUP.COURSE_ID = {$roomId}"
  			.'   and WGROUP_TYPE  = "A"'
			;
  		$groupId = $this->DB->getOne($select);

      $this->member_groups[$roomId] = intval($groupId);
    }

		return $this->member_groups[$roomId];
	}



	/**
	 * getPrivateGroupByUser
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function getPrivateGroupByUser($p_user_ident)
	{
		if ($p_user_ident == mc_Session::getUserId())
		{
			return mc_Session::getPrivateGroup();
		}

		if (empty($this->private_groups[$p_user_ident]))
		{
			$select = 'select WGROUP_IDENT'
  			.'  from WGROUP'
  			.' where WGROUP.OWNER = '.intval($p_user_ident)
  			.'   and WGROUP_TYPE  = "U"'
  		;
  		$res = $this->DB->getOne($select);

			if (empty($res) ) {
				mc_Debug::error($select, 'private user group missing');
				die();
			}

      $this->private_groups[$p_user_ident] = intval($res);
    }

		return $this->private_groups[$p_user_ident];
	}



	/**
	 * getGroupDetails
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	private function getGroupDetails($p_wgroup_ident, $p_access)
	{
		if (empty($p_wgroup_ident))
		{
			return array();
		}

		$p_wgroup_ident = intval($p_wgroup_ident);

		if (isset($this->wgroup_details[$p_wgroup_ident]))
		{
			$this->wgroup_details[$p_wgroup_ident]['wgroup.access'] = $p_access;
			return $this->wgroup_details[$p_wgroup_ident];
		}

		$select = 'select WGROUP_IDENT, WGROUP_TYPE, WGROUP.COURSE_ID'
			.' from WGROUP'
			.' where'
			.'     WGROUP_IDENT = '.$p_wgroup_ident
			.' and COURSE_ID is not null'
		;
		$res = $this->DB->getRow($select);

		if (empty($res))
		{
			mc_Debug::error($res, "group not found for select '{$select}'");
			return array();
		}

		$details = array(
			'wgroup_ident'   => intval($res[0]),
			'wgroup_type'    => trim($res[1]),
			'wgroup.room_id' => intval($res[2]),
			'wgroup.access'  => $p_access,
//			'from' => ,
//			'till' => ,
		);

		$this->wgroup_details[$p_wgroup_ident] = $details;

		return $details;
	}



	/**
	 * read
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function read($p_ressource, $l_fk_id, $p_ignore_disabled_entries = true)
	{
		return $this->getArrAccessByDsId($this->getDsId($p_ressource, $l_fk_id), $p_ignore_disabled_entries);
	}



	/**
	 * getUserAccessByDsId
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function getUserAccessByDsId($p_ds_id, $p_restricted_to_room_id = null, $p_is_restrict_to_current_user = true)
	{
		$l_res = $this->getRawPermissions($p_ds_id, $p_restricted_to_room_id, $p_is_restrict_to_current_user);

		$l_merged_access = 0;
		while(list($l_id, $l_access, $l_ds_id, $l_room_id, $l_flags) = $l_res->fetchRow())
		{
			if ($p_ds_id == $l_ds_id)
			{
				$l_merged_access = $l_merged_access | $l_access;
			}
			else if ($l_access)
			{
				// shared permissions cause a local "show" permission only!
				$l_merged_access = $l_merged_access | MC_GROUP_SHOW;
			}
		}

		return $l_merged_access;
	}



	/**
	 * getRawPermissions
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function getRawPermissions($p_ds_id
		, $p_restricted_to_room_id = null
		, $p_is_restrict_to_current_user = false
		, $p_fetch_linked_permissions = true
	)
	{
		$l_from = array();

		if ($p_fetch_linked_permissions)
		{
			$l_from[] = 'DS_LINK';
			$l_where =
				 '     DS_LINK_DEST_DS_ID = '.intval($p_ds_id)
				.' and DS_LINK.STATE = "Y"'
				.' and DS_ACCESS_DS_ID = DS_LINK_SRC_DS_ID'
			;
		}
		else
		{
			$l_where = ' DS_ACCESS_DS_ID = '.intval($p_ds_id);
		}

		$l_from[] = 'DS_ACCESS';

		if ($p_restricted_to_room_id)
		{
			$l_where .=
				 ' and ('
				.' DS_ACCESS_WGROUP_COURSE_ID = '.intval($p_restricted_to_room_id)
				.' or DS_ACCESS_WGROUP_IDENT = '.$this->Session->getPrivateGroup()
				.')'
			;
		}

		$l_where .= ' and DS_ACCESS.STATE = "Y"';

		if ($p_is_restrict_to_current_user)
		{
			$l_from[] = 'REL_G_U';
			$l_where .=
				 ' and REL_G_U_WGROUP_IDENT = DS_ACCESS_WGROUP_IDENT'
				.' and REL_G_U_USER_IDENT = '.$this->Session->getUserId()
				.' and REL_G_U.STATE = "Y"'
			;
		}

		$l_res = $this->DB->query(
			 'select'
			.'  DS_ACCESS_WGROUP_IDENT'
			.', DS_ACCESS_ACCESS'
			.', DS_ACCESS_DS_ID'
			.', DS_ACCESS_WGROUP_COURSE_ID'
			.', DS_ACCESS.FLAGS'
			.' from '.implode(', ', $l_from)
			.' where '.$l_where
		);

		return $l_res;
	}



	/**
	 * getArrAccessByDsId
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function getArrAccessByDsId($p_ds_id, $p_ignore_disabled_entries = true, $p_ignore_flags = null)
	{
		if ($p_ignore_flags === null)
		{
			$p_ignore_flags = MC_FLAG_HIDDEN;
		}

		$l_res = $this->DB->query(
			 'select'
			.'  DS_ACCESS_WGROUP_IDENT'
			.', DS_ACCESS_ACCESS'
			.' from'
			.'  DS_ACCESS'
			.' where'
			.'     DS_ACCESS_DS_ID = '.intval($p_ds_id)
			.($p_ignore_flags
				? ' and not (FLAGS & '.$p_ignore_flags.')'
				: ''
			)
			.(($p_ignore_disabled_entries)
				? ' and STATE = "Y"'
				: ''
			)
		);

		$l_arr_access = array();
		while ($l_row = $l_res->fetchRow())
		{
			$l_arr_access[intval($l_row[0])] = intval($l_row[1]);
		}

		return $l_arr_access;
	}



	/**
	 * getMgrAccessFromRequest
	 *
	 * @author	Steffen Gross <gross@metacoon.net>
	 */
	public function getMgrAccessFromRequest($p_view_ident, $p_post, $p_right_type = null, $p_mgr_access_preset = null)
	{
		$l_room_group_ident = mc_Session::getRoomGroup();

		if (empty($p_mgr_access_preset))
		{
			$p_mgr_access_preset = array();
		}

		// preset all room member entry
		if (empty($p_mgr_access_preset[$l_room_group_ident]))
		{
			$p_mgr_access_preset[$l_room_group_ident] = 0;
		}

		if ($p_right_type == 1)
		{
			// no permission access for user
			return $p_mgr_access_preset;
		} // end if right_type 1

		if ($p_right_type == 2)
		{
			// simple permission access for user (show all / private only)
			if ( !isset($p_post['RIGHT_SIMPLE']) )
			{
				return $p_mgr_access_preset;
			}

			if ($p_post['RIGHT_SIMPLE'] == 3)
			{
				// set "all : show"
				$p_mgr_access_preset[$l_room_group_ident] = MC_GROUP_SHOW;
			}

			return $p_mgr_access_preset;
		} // end if right_type 2

		// right_type == 3 or whatever => full permission access for user

		/*
			example for post scheme

			GROUP[0][ID]      = 654
			GROUP[0][SHOW]    = 1
			GROUP[0][UPDATE]  = 1
			GROUP[0][DELETE]  = 1

			GROUP[1][ID]      = 8643
			GROUP[1][SHOW]    = 1
		*/

		if (empty($p_post['GROUP']))
		{
			// original preset, no form data received yet
			return $p_mgr_access_preset;
		}

		$l_mgr_access = array();

		// ATTENTION! form data received, only STATIC preset does matter now !

		if (empty($p_mgr_access_preset['static']))
		{
			// preset
			$l_mgr_access['static'] = array();
		}
		else
		{
			// copy static permissions if exist
			$l_mgr_access['static'] = $p_mgr_access_preset['static'];
		}

		// merge static and user defined permissions

		foreach($p_post['GROUP'] as $group)
		{
			$l_wgroup_ident = intval($group['ID']);

			if (empty($l_mgr_access[$l_wgroup_ident]))
			{
				$l_mgr_access[$l_wgroup_ident] = 0;
			}

			unset($group['ID']);

			foreach($group as $idx => $val)
			{
				$l_mgr_access[$l_wgroup_ident] = ($l_mgr_access[$l_wgroup_ident] | constant('MC_GROUP_'.$idx));
			}
		}

		return $l_mgr_access;
	} // end method getMgrAccessFromRequest



	/**
	 * packArrAccessByGroup
	 */
	private function packArrAccessByGroup($arrAccess)
	{
		$arrPacked = array();

		foreach($arrAccess as $index => $access)
		{
			if (empty($arrPacked[$index]))
			{
				$arrPacked[$index] = intval($access);
				continue;
			}

			$arrPacked[$index] = $arrPacked[$index] | intval($access);
		}

		return $arrPacked;
	}



}

