<?php
/*
* $McLicense$
*
* $Id$
*
*/

require_once(MCP_LIB_PATH . 'File.php');

class Extern
{
	private $DB;
	private $File;

	private $updateSrcAction;
	private $updateSrcExtern;
	private $pageBaseDir;
	private $tmplBaseDir;
	private $arrRoomId;
	private $arrTmplId;

	function __construct($actionSrc)
	{
		global $U, $File;

		$this->DB   = &$U->DB;
		$this->File = new File();

		if ( ! is_file($actionSrc) )
		{
			throw new mc_Exception($actionSrc, "'{$actionSrc}' is not a file");
		}

		$this->updateSrcAction = $actionSrc;
		$this->updateSrcExtern = dirname($actionSrc) . '/files.extern';

		$baseExtern = MC_BASE_DIR . 'extern' . DIRECTORY_SEPARATOR;

		$this->pageBaseDir = $baseExtern . 'C';
		$this->tmplBaseDir = $baseExtern . buildPath('Templates', 'ext_room_pages') . 'T';

		$select = 'select COURSE_ID from COURSE where STATE != "T"';
		$this->arrRoomId = $this->DB->getCol($select);

		$select = 'select ROOM_TYPE_IDENT from ROOM_TYPE where STATE != "T"';
		$this->arrTmplId = $this->DB->getCol($select);
	}



	public function getPageBaseDir() { return $this->pageBaseDir; }
	public function getTmplBaseDir() { return $this->tmplBaseDir; }

	public function getRooms()     { return $this->arrRoomId; }
	public function getTemplates() { return $this->arrTmplId; }



	public function updateExternFiles()
	{
		$pageBaseDir = $this->getPageBaseDir();
		$arrRoomId = $this->getRooms();
		$this->updateFiles($pageBaseDir, $arrRoomId);

		$tmplBaseDir = $this->getTmplBaseDir();
		$arrTmplId = $this->getTemplates();
		$this->updateFiles($tmplBaseDir, $arrTmplId);
	}



	public function repairCustom()
	{
		$this->repairRoomCustom();
		$this->repairTmplCustom();
	}



	private function repairRoomCustom()
	{
		SysMsg::showMsg("repair interchanged content of room files 'room_custom.conf.php' and 'room_custom.mod.php' inside room pages");

		$pageBaseDir = $this->getPageBaseDir();
		$arrRoomId = $this->getRooms();

		$count = $this->fixCustom($pageBaseDir, $arrRoomId);

		if ($count == 0) {
			SysMsg::showInfo("nothing to repair, all fine.");
		}
		else {
			SysMsg::showInfo("configuration of {$count} rooms repaired.");
		}

		return true;
	}



	private function repairTmplCustom()
	{
		SysMsg::showMsg("repair interchanged content of files 'room_custom.conf.php' and 'room_custom.mod.php' inside room templates");

		$tmplBaseDir = $this->getTmplBaseDir();
		$arrTmplId = $this->getTemplates();

		$count = $this->fixCustom($tmplBaseDir, $arrTmplId);

		if ($count == 0) {
			SysMsg::showInfo("no room template configurations to fix.");
		}
		else {
			SysMsg::showInfo("configuration of {$count} room templates fixed.");
		}

		return true;
	}



	private function fixCustom($pathPrefix, $arrIds)
	{
		// init counter for processed changes
		$count = 0;

		$confFileName = 'room_custom.conf.php';
		$modFileName  = 'room_custom.mod.php';
		$tmpFileName  = 'room_custom.switch';

		// loop id list (rooms or templates)
		foreach ($arrIds as $id)
		{
			$path = $pathPrefix . intval($id);
			if ( ! is_dir($path) )
			{
				// directory for external pages not found
				continue;
			}

			$confPath = $path . DIRECTORY_SEPARATOR . 'conf';
			if ( ! is_dir($confPath) )
			{
				// config directory inside external pages not found
				continue;
			}

			$confPath .= DIRECTORY_SEPARATOR;

			$customConfSrc = $confPath . $confFileName;
			if ( ! is_file($customConfSrc) )
			{
				$msg = "path '{$confPath}' found but file '{$confFileName}' is not available!";
				SysMsg::showError($msg);
				continue;
			}

			$customConfContent = $this->File->readFileContent($customConfSrc);
			if (strpos($customConfContent, '$LEFT_COLUMN') === false)
			{
				// contents of files are not flipped, all fine
				continue;
			}

			// contents of files are still flipped => flip files
			rename($confPath . $confFileName, $confPath . $tmpFileName);
			rename($confPath . $modFileName,  $confPath . $confFileName);
			rename($confPath . $tmpFileName,  $confPath . $modFileName);

			$count++;
		} // end foreach ($arrIds)

		return $count ;
	}



	public function updateFiles($pathPrefix, $arrIds)
	{
		// loop id list (rooms or templates)
		foreach ($arrIds as $id)
		{
			$destPath = $pathPrefix . intval($id);
			if ( ! is_dir($destPath) )
			{
				// directory for external pages not found
				continue;
			}

			$this->File->rcopy($this->updateSrcExtern, $destPath . DIRECTORY_SEPARATOR, false, true);
		} // end foreach ($arrIds)
	}


}