<?php
/*
* $McLicense$
*
* $Id$
*
*/


define("CONF_MODULES_FILENAME",	MC_BASE_DIR . "conf/modules.conf.php");
define("CONF_SYSTEM_FILENAME",	MC_BASE_DIR . "conf/system.conf.php");
define("CONF_DEFINES_FILENAME",	MC_BASE_DIR . "conf/defines.conf.php");

class File
{
	var $DB;

	var $PVersion;

	var $TEST;
	var $LASTCHANGE;

	var $OS;
	var $CMD_QUOTE;

	var $OUCH;
	var $collision_tracker;

	var $IS_FILE_WRITABLE_ERROR;

	var $silent;

	function File($overwrite = false)
	{
		global $U;

		$this->DB = $U->DB;

		$this->PVersion = &$U->Version;
		$this->Config   = &$U->Config;

		// checking operating system
		$this->OS = php_uname();
		$this->CMD_QUOTE = (strpos($this->OS, "Windows") === false) ? '' : "\"";

		$this->isTest = $this->Config->isTestEnabled();
		$this->PFINFO = '';
		$this->LASTCHANGE = $this->PVersion->getLastChange($this->PFINFO, $this->Config);
		$this->OUCH = ! empty($overwrite);

		$this->resetCt();

		$this->IS_FILE_WRITABLE_ERROR = FALSE;

		$this->silent = FALSE;
	}


	// collision tracker functions (tracks possible collisions of user customized files with files from update)
	function resetCt()
	{
		$this->collision_tracker = array();
	}

	function getCtSize()
	{
		sizeof($this->collision_tracker);
	}

	function addCtEntry($p_file)
	{
		$this->collision_tracker[] = $p_file;
	}

	function getCtAsString($p_glue = '')
	{
		implode($p_glue, $this->collision_tracker);
	}



	public function update($fromPath, $toPath, $bakPath, $checkOnly = false)
	{
		if ($checkOnly)
		{
			return $this->checkFileUpdate($fromPath, $toPath);
		}

		if (substr($fromPath, -1) == DIRECTORY_SEPARATOR)
		{
			$fromPath = substr($fromPath, 0, -1);
		}

		if (substr($toPath, -1) == DIRECTORY_SEPARATOR)
		{
			$toPath = substr($toPath, 0, -1);
		}

		if (substr($bakPath, -1) == DIRECTORY_SEPARATOR)
		{
			$bakPath = substr($bakPath, 0, -1);
		}

		return $this->processFileUpdate($fromPath, $toPath, $bakPath);
	}



	private function processFileUpdate($fromPath, $toPath, $bakPath)
	{
		$modDir = 0760;
		if ( ! file_exists($toPath) )
		{
			if ( $this->isTestEnabled() )
			{
				SysMsg::showInfo("mkdir(".$toPath.", ".$modDir.");");
			}
			else
			{
				mkdir($toPath, $modDir, true);
			}
		}

		if ( ! file_exists($bakPath) )
		{
			if ( ! $this->isTestEnabled() )
			{
				mkdir($bakPath, $modDir, true);
			}
		}

		if ( ! is_dir($fromPath) )
		{
			throw new Exxception("'{$fromPath}' is not a directory");
		}

		$handle = @opendir($fromPath);
		if ( ! $handle )
		{
			throw new Exxception("unable to open read from path '{$fromPath}'! message: '{$php_errormsg}'");
		}

		while ( ($node = readdir($handle)) !== false)
		{
			$collision = false;

			if (($node == ".") || ($node == "..") || ($node == ".svn") )
			{
				// skip directory
				continue;
			}

			$fromNode = $fromPath.DIRECTORY_SEPARATOR.$node;
			$toNode   = $toPath.DIRECTORY_SEPARATOR.$node;
			$bakNode  = $bakPath.DIRECTORY_SEPARATOR.$node;

			if (is_dir($fromNode) )
			{
				$this->processFileUpdate($fromNode, $toNode, $bakNode);
				continue;
			}

			if ( ! is_file($fromNode) )
			{
				// skip anything other than directory or file
				continue;
			}

			// auf kollision mit von benutzer geaenderten files pruefen und ggf. vermerken

			$nodeExists = file_exists($toNode);
			if ($nodeExists)
			{
				$contentFrom = file_get_contents($fromNode);
				$contentTo = file_get_contents($toNode);

				if ($contentFrom == $contentTo)
				{
					SysMsg::showInfo("file {$toNode} is already up to date: {SKIPPED}");
					continue;
				}

				if (filemtime($toNode) > $this->LASTCHANGE)
				{
					$this->addCtEntry($toNode);
					$collision = true;
				}
			}

			if ($this->OUCH == false)
			{
				// genderte dateien drfen NICHT berschrieben werden
				if ($collision == true)
				{
					SysMsg::showWarning("not touched (file has been modified): ".$toNode);
					continue;
				}
			}

			// genderte dateien drfen berschrieben werden

			if ($this->isTestEnabled() )
			{
				if ($nodeExists) {
					SysMsg::showMsg("would backup and REPLACE file : ".$toNode);
				}
				else {
					SysMsg::showMsg("would ADD file : ".$toNode);
				}
				continue;
			}

			if ($nodeExists)
			{
				if ( ! file_exists($bakPath) )
				{
					mkdir($bakPath, 0777, true);
				}

				$state = copy($toNode, $bakNode);

				if ($state)
				{
					SysMsg::showMsg("backed up file ".$toNode);
				}
				else
				{
					SysMsg::showError("backing up file ".$toNode.' {FAILED}. file skipped.');
					continue;
				}
			}

			$state = @copy($fromNode, $toNode);

			if ($state)
			{
				if ($nodeExists) {
					SysMsg::showInfo("replaced file : ".$toNode);
				}
				else {
					SysMsg::showInfo("added file : ".$toNode);
				}
				// replace token inside copied file
				$this->replaceToken($toNode);
			}
			else
			{
				if ($nodeExists) {
					SysMsg::showError("replacing file ".$toNode.' {FAILED}. file skipped.');
				}
				else {
					SysMsg::showError("adding file ".$toNode.' {FAILED}. file skipped.');
				}
			}

		} // end while
		closedir($handle);

		return true;
	}



	private function checkFileUpdate($fromPath, $toPath, $bakPath)
	{
		if ( ! file_exists($toPath) )
		{
			$parentPath = dirname($toPath);
			if ( file_exists($parentPath) && ! is_writable($parentPath) )
			{
				SysMsg::showWarning("permission error: directory '".$parentPath."' is not writeable!");
				$this->IS_FILE_WRITABLE_ERROR = true;
				return false;
			}
		}

		if ( ! is_dir($fromPath) )
		{
			throw new Exxception("'{$fromPath}' is not a directory");
		}

		$handle = @opendir($fromPath);
		if ( ! $handle )
		{
			throw new Exxception("unable to read from path '{$fromPath}'! message: '{$php_errormsg}'");
		}

		while ( ($node= readdir($handle)) !== false)
		{
			$collision = false;

			if (($node == ".") || ($node == "..") || ($node == ".svn") )
			{
				// skip directory
				continue;
			}

			$fromNode = $fromPath.$node;
			$toNode   = $toPath.$node;

			if (is_dir($fromNode) )
			{
				$this->checkFileUpdate($fromNode.DIRECTORY_SEPARATOR, $toNode.DIRECTORY_SEPARATOR);
				continue;
			}

			if ( ! is_file($fromNode) )
			{
				// anything other than directory or file?
				continue;
			}

			// auf kollision mit von benutzer geaenderten files pruefen und ggf. vermerken
			$nodeExists = file_exists($toNode);
			if ( ! $nodeExists)
			{
				// add, check parent node for write permission
				$parentNode = dirname($toNode);

				if ( ! is_writable(dirname($parentNode)) )
				{
					SysMsg::showWarning("permission error: can not create directory '".$parentNode."' (no write permission for PHP) !");
					$this->IS_FILE_WRITABLE_ERROR = true;
				}

				continue;
			}

			// check for replace
			if ( ! is_writable($toNode) )
			{
				SysMsg::showWarning("permission error: can not replace file '".$toNode."' (no write permission for PHP) !");
				$this->IS_FILE_WRITABLE_ERROR = true;
			}

		}
		closedir($handle);

		return true;
	}



	private function replaceToken($mcf)
	{
		global $TOKEN_REPLACE_LIST;

		if (is_array($TOKEN_REPLACE_LIST) == false)
		{
			throw new Exxception('parameter $TOKEN_REPLACE_LIST is not an array');
		}

		if ( ! is_file($mcf) )
		{
			SysMsg::showError('not a file : '.$mcf);
			return false;
		}

		$content = file_get_contents($mcf);
		$hash = md5($content);

		$content = $this->replace($content, $TOKEN_REPLACE_LIST);

		$match = null;
		if ( preg_match('@\[\[\[[A-Z_]]]]@', $content, $match) ) {
			SysMsg::showWarning('unreplaced token "'.$match[0].'" found in file '.$mcf);
		}

		if ( $hash == md5($content) )
		{
			// no changes
			return true;
		}

		$mcfh = fopen($mcf, "w+");
		if ($mcfh)
		{
			fwrite($mcfh, $content);
			fclose($mcfh);
		}
		else
		{
			SysMsg::showError('can not write file '.$mcf);
		}

		return true;
	}



	function replaceFileToken($p_file_name, $p_replace, $p_source_file_name = null)
	{
		if (empty($p_source_file_name))
		{
			$p_source_file_name = $p_file_name;
		}

		$p_file_name_src  = MC_ROOT_PATH.$p_source_file_name;
		$p_file_name_dest = MC_ROOT_PATH.$p_file_name;

		if ( !is_array($p_replace) ) {
			SysMsg::showError("not an array : '".$p_replace."' ! (file ".__FILE__.", line ".__LINE__.")");
			return false;
		}

		if ( !is_file($p_file_name_src) ) {
			SysMsg::showError("not a file : '".$p_file_name_src."' ! (file ".__FILE__.", line ".__LINE__.")");
			return false;
		}

		if ( !is_writable(dirname($p_file_name_dest)) ) {
			SysMsg::showError("not writable : '".dirname($p_file_name_dest)."' ! (file ".__FILE__.", line ".__LINE__.")");
			return false;
		}

		$l_file_content   = implode("", file($p_file_name_src));
		$l_file_hash = md5($l_file_content);

		$l_file_content = strtr($l_file_content, $p_replace);

		if ( $l_file_hash != md5($l_file_content) ) {
			// changes happened
			if ( !fwrite($l_handle = fopen($p_file_name_dest, "w+"), $l_file_content) ) {
				SysMsg::showWarning("file not writeable : '".$p_file_name_dest."' ! (file ".__FILE__.", line ".__LINE__.")");
				return false;
			}
			fclose($l_handle);

			clearstatcache();

			SysMsg::showMsg('token replaced in file '.basename($p_file_name));

			return 2;
		}

		clearstatcache();

		SysMsg::showMsg('no token replaced in file '.basename($p_file_name).' (no token found)');

		return 1;
	}



	function getAffectedCustomFileChanges()
	{
		if ($this->getCtSize() == 0)
		{
			if ($this->isTestEnabled() ) {
				return patch_test_label_02."<p>";
			}

			return patch_label_02."<p>";
		}

		if ($this->OUCH) {
			$comment = patch_test_label_09a;
		}
		else {
			$comment = patch_test_label_09b;
		}

		if ($this->isTestEnabled()) {
			return "<span class='ouch_info'>".patch_test_label_09." ".$comment."<br />"
				.patch_label_13."<a class=\"link\" href='".$this->Config->getFCLogUri()."' target='_blank'>'"
				.$this->Config->getFCLogUri()."'</a>.</span><p />"
			;
		}

		$uch_logfile = fopen($this->Config->getFCLogPath(), "w+");
		$l_ct_string = $this->getCtAsString("<br>\n");

		if ($uch_logfile) {
			fwrite($uch_logfile, $l_ct_string);
			fclose($uch_logfile);

			return "<span class='ouch_info'>".patch_label_09." ".$comment."<br /> ".patch_label_13."<a class=\"link\" href='".$this->Config->getFCLogUri()."' target='_blank'>'".$this->Config->getFCLogUri()."'</a>.</span><p />";
		}

		if ( ! $this->isTestEnabled() ) {
			return "<span class='ouch_error'>".patch_label_10."</span><div>".$l_ct_string."</div><p />";
		}

		return '';
	}



	function readFileContent($f_name, $as_array = false)
	{
		$contents = @file($f_name);
		if ($contents === FALSE)
		{
			SysMsg::showError("error: cannot read file '$f_name'");
			return false;
		}

		if ( ! $as_array )
		{
			return implode('', $contents);
		}

		return $contents;
	}



	function writeFileContent($name, $new_content)
	{
		if ( ! is_writable($name) )
		{
			SysMsg::showError("error: cannot write file '$name', file is not writable!<br />");
			return false;
		}

		$backup_name = $this->createBackupFromFile($name, true);
		if ( ! $backup_name)
		{
			SysMsg::showError("error: cannot create backup from file '$name'<br />");
			return false;
		}

		if (is_array($new_content) )
		{
			$str_content = implode("", $new_content);
		}
		else
		{
			$str_content = $new_content;
		}

		$handle = fopen($name, 'w');
		if ($handle)
		{
			fwrite($handle, $str_content);
			fclose($handle);
			return $backup_name;
		}
		else
		{
			SysMsg::showError("error: cannot open file '$name' (fopen failed)<br />");
			return false;
		}
	}



	function createBackupFromFile($p_dest, $p_rename = false)
	{
		$p_rename = (bool)$p_rename;
		$new_name = $p_dest.'.'.time().'.BAK';

		if ($p_rename == true)
	{
			if (rename($p_dest, $new_name) == false)
			{
				return false;
			}
		}
		else
		{
			if (copy($p_dest, $new_name) == false)
			{
				return false;
			}
		}

		return $new_name;
	}



	function hasSection($p_content, $p_section_name)
	{
		if (preg_match("/# ".$p_section_name."[\s]+/Us", $p_content) == 0)
		{
			return false;
		}

		return true;
	}



	function appendParamToConfSection($p_old_content, $p_section_name, $p_check_for_string, $p_add_content)
	{
		$p_new_content = $p_old_content;

		$match = array();
		// ermitteln des vollstndigen bereiches zwischen zwei sektions-namen
		// (erfasst kommentar der sektion sowie alle nachfolgenden parameter, kommentare und leerzeilen)
		// ACHTUNG : kann NICHT auf die letzte sektion zugreifen!
		if (preg_match("/# ".$p_section_name."(.*)?[\s]*# [A-Z_\x20\t]/Us", $p_old_content, $match) == 0)
		{
			// no match found
			return false;
		}

		if (strlen($match[1]) <= 10)
		{
			// match too small
			return false;
		}

		if (strpos($match[1], $p_check_for_string) !== false)
		{
			// already added
			return false;
		}

		return str_replace($match[1], $match[1].$p_add_content, $p_old_content);
	}



	function fetchConfSection($p_content, $p_section)
	{

		$match = array();
		// ermitteln des vollstndigen bereiches zwischen zwei sektions-namen
		// (erfasst kommentar der sektion sowie alle nachfolgenden parameter, kommentare und leerzeilen)
		// ACHTUNG : kann NICHT auf die letzte sektion zugreifen!
		if (is_array($p_content))
		{
			$l_content = implode('', $p_content);
		}
		else
		{
			$l_content = $p_content;
		}

		preg_match("/# ".$p_section."(.*)?[\s]*# [A-Z_\x20\t]/s", $l_content, $match);
		if ( isSet($match[1]) && strlen($match[1]) > 10 )
		{
			if (is_array($p_content))
			{
				$l_match_start = strpos($l_content, $match[1]);
				$l_match_end   = $l_match_start + strlen($match[1]);
				$l_len_count = 0;
				$l_content = array();
				foreach ($p_content as $l_line)
				{
					$l_len_count += strlen($l_line);

					if ($l_len_count >= $l_match_start)
					{
						$l_content[] = $l_line;
					}
					if ($l_len_count >= $l_match_end)
					{
						$l_content[] = $l_line;
						break;
					}
				}
				return $l_content;
			}
			else
			{
				return $match[1];
			}
		}

		return false;
	}



	function addSectionToFile($p_filename, $p_section_name, $p_section_content, $p_after_section_name = null)
	{
		SysMsg::showMsg("try to add a new section to ".$p_filename);

		$l_content = $this->readFileContent($p_filename);
		if ($l_content === false)
		{
			return false;
		}

		if ($this->hasSection($l_content, $p_section_name))
		{
			SysMsg::showInfo("skipped. (section ".$p_section_name." already exists)");
			return true;
		}

		$p_section_content = '# '.$p_section_name.'
'.$p_section_content.'



';

		if (empty($p_after_section_name) || !$this->hasSection($l_content, $p_after_section_name))
		{
			SysMsg::showInfo("new section ".$p_section_name." appended at end of file");
			return $this->addContentToFile($p_filename, $p_section_content);
		}

		$l_after_section = $this->fetchConfSection($l_content, $p_after_section_name);

		$l_new_content = str_replace($l_after_section, $l_after_section.$p_section_content, $l_content);

		SysMsg::showInfo("new section ".$p_section_name." inserted after section ".$p_after_section_name);

		return $this->writeFileContent($p_filename, $l_new_content);
	}



	function removeConfSection($p_filename, $p_section)
	{
		$l_content = $this->readFileContent($p_filename, true);

		if ( !$this->searchLine($l_content, '# '.$p_section) )
		{
			SysMsg::showInfo('section '.$p_section.' in file "'.$p_filename.'" has been removed already (or did never exist). operation {SKIPPED}');
			return true;
		}

		$l_lenght = count($l_content);
		$l_content[$l_lenght] = $l_content[$l_lenght - 1];
		$l_content[$l_lenght - 1] = '# EOF_SECTIONS';
		$l_content = implode('', $l_content);
		$l_content = preg_replace("/# ".$p_section."(.*)?([\s]*# [A-Z_\x20\t])/s", '$2', $l_content);

		if (!$l_content)
		{
			SysMsg::showError('removing section '.$p_section.' from file "'.$p_filename.'" {FAILED}');
			return false;
		}

		$l_content = str_replace("# EOF_SECTIONS", '', $l_content);

		if ($this->writeFileContent($p_filename, $l_content))
		{
			SysMsg::showInfo('removing section '.$p_section.' from file "'.$p_filename.'" {SUCCEED}');
			return true;
		}

		SysMsg::showError('removing section '.$p_section.' from file "'.$p_filename.'" {FAILED}');
		return false;
	}



	function moveLine($p_content, $p_catch_string, $p_target_string, $p_add_below = true)
	{
		if (is_array($p_content) == false)
		{
			throw new Exception('error: content is not an array');
		}

		$l_new_content = array();
		$l_catch = null;

		// catch and strip row from content
		foreach ($p_content as $row)
		{
			if (strpos(trim($row), $p_catch_string) === 0)
			{
				$l_catch = $row;
			}
			else
			{
				$l_new_content[] = $row;
			}
		}

		if ($l_catch === null)
		{
			if ($this->silent == false)
			{
				SysMsg::showWarning('can\'t find a matching string for "'.htmlentities($p_catch_string).'"!');
			}
			return false;
		}

		return $this->addLine($l_new_content, $p_target_string, $l_catch, $p_add_below);
	}



	function addLine($p_content, $p_catch_string, $p_add_string, $p_add_below = true)
	{
		if (is_array($p_content) == false)
		{
			throw new Exception('error: content is not an array');
		}

		$l_new_content = array();
		$l_added = false;

		// insert catched row at target position
		foreach ($p_content as $row)
		{
			if ($p_add_below == true)
			{
				$l_new_content[] = $row;
			}
			if (strpos(trim($row), $p_catch_string) === 0)
			{
				$l_new_content[] = $p_add_string;
				$l_added = true;
			}
			if ($p_add_below == false)
			{
				$l_new_content[] = $row;
			}
		}

		if ($p_add_below == false)
		{
			if ($this->silent == false)
			{
				SysMsg::showWarning('can\'t find a matching string for "'.htmlentities($p_catch_string).'"!');
			}
			return false;
		}

		return $l_new_content;
	}



	function removeLine($p_content, $p_catch_string)
	{
		if (is_array($p_content) == false)
		{
			throw new Exception('error: content is not an array');
		}

		$l_new_content = array();
		$l_removed = false;

		// catch and strip row from content
		foreach ($p_content as $row)
		{
			if (strpos(trim($row), $p_catch_string) === 0)
			{
				// leave out
				$l_removed = true;
			}
			else
			{
				$l_new_content[] = $row;
			}
		}

		if ($l_removed == false)
		{
			if ($this->silent == false)
			{
				SysMsg::showWarning('can\'t find a matching string for "'.htmlentities($p_catch_string).'"!');
			}
			return false;
		}

		return $l_new_content;
	}



	function replaceLine($p_content, $p_catch_string, $p_replace_string)
	{
		if (is_array($p_content) == false)
		{
			throw new Exception('error: content is not an array');
		}

		$l_new_content = array();
		$l_replaced = false;

		// catch and strip row from content
		foreach ($p_content as $row)
		{
			if (strpos(trim($row), $p_catch_string) === 0)
			{
				$l_replaced = true;
				$l_new_content[] = $p_replace_string;
			}
			else
			{
				$l_new_content[] = $row;
			}
		}

		if ($l_replaced == false)
		{
			if ($this->silent == false)
			{
				SysMsg::showWarning('can\'t find a matching string for "'.htmlentities($p_catch_string).'"!');
			}
			return $p_content;
		}

		return $l_new_content;
	}



	function searchLine($p_content, $p_catch_string, $p_offset = 0)
	{
		if ( !is_array($p_content) )
		{
			throw new Exception('error: content is not an array');
		}

		$p_offset = intval($p_offset);

		// search catch string
		foreach ($p_content as $id => $row)
		{
			if ($p_offset <= $id)
			{
//				if (strpos(trim($row), $p_catch_string) === 0)
				if (strpos(trim($row), $p_catch_string) !== false)
				{
					return $id;
				}
			}
		}

		return false;
	}



	function getLineByRegExp($p_pattern, $p_content, $p_offset = 0)
	{
		if (is_array($p_content) == false)
		{
			throw new Exception('error: content is not an array');
		}

		$p_offset = intval($p_offset);

		// search catch string
		foreach ($p_content as $id => $row)
		{
			if ($p_offset <= $id)
			{
				if (preg_match($p_pattern, $row))
				{
					return $id;
				}
			}
		}

		return false;
	}



	function addContentToFile($p_filename, $p_append_content, $p_check_for_string = null)
	{

		SysMsg::showInfo("add content to ".$p_filename);

		$l_old_content = $this->readFileContent($p_filename);
		if ($l_old_content === false)
		{
			return false;
		}

		if ( !empty($p_check_for_string) && strpos($l_old_content, $p_check_for_string) !== false)
		{
			SysMsg::showInfo("skipped. (checked for ".$p_check_for_string.": already exists)");
			return true;
		}

		$tmp = rtrim($l_old_content);
		if (substr($tmp, -2) == '?'.'>')
		{
			$l_new_content  = substr($tmp, 0, -2)."\n";
			$l_new_content .= $p_append_content."\n?".'>';
		}
		else
		{
			$l_new_content .= $p_append_content;
		}

		return $this->writeFileContent($p_filename, $l_new_content);
	}



	function delete($p_filelist, $permissions_check_only = false)
	{

		if ( is_array($p_filelist) == false )
		{
			SysMsg::showError("parameter \$p_filelist is not an array !");
			return false;
		}
		else if (sizeof($p_filelist) == 0)
		{
			if ($permissions_check_only == false)
			{
				SysMsg::showInfo("no files/directories to delete (empty list).");
			}
			else
			{
				SysMsg::showInfo("no files/directories to delete (empty list).");
			}
			return true;
		}

		$p_src_path    = MC_ROOT_PATH;
		$p_backup_path = $this->Config->getBakPath();

		if (is_string($p_filelist) )
		{
			$p_filelist = array($p_filelist);
		}

		foreach ( $p_filelist as $l_filepath )
		{
			$l_src_filename  = $p_src_path.$l_filepath;
			$l_dest_filename = $p_backup_path.$l_filepath;

			if ($l_filepath[0] == "/")
			{
				SysMsg::showError("absolute path statement found ('".$l_src_filename."')! file {SKIPPED} (absolute paths are forbidden).");
			}

			if (strpos($l_filepath, '../') !== false)
			{
				SysMsg::showError("parent path statement found ('".$l_src_filename."')! file {SKIPPED} (parent paths are forbidden).");
			}


			if ( ! file_exists($l_src_filename))
			{
				if ($permissions_check_only == false)
				{
					SysMsg::showInfo("removing file '".$l_src_filename."' {SKIPPED} : file not found (already removed).");
				}
				continue;
			}

			$this->createDirectory(dirname($l_dest_filename));

			if (filemtime($l_src_filename) > $this->LASTCHANGE)
			{
				$this->addCtEntry($l_src_filename." (".filemtime($l_src_filename)." > ".$this->LASTCHANGE.")");
			}

			if ($this->isTestEnabled() || $permissions_check_only == true)
			{
				if ($permissions_check_only == false)
				{
					SysMsg::showMsg("would rename('".$l_src_filename."', '".$l_dest_filename."');");
				}
			}
			else
			{
				if ( ! is_writable($l_src_filename) ) {
					SysMsg::showError("can not backup file/directory '{$l_src_filename}' (permission denied: file not writable)");
				}
				else if ( ! is_writable( dirname($l_dest_filename) ) ) {
					SysMsg::showError("can not backup file/directory '{$l_dest_filename}' (permission denied: backup path not writable)");
				}
				else if ( rename($l_src_filename, $l_dest_filename) ) {
					SysMsg::showInfo("file/directory '{$l_src_filename}' deleted (moved to backup)");
				}
				else {
					SysMsg::showError("can not delete file/directory '{$l_src_filename}'");
				}
			}

		} // end foreach

		return true;
	}



	function move($p_path_list)
	{
		$p_src_path = MC_ROOT_PATH;

		if ( !is_array($p_path_list) )
		{
			SysMsg::showError("parameter \$p_path_list is not an array !");
			return false;
		}
		else if (sizeof($p_path_list) == 0)
		{
			SysMsg::showInfo("no files/directories to move (empty list).");
			return false;
		}

		while ( list($from_path, $to_path) = each($p_path_list) )
		{
			if ( file_exists($p_src_path.$from_path) )
			{
				if (is_dir($p_src_path.$to_path) == false)
				{
					$this->createDirectory(dirname($p_src_path.$to_path));
				}
				if ($this->isTestEnabled() )
				{
//					$this->showInfo("rename('".$p_src_path.$l_src_filename."', '".$p_src_path.$l_dest_filename."');");
					SysMsg::showInfo("rename('".$p_src_path.$from_path."', '".$p_src_path.$to_path."');");
				}
				else
				{
					if (rename($p_src_path.$from_path, $p_src_path.$to_path))
					{
						SysMsg::showInfo('path "'.$p_src_path.$from_path.'" moved to "'.$p_src_path.$to_path.'"');
					}
				}
			}
			else
			{
				SysMsg::showWarning("can't move path '".$p_src_path.$from_path."' (path doesn't exist). path {SKIPPED}.");
			}
		}

		return true;
	}



	function copy($p_src_path, $p_dest_path, $debug = true)
	{
		if ( file_exists($p_dest_path) )
		{
			return true;
		}

		if ( ! file_exists($p_src_path) )
		{
			if ($debug)
			{
				SysMsg::showError("can't copy from path '".$p_src_path."' (source doesn't exist).");
			}
			return false;
		}

		$l_dirname = dirname($p_dest_path);
		if ( ! file_exists($l_dirname) )
		{
			if ( ! $this->createDirectory($l_dirname) )
			{
				if ($debug)
				{
					SysMsg::showError("can't copy to path '".$p_dest_path."' (can't access/create target's parent directory).");
				}
				return false;
			}
		}

		if (is_file($p_src_path))
		{
			copy($p_src_path, $p_dest_path);
			return true;
		}

		if (is_dir($p_src_path))
		{
			mkdir($p_dest_path);
			chmod($p_dest_path, 0777);
			return true;
		}

		if ($debug)
		{
			SysMsg::showError("unknown source type from '".$p_dest_path."' ( is_dir == false && is_file == false ).");
		}

		return true;
	}



	public function rcopy($from, $to, $showMsg = false, $createBackup = false)
	{
		if ($this->isTestEnabled() )
		{
			SysMsg::showWarning('patch is running in test mode - directory copy skipped!');
			return true;
		}

		if ( ! $this->createDirectory($to) )
		{
			return false;
		}

		$handle = opendir($from);
		if ( ! $handle)
		{
			SysMsg::showError("can not open path '{$from}'");
			$state = false;
		}

		$state = true;
		while ( ($file = readdir($handle)) !== false)
		{
			if ($file == ".")    { continue; }
			if ($file == "..")   { continue; }
			if ($file == ".svn") { continue; }

			$srcPath  = $from . DIRECTORY_SEPARATOR . $file;
			$destPath = $to   . DIRECTORY_SEPARATOR . $file;

			if (is_dir($srcPath) )
			{
				$state = $this->rcopy($srcPath, $destPath, $showMsg, $createBackup);
				if ( ! $state )
				{
					return false;
				}
				continue;
			}

			if ( ! is_file($srcPath) )
			{
				continue; // not a file and not a directory
			}

			if (file_exists($destPath) && $createBackup)
			{
				// create backup if new content differs from current content
				$srcContent  = file_get_contents($srcPath);
				$destContent = file_get_contents($destPath);
				$isContentEqual = ($srcContent == $destContent);
				unset($srcContent, $destContent);
				if ($isContentEqual)
				{
					// file content is equal anyway => no need for replace or backup
					continue;
				}
				else
				{
					// backup file before replacement
					$bakPath = $destPath . '.' . time() . '.BAK';
					copy($destPath, $bakPath);
				}
			}

			$state = @copy($srcPath, $destPath);
			if ( ! $state )
			{
				if ($showMsg)
				{
					SysMsg::showError("copy '{$srcPath}' to '{$destPath}' {FAILED}.");
				}
				return false;
			}

			if ($showMsg)
			{
				SysMsg::showInfo("copy '{$srcPath}' to '{$destPath}' {SUCCEED}.");
			}
			if (is_file($destPath) )
			{
				@chmod($destPath, 0777);
			}

		} // end while (readdir)

		closedir($handle);

		return true;
	}



	function rdelete($p_target_path)
	{
		if(!$dir_handle = @opendir($p_target_path))
		{
			return;
		}

		while (($obj = readdir($dir_handle)) !== false)
		{
			if ($obj == '.' || $obj == '..')
			{
				continue;
			}

			if (is_dir($p_target_path.DIRECTORY_SEPARATOR.$obj))
			{
				deleteDirectory($p_target_path.DIRECTORY_SEPARATOR.$obj);
			}
			else
			{
				@unlink($p_target_path.DIRECTORY_SEPARATOR.$obj);
			}
		}

		closedir($dir_handle);
		@rmdir($p_target_path);

		return true;
	}



	function createDirectory($p_target_path)
	{

		if ( is_dir($p_target_path) )
		{
			if ( ! is_writable($p_target_path) )
			{
				// path already exists but is NOT writeable
				SysMsg::showWarning("skip creating directory '$p_target_path' (already exists). Warning : existing directory is NOT WRITEABLE!");
				$this->IS_FILE_WRITABLE_ERROR = true;
				return false;
			}
		}
		else
		{
			if ( ! $this->isTestEnabled() )
			{
				if ( ! @mkdir($p_target_path, 0777, true) )
				{
					SysMsg::showError("can't create directory '$p_target_path' !");
					$this->IS_FILE_WRITABLE_ERROR = true;
					return false;
				}
			}
		}

		return true;
	}



	function createFile($p_target_path, $p_content)
	{
		$l_dirname = basename($p_target_path);

		if ( is_dir($l_dirname) )
		{
			if ( is_writable($l_dirname) == false )
			{
				// already exists but NOT writeable
				SysMsg::showError("can not create file '$p_target_path'. path '$l_dirname' is NOT WRITEABLE!");
				return false;
			}
		}
		else
		{
			if ($this->createDirectory($l_dirname) == false)
			{
				SysMsg::showError("can not create directory '$l_dirname' !");
				return false;
			}
		}

		if (file_exists($p_target_path))
		{
			SysMsg::showError("file '$p_target_path' already exists!");
			return false;
		}

		$file_handle = @fopen($p_target_path, 'w');
		if (!$file_handle)
		{
			SysMsg::showError("opening (new) file '$p_target_path' failed!");
			return false;
		}

		fwrite($file_handle, $p_content);
		fclose($file_handle);

		return true;
	}



	function createBackupPath()
	{
		$backupPath = $this->Config->getBakPath();
		$parentPath = dirname($backupPath);

		if ( ! is_dir($parentPath) )
		{
			SysMsg::showError("'{$parentPath}' is not a directory! unable to create backup path!");
			$this->IS_FILE_WRITABLE_ERROR = true;
			return false;
		}

		if ( ! is_writable($parentPath) )
		{
			SysMsg::showError("directory '{$parentPath}' is not writeable! unable to create backup path!");
			$this->IS_FILE_WRITABLE_ERROR = true;
			return false;
		}

		if ( mkdir($backupPath, 0777) )
		{
			SysMsg::showInfo("backup directory '".$backupPath."' created.");
			return true;
		}

		SysMsg::showWarning("Error : creating backup directory '".$backupPath."' FAILED!");
		$this->IS_FILE_WRITABLE_ERROR = true;
		return false;
	}



	/**
	 * stripping comments and whitespaces from file content
	 */
	function cleanFileContent($p_content)
	{

		// fetching lines with doubleshlash comments
	//	$p_content=preg_replace("/[\x20\t]*[\/]{2}[\x20\S]*[\r\n]{1}/", "", $p_content);
		$p_content = preg_replace("/[\x20\t]*\/[\/]+.*[\r\n]{1,2}/", "", $p_content);

		// fetching lines with doublecross comments
	//	$p_content=preg_replace("/[\r\n]{1}[\x20\t]*[#]+[\x20\S]*[\r\n]{1}/", "\n", $p_content);
		$p_content = preg_replace("/[\r\n]{1,2}[\x20\t]*[#]+.*[\r\n]{1,2}/", "\n", $p_content);

		// fetching slash-star comment areas
		// execute AFTER double-slash- / double-cross-comments
		$p_content = preg_replace("@[\x20\t]*[/]+\*.*?\*/@s", "", $p_content);

		// fetching whitespace areas (line breaks, spaces, tabs) ending with at least one line break
	//	$p_content=preg_replace("/[\s]*[\r\n]{1}[\x20\t]*/", "\n", $p_content);
		$p_content = preg_replace("/[\s]*[\r\n]{1}[\x20\t]*/", "\n", $p_content);

		return $p_content;
	}



	/**
	 *
	 */
	function isUTF8($p_content)
	{
		$l_charset = mb_detect_encoding($p_content.'xyz', 'UTF-8, ISO-8859-1');

		if ($l_charset != 'UTF-8')
		{
			return false;
		}

		return true;
	}



	/**
	 *
	 */
	function uft8Encode($p_content, $p_ignore_charset = false)
	{
		if ($p_ignore_charset || $this->isUTF8($p_content) == false)
		{
			$p_content = utf8_encode($p_content);
		}

		return $p_content;
	}



	/**
	 *
	 */
	function switchMsg($p_state = null)
	{
		if ($p_state !== null)
		{
			$this->silent	= empty($p_state);
		}

		return $p_state;
	}



	/**
	 *
	 */
	function modifyLangFile ($p_path, $l_arr)
	{
		foreach($l_arr as $token => $settings)
		{
			$target = $settings['position'];
			$action = $settings['action'];
			$patternSingle = null;

			if (empty($settings['if_exists']) == false)
			{
				$pattern = $settings['if_exists'];
				$match_must_exist = true;
			}
			else if (empty($settings['if_not_exists']) == false)
			{
				$pattern = $settings['if_not_exists'];
				$match_must_exist = false;
			}
			else
			{
				$pattern = 'define("'.$token.'"';
				$patternSingle = "define('".$token."'";

				if ($action == 'replace') {
					$match_must_exist = true;
				}
				else {
					$match_must_exist = false;
				}
			}


			foreach($settings['wording'] as $lang => $wording)
			{
				$l_path = MC_ROOT_PATH.strtr($p_path, array(
					'{LANG}' => strtoupper($lang),
					'{lang}' => strtolower($lang),
				));

				$l_content_old = $this->readFileContent($l_path, true);

				if ( ! is_array($wording) )
				{
					$wording  = array('define("'.$token.'", "'.addcslashes($wording, '\\"').'");'."\n");
				}

				$match = $this->searchLine($l_content_old, $pattern);
				if ($match === false && ! empty($patternSingle) ) {
					$match = $this->searchLine($l_content_old, $patternSingle);
				}

				if ($match_must_exist)
				{
					if ($match === false)
					{
						SysMsg::showInfo('token "'.$token.'" ('.$lang.') does not match (match is required for update). entry {SKIPPED}.');
						continue;
					}
				}
				else // match must not exist
				{
					if ($match !== false)
					{
						SysMsg::showInfo('token "'.$token.'" ('.$lang.') matches already (mismatch is required for update). entry {SKIPPED}.');
						continue;
					}
				}

				$l_position = $this->searchLine($l_content_old, $target);
				if ($l_position === false)
				{
					SysMsg::showWarning('position "'.$target.'" for token "'.$token.'" ('.$lang.') not found (entry has already changed?). entry {SKIPPED}.');
					continue;
				}

				$l_content_new = $l_content_old;

				switch($action)
				{
					case 'add' :
						array_splice($l_content_new, ($l_position + 1), 0, $wording);
						break;

					case 'replace' :
						array_splice($l_content_new, $l_position, 1, $wording);
						break;

					default :
						SysMsg::showError('unknown "'.$action.'" at token ('.$lang.'::'.$token.'). entry {SKIPPED}! (allowed actions: add|replace)');
						continue 2;
				}

				if ($this->writeFileContent($l_path, implode('', $l_content_new)))
				{
					SysMsg::showInfo($action.' label for token "'.$token.'" ('.$lang.') {SUCCEEDED}.');
				}
				else
				{
					SysMsg::showError($action.' label for token "'.$token.'" ('.$lang.') {FAILED}!');
				}

			} // end foreach $settings

		} // end foreach $l_arr

		return true;
	}



	/**
	 * isTestEnabled
	 */
	function isTestEnabled()
	{
		return $this->isTest;
	}



	/**
	 * replace
	 */
	function replace($value, &$replace)
	{
		$pattern = array_keys($replace);
		$value = str_replace($pattern, $replace, $value);

		return $value;
	}



}

