<?php
/**
 * @package core
 */
/*
* $McLicense$
*
* $Id$
*
*/


/*
// http://de.php.net/manual/de/reserved.variables.php

$GLOBALS   Referenziert alle Variablen, die im globalen Gltigkeitsbereich vorhanden sind

$_SERVER   Informationen ber Server und Ausfhrungsumgebung
$_ENV      Umgebungsvariablen

$_GET      HTTP GET-Variablen
$_POST     HTTP POST-Variablen
$_COOKIE   HTTP Cookies
$_REQUEST  HTTP Request-Variablen
$_FILES    HTTP Dateiupload-Variablen

$_SESSION  Sessionvariablen

$HTTP_RAW_POST_DATA  Raw POST-Daten

$php_errormsg  Die vorangegangene Fehlermeldung
$http_response_header  HTTP Response-Header

$argc  Die Anzahl der an das Skript bergebenen Argumente
$argv  Array der an das Skript bergebenen Argumente

*/

// define source types (binary code)
define('MC_REQ_SRC_POST',   1); // _POST
define('MC_REQ_SRC_GET',    2); // _GET
define('MC_REQ_SRC_FORM',   3); // default, merges _GET, _POST
define('MC_REQ_SRC_COOKIE', 4); // _COOKIE
define('MC_REQ_SRC_ALL',    7); // merges _GET, _POST, _COOKIE


// define regular expressions to validate request parameters
//define('MC_REGEXP_SESSION',   '/^[0-9a-f]{32}$/'); // old session ids
define('MC_REGEXP_SESSION',   '/^[0-9a-zA-Z-,]{26,40}$/');
//define('MC_REGEXP_VIEW',      '/^V[A-Z0-9_]*$/');
//define('MC_REGEXP_VIEW',      '/^[A-Z0-9_]*$/');


/**
 * Request
 *
 * @author gross
 * @version 1.0
 * @package core
 * @subpackage classes.new
 */
class mc_Request
{
	/**
	 * request data, see constructor for default content
	 */
	public $data;

	/**
	 *
	 */
	private $param_type = array(
		//'name' => 'type',
		'ENT'    => 'char',
		'ENTSOURCE' => 'char',
		'LAYOUT' => 'char',
		'GROUP'  => 'array',
	);

	/**
	 * non-listed parameters default is NULL
	 */
	private $param_default = array(
		//'name' => 'defaultvalue',
		'GROUP'  => array(),
	);

	/**
	 *
	 */
	private $param_regexp = array(
		//'name' => 'regexp',
		//'ENT'  => MC_REGEXP_VIEW,
		//'ENTSOURCE' => MC_REGEXP_VIEW,
	);

	/**
	 * system settings
	 */
	public $request_order;
	public $variables_order;



	/**
	 *
	 */
	public function __construct()
	{
		// get and prepare request data depending on magic quote settings
//		$this->data = array_merge($_GET, $_POST);
		$this->request_order   = ini_get('request_order');
		$this->variables_order = ini_get('variables_order');

		if ($this->request_order != 'GPS' && ( !empty($this->request_order) || $this->variables_order != 'GPS') )
		{
			$_REQUEST = array_merge($_GET, $_POST, $_SERVER);
		}
		$this->data = &$_REQUEST;
		$this->data = transcribe($this->data); // method transcribe is from file "php5.inc.php"
	} // end constructor



	/**
	 *
	 */
	public function exists($p_varname)
	{
		if (isset($this->data[$p_varname]))
		{
			return true;
		}

		return false;
	} // end method has



	/**
	 *
	 */
	public function get($p_varname, $p_default = null)
	{
		// check if parameter is defined
		if (empty($this->param_type[$p_varname]))
		{
			mc_Debug::error($p_varname, 'parameter type missing');
			return null;
		}

		if (isset($this->param_type[$p_varname]))
		{
			$l_type = $this->param_type[$p_varname];
		}
		else
		{
			$l_type = null;
		}

		switch (true)
		{
			// check request data
			case isset($this->data[$p_varname]) :
				$l_value = $this->data[$p_varname];
				break;

			// return default preset if preset exists
			case isset($this->param_default[$p_varname]) :
				return $this->param_default[$p_varname];

			// return type default
			default:
				return $this->getDefault($l_type, $p_default, $p_varname, $this->param_default);
		} // end switch

		if (empty($this->param_regexp[$p_varname]))
		{
			// return type casted value if regular expression check is not required
			return $this->cast($l_value, $this->param_type[$p_varname]);
		}

		if (preg_match($this->param_regexp[$p_varname], $l_value, $l_match))
		{
			if (sizeof($l_match) > 0)
			{
				// return type casted match of regular expression
				return $this->cast($l_match[0], $this->param_type[$p_varname]);
			}
		}
		// regular expression match was empty

		// return type default
		return $this->getDefault($l_type, $p_default, $p_varname, $this->param_default);
	} // end method get



	/**
	 *
	 */
	public function set($p_var, $p_value = null)
	{
		if ( !is_array($p_var) )
		{
			$_GET[$p_var] = $p_value;
			$_POST[$p_var] = $p_value;
			$this->data[$p_var] = $p_value; // reference to _REQUEST

			return;
		} // end if

		foreach($p_var as $p_key => $p_value)
		{
			$_GET[$p_key] = $p_value;
			$_POST[$p_key] = $p_value;
			$this->data[$p_key] = $p_value; // reference to _REQUEST
		} // end foreach

		return;
	} // end method set



	/**
	 *
	 */
	public function erase($p_varname)
	{
		if(isset($_GET[$p_varname]))  { unset($_GET[$p_varname]); }
		if(isset($_POST[$p_varname])) { unset($_POST[$p_varname]); }
		if(isset($this->data[$p_varname])) { unset($this->data[$p_varname]); } // reference to _REQUEST
	} // end method set



	/**
	 *
	 */
	public static function fetch($p_var, $p_type, $p_default = null, $p_regmatch = null, $p_source = MC_REQ_SRC_FORM)
	{
		$p_var  = trim($p_var);
		$p_type = strtolower($p_type);

		if (is_array($p_source))
		{
			if ( isset($p_source[$p_var]) )
			{
				$l_value = $p_source[$p_var];
			}
			else
			{
				return mc_Request::getDefault($p_type);
			}
		}
		else
		{
			if ($p_source & MC_REQ_SRC_COOKIE && isset($_COOKIE[$p_var]) )
			{
				$l_value = transcribe($_COOKIE[$p_var]);
			}
			elseif ($p_source & MC_REQ_SRC_POST && isset($_POST[$p_var]) )
			{
				$l_value = transcribe($_POST[$p_var]);
			}
			elseif ($p_source & MC_REQ_SRC_GET && isset($_GET[$p_var]) )
			{
				$l_value = transcribe($_GET[$p_var]);
			}
			else
			{
				return mc_Request::getDefault($p_type, $p_default);
			}
		}

		// perform regular expression match
		if ($p_regmatch)
		{
			if (preg_match($p_regmatch, $l_value, $l_match) == 0)
			{
				return mc_Request::getDefault($p_type, $p_default);
			}
			$l_value = $l_match[0];
		}

		return mc_Request::cast($l_value, $p_type);
	} // end method fetch



	/**
	 *
	 */
	private static function getDefault($p_type, $p_default = null, $p_varname = null, $p_arr_default = array())
	{
		// check for global default
		// developers note : this array is not called as an object member for using getDefault as static function via scope
		if ($p_varname && isset($p_arr_default[$p_varname]))
		{
			// return global default preset (from constructor)
			return $p_arr_default[$p_varname];
		}

		// return type default if default is null
		if ($p_default === null)
		{
			switch ($p_type)
			{
				case 'array':
					return array();

				default :
					return null;
			}
		}

		// return local default (if default is NOT null)
		return $p_default;
	} // end method getDefault



	/**
	 *
	 */
	private static function cast($p_value, $p_type)
	{
		// parameter type check or cast
		switch ($p_type)
		{

/**********************************************/
/*  !!! !!! !!!  IMPORTANT NOTE  !!! !!! !!!  */
/**********************************************/
/*                                            */
/*    p_type is referred to as a string to    */
/*  enable direct type casts from COL_TYPE !  */
/*                                            */
/**********************************************/

			case 'int':
			case 'integer':
//				if (is_int($p_value)) { mc_Debug::error($p_value, 'parameter is not an integer!'); }
				return intval($p_value);

/*
			case 'float':
				// ???
				break;
*/

			case 'char':
			case 'string':
				if (!is_string($p_value)) { mc_Debug::error($p_value, 'parameter is not a string!'); }
				return $p_value;
				break;

			case 'string_lower':
				if (!is_string($p_value)) { mc_Debug::error($p_value, 'parameter is not a string!'); }
				return strtolower($p_value);
				break;

			case 'string_upper':
				if (!is_string($p_value)) { mc_Debug::error($p_value, 'parameter is not a string!'); }
				return strtoupper($p_value);
				break;

			case 'date':
//				if (!is_array($p_value)) { mc_Debug::error($p_value, 'parameter is not an array!'); }
				return $p_value;
				break;

			case 'array':
				if (!is_array($p_value)) { mc_Debug::error($p_value, 'parameter is not an array!'); }
				return $p_value;
				break;

			case 'bool':
			case 'boolean':
				return !empty($p_value);

			default :
				die(mc_Debug::error($p_type, 'variable type unknown'));
		}

		return null;
	} // end method cast



	/**
	 *
	 */
	public static function fetchMulti($p_arr_var, $p_arr_return)
	{
		foreach ($p_arr_var as $p_var)
		{
			$p_arr_return[$p_var] = mc_Request::fetch(
				$p_var[0],
				$p_var[1],
				(isset($p_var[2]) ? $p_var[2] : null),
				(isset($p_var[3]) ? $p_var[3] : null),
				(isset($p_var[4]) ? $p_var[4] : null)
			);
		}
		return true;
	} // end method fetchMulti



} // end class mc_Request

