<?php
/*
* $McLicense$
*
* $Id$
*
*/

/*
session_set_save_handler('session_handler_open', 'session_handler_close',
                         'session_handler_fetch', 'session_handler_store',
                         'session_handler_delete', 'session_handler_gc');
*/

// levels are derived from admin states: 1=room, 2=platform, 4=system
define('SU_FLAG_ROOM',     1);
define('SU_FLAG_PLATFORM', 2);
define('SU_FLAG_SYSTEM',   4);
define('SU_LEVEL_NONE',     0);
define('SU_LEVEL_ROOM',     1); // SU_FLAG_ROOM
define('SU_LEVEL_PLATFORM', 3); // SU_FLAG_ROOM + SU_FLAG_PLATFORM
define('SU_LEVEL_SYSTEM',   7); // SU_FLAG_ROOM + SU_FLAG_PLATFORM + SU_FLAG_SYSTEM



/**
 * handles the current metacoon session
 *
 * @author steffen gross / matthias hupfer
 * @version 1.0
 * @package core
 * @subpackage classes.new
 */
final class mc_Session
{
	/**
	 * name of session
	 */
	protected $name;

	/**
	 * id of session
	 */
	protected $id;

	/**
	 *
	 */
	private function __construct($p_name = null, $p_id = null)
	{
		if (empty($p_name))
		{
			$p_name = session_name();
		}

		if (empty($p_name))
		{
			throw new Exception('session name is empty');
		}

		if (empty($p_id))
		{
			$p_id = session_id();
		}

		if (empty($p_id))
		{
			throw new Exception('session id is empty');
		}

		$this->name = $p_name;
		$this->id   = $p_id;

		$this->updateTsLog();
	}



	/**
	 * getName
	 */
	public function getName()
	{
		return $this->name;
	}



	/**
	 * getId
	 */
	public function getId()
	{
		return $this->id;
	}



	/**
	 * checks if session user is known
	 */
	private function updateTsLog()
	{
		return true;
	}



	/**
	 *
	 */
	final public function destroy()
	{
		$_SESSION = array();
		session_destroy();
		return true;
	}



	/**
	 *
	 */
	final public function close()
	{
		session_write_close();
	}



	/**
	 *
	 */
	final public function clear()
	{
		$_SESSION = array();
	}



	/**
	 *
	 */
	public function regenerateId()
	{
		session_regenerate_id(false); // generate new (second) id for current session data
		$this->id = session_id();    // fetch new id / overwrite old (first) id

		$_SESSION = array();   // delete all data that have been copied from old id
		session_write_close(); // close all sessions

		return $this->id;
	}



	/**
	 *
	 */
	public function setTemp($p_name, $p_value)
	{
		$_SESSION['TEMP'][$p_name] = $p_value;
	}

	/**
	 *
	 */
	public function getTemp($p_name)
	{
		if (isset($_SESSION['TEMP'][$p_name]))
		{
			return $_SESSION['TEMP'][$p_name];
		}

		die(mc_Debug::error($p_name, 'unknown temporary parameter'));
//		die('unknown temporary parameter "'.$p_name.'" '.mc_Debug::showBacktrace(true));
	}



	/**
	 * environment requests
	 */
	public function getAppId()
	{
		if (empty($_SESSION['APP_IDENT']) )
		{
			return $GLOBALS['APP_IDENT'];
		}

		return $_SESSION['APP_IDENT'];
	}

	/**
	 * @DEPRECATED
	 */
	public function getApp()
	{
		return self::getAppId();
	}

	public function getVersion()        { return $_SESSION['MC_VERSION']; }
	public function getAuthType()       { return $_SESSION['USER_AUTH']['AUTH_TYPE']; }
	public function hasAuthType($t)     { return (($_SESSION['USER_AUTH']['AUTH_TYPE'] & $t) ? true : false); }
	public function getBlockedCols()    { return $_SESSION['USER_AUTH']['BLOCKEDCOLS']; }
//	public function getCOMPLEMENTFORM()    { return $_SESSION['USER_AUTH']['COMPLEMENTFORM']; } // ???
	public function getCaller()         { return $_SESSION['CALLED_BY']; }
	public function getCourseInlist()   { return $_SESSION['COURSE_INLIST']; }
	public function getCourseRAInlist() { return $_SESSION['COURSE_INLIST_C_ADMIN']; }
	public function getMaxUploadSize()  { return $_SESSION['MAX_MB_UPLOAD_SIZE']; }
	public function getRestriction()    { return (($_SESSION['RESTRICT'] == 'Y') ? mc_Session::getRoomId() : null); } // deprecated
	public function getUrlRestriction() { return (($_SESSION['RESTRICT'] == 'Y') ? 'R=Y' : 'R=N'); } // deprecated
	public function isRestricted()      { return (($_SESSION['RESTRICT'] == 'Y') ? true : false); } // deprecated
/*
	_public function isRestricted($p_view_name)
	{
		$_SESSION['VIEW'][mc_Session::getRoomId()][$p_view_name]['lastcheck'] = time();
		if (mc_Session::isStandalone()) { return true; }
		if (empty($_SESSION['VIEW'][mc_Session::getRoomId()][$p_view_name]['RESTRICT'])) { return true; }
		return (($_SESSION['VIEW'][mc_Session::getRoomId()][$p_view_name]['RESTRICT'] == 'Y') ? true : false);
	}
*/



	/**
	 * lounge requests
	 */
	public function getLounge()       { return self::getLoungeId(); } // deprecated
	public function getLoungeId()     { return (isset($_SESSION['LOUNGE_ID']) ? $_SESSION['LOUNGE_ID'] : 0); }
	public function getLoungeInlist() { return (empty($_SESSION['LOUNGE_INLIST']) ? '-1' : $_SESSION['LOUNGE_INLIST']); }
	public function getStructOfLounge($p_lounge_id) { return (empty($_SESSION['LOUNGE_MAP'][intval($p_lounge_id)]) ? false : $_SESSION['LOUNGE_MAP'][intval($p_lounge_id)]); }



	/**
	 * room requests
	 */
	public function getRoom()  { return self::getRoomId(); } // deprecated
	public function getRoomId()  { return (isset($_SESSION['COURSE_ID']) ? $_SESSION['COURSE_ID'] : mc_Session::getLoungeId()); }
	public function getTheme()
	{
		if (empty($_SESSION['COURSE_CONF']['COURSE_THEME']))
		{
			return 'default_theme';
		}
		return $_SESSION['COURSE_CONF']['COURSE_THEME'];
	}
	public function getRoomNode()  { return $_SESSION['VT_ROOM_NODE_ID']; }
	public function getRightType() { return $_SESSION['COURSE_CONF']['COURSE_RIGHT_TYPE']; }
	public function getConf($p_colname)
	{
		if (!isset($_SESSION['COURSE_CONF'][$p_colname]))
		{
			return(mc_Debug::error('unknown room configuration index "'.$p_colname.'"'));
		}
		return $_SESSION['COURSE_CONF'][$p_colname];
	}// end method getConf
	public function getRoomExtensions()       { return (empty($_SESSION['COURSE_CONF']['COURSE_EXTENSIONS']) ? array() : $_SESSION['COURSE_CONF']['COURSE_EXTENSIONS']); }
	public function getRoomExtensionNames()   { return (empty($_SESSION['COURSE_CONF']['COURSE_EXTENSIONS']) ? array() : array_keys($_SESSION['COURSE_CONF']['COURSE_EXTENSIONS'])); }
	public function getRoomExtensionIdList()  { return (empty($_SESSION['COURSE_CONF']['COURSE_EXTENSIONS_ID_LIST']) ? '' : $_SESSION['COURSE_CONF']['COURSE_EXTENSIONS_ID_LIST']); }
	public function getRoomExtension($p_name) { return (empty($_SESSION['COURSE_CONF']['COURSE_EXTENSIONS'][$p_name]) ? false : $_SESSION['COURSE_CONF']['COURSE_EXTENSIONS'][$p_name]); }
	public function hasRoomExtension($p_name) { return (empty($_SESSION['COURSE_CONF']['COURSE_EXTENSIONS'][$p_name]) ? false : true); }
//public function isStandalone() { return (empty($_SESSION['COURSE_CONF']['COURSE_STANDALONE']) ? false : true); }
	public function isStandalone()
	{
		if (mc_Session::isLounge() && mc_Session::getRoomId() > 0)
		{
			// in this case (due to iDA-mod) standalone is 0, NOT standalone is -1 !!!
			return (empty($_SESSION['COURSE_CONF']['COURSE_STANDALONE']) ? true : false);
		}
		return (empty($_SESSION['COURSE_CONF']['COURSE_STANDALONE']) ? false : true);
	}
	public function isVirtual()    { return (empty($_SESSION['IS_VIRTUAL']) ? false : true); }
	public function isLounge($p_room_id = null)
	{
		if ($p_room_id === null)
		{
			$p_room_id = mc_Session::getRoomId();
		}
		return ((mc_Session::getLoungeId() == $p_room_id) ? true : false);
	}
	public function isRoomModel($p_model)     { return ($_SESSION['COURSE_CONF']['ROOM_MODEL_NAME'] == $p_model) ? true : false; }
	public function isMemberOfRoom($p_room_id)
	{
		if ( !is_numeric($p_room_id) )
		{
			return false;
		}
		return ((array_search(intval($p_room_id), explode(',', $_SESSION['COURSE_INLIST'])) === false) ? false : true);
	}



	/**
	 * group requests
	 */
	public function getRoomGroup()        { return $_SESSION['COURSE_GROUP']; }
	public function getPrivateGroup()     { return (empty($_SESSION['USER_PROFILE']['GROUP']) ? 0 : $_SESSION['USER_PROFILE']['GROUP']); }
	public function getAllUserGroupIds()
	{
		if (mc_Session::isNotAGroupMember())
		{
			return array(-1);
		}
		return $_SESSION['group']['all_list'];
	}
	public function getRoomUserGroups()
	{
		if (empty($_SESSION['GROUP_INLIST_COURSE']))
		{
			return array();
		}
		return $_SESSION['GROUP_INLIST_COURSE'];
	}
	public function getRoomUserGroupIds() { return array_keys(mc_Session::getRoomUserGroups()); }
	public function getRoomGroups()       { return $_SESSION['group']['room_list']; }
	public function getRoomGroupIds()     { return array_keys($_SESSION['group']['room_list']); }
	public function getRoomGroupCap($gid)
	{
		$gid = intval($gid);
		if (empty($_SESSION['group']['room_list'][$gid]))
		{
			return '';
		}
		return $_SESSION['group']['room_list'][$gid]['WGROUP_CAP'];
	}
	public function isMemberOfGroup($gid) { return in_array(intval($gid), $_SESSION['group']['all_list']); }
	public function isNotAGroupMember()
	{
		if (empty($_SESSION['group']))
		{
			return true;
		}

		if (empty($_SESSION['group']['all_list']))
		{
			return true;
		}

		return false;
	}



	/**
	 * user requests
	 */
	public function getUser()           { return mc_Session::getUserId(); } // deprecated; for BC only
	public function getUserId()         { return (empty($_SESSION['USER_IDENT'])   ? -1 : $_SESSION['USER_IDENT']); }
	public function getUserName()       { return (empty($_SESSION['USER_NAME'])    ? 'anonymous' : $_SESSION['USER_NAME']); }
	public function getSelname()        { return (empty($_SESSION['USER_SELNAME']) ? 'anonymous' : $_SESSION['USER_SELNAME']); }
	public function getEmail()          { return $_SESSION['USER_EMAIL']; }
	public function getChatname()       { return $_SESSION['CHAT']['NICKNAME']; }
//public function getAccountId()      { return (empty($_SESSION['USER_PROFILE']['USER_ID'])      ? -1 : $_SESSION['USER_PROFILE']['USER_ID']); }
	public function getAddressId()      { return (empty($_SESSION['USER_PROFILE']['SIMPLADR_ID'])  ? -1 : $_SESSION['USER_PROFILE']['SIMPLADR_ID']); }
	public function getLang()           { return (empty($_SESSION['USER_PROFILE']['LANGUAGE_ISO']) ? $GLOBALS['l_strLang'] : $_SESSION['USER_PROFILE']['LANGUAGE_ISO']); }
	public function getLangId()         { return (empty($_SESSION['USER_PROFILE']['LANGUAGE'])     ? $GLOBALS['l_intLang'] : $_SESSION['USER_PROFILE']['LANGUAGE']); }
	public function getTimezone()       { return (empty($_SESSION['USER_PROFILE']['TIMEZONE']) ? null : $_SESSION['USER_PROFILE']['TIMEZONE']); }
	public function getUserTheme()
	{
		if (empty($_SESSION['USER_PROFILE']['THEME']))
		{
			return 'default_theme';
		}
		return $_SESSION['USER_PROFILE']['THEME'];
	}
	public function getStyle()          { return $_SESSION['USER_PROFILE']['STYLE']; }
	public function getHomeNode()       { return $_SESSION['USER_PROFILE']['HOMENODE']; }
	public function getOpenFMNodes()    { return $_SESSION['USER_PROFILE']['OPENNODES']; }
	public function getOpenCBNodes()    { return $_SESSION['USER_PROFILE']['COPENNODES']; }

	public function isUser($p_user)
	{
		if (is_numeric($p_user))
		{
			return ((intval($p_user) == mc_Session::getUserId()) ? true : false);
		}

		return ((trim($p_user) == mc_Session::getUserName()) ? true : false);
	}






	/**
	 * role setters
	 */
	public function setSuLevel($level)
	{
		switch ($level)
		{
			case SU_LEVEL_SYSTEM :
				$_SESSION['IS_ADMIN'] = SU_LEVEL_SYSTEM;
				break;

			case SU_LEVEL_PLATFORM :
				$_SESSION['IS_ADMIN'] = SU_LEVEL_PLATFORM;
				break;

			case SU_LEVEL_ROOM :
				$_SESSION['IS_ADMIN'] = SU_LEVEL_ROOM;
				break;

			case SU_LEVEL_NONE :
				$_SESSION['IS_ADMIN'] = SU_LEVEL_NONE;
				break;

			default :
				throw new Exception("unknown SU level : '{$level}'");
		}
	}

	public function enableSuIgnore()
	{
		$_SESSION['SU_IGNORE_LEVEL'] = true;
	}

	public function disableSuIgnore()
	{
		$_SESSION['SU_IGNORE_LEVEL'] = false;
	}

	public function isSuIgnored()
	{
		if ( empty($_SESSION['SU_IGNORE_LEVEL']) )
		{
			return false;
		}

		return true;
	}



	/**
	 * role requests
	 */
	public function getSuLevel()
	{
		if ( empty($_SESSION['IS_ADMIN']) )
		{
			return 0;
		}

		return $_SESSION['IS_ADMIN'];
	}

	public function isAdmin($f = 255)
	{
		// check for superuser switch
		if ( mc_Session::isSuIgnored() )
		{
			return false;
		}

		$currentSuLevel = mc_Session::getSuLevel();

		if (intval($f) & $currentSuLevel)
		{
			return true;
		}

		return false;
	}

	public function isMasterOf($userSuLevel)
	{
		// superuser level uses the binary values
		// 4 = system   admin
		// 2 = platform admin
		// 1 = room     admin of current room

		// fetch superuser level of current user
		$currentSuLevel = mc_Session::getSuLevel();

		// strip current users level from other users level to get his EXTRA level
		$userExtraSuLevel = $userSuLevel & ~$currentSuLevel;

		// is current users full level not smaller than other users EXTRA level ?
		if ($currentSuLevel >= $userExtraSuLevel)
		{
			// current user is allowed to master other user
			return false;
		}

		// current user has NO permission to master other user
		return false;
	}

	public function isRoomAdmin($roomId = null)
	{
		if ($roomId === null)
		{
			$roomId = mc_Session::getRoomId();
		}

		return in_array(intval($roomId), $_SESSION['user']['roles']['KURSADMIN']);
	}

	public function isTutor()
	{
		$roomId = mc_Session::getLoungeId();
		return in_array($roomId, $_SESSION['user']['roles']['LEHRENDE']);
	}

	public function isRoomTutor($roomId = null)
	{
		if ($roomId === null) {
			$roomId = mc_Session::getRoomId();
		}
		else {
			$roomId = intval($roomId);
		}

		return in_array($roomId, $_SESSION['user']['roles']['LEHRENDE']);
	}

	public function isSuperUser($roomId = null)
	{
		// check for superuser switch
		if ( mc_Session::isSuIgnored() )
		{
			return false;
		}

		// check for superuser state
		if ( ! mc_Session::isAdmin() )
		{
			return false;
		}

		if ( mc_Session::isAdmin(SU_FLAG_PLATFORM | SU_FLAG_SYSTEM) )
		{
			// global superuser (usually system and platform administrators)
			return true;
		}

		if ($roomId === null)
		{
			return false;
		}

		if ( mc_Session::isRoomAdmin($roomId) )
		{
			// room admin (local superuser)
			return true;
		}

		return false;
	}



	/**
	 * element requests
	 */
	public function getElementIdent($p_name)
	{
		if (empty($_SESSION['elements']['map'][$p_name]))
		{
			mc_Debug::error($p_name, 'element name is unknown');
			return -1;
		}
		return $_SESSION['elements']['map'][$p_name];
	}
	public function getElementTypes()
	{
		return (array_keys($_SESSION['elements']['type']));
	}
	public function getElementGroupPermissions($p_element_ident = null)
	{
		if (empty($p_element_ident))
		{
			return $_SESSION['group']['permissions'];
		}

		if (empty($_SESSION['group']['permissions'][$p_element_ident]))
		{
			return array();
		}

		return $_SESSION['group']['permissions'][$p_element_ident];
	}



	/**
	 * file manager requests
	 */
	public function getHtmlCourseNode() { return $_SESSION['VT_HTML_COURSE_NODE_ID']; }
	public function getVTRootNodes()    { return $_SESSION['VT_ROOT_NODES']; }



	/**
	 *
	 */
	public function addSid($p_link)
	{
		$l_info = parse_url($p_link);
		if (empty($l_info['query']))
		{
			return $p_link.'?'.SID;
		}

		if (strpos(SID, $l_info['query']) === false)
		{
			return $p_link.'&'.SID;
		}

		return $p_link;
	}



	/**
	 *
	 */
	final public function getFormSid()
	{
		if (ini_get('session.use_trans_sid') == 0)
		{
			return ('<input type="hidden" name="'.session_name().'" value="'.session_id().'">');
		}
		return '';
	}



	/**
	 *
	 */
	public function start($p_name = null, $p_user_id = null)
	{
		global $U;

		@ini_set('default_charset', MC_CHAR_SET);
//		@ini_set('session.gc_maxlifetime', $U->Config->get('SESSION_MAX_LIFETIME'));

		if (empty($p_name) )
		{
			$p_name = MCP_SESSION_NAME;
		}

		session_name($p_name);

		$l_sid = $U->Request->fetch($p_name, 'string', null, MC_REGEXP_SESSION, MC_REQ_SRC_FORM);
		if (empty($l_sid) )
		{
			//die("Session id is empty! You need an internal administrator session to execute updates. Sorry.");
		} else {
		    session_id($l_sid);
		}

		// session id is available, start given session
				session_start();


		$Session = new mc_Session($p_name, $l_sid);

		return $Session;
	}



	/**
	 * checks if a user is not logged in
	 */
	public function isAnonymous()
	{
		if (mc_Session::getUserId() == -1)
		{
			return true;
		}

		return false;
	}



	/**
	 * checks if session user is known
	 */
	public function hasUser()
	{
		if (mc_Session::getUserId() < 1)
		{
			return false;
		}

		return true;
	}


	// disabled functions

	public function logout($p_sid = null)
	{
		die(__CLASS__.'::'.__METHOD__.' is not available in updater');
	}

	public function setRestriction($p_request_value)
	{
		die(__CLASS__.'::'.__METHOD__.' is not available in updater');
	}

	public function create($p_name, $p_user_id = null)
	{
		die(__CLASS__.'::'.__METHOD__.' is not available in updater');
	}

	public function initAnonymousSession()
	{
		die(__CLASS__.'::'.__METHOD__.' is not available in updater');
	}

	public function initUserSession($p_user_id)
	{
		die(__CLASS__.'::'.__METHOD__.' is not available in updater');
	}

	public function getExtLang()
	{
		die(__CLASS__.'::'.__METHOD__.' is not available in updater');
	}

	public function getExtLangId()
	{
		die(__CLASS__.'::'.__METHOD__.' is not available in updater');
	}

	public function getDefaultLang()
	{
		die(__CLASS__.'::'.__METHOD__.' is not available in updater');
	}


}

