<?php
/*
* $McLicense$
*
* $Id$
*
*/


define('mc_sysmsg_failed',    'failed');
define('mc_sysmsg_canceled',  'canceled');
define('mc_sysmsg_skipped',   '<b style="color:olive;">skipped</b>');
define('mc_sysmsg_succeeded', 'succeeded');
define('mc_sysmsg_done',      'done');



/**
 * SysMsg
 *
 * @author [Autor]
 * @version 1.0
 * @package core
 * @subpackage classes.new
 */
class SysMsg
{
	/**
	 * displays normal messages
	 *
	 * formatting html tags are allowed.
	 *
	 * @param	string	$msg	message to show
	 * @param	string	$send_header	optional: send page header before displaying the message
	 */
	public static function showMsg($msg, $sendHeader = false)
	{
		return SysMsg::showMessage($msg, $sendHeader, true, 'msg');
	}



	/**
	 * displays informations on success
	 *
	 * formatting html tags are allowed.
	 *
	 * @param	string	$msg	information to show
	 * @param	string	$send_header	optional: send page header before displaying the message
	 */
	public static function showInfo($msg, $sendHeader = false, $showSmall = true)
	{
		return SysMsg::showMessage($msg, $sendHeader, $showSmall, 'info');
	}



	/**
	 * displays warnings
	 *
	 * formatting html tags are allowed.
	 *
	 * @param	string	$msg	warning to show
	 * @param	string	$send_header	optional: send page header before displaying the message
	 */
	public static function showWarning($msg, $sendHeader = false, $showSmall = true)
	{
		return SysMsg::showMessage($msg, $sendHeader, $showSmall, 'warning');
	}



	/**
	 * displays error
	 *
	 * formatting html tags are allowed.
	 *
	 * @param	string	$msg	error to show
	 * @param	string	$send_header	optional: send page header before displaying the message
	 */
	public static function showError($msg, $sendHeader = false, $showSmall = true)
	{
		return SysMsg::showMessage($msg, $sendHeader, $showSmall, 'error');
	}



	/**
	 * displays notes (mostly for development)
	 *
	 * @param	string	$msg	note to show
	 * @param	string	$send_header	optional: send page header before displaying the message
	 */
	public static function showNote($msg, $sendHeader = false, $showSmall = true)
	{
		return SysMsg::showMessage($msg, $sendHeader, $showSmall, 'note');
	}



	/**
	 * displays tasks (mostly for development)
	 *
	 * @param	string	$msg	task to show
	 * @param	string	$send_header	optional: send page header before displaying the message
	 */
	public static function showTask($msg, $sendHeader = false, $showSmall = true)
	{
		return SysMsg::showMessage($msg, $sendHeader, $showSmall, 'task');
	}



	/**
	 * displays actions (much like showMsg)
	 *
	 * @param	string	$msg	action to show
	 * @param	string	$send_header	optional: send page header before displaying the message
	 */
	public static function showAction($msg, $sendHeader = false, $showSmall = true)
	{
		return SysMsg::showMessage($msg, $sendHeader, $showSmall, 'action');
	}



	/**
	 * displays collisions of any kind
	 *
	 * @param	string	$msg	hit to show
	 * @param	string	$send_header	optional: send page header before displaying the message
	 */
	public static function showHit($msg, $sendHeader = false, $showSmall = true)
	{
		return SysMsg::showHit($msg, $sendHeader, $showSmall, 'hit');
	}



	/**
	 * displays notes (mostly for development)
	 *
	 * @param	string	$msg	devnote to show
	 * @param	string	$writeLogOnly	optional: if true, message will still be logged but will not be displayed
	 */
	public static function showDevNote($msg, $writeLogOnly = false)
	{
		// deprecated feature
	}




	/**
	 * displays all the messages depending on their parameters
	 *
	 * @param	string	$msg	benutzerdefinierte Warnung
	 */
	protected static function showMessage($msg, $sendHeader, $showSmall, $msgType)
	{
		$iconFile = null;
		$class    = '';
		$style    = '';
		$smallClass = '';

		switch ($msgType)
		{
			case 'info' :
			case 'warning' :
			case 'error' :
			case 'task' :
				$iconFile = 'i_msg_'.$msgType.'.gif';
				$class    = ' user_'.$msgType;
				$smallClass = ' small_'.$msgType;
				break;

			case 'note' :
				$style = 'style="color:gray;"';
				$msg = 'Note : '.$msg;
				break;

			case 'action' :
				$class = ' action';
				break;

			case 'hit' :
				$class = ' hit';
				break;

			case 'devnote' :
				// deprecated feature
				//break;

			case 'msg' :
			default :
				break;
		} // end switch

		if (empty($icon_url) == false && defined("MC_IMG_URI"))
		{
			$url = MC_IMG_URI . 'msg/' . $iconFile;
			$style = 'style="background-image : url('.$url.'); background-repeat: no-repeat; padding-left: 30px;"';
		}

		if ( ! empty($showSmall) ) {
			$smallClass = '';
		}

		$content = '<div class="user_message'.$class.$smallClass.'" '.$style.'>'.SysMsg::stripHTML($msg).'</div>'."\n";

		return SysMsg::display($content, $sendHeader);
	}



	/**
	 * displays any type of message w/o header
	 *
	 * @param	string	$msg	message to show
	 * @param	string	$sendHeader	show header or not
	 */
	protected static function display($msg, $sendHeader = false)
	{
		if ($sendHeader)
		{
			$msg = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN">
<HTML>
<HEAD>
<TITLE></TITLE>
<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset='.MC_CHAR_SET.'">
<link href="'.MC_ROOT_URI.'design/m2xcross/style/intern_style.css" rel="stylesheet" type="text/css">
</head>
<BODY>
'.$msg.'
</BODY>
</HTML>';
		}

		$l_replace = array(
			'{FAIL}'   => mc_sysmsg_failed,
			'{FAILED}' => mc_sysmsg_failed,

			'{CANCEL}'   => mc_sysmsg_canceled,
			'{CANCELED}' => mc_sysmsg_canceled,

			'{SKIP}'    => mc_sysmsg_skipped,
			'{SKIPPED}' => mc_sysmsg_skipped,

			'{SUCCEED}'   => mc_sysmsg_succeeded,
			'{SUCCEEDED}' => mc_sysmsg_succeeded,

			'{DONE}' => mc_sysmsg_done,
		);

		echo str_replace(array_keys($l_replace), $l_replace, $msg);

		return;
	}



	/**
	 * strips unwanted tags from the messages
	 *
	 * @param string	$string string
	 */
	protected static function stripHTML($string)
	{
		if ( is_string($string) ) {
			$pattern = "<hr><b><i><br><p><pre><nobr><h4><h3>";
			return strip_tags($string, $pattern);
		}

		return $string;
	}


}

