<?php
/*
* $McLicense$
*
* $Id$
*
*/


require(MCP_LIB_PATH.'Config.class.php');

class UConf
	extends mc_Config
{
	// names of sections of file conf.ini
	private $section_main  = 'UPDATE';
	private $section_dbsrv = 'MYSQL';
	private $section_php   = 'PHP';

	// names of sections of description files
	private $usection_desc    = 'DESCRIPTION';
	private $usection_require = 'REQUIREMENT';
	private $usection_details = 'DETAILS';

	private $local_php_major_version = 0;

	private $current_dirname  = null;
	private $current_version  = null;
	private $current_revision = null;
	private $current_build    = null;

	private $conf = false;
	private $fc_log_name = 'overwritten_custom_files.html';

	private $actions;
	private $script_list;
	private $deleted_files;
	private $moved_files;

	private $__UPDATES = array();
	private $__MODS    = array();

	function __construct($DB)
	{
		parent::__construct($DB);

		$this->local_php_major_version = intval(reset(explode('.', PHP_VERSION) ) );

		if ( ! file_exists(MCP_PATH."conf.ini") )
		{
			die('missing file "'.MCP_PATH.'conf.ini" (in '.__FILE__.':'.__LINE__.')');
		}

		$this->conf = parse_ini_file(MCP_PATH."conf.ini", true);
		$this->initOptions();

		$this->table_list    = array(); // for backups
		$this->script_list   = array();
		$this->deleted_files = array();
		$this->moved_files   = array();

		if ( ! file_exists(MCP_PATH."register.ini") )
		{
			die('file register.ini not found');
		}

		$this->actions = false;

		$register_tmp = parse_ini_file(MCP_PATH."register.ini");

		if (empty($register_tmp['updates']) ) {
			$arrTmpRegisteredUpdates = array();
		}
		else {
			$arrTmpRegisteredUpdates = explode(' ', $register_tmp['updates']);
		}

		if (empty($register_tmp['mods']) ) {
			$arrTmpRegisteredMods = array();
		}
		else {
			$arrTmpRegisteredMods = explode(' ', $register_tmp['mods']);
		}

		unset($register_tmp);

		// read configuration of registered updates
		$arrTmpUpdates = array();

		foreach($arrTmpRegisteredUpdates as $updateDirname)
		{
			if (empty($updateDirname) )
			{
				continue;				
			}
			
			$updatePath = MCP_PATH . $updateDirname . DIRECTORY_SEPARATOR;
			$descFile = $updatePath . $this->getDescFile();

			if ( ! file_exists($descFile) )
			{
				SysMsg::showError("description file missed for registered mod '{$updateDirname}'", true);
				exit(0);
			}

			$arrIni = parse_ini_file($descFile, true);

			if ( ! array_key_exists($this->usection_desc, $arrIni) )
			{
				throw new Exception("missing section [{$this->usection_desc}] in file '{$descFile}'");				
			}
			// define description shortcut
			$description = &$arrIni[$this->usection_desc];

			if ( ! array_key_exists($this->usection_require, $arrIni) )
			{
				throw new Exception("missing section [{$this->usection_require}] in file '{$descFile}'");				
			}
			// define requirements shortcut
			$requirements = &$arrIni[$this->usection_require];

			if ( empty($requirements['php_major_version']) ) {
				$req_php_major_version = 5; // php 5
			}
			else {
				$req_php_major_version = intval($requirements['php_major_version']);
			}

			if ($req_php_major_version != $this->local_php_major_version)
			{
				// revision update does not match local major version
				SysMsg::showError("update {$updateDirname} skipped!<br>"
					."required php major version ({$req_php_major_version}) does NOT match the"
					."installed php major version ({$this->local_php_major_version})"
					, true
				);
				exit(0);
			}

			if ( empty($requirements['environment_version']) ) {
				$requiredEnvironmentVersion = 1;
			}
			else {
				$requiredEnvironmentVersion = intval($requirements['environment_version']);
			}

			if (MCP_ENVIRONMENT_VERSION != $requiredEnvironmentVersion)
			{
				// skip updates with wrong environment requirements
				continue;
			}

			$requiredVersion  = trim($requirements['version']);
			$requiredRevision = intval($requirements['revision']);
			$requiredBuild    = intval($requirements['build']);
			$requiredName     = "{$requiredVersion}.R{$requiredRevision}.B{$requiredBuild}";

			$destinedVersion  = trim($description['version']);
			$destinedRevision = intval($description['revision']);
			$destinedBuild    = intval($description['build']);
			$destinedName     = "{$destinedVersion}.R{$destinedRevision}.B{$destinedBuild}";

			if (empty($description['bundle']) ) {
				$bundle = "2.0";
			}
			else {
				$bundle = $description['bundle'];
			}

			// fetch details of destined version
			$arrIni['dirname']  = $updateDirname;
			$arrIni['path']     = $updatePath;
			$arrIni['version']  = $destinedVersion;
			$arrIni['revision'] = $destinedRevision;
			$arrIni['build']    = $destinedBuild;
			$arrIni['bundle']   = $bundle;
			$arrIni['requiredname']   = $requiredName;
			$arrIni['destinedname']   = $destinedName;
			$arrIni['environment_version'] = $requiredEnvironmentVersion;
			$arrIni['infosource_url'] = $this->getInfoSource() . implode('/', array(
				  urlencode($description['type'])
				, urlencode($destinedVersion)
				, $destinedRevision
				, $destinedBuild
			));

			$index = 0;
			foreach($arrTmpUpdates as $listedUpdate)
			{
				$listedName = $listedUpdate['requiredname'];
				if ( version_compare($requiredName, $listedName, "<") )
				{
					break;
				}

				$index++;
			}

			array_splice($arrTmpUpdates, $index, 0, array($arrIni));
		}

		foreach($arrTmpUpdates as $data)
		{
			$updateDirname = $data['dirname'];

			$this->__UPDATES[$updateDirname] = $data;

			$actionFilePath = $data['path'].$this->getActionFile();

			include_once($actionFilePath);

			$this->script_list[$updateDirname]   = (empty($MCP_ACTIONS['PROCESS'])   ? array() : $MCP_ACTIONS['PROCESS']);
			$this->deleted_files[$updateDirname] = (empty($MCP_ACTIONS['DELETE'])    ? array() : $MCP_ACTIONS['DELETE']);
			$this->moved_files[$updateDirname]   = (empty($MCP_ACTIONS['MOVE'])      ? array() : $MCP_ACTIONS['MOVE']);
			$this->table_list[$updateDirname]    = (empty($MCP_ACTIONS['BAKTABLES']) ? array() : $MCP_ACTIONS['BAKTABLES']);
		}
		unset($arrTmpUpdates, $arrTmpRegisteredUpdates);

		// read gonfiguration of registered mods
		$arrTmpMods = array();
		$modType = 'mod'; // type is always MOD 
		foreach($arrTmpRegisteredMods as $modDirname)
		{
			if (empty($modDirname) )
			{
				continue;				
			}

			$modPath = MCP_PATH . $modDirname . DIRECTORY_SEPARATOR;

			$descFile = $modPath . $this->getDescFile();
			if ( ! file_exists($descFile) )
			{
				SysMsg::showError("description file missed for registered mod '{$modDirname}'", true);
				exit(0);
			}

			$arrIni = parse_ini_file($descFile, true);

			$arrIni[$this->usection_desc]['type']     = $modType;
			$arrIni[$this->usection_require]['check'] = MCP_VRB_RANGE;

			$requiredVersion  =   trim($arrIni[$this->usection_require]['version']);
			$requiredRevision = intval($arrIni[$this->usection_require]['revision']);
			$requiredBuild    = intval($arrIni[$this->usection_require]['build']);
			$requiredName     = "{$requiredVersion}.R{$requiredRevision}.B{$requiredBuild}";

			$arrIni['dirname'] = $modDirname;
			$arrIni['path']    = $modPath;
			$arrIni['requiredname']   = $requiredName;
			$arrIni['infosource_url'] = $this->getInfoSource() . implode('/', array(
				  urlencode($modType)
				, NULL //urlencode($destinedVersion)
				, NULL // $destinedRevision
				, NULL // $destinedBuild
			));
			#$arrIni['version']  = $destinedVersion;
			#$arrIni['revision'] = $destinedRevision;
			#$arrIni['build']    = $destinedBuild;
			#$arrIni['destinedname']   = $destinedName;
			$this->__MODS[$modDirname] = $arrIni;

			$actionFilePath = $modPath.$this->getActionFile();
			include_once($actionFilePath);

			$this->script_list[$modDirname]   = (empty($MCP_ACTIONS['PROCESS'])   ? array() : $MCP_ACTIONS['PROCESS']);
			$this->deleted_files[$modDirname] = (empty($MCP_ACTIONS['DELETE'])    ? array() : $MCP_ACTIONS['DELETE']);
			$this->moved_files[$modDirname]   = (empty($MCP_ACTIONS['MOVE'])      ? array() : $MCP_ACTIONS['MOVE']);
			$this->table_list[$modDirname]    = (empty($MCP_ACTIONS['BAKTABLES']) ? array() : $MCP_ACTIONS['BAKTABLES']);
		}
		unset($arrTmpMods, $arrTmpRegisteredMods);
		
		// constructor end 
	}



	function getDbServerBasedir()
	{
		if (empty($this->conf[$this->section_dbsrv]['mysql.basedir']) )
		{
			return null;
		}
		return trim($this->conf[$this->section_dbsrv]['mysql.basedir']);
	}



	function getDbServerParams()
	{
		if (empty($this->conf[$this->section_dbsrv]['mysql.params']))
		{
			return '';
		}
		return trim($this->conf[$this->section_dbsrv]['mysql.params']);
	}



	function isEnabled($switchName, $categoryName = null)
	{
		if (empty($categoryName)) {
			$categoryName = $this->section_main;
		}

		if (empty($this->conf[$categoryName])) {
			return false;
		}

		if (empty($this->conf[$categoryName][$switchName])) {
			return false;
		}

		return true;
	}



	function getLibsRequired()
	{
		return $this->formattedLibList($this->conf[$this->section_php]['libnames.required']);
	}



	function getLibsOptional()
	{
		return $this->formattedLibList($this->conf[$this->section_php]['libnames.optional']);
	}



	function formattedLibList($list)
	{
		$list = explode(' ', $list);
		$result = array();
		foreach($list as $lib)
		{
			if (empty($lib) == false)
			{
				if (empty($this->conf[$this->section_php]['libdesc.'.$lib]))
				{
					$desc = '';
				}
				else
				{
					$desc = trim($this->conf[$this->section_php]['libdesc.'.$lib]);
				}

				$result[$lib] = $desc;
			}
		}

		return $result;
	}



	function getInfoSource()
	{
		return $this->conf[$this->section_main]['infosource_url'];
	}



	function getDescFile()
	{
		if (empty($this->conf[$this->section_main]['description_filename']) )
		{
			return '_description.ini';
		}

		return $this->conf[$this->section_main]['description_filename'];
	}



	function getActionFile()
	{
		if (empty($this->conf[$this->section_main]['actions_filename']) )
		{
			return '_actions.php';
		}

		return $this->conf[$this->section_main]['actions_filename'];
	}



	function getCheckFile()
	{
		if (empty($this->conf[$this->section_main]['check_filename']) )
		{
			return '_check_app.php';
		}

		return $this->conf[$this->section_main]['check_filename'];
	}



	function getDeletedFiles($updateDirname)
	{
		if (empty($this->deleted_files[$updateDirname]) )
		{
			return array();
		}
		return $this->deleted_files[$updateDirname];
	}



	function getMovedFiles($updateDirname)
	{
		if (empty($this->deleted_files[$updateDirname]) )
		{
			return array();
		}
		return $this->moved_files[$updateDirname];
	}



	function getBakTables($updateDirname)
	{
		return $this->table_list[$updateDirname];
	}



	function getScriptList()
	{
		return $this->script_list;
	}



	function getTimeName()
	{
		return basename(MC_ROOT_PATH) . '_' . date('Y-m-d_H\hi\ms\s', MCP_TIMESTAMP);
	}



	function getBackupDir($separator)
	{
		$dirName = $this->getCurrentDirname();

		$arrBackupDir = array();
		$arrBackupDir[] = MCP_BAK_DIR;
		$arrBackupDir[] = $this->getTimeName();

		if ($dirName) {
			$arrBackupDir[] = $dirName;
		}

		$bakDir = implode($separator, $arrBackupDir) . $separator;

		return $bakDir;
	}



	function getBakPath() {
		$bakPath = MCP_PATH . $this->getBackupDir(DIRECTORY_SEPARATOR);
		return $bakPath;
	}



	function getBakUri()  {
		$bakUri = MCP_URI . $this->getBackupDir('/');
		return $bakUri;
	}



	// @DEPRECATED
	function getBackupPath() { return $this->getBakPath(); }

	// @DEPRECATED
	function getBackupUri()  { return $this->getBakUri(); }



	function getFCLogPath()
	{
		return $this->getBackupPath().$this->fc_log_name;
	}



	function getFCLogUri()
	{
		return $this->getBackupUri().$this->fc_log_name;
	}



	function getLogPath()
	{
		return MCP_PATH.MCP_LOG_DIR.DIRECTORY_SEPARATOR;
	}



	function getLogUri()
	{
		return MCP_URI.MCP_LOG_DIR.'/';
	}


	function getFirstUpdate()
	{
		return (reset($this->__UPDATES));
	}



	function getLastUpdate()
	{
		return (end($this->__UPDATES));
	}



	function getUpdateDirnames()
	{
		return array_keys($this->__UPDATES);
	}

	function setCurrentUpdate($updateDirname)
	{
		$this->current_dirname = $updateDirname;
	}

	function getCurrentDirname()
	{
		return $this->current_dirname;
	}

	function getUpdate($updateDirname = null)
	{
		if (empty($updateDirname) ) {
			$updateDirname = $this->getCurrentDirname();
		}
		return $this->__UPDATES[$updateDirname];
	}

	function getUpdateType($updateDirname = null)
	{
		if (empty($updateDirname) ) {
			$updateDirname = $this->getCurrentDirname();
		}
		return $this->__UPDATES[$updateDirname]['type'];
	}

	function getUpdatePath($updateDirname = null)
	{
		if (empty($updateDirname) ) {
			$updateDirname = $this->getCurrentDirname();
		}

		if (empty($this->__UPDATES[$updateDirname]) ) {
			return dirname(__FILE__) . "/../{$updateDirname}/";			
		}
		
		return $this->__UPDATES[$updateDirname]['path'];
	}

	function getUpdateVersion($updateDirname = null)
	{
		if (empty($updateDirname) ) {
			$updateDirname = $this->getCurrentDirname();
		}
		return $this->__UPDATES[$updateDirname]['version'];
	}

	function getUpdateRevision($updateDirname = null)
	{
		if (empty($updateDirname) ) {
			$updateDirname = $this->getCurrentDirname();
		}
		return $this->__UPDATES[$updateDirname]['revision'];
	}

	function getUpdateBuild($updateDirname = null)
	{
		if (empty($updateDirname) ) {
			$updateDirname = $this->getCurrentDirname();
		}
		return $this->__UPDATES[$updateDirname]['build'];
	}

	function getUpdateBundle($updateDirname)
	{
		if (empty($updateDirname) ) {
			throw new Exception('parameter is empty');
		}
		if (empty($this->__UPDATES[$updateDirname]['bundle']) ) {
			return '2.0';
		}
		return $this->__UPDATES[$updateDirname]['bundle'];
	}

	function getUpdateRequiredName($updateDirname = null)
	{
		if (empty($updateDirname) ) {
			$updateDirname = $this->getCurrentDirname();
		}
		return $this->__UPDATES[$updateDirname]['requiredname'];
	}

	function getUpdateRequiredEnvironmentVersion($updateDirname = null)
	{
		if (empty($updateDirname) ) {
			$updateDirname = $this->getCurrentDirname();
		}
		return $this->__UPDATES[$updateDirname]['environment_version'];
	}

	function getUpdateDestinedName($updateDirname = null)
	{
		if (empty($updateDirname) ) {
			$updateDirname = $this->getCurrentDirname();
		}
		return $this->__UPDATES[$updateDirname]['destinedname'];
	}

	function getUpdateInfoSource($updateDirname = null)
	{
		if (empty($updateDirname) ) {
			$updateDirname = $this->getCurrentDirname();
		}
		return $this->__UPDATES[$updateDirname]['infosource_url'];
	}

	function getUpdateInfoPath($updateDirname = null)
	{
		if (empty($updateDirname) ) {
			$updateDirname = $this->getCurrentDirname();
		}
		$path = MCP_PATH.$updateDirname.DIRECTORY_SEPARATOR.MCP_INFO_SRC_DIR;
		return $path;
	}



	// @DEPRECATED
	function getCurrentVersion()  { return $this->getUpdateVersion(); }
	// @DEPRECATED
	function getCurrentRevision() { return $this->getUpdateRevision(); }
	// @DEPRECATED
	function getCurrentBuild()    { return $this->getUpdateBuild(); }

	function getCurrentDescParam($param)
	{
		$updateDirname = $this->getCurrentDirname();

		if ( isset($this->__UPDATES[$updateDirname][$this->usection_desc][$param]) )
		{
			return $this->__UPDATES[$updateDirname][$this->usection_desc][$param];
		}

		$msg = "param '__UPDATES[{$updateDirname}][{$this->usection_desc}][{$param}]' is not available";
		mc_Debug::error($this->__UPDATES, $msg);
		die($msg);
	}

	function getCurrentDetailsParam($param)
	{
		$updateDirname = $this->getCurrentDirname();

		if ( isset($this->__UPDATES[$updateDirname][$this->usection_details][$param]) )
		{
			return $this->__UPDATES[$updateDirname][$this->usection_details][$param];
		}

		$msg = "param '__UPDATES[{$updateDirname}][{$this->usection_details}][{$param}]' is not available";
		mc_Debug::error($this->__UPDATES, $msg);
		die($msg);
	}



	function getMods()
	{
		return $this->__MODS;
	}



	private function initOptions()
	{
		// need to check for action because checkboxes are empty when not submitted
		if ( empty($_REQUEST['update']) )
		{
			// use preset from conf.ini
			return;
		}

		if ( ! empty($_REQUEST['enable_testmode']) ) {
			$this->conf['UPDATE']['enable_testmode'] = true;
		}
		else {
			$this->conf['UPDATE']['enable_testmode'] = false;
		}

		if ( ! empty($_REQUEST['backup_database']) ) {
			$this->conf['UPDATE']['backup_database'] = true;
		}
		else {
			$this->conf['UPDATE']['backup_database'] = false;
		}

		if ( ! empty($_REQUEST['register_version']) ) {
			$this->conf['UPDATE']['register_version'] = true;
		}
		else {
			$this->conf['UPDATE']['register_version'] = false;
		}
	}



	function getForm()
	{
		if (empty($this->conf['UPDATE']['enable_testmode']) ) {
			$testCheck = '';
		}
		else {
			$testCheck = 'checked="checked"';
		}

		if (empty($this->conf['UPDATE']['backup_database']) ) {
			$bakDbCheck = '';
		}
		else {
			$bakDbCheck = 'checked="checked"';
		}

		if (empty($this->conf['UPDATE']['register_version']) )
		{
			$versionCheck = '';
			$versionValue = 0;
		}
		else
		{
			$versionCheck = 'checked="checked"';
			$versionValue = 1;
		}

		if ($this->isTestEnabled() )
		{
			$html = <<<TESTOPTIONS
<br>
<table border="0">
	<tr>
		<td valign="top"><input name="enable_testmode"  type="checkbox" value="1" id="enable_testmode"  {$testCheck}></td>
		<td><label for="enable_testmode">{label_11}<label></td>
	</tr>
	<tr>
		<td valign="top"><input name="register_version" type="checkbox" value="1" id="register_version" {$versionCheck}></td>
		<td><label for="register_version">log version<label></td>
	</tr>
	<tr>
		<td valign="top"><input name="backup_database"  type="checkbox" value="1" id="backup_database"  {$bakDbCheck}></td>
		<td><label for="backup_database">backup database<label></td>
	</tr>
</table>
<br>
TESTOPTIONS;
		}

		$html = <<<OPTIONS
<br>
<table border="0">
	<tr>
		<td valign="top">
			<input name="register_version" type="hidden"   value="{$versionValue}">
			<input name="backup_database"  type="checkbox" value="1" id="backup_database" {$bakDbCheck}>
		</td>
		<td><label for="backup_database">backup database<label></td>
	</tr>
</table>
<br>
OPTIONS;

		return $html;
	}



}
