<?php
/**
 * @package core
 */
/*
* $McLicense$
*
* $Id: PComponentFunctions.php 969 2012-02-03 18:43:35Z gross $
*
*/


/**
 * @author
 * @package core
 * @subpackage classes.new
 * @version
 */
class PComponentFunctions
{
	/**
	 * array of names of registered functions
	 * (int) flag position => (string) name
	 *
	 * @var array
	 */
	private $arrFunctions = array();

	/**
	 * flipped property $arrFunctions for faster index check
	 *
	 * @var array
	 */
	private $arrFlippedFunctions = array();

	/**
	 * set of new function settings, waiting for commit
	 * array {
	 *   (string) viewName => array {
	 *     (string) action => array {
	 *	     (string) function => state [TRUE|FALSE|NULL]
	 *     }
	 *   }
	 * }
	 *
	 * @var array
	 */
	private $arrNewFunctions = array();



	/**
	 * constructor
	 */
	function __construct()
	{
		$this->registerFunctions();
/*
		$this->setAction($action);
		$this->setDefaultMap($default_map);

		$map = trim($map);
		if ($map === "")
		{
			$map = $this->getDefaultMap();
		}

		$this->parseFunctionsMap($map);
*/
	}



	/**
	 * registerFunctions
	 *
	 * @return bool
	 */
	private function registerFunctions()
	{
		// IMPORTANT NOTE about index and order of functions below:
		// the INDEX means the position in the string (e.g. in VIEW_SHOWL_FUNCTIONS)
		// the line ORDER of entries means their position/order in the function list

		$arrFunctions = array();

		// "show_functions" is the master flag
		// when off NO functions are visible at all
		$arrFunctions[ 0] = 'show_functions';

		$arrFunctions[ 1] = 'insert';
		$arrFunctions[11] = 'Import';
		$arrFunctions[10] = 'insreply';
		$arrFunctions[ 2] = 'update';
		$arrFunctions[ 3] = 'delete';
		$arrFunctions[ 4] = 'upload';
		$arrFunctions[ 5] = 'link';
		$arrFunctions[ 6] = 'mboard';
		$arrFunctions[ 7] = 'poll';
		// only available for manual archive administration
		$arrFunctions[17] = 'archive';
		$arrFunctions[ 8] = 'release';
		// the true name of "download_liste" is "download_list_xls"
		// but atm there are to much layouts using the deprecated name
		$arrFunctions[ 9] = 'download_liste'; // name is deprecated
		$arrFunctions[18] = 'download_single_pdf';
		$arrFunctions[12] = 'custom_1';
		$arrFunctions[13] = 'custom_2';
		$arrFunctions[14] = 'custom_3';
		$arrFunctions[15] = 'custom_4';
		$arrFunctions[16] = 'custom_5';
		$arrFunctions[19] = 'help';

		$this->arrFunctions = $arrFunctions;
		$this->arrFlippedFunctions = array_flip($arrFunctions);

		return TRUE;
	}



	/**
	 * returns an array of all registered functions
	 * (int) position => (string) name
	 *
	 * @return array
	 */
	private function getFunctions()
	{
		return $this->arrFunctions;
	}



	/**
	 * returns if a function is registered or not
	 *
	 * @return bool
	 */
	private function isRegisteredFunction($function)
	{
		if (isset($this->arrFlippedFunctions[$function]) )
		{
			return TRUE;
		}
		else
		{
			return FALSE;
		}
	}



	/**
	 * adds an flag to a function map
	 * added flags needs confirm to get stored in database
	 *
	 * @return bool
	 */
	public function addFlag($viewname, $action, $function, $flag=NULL)
	{
		if (is_array($function) )
		{
			foreach ($function as $func => $flag)
			{
				$this->addFlag($viewname, $action, $func, $flag);
			}
			return TRUE;
		}
		
		
		if ($action != 'list' && $action != 'ins' && $action != 'single' && $action != 'upd')
		{
			SysMsg::showMsg("valid actions: list, ins, single, upd");
			throw new Exception("call of invalid action '{$action}'!");
		}

		if ( ! $this->isRegisteredFunction($function) )
		{
			$arrFunctions = $this->getFunctions();
			$strFunctions = implode("<br>", $arrFunctions);
			SysMsg::showMsg("registered functions are: <br>" . $strFunctions);
			throw new Exception("'{$function}' is not a registered function!");
		}

		if ($flag !== TRUE && $flag !== FALSE && $flag !== NULL) 
		{
			SysMsg::showMsg("valid states are: TRUE, FALSE, NULL");
			throw new Exception("invalid state '{$state}'!");
		}

		if ( ! isset($this->arrNewFunctions[$viewname]) )
		{
			$this->arrNewFunctions[$viewname] = array();
		}

		if ( ! isset($this->arrNewFunctions[$viewname][$action]) )
		{
			$this->arrNewFunctions[$viewname][$action] = array();
		}

		if (isset($this->arrNewFunctions[$viewname][$action][$function]) )
		{
			throw new Exception("function '{$function}' for action '{$action}' of view '{$viewname}' is already in queue!");
		}

		$this->arrNewFunctions[$viewname][$action][$function] = $flag;

		return TRUE;
	} 



	/**
	 * gets column of table view which stores the flags for requested action
	 *
	 * @return string
	 */
	public function getActionColumn($action)
	{
		switch ($action)
		{
			case 'list':
				return 'VIEW_SHOWL_FUNCTIONS';

			case 'ins':
				return 'VIEW_INSERT_FUNCTIONS';

			case 'single':
				return 'VIEW_SHOW_FUNCTIONS';

			case 'upd':
				return 'VIEW_UPDATE_FUNCTIONS';

			default:
				throw new Exception("call of invalid action '{$action}'!");

		}
	}



	/**
	 * stores all added flags in database
	 *
	 * @return bool
	 */
	public function commit()
	{
		global $U;
		
		if (empty($this->arrNewFunctions) )
		{
			SysMsg::showInfo("empty list. there are no items to commit");
			return TRUE;
		}
		
		#$this->arrNewFunctions[$viewname][$action][$function] = $flag;
		foreach ($this->arrNewFunctions as $viewName => $arrActions)
		{
			$query = "SELECT VIEW_SHOWL_FUNCTIONS, VIEW_INSERT_FUNCTIONS, VIEW_SHOW_FUNCTIONS, VIEW_UPDATE_FUNCTIONS"
				." FROM VIEW"
				." WHERE VIEW_NAME = " . $U->DB->quote($viewName, 'TEXT')
			;
			$arrViewFunctions = $U->DB->getRow($query, DBMC_FETCHMODE_ASSOC);
			if (sizeof($arrViewFunctions) == 0)
			{
				throw new Exception("invalid view '{$viewName}'");
			}

			$arrUpdate = array();			
			foreach ($arrActions as $action => $arrNewFlags)
			{
				$actionColumn = $this->getActionColumn($action);
				$oldFunctions = $arrViewFunctions[$actionColumn];
				$newFunctions = $this->buildFunction($oldFunctions, $arrNewFlags);
				$arrUpdate[] = "{$actionColumn} = " . $U->DB->quote($newFunctions, 'TEXT');
			}

			$query = "UPDATE VIEW SET " . implode(", ", $arrUpdate)
				." WHERE VIEW_NAME = " . $U->DB->quote($viewName, 'TEXT')
			;
			$U->DB->query($query);
			SysMsg::showInfo("functions of view '{$viewName}' updated! : " . $query);
		}

		$this->arrNewFunctions = array();
		
		return TRUE;
	}



	/**
	 * builds new functions list from preset and list of flags  
	 *
	 * @return string
	 */
	private function buildFunction($oldFunctions, $arrNewFlags)
	{
		$count = count($this->arrFunctions);
		$wildcard = '?';
		$wildcardPreset = str_pad("", $count, $wildcard);

		$newFunctions = $wildcardPreset;
		foreach ($this->arrFunctions as $index => $function)
		{
			if (array_key_exists($function, $arrNewFlags) )
			{
				$flag = $arrNewFlags[$function];

				if ($flag === TRUE)
				{
					$newFunctions[$index] = '1';
					continue;
				}

				if ($flag === FALSE)
				{
					$newFunctions[$index] = '0';
					continue;
				}

				if ($flag === NULL)
				{
					$newFunctions[$index] = $wildcard;
					continue;
				}

				throw new Exception("invalid flag"); 
			}

			if (isset($oldFunctions[$index]) )
			{
				$newFunctions[$index] = $oldFunctions[$index];
				continue; 
			}		
		}
		
		if ($newFunctions == $wildcardPreset)
		{
			return NULL;			
		}
		
		while (substr($newFunctions, -1) == $wildcard)
		{
			$newFunctions = substr($newFunctions, 0, -1);
		}
		
		return $newFunctions;
	}



}

