<?php
/*
* $McLicense$
*
* $Id: PhpBB2.php 472 2011-03-24 14:24:19Z gross $
*
*/

require_once(MCP_LIB_PATH . "core/PGroup.php");

if (defined("MC_PHPBB2_TABLE_PREFIX") == false)
{
	die("MC_PHPBB2_TABLE_PREFIX not defined. please check conf/modules.conf.php settings");
}

/**
 * PhpBB2
 *
 * -
 *
 * @author [Autor]
 * @version 1.0
 * @package core
 * @subpackage classes.new
 */
class PPhpBB2
	extends PGroup
{
	var $def_group_type;

	var $lang_map;

  /**
  * Initialisieren zugehöriger Membervariablen
  */
	function __construct()
  {
  	parent::__construct();

		$this->def_group_type=2;

		$this->lang_map = array();
		$this->lang_map[1]    = "german_formal";
		$this->lang_map["DE"] = "german_formal";
		$this->lang_map[2]    = "english";
		$this->lang_map["EN"] = "english";
  } // Ende Konstruktor


	public function repair()
	{
		SysMsg::showMsg('clean up and repair accounts and group memberships of bulletin board. this may take a while, hang on...');

		// remove all groups without having valid mc users or mc groups as reference
		$this->removeOrphanedGroups();

		// create missing board accounts
		// each mc account needs a related board account
		// creates group and std-relation for account (no special group memberships!)
		$this->repairAccounts();

		// creates missing groups
		// each mc group of type MC_GROUP_TYPE_VISIBLE needs a related board group
		$this->repairGroups();

		// fixes all relations between board users and board groups
		// according to the relations between mc users and mc groups of type MC_GROUP_TYPE_VISIBLE
		$this->repairGroupMemberships();

		return;
	} // end method repairGroups




	private function removeOrphanedGroups()
	{
		SysMsg::showMsg('running check for orphaned groups');

		// fetch orphaned account groups
   	$sel =" select ".MC_PHPBB2_TABLE_PREFIX."groups.group_id, ".MC_PHPBB2_TABLE_PREFIX."groups.group_mc_user_ident";
  	$sel.=" from ".MC_PHPBB2_TABLE_PREFIX."groups";
  	$sel.=" left join USER";
  	$sel.=" on USER.USER_IDENT = ".MC_PHPBB2_TABLE_PREFIX."groups.group_mc_user_ident";
		$sel.=" where ".MC_PHPBB2_TABLE_PREFIX."groups.group_mc_user_ident	> 0";
		$sel.=" and USER.USER_IDENT is null";
  	$result = $this->DB->query($sel);

		$count = $result->numRows();

		while ($row = $result->fetchRow())
		{
			$bbGroupId = intval($row[0]);
			$bbUserId  = intval($row[1]); // NOTE: PhpBB Account Id == MC Account Id !

			// board account is orphaned (parent mc account does not exist)
			// IMPORTANT NOTE:
			// record in user table remains, otherwise all posts of the user disappear !

			// remove group and all relative entries
	  	$this->DB->query(
				"delete from ".MC_PHPBB2_TABLE_PREFIX."auth_access where group_id = {$bbGroupId}"
			);
	  	$this->DB->query(
				"delete from ".MC_PHPBB2_TABLE_PREFIX."user_group  where group_id = {$bbGroupId}"
			);
	  	$this->DB->query(
				"delete from ".MC_PHPBB2_TABLE_PREFIX."groups      where group_id = {$bbGroupId}"
			);

			// remove all group relations from orphaned board account
	  	$this->DB->query(
				"delete from ".MC_PHPBB2_TABLE_PREFIX."user_group  where user_id  = {$bbUserId}"
			);

		} // end while
		$result->free();

		if ( empty($count) ) {
			SysMsg::showInfo("no orphaned accounts found.");
		}
		else {
			SysMsg::showInfo("groups, relations and access of {$count} orphaned account(s) removed.");
		}

		// fetch orphaned mc groups (NOTE: WGROUP_ID == WGROUP_IDENT)
   	$sel =" select ".MC_PHPBB2_TABLE_PREFIX."groups.group_id";
  	$sel.=" from ".MC_PHPBB2_TABLE_PREFIX."groups";
  	$sel.=" left join WGROUP";
  	$sel.=" on WGROUP.WGROUP_ID = ".MC_PHPBB2_TABLE_PREFIX."groups.group_mc_wgroup_ident";
		$sel.=" where ".MC_PHPBB2_TABLE_PREFIX."groups.group_mc_wgroup_ident	> 0";
		$sel.=" and WGROUP.WGROUP_ID	is null";
  	$result = $this->DB->query($sel);

		$count = $result->numRows();

		while ($row = $result->fetchRow())
		{
			$bbGroupId = intval($row[0]);

			// kill orphaned group and all relative entries
	  	$this->DB->query(
				"delete from ".MC_PHPBB2_TABLE_PREFIX."auth_access where group_id = {$bbGroupId}"
			);
	  	$this->DB->query(
				"delete from ".MC_PHPBB2_TABLE_PREFIX."groups      where group_id = {$bbGroupId}"
			);
	  	$this->DB->query(
				"delete from ".MC_PHPBB2_TABLE_PREFIX."user_group  where group_id = {$bbGroupId}"
			);

		} // end while
		$result->free();

		if ( empty($count) ) {
			SysMsg::showInfo("no orphaned groups found.");
		}
		else {
			SysMsg::showInfo("relations and access of {$count} orphaned groups(s) removed.");
		}

		return true;
	}// Ende function removeOrphanedGroups



	private function repairAccounts()
	{
		SysMsg::showMsg('running check for missing board accounts');

		// fetch and check board user accounts
   	$sel =" select distinct USER.USER_IDENT";
  	$sel.=" from USER";
  	$sel.=" left join ".MC_PHPBB2_TABLE_PREFIX."users";
  	$sel.=" on ".MC_PHPBB2_TABLE_PREFIX."users.user_id = USER.USER_IDENT";
   	$sel.=" where USER.USER_IDENT > 0";
   	$sel.="   and USER.USER_IDENT is not null";
   	$sel.="   and ".MC_PHPBB2_TABLE_PREFIX."users.user_id is null";
  	$result = $this->DB->query($sel);

		$count = 0;
		while ($row = $result->fetchRow())
		{
			$mcUserId = intval($row[0]);
			if ( $this->createAccountFromMcUser($mcUserId) )
			{
				$count++;
			}
		} // end while
		$result->free();

		if (empty($count) ) {
			SysMsg::showInfo('no missing board accounts found.');
		}
		else {
			SysMsg::showInfo($count.' missing account(s) created');
		}

		return true;
	}// Ende function repairAccounts




	private function repairGroups()
	{
		// check private groups (user_id == USER_IDENT == group_mc_user_ident)
   	$sel =" select ".MC_PHPBB2_TABLE_PREFIX."users.user_id";
  	$sel.=" from ".MC_PHPBB2_TABLE_PREFIX."users";
  	$sel.=" left join ".MC_PHPBB2_TABLE_PREFIX."groups";
  	$sel.=" on ".MC_PHPBB2_TABLE_PREFIX."groups.group_mc_user_ident = ".MC_PHPBB2_TABLE_PREFIX."users.user_id";
		$sel.=" where ".MC_PHPBB2_TABLE_PREFIX."groups.group_mc_user_ident	is null";
  	$result = $this->DB->query($sel);

		$count = 0;
		while ($row = $result->fetchRow())
		{
			$bbUserId = intval($row[0]);
			if ( $this->createAccountGroup($bbUserId) )
			{
				$count++;
			}
		} // end while

		if ($count == 0)
		{
			SysMsg::showInfo('no missing private groups found.');
		}
		else
		{
			SysMsg::showInfo($count.' missing private group(s) created');
		}

		// check visible groups
   	$sel =" select WGROUP.WGROUP_ID, WGROUP.WGROUP_CAP, WGROUP.WGROUP_NAME, WGROUP.COURSE_ID";
  	$sel.=" from WGROUP";
  	$sel.=" left join ".MC_PHPBB2_TABLE_PREFIX."groups";
  	$sel.=" on ".MC_PHPBB2_TABLE_PREFIX."groups.group_mc_wgroup_ident = WGROUP.WGROUP_IDENT";
		$sel.=" where WGROUP.WGROUP_IDENT > 0";
		$sel.="   and WGROUP.STATE in ('Y','S')";
		$sel.="   and WGROUP.WGROUP_TYPE in (".MC_GROUP_TYPE_VISIBLE.")";
		$sel.="   and ".MC_PHPBB2_TABLE_PREFIX."groups.group_id	is null";
  	$result = $this->DB->query($sel);

		$lang = $GLOBALS['l_strLang'];

		$count = 0;
		while ($row = $result->fetchRow())
		{
			$mcGroupId = intval($row[0]);
			$wgroupCap = mc_U::parseMultiLang($row[1], $lang);
			$wgroupName = $row[2];
			$roomId = intval($row[3]);

			$bbGroupId = $this->createGroupFromMcGroup($mcGroupId, null, $wgroupCap);
			if ($bbGroupId)
			{
				$count++;
				$arrForumId = $this->getForumsByRoomId($roomId);
				if ( ! empty($forumId) )
				{
					$isMod = ($wgroupName == MC_GROUP_NAME_ROOM_ADMIN) ? TRUE : FALSE;
					$this->addGroupToForum($bbGroupId, $arrForumId, $isMod);
				}
			}
		} // end while

		if ($count == 0)
		{
			SysMsg::showInfo('no missing room groups found.');
		}
		else
		{
			SysMsg::showInfo($count.' missing room group(s) created');
		}

		return true;
	}// Ende function repairGroups



	private function getForumsByRoomId($roomId)
	{
		$select = "select ".MC_PHPBB2_TABLE_PREFIX."forums.forum_id"
			." from"
			."  ".MC_PHPBB2_TABLE_PREFIX."forums"
			.", ".MC_PHPBB2_TABLE_PREFIX."categories"
			." where ".MC_PHPBB2_TABLE_PREFIX."categories.cat_id = {$roomId}"
			."   and ".MC_PHPBB2_TABLE_PREFIX."forums.cat_id = ".MC_PHPBB2_TABLE_PREFIX."categories.cat_id"
		;
		$arrForumId = $this->DB->getCol($select);

		return $arrForumId;
	}// Ende function getForumsByRoomId



	private function repairGroupMemberships()
	{
		$del = "delete from ".MC_PHPBB2_TABLE_PREFIX."user_group
 where ".MC_PHPBB2_TABLE_PREFIX."user_group.group_id = 0
    or ".MC_PHPBB2_TABLE_PREFIX."user_group.user_id = 0";
  	$this->DB->query($del);

		$sel = "select user_id, group_id, count(1) as c
 from mc_phpbb_user_group
 group by group_id, user_id
 having c > 1";
  	$result = $this->DB->query($sel);
		$count = 0;

		while ($row = $result->fetchRow() )
		{
			$bbUserId  = intval($row[0]);
			$bbGroupId = intval($row[1]);
			$duplicates = intval($row[2]);

			// remove all relation entries
			$del = "delete from ".MC_PHPBB2_TABLE_PREFIX."user_group
 where ".MC_PHPBB2_TABLE_PREFIX."user_group.group_id = {$bbGroupId}
   and ".MC_PHPBB2_TABLE_PREFIX."user_group.user_id = {$bbUserId}";
  	$this->DB->query($del);

			// recreate a single relation entry
			if ( $this->addUserToGroup($bbUserId, $bbGroupId, false) ) {
				$count += ($duplicates - 1);
			}
		}

		if ($count == 0)
		{
			SysMsg::showInfo('no duplicated memberships found.');
		}
		else
		{
			SysMsg::showInfo($count.' duplicated membership(s) removed');
		}

		$sel = "select REL_G_U_USER_IDENT, ".MC_PHPBB2_TABLE_PREFIX."groups.group_id
 from REL_G_U, ".MC_PHPBB2_TABLE_PREFIX."groups
 where ".MC_PHPBB2_TABLE_PREFIX."groups.group_mc_wgroup_ident = REL_G_U.REL_G_U_WGROUP_IDENT";
  	$result = $this->DB->query($sel);

		$count = 0;

		while ($row = $result->fetchRow() )
		{
			$bbUserId  = intval($row[0]);
			$bbGroupId = intval($row[1]);

			$sel = "select count(1)
 from ".MC_PHPBB2_TABLE_PREFIX."user_group
 where ".MC_PHPBB2_TABLE_PREFIX."user_group.group_id = {$bbGroupId}
   and ".MC_PHPBB2_TABLE_PREFIX."user_group.user_id = {$bbUserId}";
  		$relExists = $this->DB->getOne($sel);

			if ($relExists) {
				continue;
			}

			// 3rd parameter: no check, relation is missing anyway
			if ( $this->addUserToGroup($bbUserId, $bbGroupId, false) ) {
				$count++;
			}
		} // end while

		if ($count == 0)
		{
			SysMsg::showInfo('no missing memberships found.');
		}
		else
		{
			SysMsg::showInfo($count.' missing membership(s) created');
		}

		return true;
	}// Ende function repairGroupMemberships



	function createGroup(
		  $p_group_name
		, $p_group_desc
		, $p_group_mod
		, $p_group_single
		, $p_mc_uid
		, $p_mc_gid
		, $p_group_type = null
	)
	{
		if ($p_group_type === null) {
			$p_group_type = $this->def_group_type;
		}
		else {
			$p_group_type = intval($p_group_type);
		}

		$p_group_mod = intval($p_group_mod);
		$p_group_single = intval($p_group_single);

		if (empty($p_mc_uid) ) {
			$p_mc_uid = 'null';
		}
		else {
			$p_mc_uid = intval($p_mc_uid);
		}

		if (empty($p_mc_gid) ) {
			$p_mc_gid = 'null';
		}
		else {
			$p_mc_gid = intval($p_mc_gid);
		}

		$sel = "select max(group_id) from ".MC_PHPBB2_TABLE_PREFIX."groups";
		$max_group_id = $this->DB->getOne($sel);

		$new_group_id = intval($max_group_id) + 1;

		$groupName = substr($p_group_name, 0, 40);
		$groupDesc = substr($p_group_desc, 0, 255);

  	$ins = "insert into ".MC_PHPBB2_TABLE_PREFIX."groups set"
			."  group_id = ".$new_group_id
			.", group_type = ".$p_group_type
			.", group_name = \"".mysql_real_escape_string($groupName)."\""
			.", group_description = \"".mysql_real_escape_string($groupDesc)."\""
			.", group_moderator   = ".$p_group_mod
			.", group_single_user = ".$p_group_single
			.", group_mc_user_ident   =".$p_mc_uid
			.", group_mc_wgroup_ident =".$p_mc_gid
		;
  	$res = $this->DB->query($ins);

    return $new_group_id;
	}// Ende function createGroup



	private function createGroupFromMcGroup($mcGroupId, $lang = null, $wgroupCap = null)
	{
		if ($wgroupCap)
		{
			$bbGroupId = $this->createGroup($wgroupCap, '', 0, 0, 0, $mcGroupId);
			return $bbGroupId;
		}

		if (empty($lang))
		{
			$lang = $GLOBALS['l_strLang'];
		}

		$mcGroupId = intval($mcGroupId);

		$sel = "select WGROUP_CAP";
		$sel.=" from WGROUP";
		$sel.=" where WGROUP_ID=".$mcGroupId;
		$ser_wgroup_cap = $this->DB->getOne($sel);

		$wgroupCap = mc_U::parseMultiLang($ser_wgroup_cap, $lang);

		$bbGroupId = $this->createGroup($wgroupCap, '', 0, 0, 0, $mcGroupId);

		return $bbGroupId;
	}// Ende function createGroupFromMcGroup



	private function getGroupByMcGroup($mcGroupId)
	{
  	$bbGroupId = $this->DB->getCol(
	   	 " select group_id"
	   	." from ".MC_PHPBB2_TABLE_PREFIX."groups"
	   	." where group_mc_wgroup_ident = ".intval($mcGroupId)
	  );

		if (empty($bbGroupId) )
		{
			return 0;
		}

		$bbGroupId = intval($bbGroupId);

		return $bbGroupId;
	}// Ende function getGroupByMcGroup



	function addUserToGroup($bbUserId, $bbGroupId, $checkIfExists = true)
	{
		$bbUserId = intval($bbUserId);
		$bbGroupId = intval($bbGroupId);

		if ($checkIfExists)
		{
	  	$id = $this->DB->getOne(
		   	 " select count(group_id)"
		   	." from ".MC_PHPBB2_TABLE_PREFIX."user_group"
		   	." where user_id = {$bbUserId} and group_id = {$bbGroupId}"
		  );

			if ( ! empty($id) )
			{
				// relation already exists
				return true;
			}
		}

  	$ins = "insert into ".MC_PHPBB2_TABLE_PREFIX."user_group set"
  		."  group_id = ".$bbGroupId
  		.", user_id  = ".$bbUserId
  		.", user_pending  = 0"
  	;
  	$this->DB->query($ins);

		return true;
	}// Ende function addMcUserToMcGroup



	function addUserToMcGroup($bbUserId, $mcGroupId)
	{
		$bbUserId = intval($bbUserId);

		$bbGroupId = $this->getGroupByMcGroup($mcGroupId);
		if (empty($bbGroupId) )
		{
			mc_Debug::error($mcGroupId, "missing board group for mc group {$mcGroupId}");
			return false;
		}

		$this->addUserToGroup($bbUserId, $bbGroupId);

		return true;
	}// Ende function addMcUserToMcGroup



	function addGroupToForum($bbGroupId, $arrForumId, $isMod=false)
	{
		if ($isMod) {
			$auth_mod = 2;
		}
		else {
			$auth_mod = 0;
		}

		foreach($arrForumId as $forumId)
		{
			$forumId = intval($forumId);
			$bbGroupId = intval($bbGroupId);

	  	$ins = "insert into ".MC_PHPBB2_TABLE_PREFIX."auth_access";
	  	$ins.=" set";
			$ins.="  group_id    = ".$bbGroupId;
			$ins.=", forum_id    = ".$forumId;
			$ins.=", auth_view   = 2";
			$ins.=", auth_read   = 2";
			$ins.=", auth_post   = 2";
			$ins.=", auth_reply  = 2";
			$ins.=", auth_edit   = 2";
			$ins.=", auth_delete = 2";
			$ins.=", auth_sticky = 2";
			$ins.=", auth_announce    = 2";
			$ins.=", auth_vote        = 2";
			$ins.=", auth_pollcreate  = 2";
			$ins.=", auth_attachments = 2";
			$ins.=", auth_mod         = ".$auth_mod;
	  	$res = $this->DB->query($ins);
		}

		return true;
	}// Ende function addGroupToForum



	function removeMcGroupFromForum($mcGroupId, $forumId)
	{
		$mcGroupId = intval($mcGroupId);
		$forumId = intval($forumId);

  	$bbGroupId = $this->getGroupFromMcGroup($mcGroupId);
		if (empty($bbGroupId) ) {
			// group not found
			return false;
		}

  	$del = "delete from ".MC_PHPBB2_TABLE_PREFIX."auth_access";
		$del.=" where";
		$del.="     forum_id = ".$forumId;
		$del.=" and group_id = ".$bbGroupId;
  	$res = $this->DB->query($del);

		return true;
	}// Ende function removeMcGroupFromForum



	function createAccountGroup($bbUserId)
	{
		$bbGroupId = $this->createGroup('', 'Personal User', 0, 1, $bbUserId, 0);
		if ( empty($bbGroupId) )
		{
  		mc_Debug::error(null, "personal user group not created");
  		return false;
		}

		$this->addUserToGroup($bbUserId, $bbGroupId);

		return $bbGroupId;
	} // end of method createAccountGroup



	function createAccountFromMcUser($mcUserId)
	{
		$appId = mc_Session::getApp();
		$mcUserId = intval($mcUserId);

		$sel = "select USER_IDENT, USER_NAME, USER_PWD, USER_EMAILADR, USER_LANG";
		$sel.=" from USER";
		$sel.=" where ";
		$sel.="     USER_APP_IDENT=".$appId;
		$sel.=" and USER_IDENT=".$mcUserId;
		$sel.=" and STATE not in ('T', 'N')";
		$user = $this->DB->getRow($sel, DBMC_FETCHMODE_ASSOC);

		if (empty($user) )
		{
			return false;
		}

		$bbUserId = intval($user['USER_IDENT']);
		if ( empty($user['USER_LANG']) || empty($this->lang_map[$user['USER_LANG']]) )
		{
			$lang = $this->lang_map['DE'];
		}
		else
		{
			$lang = $this->lang_map[$user['USER_LANG']];
		}

		$ins = "insert into ".MC_PHPBB2_TABLE_PREFIX."users"
			." set"
			."  user_id         = ".$bbUserId
			#.", user_active     = 1,"
			.", username        = \"".mysql_real_escape_string($user['USER_NAME']).'"'
			.", user_password   = \"".mysql_real_escape_string($user['USER_PWD']).'"'
			.", user_lang       = \"".mysql_real_escape_string($lang).'"'
			.", user_email      = \"".mysql_real_escape_string($user['USER_EMAILADR']).'"'
			.", user_regdate    = unix_timestamp()"
			.", user_viewemail  = 0"
			.", user_allowhtml  = 0"
			#.", user_dateformat = 'd-m-y, H:i'"
			.", user_dateformat = 'd M y, H:i'"
			.", user_style      = 2"
			.", user_timezone   = 2"
		;
		$this->DB->query($ins);

		$this->createAccountGroup($bbUserId);

		$this->addUserToGroup($bbUserId, 3); // all platform members

    return true;
	}// Ende function createAccountFromMcUser


} // end class


