<?php
/*
* $McLicense$
*
* $Id$
*
*/


class PVTreeNode
{
	var $DB;

	private $id = 0;

	private $node;

	public function getInstance($id)
	{
		$l_PVTreeNode = new PVTreeNode($id);
		if ($l_PVTreeNode->getId() == 0)
		{
			// failed
			return false;
		}

		// succeed
		return $l_PVTreeNode;
	} // end method getInstance


	public function __construct($id)
	{
		global $U;
		$this->DB = &$U->DB;

		try {
			$this->node = $this->DB->getRow(
				 "select * "
				." from VTREE_NODE"
				." where"
				.((is_numeric($id)) ? " VTREE_NODE_ID = ".intval($id) : ' VTREE_NODE_IDENTIFIER = "'.addslashes(trim($id)).'"')
				." and VTREE_NODE_DELETED != 1"
			, DBMC_FETCHMODE_ASSOC);
		} catch (Exception $e) {
			$this->node = null;
		}

		if (empty($this->node))
		{
			$this->node = null;
			$this->id = 0;
			return;
		}

		$this->id = intval($this->node['VTREE_NODE_ID']);

		return;
	} // end method __construct


	function getId()         {return intval($this->id);}
	function getParentID()   {return $this->node['VTREE_NODE_PARENT_ID'];}
	function getCourse()     {return intval($this->node['VTREE_NODE_COURSE_ID']);}
	function getFilePath()   {return $this->node['VTREE_NODE_FILEPATH'];}
	function getFileName()   {return $this->node['VTREE_NODE_FILENAME'];}
	function getSaveName()   {return $this->node['VTREE_NODE_SAVENAME'];}
	function getIdentifier() {return $this->node['VTREE_NODE_IDENTIFIER'];}
	function getFileType()   {return $this->node['VTREE_NODE_FILETYPE'];}
	function getFileSize()   {return $this->node['VTREE_NODE_FILESIZE'];}
	function getNodeDesc()   {return $this->node['VTREE_NODE_DESC'];}
	function getNodeType()   {return $this->node['VTREE_NODE_TYPE'];}
	function getMimeType()   {return $this->node['VTREE_NODE_MIMETYPE'];}
	function getVersion()    {return $this->node['VTREE_NODE_VERSION'];}
	function isDeleted()     {return $this->node['VTREE_NODE_DELETED'];}
	function isLocked()      {return $this->node['VTREE_NODE_LOCK_ID'];}
	function getOwner()      {return $this->node['OWNER'];}
	function getUpdUser()    {return $this->node['UPDUSER'];}
	function getWGroup()     {return $this->node['WGROUP'];}
	function getAccess()     {return $this->node['ACCESS'];}
	function getExtemp()     {return $this->node['VTREE_NODE_LOCK_EXTEMP'];}
	function getInsDat()     {return $this->node['INSDAT'];}
	function getUpdDat()     {return $this->node['UPDDAT'];}
	function getFullName()   {return ($this->getFileType()) ? $this->getFileName().".".$this->getFileType() : $this->getFileName();}
	function isZip()         {return (strtolower($this->getFileType()) == "zip") ? true : false;}



	function getCanonicalName()
	{
		if (empty($this->id))
		{
			mc_Debug::error(null, 'empty node');
			return false;
		}

		if ($this->getNodeType() < 10)
		{
			if ($this->getNodeType() == 1)
			{
				$path = $this->getFullName()."/";
				if ($this->getParentID() == 0)
				{
					$path = $this->getFilePath()."/".$path;
					return $path;
				}
			}
			else
			{
				$path = $this->getFilePath()."/";
				return $path;
			}
		}
		else if ($this->getNodeType() == 12) // alias
		{
			$path = $this->getSaveName();
		}
		else
		{
			$path = $this->getFullName();
		}

		$l_id = $this->getParentID();

		if (empty($l_id))
		{
			SysMsg::showWarning('no parent node id found for node '.$this->getId(), true);
			die();
		}

		while ($l_id)
		{
			$l_parent_node = PVTreeNode::getInstance($l_id);
			if (empty($l_parent_node))
			{
				SysMsg::showWarning('no parent node found for node '.$this->getId(), true);
				break;
			}

			$l_parent_node_type = $l_parent_node->getNodeType();
			if ($l_parent_node_type == 2 || $l_parent_node_type == 3)
			{
				break;
			}
			$path = $l_parent_node->getFullName()."/".$path;
			$l_id = $l_parent_node->getParentID();
		}

		$path = $l_parent_node->getFilePath()."/".$path;

		return $path;
	} // end method getCanonicalName



	function delete($p_arr_link_type_ident = null)
	{
		if (empty($this->id))
		{
			return false;
		}

		if ($p_arr_link_type_ident === null)
		{
			$p_arr_link_type_ident = $this->DB->getCol('select LINK_TYPE_IDENT from LINK_TYPE where LINK_TYPE_NAME in ("fileman_jump_to_node", "fileman_download")');
		}

		if ( !empty($p_arr_link_type_ident) )
		{
			$this->DB->query('delete from LINK where LINK_LINK_TYPE_IDENT in ('.implode(', ', $p_arr_link_type_ident).') and LINK_D_ID = '.$this->getId());
		}

		$l_arr_childs = $this->DB->getCol('select VTREE_NODE_ID from VTREE_NODE where VTREE_NODE_PARENT_ID = '.$this->getId());
		foreach($l_arr_childs as $l_child_id)
		{
			$l_child_node = PVTreeNode::getInstance($l_child_id);
			if (empty($l_child_node))
			{
				continue;
			}

			$l_child_node->delete($p_arr_link_type_ident);
			$this->DB->query('delete from VTREE_NODE where VTREE_NODE_ID = '.$this->getId());
		}

		return $this->getCanonicalName();
	} // end method delete



} // end class PVTreeNode

