<?php
/*
* $McLicense$
*
* $Id$
*
*/

class PVcol
{
	private $DB;

	private $arrVcolFields = array();

	private $arrVcolValues = array();


	public function __construct ()
	{
		global $U;
		$this->DB = &$U->DB;
	}



	/**
	 * defines the set of vcol fields for which addUpdate will provide the values.
	 * fields names are given without prefix 'VCOL_' as list of comma separated arguments e.g.
	 * ->setFields('VCOL_LIST', 'VCOL_SHOW', 'STATE')
	 * throws an exception if there are still uncommitted updates in queue
	 *  
	 */
	public function setFields()
	{
		$num = func_num_args();
		if ($num == 0)
		{
			throw new Exception("no arguments given");			
		}

		if ( ! empty($arrVcolValues) )
		{
			throw new Exception("there are uncommitted updates in queue! use function commit() in order to clear the queue.");			
		}

		$this->clear();

		$arrVcolFields = func_get_args();
		foreach($arrVcolFields as $field)
		{
			switch($field)
			{
				#case 'EINHEIT':
				#case 'TYPE':
				case 'LIST':
				case 'INPUT':
				case 'SHOW':
				case 'UPDATE':
				case 'SEARCH':
				case 'LLIST':
				case 'LSEARCH':
				case 'UPERM':
				case 'FELDART':
				#case 'TEXTLEN':
				#case 'URL':
				#case 'TARGET':
				#case 'ADDSID':
				#case 'MAIL':
				#case 'WEB':
				#case 'FORMNAME':
				case 'FORMTYP':
				case 'FORMSTYP':
				case 'FORMLEN':
				case 'FORMMLEN':
				#case 'FORMCHECKV':
				case 'FORMROWS':
				case 'FORMCOLS':
				case 'FORMVALUE':
				case 'FORMFIELD':
				#case 'REGEXP_TPL_NAME':
				#case 'CONDITION':
				#case 'JSCRIPT':
				case 'ORDER':
				case 'CSS':
					// field name is valid
					$this->arrVcolFields[] = 'VCOL_'.$field;
					break;

				case 'STATE':
					$this->arrVcolFields[] = $field;
					break;
				default: 
					throw new Exception("'{$field}' is not a registered field name");
			}
		}

		return TRUE;
	}



	/**
	 * resets all list of fields and removes all uncommitted updates
	 */
	private function clear()
	{
		$this->arrVcolFields = array();
		$this->arrVcolValues = array();
		return TRUE;
	}



	/**
	 * called with view name an column name as first two arguments.
	 * more arguments depend on current setting from function setFields.
	 * e.g.:
	 * ->setFields(                           'LIST', 'SHOW', 'STATE');
	 * ->addUpdate('VCOURSE', 'COURSE_TITEL', 'Y',    'Y',    'Y')
	 *  
	 */
	public function addUpdate()
	{
		$num = func_num_args();
		if ($num < 3)
		{
			throw new Exception("number of arguments is to low! use addUpdate(viewName, colname, value1 [, value2 [, ...]]]);");			
		}

		$arrArguments = func_get_args();
		$viewName = array_shift($arrArguments);
		$colName = array_shift($arrArguments);

		if (sizeof($arrArguments) != sizeof($this->arrVcolFields) )
		{
			$strValues = print_r($arrArguments, TRUE);
			$strFields = print_r($this->arrVcolFields, TRUE);
			throw new Exception("number of values ( {$strValues} ) does now match number of required fields ( {$strFields} )");
		}

		if (empty($this->arrVcolValues[$viewName]) )
		{
			$this->arrVcolValues[$viewName] = array();
		}
		
		if (empty($this->arrVcolValues[$viewName][$colName]) )
		{
			$this->arrVcolValues[$viewName][$colName] = array();
		}
		else
		{
			throw new Exception("values for view '{$viewName}', column '{$colName}' are already in queue!");
		}

		foreach($this->arrVcolFields as $idx => $field)
		{
			$value = $arrArguments[$idx];
			switch($field)
			{
				case 'VCOL_INPUT':
				case 'VCOL_LLIST':
				case 'VCOL_LSEARCH':
				case 'VCOL_UPERM':
				case 'STATE':
					if ($value == 'Y' || $value == 'N')
					{
						break;
					}
					throw new Exception("invalid value '{$value}' for field '{$field}'");


				case 'VCOL_SHOW':
					// C = ???
					if ($value == 'Y' || $value == 'N' || $value == 'C')
					{
						break;
					}
					throw new Exception("invalid value '{$value}' for field '{$field}'");
 
				case 'VCOL_LIST':
					if ($value == 'Y' || $value == 'N' || is_int($value) )
					{
						break;
					}
					throw new Exception("invalid value '{$value}' for field '{$field}'");

				case 'VCOL_UPDATE':
					// R = readonly
					if ($value == 'Y' || $value == 'N' || $value == 'R' || is_int($value) )
					{
						break;
					}
					throw new Exception("invalid value '{$value}' for field '{$field}'");

				case 'VCOL_SEARCH':
					if ($value == 'Y' || $value == 'N' || $value == 'S')
					{
						break;
					}
					throw new Exception("invalid value '{$value}' for field '{$field}'");

				case 'VCOL_FELDART':
					// N is not in use
					// Y = standard value, no madatory field
					// P = madatory field
					// C = conditional field, for FORMTYP='BILD' ONLY !!! (displays image tage only if not a default image)
					if ($value == 'Y' || $value == 'N' || $value == 'P' || $value == 'C')
					{
						break;
					}
					throw new Exception("invalid value '{$value}' for field '{$field}'");

				case 'VCOL_FORMLEN':
				case 'VCOL_FORMMLEN':
				case 'VCOL_ORDER':
					if ($value === NULL || is_int($value) && $value >= 0)
					{
						break;
					}
					throw new Exception("invalid value '{$value}' for field '{$field}'");

				case 'VCOL_FORMROWS':
				case 'VCOL_FORMCOLS':
					if (is_int($value) && $value >= 0)
					{
						break;
					}
					throw new Exception("invalid value '{$value}' for field '{$field}'");

				case 'VCOL_FORMTYP':
					// alphabetical order!
					if ($value === NULL) { break; }
					if ($value === ''  ) { break; }
					if ($value == 'BILD'       ) { break; }
					if ($value == 'GROUPBOX'   ) { break; }
					if ($value == 'INPUT'      ) { break; }
					if ($value == 'METADATA'   ) { break; }
					if ($value == 'SELECT'     ) { break; }
					if ($value == 'TEXTAREA'   ) { break; }
					if ($value == 'TEXTAREAEXT') { break; }
					throw new Exception("invalid value '{$value}' for field '{$field}'");

				case 'VCOL_FORMSTYP':
					// alphabetical order!
					if ($value === NULL) { break; }
					if ($value === ''  ) { break; }
					if ($value == 'CHECKBOX'  ) { break; }
					if ($value == 'ComponentRecordSet') { break; }
					if ($value == 'EXTERN'    ) { break; }
					if ($value == 'HIDDEN'    ) { break; }
					if ($value == 'multimedia') { break; }
					if ($value == 'MULTIPLE'  ) { break; }
					if ($value == 'NewsPict'  ) { break; }
					if ($value == 'normal'    ) { break; }
					if ($value == 'PASSWORT'  ) { break; }
					if ($value == 'RADIO'     ) { break; }
					if ($value == 'SELF'      ) { break; }
					if ($value == 'TagSet'    ) { break; }
					if ($value == 'TEXT'      ) { break; }
					if ($value == 'webpage'   ) { break; }
					if ($value == 'website'   ) { break; }
					throw new Exception("invalid value '{$value}' for field '{$field}'");

				case 'VCOL_FORMFIELD':
					if ($value === NULL) { break; }

					$trimmedValue = trim($value);
					if ($trimmedValue === '') { break; }
					if (substr($trimmedValue, 0, 1) == '%') { break; }
					if (substr($trimmedValue, 0, 1) == '#') { break; }
					if (strtoupper( substr($trimmedValue, 0, 6) ) == 'SELECT') { break; }

					throw new Exception("invalid value '{$value}' for field '{$field}'");

				case 'VCOL_FORMVALUE':
				case 'VCOL_CSS':
					// beside some standardized values nearly every possible string is a valid preset
					break;

				#case 'VCOL_EINHEIT':
				#case 'VCOL_TYPE':
				#case 'VCOL_TEXTLEN':
				#case 'VCOL_URL':
				#case 'VCOL_TARGET':
				#case 'VCOL_ADDSID':
				#case 'VCOL_MAIL':
				#case 'VCOL_WEB':
				#case 'VCOL_FORMNAME':
				#case 'VCOL_FORMCHECKV':
				#case 'VCOL_REGEXP_TPL_NAME':
				#case 'VCOL_CONDITION':
				#case 'VCOL_JSCRIPT':
				default: 
					throw new Exception("'{$field}' is not a registered field name");
			}

			// check passed, store value
			$fieldUpdate = "VCOL.{$field} = " . $this->DB->quote($value, 'TEXT');
			$this->arrVcolValues[$viewName][$colName][] = $fieldUpdate;
		}

		return TRUE;
	}



	/**
	 * commits all stored updates and empties the queue 
	 */
	public function commit()
	{
		
		if (empty($this->arrVcolValues) )
		{
			throw new Exception("empty queue: there are no updates to commit.");			
		}
		
		foreach($this->arrVcolValues as $viewName => $arrColumns)
		{
			$qViewName = $this->DB->quote($viewName, 'TEXT');
			foreach($arrColumns as $colName => $arrFieldUpdates)
			{
				$qColName = $this->DB->quote($colName, 'TEXT');
				$strFieldUpdates = implode(', ', $arrFieldUpdates);
				$query = "
UPDATE VIEW, COL, VCOL SET
 {$strFieldUpdates}
 WHERE VIEW.VIEW_NAME = {$qViewName}
 AND COL.COL_TAB_IDENT = VIEW.VIEW_TAB_IDENT
 AND COL.COL_NAME = {$qColName}
 AND VCOL.VCOL_VIEW_IDENT = VIEW.VIEW_IDENT
 AND VCOL.VCOL_COL_IDENT = COL.COL_IDENT
";
				$this->DB->query($query);
				SysMsg::showInfo("updated vcol for {$viewName}:{$colName}");
			}
			
		}
		$this->arrVcolValues = array();

		return TRUE;
	}


}


