<?php
/*
* $McLicense$
*
* $Id$
*
*/


define("MCP_RIGHT_SHOW",   1);
define("MCP_RIGHT_INSERT", 2);
define("MCP_RIGHT_UPDATE", 3);
define("MCP_RIGHT_DELETE", 4);
define("MCP_RIGHT_FILE",   5);
define("MCP_RIGHT_LINK",   6);
define("MCP_RIGHT_POLL",   7);
define("MCP_RIGHT_MBOARD", 8);

require_once(MCP_LIB_PATH . "QueryLog.php");

class PWright
{
	private $DB;

	private $defaultComponentSiud = 'YYYYNNNNN';

	function __construct()
	{
		global $U;
		$this->DB = &$U->DB;
	}



	function setDefaultComponentSiud($siud)
	{
		if (strlen($siud) != 9)
		{
			throw new Exception("invalid SIUD length of '{$siud}'! requires 9 characters");
		}
		$this->defaultComponentSiud = $siud;
		return TRUE;
	}



	function getDefaultComponentSiud() 
	{
		return $this->defaultComponentSiud;
	}



	function updateRoomElementPermissions($elementName, $right, $state, $roomId = NULL)
	{
		$right = intval($right);

		if ($right < 1 || $right > 9)
		{
			throw new Exception("invalid permission index '{$right}'");
		}
		$SiudIdx = $right - 1;

		if ($state == 'Y' || $state == TRUE) 
		{
			$state = 'Y';
		}
		else 
		{
			$state = 'N';
		}

		$qElementName = $this->DB->quote($elementName, 'TEXT');
		$query = "
select WRIGHT_SIUD, WRIGHT_ID
 from WRIGHT, ELEMENT
 where ELEMENT_NAME = {$qElementName}
 and WRIGHT_ELEMENT_IDENT = ELEMENT_IDENT
";
		if ($roomId !== NULL)
		{
			$query .= " and COURSE_ID = ".intval($roomId);
		}
		$result = $this->DB->query($query);

		if ($result->numRows() == 0) 
		{
			return TRUE;
		}

		while ($row = $result->fetchRow(DBMC_FETCHMODE_ASSOC) )
		{
			$id = intval($row['WRIGHT_ID']);
			$SIUD = $row['WRIGHT_SIUD']; 
			$SIUD[$SiudIdx] = $state;
			$qSIUD = $this->DB->quote($SIUD, 'TEXT');

			$query = "update WRIGHT set WRIGHT_SIUD = {$qSIUD} where WRIGHT_ID = {$id}";
			$this->DB->query($query);
		}
		$result->free();

		return TRUE;
	}



	function updateTemplateElementPermissions($elementName, $right, $state, $templateId = NULL)
	{
		$right = intval($right);

		if ($right < 1 || $right > 9)
		{
			throw new Exception("invalid permission index '{$right}'");
		}
		$SiudIdx = $right - 1;

		if ($state == 'Y' || $state == TRUE) 
		{
			$state = 'Y';
		}
		else 
		{
			$state = 'N';
		}

		$qElementName = $this->DB->quote($elementName, 'TEXT');
		$query = "
select TPL_WRIGHT_WRIGHT_SIUD, TPL_WRIGHT_ID
 from TPL_WRIGHT, ELEMENT
 where ELEMENT_NAME = {$qElementName}
 and TPL_WRIGHT_WRIGHT_ELEMENT_IDENT = ELEMENT_IDENT
";
		if ($templateId !== NULL)
		{
			$query .= " and TPL_WRIGHT_ROOM_TYPE_IDENT = ".intval($templateId);
		}
		$result = $this->DB->query($query);

		if ($result->numRows() == 0) 
		{
			return TRUE;
		}

		while ($row = $result->fetchRow(DBMC_FETCHMODE_ASSOC) )
		{
			$id = intval($row['TPL_WRIGHT_ID']);
			$SIUD = $row['TPL_WRIGHT_WRIGHT_SIUD']; 
			$SIUD[$SiudIdx] = $state;
			$qSIUD = $this->DB->quote($SIUD, 'TEXT');

			$query = "update TPL_WRIGHT set TPL_WRIGHT_WRIGHT_SIUD = {$qSIUD} where TPL_WRIGHT_ID = {$id}";
			$this->DB->query($query);
		}
		$result->free();

		return TRUE;
	}



	function mapElementPermission($context, PElement $fromElement, PElement $toElement, $fkId = null)
	{

		if ($context == 'room')
		{
			$table = 'WRIGHT';
			$fieldFk    = 'COURSE_ID';
			$fieldApp   = 'WRIGHT_APP_IDENT';
			$fieldGroup = 'WRIGHT_WGROUP_IDENT';
			$fieldType  = 'WRIGHT_TYPE';
			$fieldSIUD  = 'WRIGHT_SIUD';
			$fieldElement = 'WRIGHT_ELEMENT_IDENT';
		}
		else if ($context == 'template')
		{
			$table = 'TPL_WRIGHT';
			$fieldFk    = 'TPL_WRIGHT_ROOM_TYPE_IDENT';
			$fieldApp   = 'TPL_WRIGHT_APP_IDENT';
			$fieldGroup = 'TPL_WRIGHT_TPL_WGROUP_IDENT';
			$fieldType  = 'TPL_WRIGHT_WRIGHT_TYPE';
			$fieldSIUD  = 'TPL_WRIGHT_WRIGHT_SIUD';
			$fieldElement = 'TPL_WRIGHT_WRIGHT_ELEMENT_IDENT';
		}
		else
		{
			throw new Exception("unknown context '{$context}'!");
		}

		$fromKind = $fromElement->getKind();
		$toKind   = $toElement->getKind();

		if ($fromKind == $toKind)
		{
			$mapType = 'equal';
		}
		else if ($fromKind == 3 && $toKind == 0)
		{
			$mapType = 'virtualToComponent';
		}
		else
		{
			throw new Exception("unknown map process requested: element kind '{$fromKind}' to element kind '{$toKind}'");
		}

		$fromElementIdent = $fromElement->getIdent();
		$toElementIdent   = $toElement->getIdent();

		$QueryLog = new QueryLog();

		$select = "select {$fieldApp}, {$fieldGroup}, {$fieldSIUD}, {$fieldFk}"
			." from {$table}"
			." where {$fieldElement} = {$fromElementIdent}"
			." and {$table}.STATE = 'Y'"
		;

		if ($fkId) 
		{
			$select .= " and {$fieldFk} = ".intval($fkId);
		}

		$permissionResult = $this->DB->query($select);

		if ($permissionResult->numRows() == 0)
		{
			// no permissions
			return;
		}

		while( $permission = $permissionResult->fetchRow() )
		{
			$appId   = intval($permission[0]);
			$groupId = intval($permission[1]);
			$SIUD    = trim($permission[2]);
			$roomId  = intval($permission[3]);

			if (empty($SIUD) || empty($groupId) || empty($appId) )
			{
				// corrupted permission entry
				continue;
			}

			// check if permission already exists
			$select = "select {$table}_ID"
				." from {$table}"
				." where {$fieldFk} = {$fkId}"
				." and {$fieldElement} = {$toElementIdent}"
				." and {$fieldGroup} = {$groupId}"
				." and {$table}.STATE = 'Y'"
			;
			$wid = $this->DB->getOne($select);

			if ($wid)
			{
				// permission already exists
				continue;
			}

			switch ($mapType)
			{
				case 'equal' :
					$newSIUD = $SIUD;
					break;

				case 'virtualToComponent' :
					if ($SIUD[0] != 'Y') {
						// permission denied
						continue 2; // switch, while (permission)
					}
					$newSIUD = 'YYYYNNNNN';
					break;

				default :
					throw new mc_Exception($mapType, "unknown map type");
			} // switch

			$insert = "insert into {$table} set"
				."  {$fieldApp} = {$appId}"
				.", {$fieldElement} = {$toElementIdent}"
				.", {$fieldGroup} = {$groupId}"
				.", {$fieldSIUD} = '{$newSIUD}'"
				.", {$fieldType} = {$toKind}"
				.", {$fieldFk} = {$roomId}"
				.", STATE = 'Y'"
			;

			if ($QueryLog->exists($insert) )
			{
				// query was already executed (maybe pemission was deleted afterwards)
				continue;
			}

			$insRes = $this->DB->query($insert);

			$QueryLog->logQuery($table, $insert);
		} // end foreach (groups)
		
		return;
	}



	/**
	 * DEPRECATED function name
	 */
	function rewriteElementSIUD($p_element_name, $p_right, $p_state, $p_course_id = null)
	{
		return $this->updateElementPermission($p_element_name, $p_right, $p_state, $p_course_id);
	}



	function setRoomElementPermissions($roomId, $elementName, $group, $SIUD, $skipAmbiguousGroups=FALSE)
	{
		if ( ! is_numeric($roomId) )
		{
			throw new Exception("parameter roomId is not numeric");
		}
		$roomId = intval($roomId);
	
		if (is_numeric($group) )
		{
			$arrGroupId = array();
			$arrGroupId[] = intval($group);
		}
		else
		{
			$query = "SELECT WGROUP_ID FROM WGROUP"
				." WHERE COURSE_ID = {$roomId}"
				." AND WGROUP_NAME = ".$this->DB->quote($group, 'TEXT')
			;
			$arrGroupId = $this->DB->getCol($query);
			$groupCount = count($arrGroupId);
			if (empty($groupCount) )
			{
				SysMsg::showWarning("group '{$group}' not found for room #{$roomId}");
				return FALSE;
			}
			if ($skipAmbiguousGroups && $groupCount > 1)
			{
				SysMsg::showWarning("group '{$group}' is ambiguous for room #{$roomId}");
				return FALSE;
			}
		}
		
		$query = "SELECT ELEMENT_ID FROM ELEMENT"
			." WHERE ELEMENT_NAME = ".$this->DB->quote($elementName, 'TEXT')
			." AND STATE = 'Y'"
		;
		$elementId = $this->DB->getOne($query);
		if (empty($elementId) ) 
		{
			throw new Exception("element '{$elementName}' not found!");
		}

		$SIUD = trim($SIUD);
		$siudLength = strlen($SIUD);
		if ($siudLength == 9) { // component
			$wrightType = 0;
		}
		else if ($siudLength == 4) { // mboard
			$wrightType = 1;
		}
		else if ($siudLength == 1) { // virtual
			$wrightType = 3;
		}
		else {
			throw new Exception("length of SIUD '{$SIUD}' is invalid!");
		}
		
		foreach($arrGroupId as $groupId)
		{
			$groupId = intval($groupId);

			$query = "SELECT WRIGHT_ID, WRIGHT_SIUD"
				." FROM WRIGHT"
				." WHERE WRIGHT_ELEMENT_IDENT = {$elementId}"
				." AND WRIGHT_WGROUP_IDENT = {$groupId}"
			;
			$wright = $this->DB->getRow($query);
			$wrightId = intval($wright[0]);
			$wrightSIUD = trim($wright[1]);

			if (strpos($SIUD, 'Y') === FALSE)
			{
				if (empty($wrightId) )
				{
					SysMsg::showInfo("permission for {$elementName}:r{$roomId}:g{$groupId} has been removed already");
					continue;
				}
			
				// delete permission
				$this->DB->query("DELETE FROM WRIGHT WHERE WRIGHT_ID = {$wrightId}");
				SysMsg::showInfo("permission for {$elementName}:r{$roomId}:g{$groupId} deleted");
				continue;
			}
			
			if (empty($wrightId) )
			{
				// permission does not exist, requires insert
				$arrFields = array(
					'WRIGHT_APP_IDENT' => 1,
					'WRIGHT_ELEMENT_IDENT' => $elementId,
					'WRIGHT_WGROUP_IDENT' => $groupId,
					'WRIGHT_SIUD' => $this->DB->quote($SIUD, 'TEXT'),
					'WRIGHT_TYPE' => $wrightType,
					'COURSE_ID' => $roomId,
					'OWNER' => mc_Session::getUser(),
					'INSDAT' => date('Y-m-d'),
					'STATE' => '"Y"',
				);
				$this->DB->insert('WRIGHT', $arrFields, FALSE);
				SysMsg::showInfo("permission added: {$elementName}:r{$roomId}:g{$groupId} = '{$SIUD}'");
				continue;
			}

			if ($wrightSIUD == $SIUD)
			{
				// permission exists and is the same as required
				SysMsg::showInfo("permission {$elementName}:r{$roomId}:g{$groupId} is already up to date");
				continue;
			}

			// permission exists but requires update
			$query = "UPDATE WRIGHT SET"
				." WRIGHT_SIUD = ".$this->DB->quote($SIUD, 'TEXT')
				." WHERE WRIGHT_ID = {$wrightId}"
			;
			$this->DB->query($query);
			SysMsg::showInfo("permission updated: {$elementName}:r{$roomId}:g{$groupId} = '{$SIUD}'");
		}

		return TRUE;
	}


}


