<?php
/*
* $McLicense$
*
* $Id$
*
*/


if ( defined("MC_ROOT_PATH") == false)
{
	include_once("../../dblog.inc.php");
}

require_once(MCP_PATH."_classes/Plattform.php");

class PVTreeNode
	extends Plattform
{
	var $type;
	var $canonicalName = "";



	function getInstance($id)
	{
		$VTreeNode = new PVTreeNode ($id);

		if ( ! is_object($VTreeNode) ) {
			return false;
		}

		if ( $VTreeNode->isValid() )
		{
			// succeed
			return $VTreeNode;
		}

		return false;
	} // end getInstance



	function PVTreeNode ($id)
	{
		$this->Plattform();

		$this->type = array(
			"DIR"          =>  1,
			"FILEDIR_DIR"  =>  2,
			"ALIAS_DIR"    =>  3,
			"FILE"         => 10,
			"LINK"         => 11,
			"ALIAS_FILE"   => 12,
			"FILEDIR_FILE" => 21,
		);

		$id = intval($id);

		$this->node = $this->DB->getRow(
			'select * from VTREE_NODE where VTREE_NODE_ID = '.$id
			, DB_FETCHMODE_ASSOC
		);
		if (Plattform::isError($this->node)) {
			$this->node = null;
		}

    return;
	}



	##
	## member functions
	##


	/**
	 * getId
	 */
	function getId() {
		return intval($this->node['VTREE_NODE_ID']);
	}

	/**
	 * getParentId
	 */
	function getParentId() {
    return intval($this->node['VTREE_NODE_PARENT_ID']);
	}

	/**
	 * getFilePath
	 */
	function getFilePath() {
		return trim($this->node['VTREE_NODE_FILEPATH']);
	}


	/**
	 * getFileName
	 */
	function getFileName() {
		return trim($this->node['VTREE_NODE_FILENAME']);
	}

	/**
	 * getFileType
	 */
	function getFileType() {
		return trim($this->node['VTREE_NODE_FILETYPE']); // extension
	}


	/**
	 * getSaveName
	 */
	function getSaveName() {
		return trim($this->node['VTREE_NODE_SAVENAME']);
	}

	/**
	 * getNodeType
	 */
	function getNodeType() {
		return intval($this->node['VTREE_NODE_TYPE']);
	}

	/**
	 * isDeleted
	 */
	function isDeleted() {
		return intval($this->node['VTREE_NODE_DELETED']);
	}


	##
	## additional functions
	##


	/**
	 * isValid
	 */
	function isValid() {
		if ( empty($this->node) ) {
			return false;
		}

		return true;
	} // end method isValid



	/**
	 * getTypeId
	 */
	function getTypeId($typeName) {
		return $this->type[$typeName];
	} // end method getTypeId



	/**
	 * isNodeType
	 */
	function isNodeType($typeName) {
		$selfTypeId = $this->getNodeType();
		$requestedId = $this->getTypeId($typeName);

		if ($selfTypeId == $requestedId) {
			return true;
		}
		else {
			return false;
		}
	} // end method isNodeType



	/**
	 * getTypeId
	 */
	function getMimeTypeId ($extension) {
		$id = $this->DB->getOne(
			 'select MIMETYPE_IDENT from MIMETYPE where'
			.' MIMETYPE_EXT = "'.addcslashes($extension, '\\"').'"'
		);
		if (Plattform::isError($id)) {
			return 0;
		}

		$id = intval($id);

		return $id;
	} // end method getMimeTypeId



	/**
	 * getFullName
	 */
	function getFullName () {
		$saveName = $this->getSaveName();

		if ($saveName) {
			return $saveName;
		}

		$extension = $this->getFileType();
		$givenName = $this->getFileName();

		if ($extension) {
			$fullName = $givenName.'.'.$extension;
		}
		else {
			$fullName = $givenName;
		}

		return $fullName;
	} // end function getFullName



	/**
	 * getCanonicalPath
	 */
	function getCanonicalName() {

		if ($this->canonicalName) {
			return $this->canonicalName;
		}

		$this->readCanonicalName();

		return $this->canonicalName;
	}

	/**
	 * setCanonicalName
	 */
	function setCanonicalName($path) {
		$this->canonicalName = $path;
	}



	/**
	 * readCanonicalName
	 */
	function readCanonicalName() {

		if ( $this->isNodeType('FILEDIR_DIR') || $this->isNodeType('ALIAS_DIR') ) {
			// directory with full path
			// name is just a label but not part of the path
			$path = $this->getFilePath().DIRECTORY_SEPARATOR;
			$this->setCanonicalName($path);

			return $path;
		}

		$parentId = $this->getParentId();
		$selfName = $this->getFullName();

		if ( empty($parentId) ) {
			// other directory: name is part of the path!
			$path = $this->getFilePath().DIRECTORY_SEPARATOR.$selfName.DIRECTORY_SEPARATOR;
			$this->setCanonicalName($path);

			return $path;
		}

		$VTreeNode = VTreeNode::getInstance($parentId);
		if (empty($VTreeNode) || ! $VTreeNode->isValid() ) {
			return '';
		}

		$cPath = $VTreeNode->readCanonicalName();

		$selfType = $this->getNodeType();
		if ($selfType < 10) {
			// current node is directory
			$cPath = $cPath.$selfName.DIRECTORY_SEPARATOR;
		}
		else {
			// current node is a file
			$cPath = $cPath.$selfName;
		}

		$this->setCanonicalName($path);

		return $path;
	}



	/**
	 * createNode
	 */
	#private
	function createNode ($fileGivenName, $fileExtension, $fileSize, $nodeTypeId, $roomId, $owner, $group, $access, $fileDesc = null, $filePath = null)
	{
		$parentId = $this->getId();
		$filePath = trim($filePath);
		$fileGivenName = trim($fileGivenName);
		$fileExtension = trim($fileExtension);
		$roomId = intval($roomId);
		$fileSize = intval($fileSize);
		$fileDesc = trim($fileDesc);
		$nodeTypeId = intval($nodeTypeId);
		$state = 'Y';

		if ($nodeTypeId < 10) {
			$mimeTypeId = 0;
			if ( $this->getTypeId('FILEDIR_DIR') == $nodeTypeId ) {
				$state = 'S';
			}
		}
		else {
			$mimeTypeId = $this->getMimeTypeId($fileExtension);
		}

		$result = $this->DB->query(
			 'insert into VTREE_NODE set'
			.'  VTREE_NODE_PARENT_ID = '.$parentId
			.', VTREE_NODE_APP_IDENT = '.$this->APP_IDENT
			.', VTREE_NODE_COURSE_ID = '.$roomId
			.', VTREE_NODE_FILEPATH = "'.addcslashes($filePath, '\\"').'"'
			.', VTREE_NODE_FILENAME = "'.addcslashes($fileGivenName, '\\"').'"'
			.', VTREE_NODE_FILETYPE = "'.addcslashes($fileExtension, '\\"').'"'
			.', VTREE_NODE_FILESIZE = '.$fileSize
			.', VTREE_NODE_MIMETYPE = '.$mimeTypeId
			.', VTREE_NODE_TYPE = '.$nodeTypeId
			.', VTREE_NODE_DESC = "'.addcslashes($fileDesc, '\\"').'"'
			.', VTREE_NODE_VERSION = 1'
			.', VTREE_NODE_DELETED = 0'
			.', VTREE_NODE_LOCK_ID = 0'
			.', OWNER  = '.intval($owner)
			.', WGROUP = '.intval($group)
			.', ACCESS = '.intval($access)
			.', INSDAT = '.time()
			.', STATE = "'.$state.'"'
		);
		if (Plattform::isError($result)) {
			return 0;
		}

		$insId = @$this->DB->getOne('select last_insert_id()');
		$insId = @intval($insId);

		return $insId;
	} // end method createFileNode



	/**
	 * createFileNode
	 */
	#private
	function createFileNode ($fileName, $fileSize, $roomId, $owner, $group, $access, $fileDesc = null)
	{
		$fileName = trim($fileName);

		$pos = strrpos($fileName, ".");
		if ($pos) {
			$fileExtension = substr($fileName, $pos + 1);
			$fileGivenName = substr($fileName, 0, $pos);
		}
		else {
			$fileExtension = "";
			$fileGivenName = $fileName;
		}

		$id = $this->getChild($fileGivenName, $fileExtension, "");

		if ($id) {
			// node already exists
			return true;
		}

		$nodeTypeId = $this->getTypeId('FILEDIR_FILE');

		$insId = $this->createNode($fileGivenName, $fileExtension, $fileSize, $nodeTypeId, $roomId, $owner, $group, $access, $fileDesc);

		return $insId;
	} // end method createFileNode



	/**
	 * createElementNode
	 */
	#private
	function createElementNode ($filePath, $fileName, $fileDesc = null)
	{
		$filePath = trim($filePath);
		$fileName = trim($fileName);

		$pos = strrpos($fileName, ".");
		if ($pos) {
			$fileExtension = substr($fileName, $pos + 1);
			$fileGivenName = substr($fileName, 0, $pos);
		}
		else {
			$fileExtension = "";
			$fileGivenName = $fileName;
		}

		$nodeTypeId = $this->getTypeId('FILEDIR_DIR');

		$dirName  = dirname($filePath);
		$baseName = basename($filePath);

		if ( ! is_writable($dirName) ) {
			die("can't create directory '$filePath' : directory '$dirName' is not writable!");
		}

		if ( ! file_exists($filePath) ) {
			if ( ! mkdir($filePath) ) {
				die(Plattform::error($filePath, "can't create directory"));
			}
			chmod($filePath, 0777);
		}

		$id = $this->getChild('', '', $filePath);

		if ($id) {
			// node already exists
			return $id;
		}

		$owner = 1;
		$group = 2;
		$access = 7;
		$roomId = 0;
		$fileSize = 0;

		$insId = $this->createNode($fileGivenName, $fileExtension, $fileSize, $nodeTypeId, $roomId, $owner, $group, $access, $fileDesc, $filePath);

		if (empty($insId) ) {
			// failure
			return 0;
		}


		return $insId;
	} // end method createElementNode



	/**
	 * getChildByName
	 */
	#private
	function getChild($fileGivenName, $fileExtension, $filePath = null)
	{
		$parentId = $this->getId();

		$id = $this->DB->getOne(
			 'select VTREE_NODE_ID from VTREE_NODE'
			.' where'
			.'     VTREE_NODE_PARENT_ID = '.$parentId
			.($filePath
				? ' and VTREE_NODE_FILEPATH = "'.addcslashes($filePath, '\\"').'"'
				: ''
			)
			.($fileGivenName
				?  ' and VTREE_NODE_FILENAME = "'.addcslashes($fileGivenName, '\\"').'"'
					.' and VTREE_NODE_FILETYPE = "'.addcslashes($fileExtension, '\\"').'"'
				: ''
			)
			.' and VTREE_NODE_DELETED != 1'
		);
		if (Plattform::isError($id)) {
			return true; // pretending SOME node has been found
		}

		$id = intval($id);

		return $id;
	} // end function getChild



} // end class VTreeNode

