<?php

/**
 * This product Copyright 2010 metaVentis GmbH.  For detailed notice,
 * see the "NOTICE" file with this distribution.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

require_once (dirname(__FILE__).'/Plattform.php');

/**
 * handles
 *
 * @author steffen gross / matthias hupfer
 * @version 1.0
 * @package core
 * @subpackage classes.new
 */
class ESModule
extends Plattform
{

	protected $ESMODULE_ID;
	protected $ESMODULE_NAME;
	protected $ESMODULE_DESC;
	protected $ESMODULE_TYPE;
	protected $ESMODULE_URI;
	protected $ESMODULE_DISPATCHER_URI;
	protected $ESMODULE_TMP_FILEPATH;
	protected $ESMODULE_CONF;

	protected $parsed_conf;

	/**
	 *
	 */
	public function __construct($p_id = null)
	{
		parent::__construct();

		$this->ESMODULE_ID  =  0;
		$this->ESMODULE_NAME = '';
		$this->ESMODULE_DESC = '';
		$this->ESMODULE_TYPE = 0;
		$this->ESMODULE_URI = '';
		$this->ESMODULE_DISPATCHER_URI = '';
		$this->ESMODULE_TMP_FILEPATH = '';
		$this->ESMODULE_CONF = '';

		if ($p_id)
		{
			$this->setModuleID($p_id);
		}

		return true;
	}

	final public function setModuleID($p_id)
	{
		$sel   =  'SELECT';
			$sel  .= ' '.$this->DB->quoteIdentifier('ESMODULE_ID');
			$sel  .= ', '.$this->DB->quoteIdentifier('ESMODULE_NAME');
			$sel  .= ', '.$this->DB->quoteIdentifier('ESMODULE_DESC');
			$sel  .= ', '.$this->DB->quoteIdentifier('ESMODULE_TYPE');
			$sel  .= ', '.$this->DB->quoteIdentifier('ESMODULE_URI');
			$sel  .= ', '.$this->DB->quoteIdentifier('ESMODULE_DISPATCHER_URI');
			$sel  .= ', '.$this->DB->quoteIdentifier('ESMODULE_TMP_FILEPATH');
			$sel  .= ', '.$this->DB->quoteIdentifier('ESMODULE_CONF');
			$sel  .= ' FROM '.$this->DB->quoteIdentifier('ESMODULE');
			$sel  .= ' WHERE '.$this->DB->quoteIdentifier('ESMODULE_ID').' = '.$this->DB->quote((int)$p_id, 'integer');

		$ret = $this->DB->getRow($sel, DBMC_FETCHMODE_ASSOC);

		if ( Plattform::isError($ret) )
		{
			return false;
		}

		if ( empty($ret) )
		{
			return false;
		}

		$this->ESMODULE_ID   =  $ret['ESMODULE_ID'];
		$this->ESMODULE_NAME =  $ret['ESMODULE_NAME'];
		$this->ESMODULE_DESC =  $ret['ESMODULE_DESC'];
		$this->ESMODULE_TYPE =  $ret['ESMODULE_TYPE'];
		$this->ESMODULE_URI =   $ret['ESMODULE_URI'];
		$this->ESMODULE_DISPATCHER_URI =  $ret['ESMODULE_DISPATCHER_URI'];
		$this->ESMODULE_TMP_FILEPATH   =  $ret['ESMODULE_TMP_FILEPATH'];
		$this->ESMODULE_CONF =  $ret['ESMODULE_CONF'];

		return true;
	}

	/**
	 * Load data for named module.
	 *
	 * @param string $ModuleName
	 *
	 * @return bool
	 */
	public function loadModuleData()
	{
		if ( empty($this->ESMODULE_NAME) )
		{
			throw new Exception('Error loading module-data. No module-name set.');
		}

		$ret = $this->DB->getRow(
			 'select'
			.'  '.$this->DB->quoteIdentifier('ESMODULE_ID')
			.', '.$this->DB->quoteIdentifier('ESMODULE_DESC')
			.', '.$this->DB->quoteIdentifier('ESMODULE_TYPE')
			.', '.$this->DB->quoteIdentifier('ESMODULE_URI')
			.', '.$this->DB->quoteIdentifier('ESMODULE_CONF')
			.', '.$this->DB->quoteIdentifier('ESMODULE_DISPATCHER_URI')
			.', '.$this->DB->quoteIdentifier('ESMODULE_TMP_FILEPATH')
			.' from '.$this->DB->quoteIdentifier('ESMODULE')
			.' where'
			.'  '.$this->DB->quoteIdentifier('ESMODULE_NAME').' = ' . $this->DB->quote($this->ESMODULE_NAME, 'text')
		, DBMC_FETCHMODE_ASSOC);

		if (Plattform::isError($ret))
		{
			return false;
		}

		if (empty($ret))
		{
			return false;
		}

		$this->ESMODULE_ID   = $ret['ESMODULE_ID'];
		$this->ESMODULE_DESC = $ret['ESMODULE_DESC'];
		$this->ESMODULE_TYPE = $ret['ESMODULE_TYPE'];
		$this->ESMODULE_URI  = $ret['ESMODULE_URI'];
		$this->ESMODULE_DISPATCHER_URI = $ret['ESMODULE_DISPATCHER_URI'];
		$this->ESMODULE_TMP_FILEPATH   = $ret['ESMODULE_TMP_FILEPATH'];
		$this->ESMODULE_CONF = $ret['ESMODULE_CONF'];

		return true;
	}

	/**
	 *
	 * @param string $ESMODULE_NAME
	 *
	 * @param bool
	 */
	final public function setName($ESMODULE_NAME)
	{
		$this->ESMODULE_NAME = $ESMODULE_NAME;
		return $this->loadModuleData();
	}

	/**
	 *
	 */
	final public function setModuleByMimetype($p_mimetype)
	{
		// use only MIME's "type/subtype" specification, skip optional parameters
		$MimeTypeParts = explode(';', $p_mimetype);
		$MimeType = $MimeTypeParts[0];

		$sel   =  'SELECT';
			$sel  .= ' '.$this->DB->quoteIdentifier('ESMODULE_NAME');
			$sel  .= ' FROM '.$this->DB->quoteIdentifier('ESMODULE').', '.$this->DB->quoteIdentifier('REL_ESMODULE_MIMETYPE');
			$sel  .= ' WHERE '.$this->DB->quoteIdentifier('ESMODULE_ID').' = '.$this->DB->quoteIdentifier('REL_ESMODULE_MIMETYPE_ESMODULE_ID');
			$sel  .= ' AND '.$this->DB->quoteIdentifier('REL_ESMODULE_MIMETYPE_TYPE').' = '.$this->DB->quote($MimeType, 'text');

		$ret = $this->DB->getRow($sel, DBMC_FETCHMODE_ASSOC);

		if ( Plattform::isError($ret) )
		{
			return false;
		}

		if ( empty($ret) )
		{
			return false;
		}

		$ESMODULE_NAME = $ret['ESMODULE_NAME'];
		if ( empty($ESMODULE_NAME) )
		{
			throw new Exception('Cannot load module having empty module-name.');
		}

		$this->ESMODULE_NAME = $ESMODULE_NAME;

		return true;
	}

	/**
	 *
	 * @param string $p_RESOURCE_TYPE
	 * @param string $p_RESOURCE_VERSION
	 *
	 * @return bool
	 */
	final public function setModuleByResource($p_RESOURCE_TYPE, $p_RESOURCE_VERSION)
	{
		switch (true)
		{
			case ($p_RESOURCE_TYPE == 'ADL SCORM' && $p_RESOURCE_VERSION == '1.2') :
				$this->ESMODULE_NAME = 'scorm12';
				break;

			case ($p_RESOURCE_TYPE == 'imsqti' && $p_RESOURCE_VERSION == 'xmlv2p1') :
				$this->ESMODULE_NAME = 'qti21';
				break;

			case ($p_RESOURCE_TYPE == 'moodle' && substr($p_RESOURCE_VERSION, 0, 1) == '1') :
				$this->ESMODULE_NAME = 'moodle';
				break;

			case ($p_RESOURCE_TYPE == 'moodle' && substr($p_RESOURCE_VERSION, 0, 1) == '2') :
				$this->ESMODULE_NAME = 'moodle2';
                $this->ESMODULE_TMP_FILEPATH = 'modules/moodle2/files';
				break;

			case ($p_RESOURCE_TYPE == 'Edu-Sharing Scenario' && substr($p_RESOURCE_VERSION, 0, 1) == '1') :
				$this->ESMODULE_NAME = 'scenario';
				break;

			case ($p_RESOURCE_TYPE == 'eduapplet' && substr($p_RESOURCE_VERSION,0,1) == '1') :
				$this->ESMODULE_NAME = 'applet';
				break;

			case ($p_RESOURCE_TYPE == 'eduhtml' && substr($p_RESOURCE_VERSION,0,1) == '1') :
				$this->ESMODULE_NAME = 'html';
				break;

			default :
				error_log('Could not set module by resource-type/-version.');
				return false;
		}

		error_log('Using module "'.$this->ESMODULE_NAME.'" from resource-type/-version.');

		return true;
	}


	/**
	 *
	 */
	final public function setCache()
	{
		return true;
	}


	/**
	 *
	 */
	final public function getName()
	{
		return $this->ESMODULE_NAME;
	}

	/**
	 *
	 */
	final public function getModuleId()
	{
		return $this->ESMODULE_ID;
	}

	/**
	 *
	 */
	final public function getTmpFilepath()
	{
		return $this->ESMODULE_TMP_FILEPATH;
	}

	/**
	 *
	 */
	final public function getConf($p_param, $p_default = null)
	{
		if (empty($this->ESMODULE_CONF))
		{
			return $p_default;
		}

		if (empty($this->parsed_conf))
		{
			$this->parsed_conf = simplexml_load_string($this->ESMODULE_CONF);
		}

		if (property_exists($this->parsed_conf, $p_param))
		{
			return $this->parsed_conf->{$p_param};
		}

		return $p_default;
	}



}

