<?php
/**
 * This product Copyright 2010 metaVentis GmbH.  For detailed notice,
 * see the "NOTICE" file with this distribution.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

include_once "Plattform.php";
include_once "ESModule.php";

/**
 * handles
 *
 * @author steffen gross / matthias hupfer
 * @version 1.0
 * @package core
 * @subpackage classes.new
 */
class ESObject
extends Plattform {

    /**
     * License-constants
     *
     * @var string
     */
    const COMMON_LICENSE_CUSTOM = 'CUSTOM';
    const COMMON_LICENSE_EDU_NC = 'EDU_NC';
    const COMMON_LICENSE_EDU_NC_ND = 'EDU_NC_ND';
    const COMMON_LICENSE_EDU_P_NR = 'EDU_P_NR';
    const COMMON_LICENSE_EDU_P_NR_ND = 'EDU_P_NR';

    const CC_BY = 'CC_BY';
    const CC_BY_NC = 'CC_BY_NC';
    const CC_BY_NC_ND = 'CC_BY_NC_ND';
    const CC_BY_NC_SA = 'CC_BY_NC_SA';
    const CC_BY_ND = 'CC_BY_ND';
    const CC_BY_SA = 'CC_BY_SA';

    /**
     * Conversion-constants
     *
     */
    const CONVERSION_STATUS_WAIT = 'CONVERSION_STATUS_WAIT';
    const CONVERSION_STATUS_PROCESSING = 'CONVERSION_STATUS_PROCESSING';
    const CONVERSION_STATUS_PROCESSED = 'CONVERSION_STATUS_PROCESSED';

    /**
     *
     * @var int
     */
    protected $ESOBJECT_ID = null;

    /**
     *
     * @var string
     */
    protected $ESOBJECT_ESAPPLICATION_ID = null;

    /**
     *
     * @var int
     */
    protected $ESOBJECT_ESMODULE_ID = null;

    /**
     *
     * @var string
     */
    protected $ESOBJECT_TITLE = null;

    /**
     *
     * @var string
     */
    protected $ESOBJECT_REP_ID = null;

    /**
     *
     * @var string
     */
    protected $ESOBJECT_OBJECT_ID = '';

    /**
     *
     * @var string
     */
    protected $ESOBJECT_OBJECT_VERSION = null;

    /**
     *
     * @var string
     */
    protected $ESOBJECT_VERSIONED_OBJECT_ID = null;

    /**
     *
     * @var
     */
    protected $ESOBJECT_MIMETYPE = '';

    /**
     *
     * @var int
     */
    protected $ESOBJECT_TIMESTAMP = 0;

    /**
     *
     * @var string
     */
    protected $ESOBJECT_PATH = null;

    /**
     * @var the remote's lms-identifier, e.g. 'alf' or 'moodle197'
     */
    protected $ESOBJECT_LMS_ID = '';

    /**
     * @var the remote's lms course-identifier this resource belongs to
     */
    protected $ESOBJECT_COURSE_ID = '';

    /**
     * Unique resource-identifier from used LMS.
     * @var int
     */
    protected $ESOBJECT_RESOURCE_ID = 0;

    /**
     *
     * @var string
     */
    protected $ESOBJECT_RESOURCE_TYPE;

    /**
     *
     * @var string
     */
    protected $ESOBJECT_RESOURCE_VERSION;

    /**
     *
     * @var string
     */
    protected $ESOBJECT_PREVIEW_PATH;

    /**
     *
     * @var string
     */
    protected $ESOBJECT_FILE_PATH;

    /**
     *
     * @var string
     */
    protected $ESOBJECT_ALF_TIMESTAMP;

    /**
     *
     * @var string
     */
    protected $ESOBJECT_ALF_FILENAME;

    /**
     *
     * @var string
     */
    protected $ESOBJECT_LICENSE = '';

    /**
     *
     * @var string
     */
    protected $ESOBJECT_LICENSE_AUTHOR = '';

    /**
     * @var ESModule
     */
    public $ESModule = null;

    /**
     *
     * @var Node
     */
    public $AlfrescoNode = null;

    /**
     *
     * @param string $ObjectId The initial object-id (from SpacesStore)
     */
    public function __construct($ObjectId, $ObjectVersion = null) {
        parent::__construct();

        $this -> ESOBJECT_ID = 0;
        $this -> ESOBJECT_ESAPPLICATION_ID = 0;
        $this -> ESOBJECT_ESMODULE_ID = 0;
        $this -> ESOBJECT_TITLE = '';
        $this -> ESOBJECT_REP_ID = '';
        $this -> ESOBJECT_OBJECT_ID = $ObjectId;
        $this -> ESOBJECT_OBJECT_VERSION = $ObjectVersion;
        $this -> ESOBJECT_MIMETYPE = '';
        $this -> ESOBJECT_TIMESTAMP = '';
        $this -> ESOBJECT_PATH = '';
        $this -> ESOBJECT_RESOURCE_ID = 0;
        $this -> ESOBJECT_RESOURCE_TYPE = '';
        $this -> ESOBJECT_RESOURCE_VERSION = '';
        $this -> ESOBJECT_PREVIEW_PATH = '';
        $this -> ESOBJECT_FILE_PATH = '';
        $this -> ESOBJECT_ALF_TIMESTAMP = null;
        $this -> ESOBJECT_ALF_FILENAME = '';

        $this -> ESModule = new ESModule();

        return true;
    }

    /**
     * Cleanup.
     *
     */
    public function __destruct() {
        $this -> AlfrescoNode = null;
        $this -> ESModule = null;
        $this -> Logger = null;
    }

    /**
     *
     */
    public function __get($name) {
        if (!property_exists($this, $name)) {
            throw new Exception('Accessing non-existing property "' . $name . '".');
        }

        return $this -> $name;
    }

    /**
     * Get full name from vCard-string.
     *
     * @param string $vcard
     *
     * @return string
     */
    protected function getFNfromVcard($vcard) {
        $regex = '/\nFN:(?<fn>[^\n]+)\n?/iu';

        $matches = array();
        if (!preg_match($regex, $vcard, $matches)) {
            return false;
        }

        return $matches['fn'];
    }

    /**
     * Removes this object from database.
     *
     * @return bool
     */
    final public function deleteFromDb() {
        $Sql = 'delete from ' . $this -> DB -> quoteIdentifier('ESOBJECT') . ' where ' . $this -> DB -> quoteIdentifier('ESOBJECT_ID') . ' = ' . $this -> DB -> quote((int)$this -> ESOBJECT_ID, 'integer') . ' and ' . $this -> DB -> quoteIdentifier('ESOBJECT_REP_ID') . ' = ' . $this -> DB -> quote($this -> ESOBJECT_REP_ID, 'text');

        if (!empty($this -> ESOBJECT_LMS_ID)) {
            $Sql .= ' and ' . $this -> DB -> quoteIdentifier('ESOBJECT_LMS_ID') . ' = ' . $this -> DB -> quote($this -> ESOBJECT_LMS_ID, 'text');
            $Sql .= ' and ' . $this -> DB -> quoteIdentifier('ESOBJECT_COURSE_ID') . ' = ' . $this -> DB -> quote($this -> ESOBJECT_COURSE_ID, 'text');
        }

        if (!empty($this -> ESOBJECT_OBJECT_VERSION)) {
            $Sql .= ' and ' . $this -> DB -> quoteIdentifier('ESOBJECT_OBJECT_VERSION') . ' = ' . $this -> DB -> quote((int)$this -> ESOBJECT_OBJECT_VERSION, 'text');
        }

        $res = $this -> DB -> query($Sql);

        if (Plattform::isError($res)) {
            return false;
        }

        return true;
    }

    /**
     *
     * @return bool
     */
    final public function readObjectData($p_rep_id, $p_lms_id, $p_course_id = '', $p_resource_id = '', $requiredInstanceVersion = '') {
        $Sql = 'select' . '  ' . $this -> DB -> quoteIdentifier('ESOBJECT_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_ESAPPLICATION_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_ESMODULE_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_TITLE') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_REP_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_OBJECT_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_OBJECT_VERSION') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_VERSIONED_OBJECT_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_MIMETYPE') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_LMS_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_COURSE_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_RESOURCE_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_RESOURCE_TYPE') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_RESOURCE_VERSION') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_TIMESTAMP') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_PATH') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_PREVIEW_PATH') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_FILE_PATH') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_ALF_TIMESTAMP') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_ALF_FILENAME') . ' from ' . $this -> DB -> quoteIdentifier('ESOBJECT') . ' where' . ' ' . $this -> DB -> quoteIdentifier('ESOBJECT_OBJECT_ID') . ' = ' . $this -> DB -> quote($this -> getObjectId(), 'text') . ' and ' . $this -> DB -> quoteIdentifier('ESOBJECT_REP_ID') . ' = ' . $this -> DB -> quote($p_rep_id, 'text');

        if (!empty($requiredInstanceVersion)) {
            $Sql .= ' AND ' . $this -> DB -> quoteIdentifier('ESOBJECT_OBJECT_VERSION') . ' = ' . $this -> DB -> quote($requiredInstanceVersion, 'text');
        } else {
            $Sql .= ' AND ' . $this -> DB -> quoteIdentifier('ESOBJECT_OBJECT_VERSION') . ' IS NULL';
        }

        /*
         * if course-id given, a resource-id must be provided as the same object
         * might exist twice in the same course.
         *
         * if a resource-id is given but no course-id specified, its just an
         * invalid request as repositories, which don't submit a course-id shall
         * not request a resource-id
         */
        /*
         if ( ( ! empty($p_course_id) ) OR ( ! empty($p_resource_id) ) )
         {
         if ( empty($p_course_id) OR empty($p_resource_id) )
         {
         error_log('Invalid state: Course- or resource-identifier specified, but not both of them.');
         throw new Exception('Required resource- or course-identifier NOT found.');
         }

         $Sql .= ' and '.$this->DB->quoteIdentifier('ESOBJECT_LMS_ID').' = '.$this->DB->quote($p_lms_id, 'text');
         $Sql .= ' and '.$this->DB->quoteIdentifier('ESOBJECT_COURSE_ID').' = '.$this->DB->quote($p_course_id, 'text');
         $Sql .= ' and '.$this->DB->quoteIdentifier('ESOBJECT_RESOURCE_ID').' = '.$this->DB->quote($p_resource_id, 'text');
         }
         */

        switch( $this->DB->getPhptype() ) {
            case 'mysql' :
                $Sql .= ' LIMIT 0, 1';
                break;
            case 'pgsql' :
                $Sql .= ' LIMIT 1 OFFSET 0';
                break;

            default :
                throw new Exception('Unhandled db-driver.');
        }

        $res = $this -> DB -> query($Sql);
        if (Plattform::isError($res)) {
            return false;
        }

        $row = $res -> fetchRow(DBMC_FETCHMODE_ASSOC);
        if (!$row) {
            return false;
        }

        $this -> setInstanceData($row);

        return true;
    }

    public function setInstanceData(array $row) {
        $this -> ESOBJECT_ID = $row['ESOBJECT_ID'];
        $this -> ESOBJECT_ESAPPLICATION_ID = $row['ESOBJECT_ESAPPLICATION_ID'];
        $this -> ESOBJECT_ESMODULE_ID = $row['ESOBJECT_ESMODULE_ID'];
        $this -> ESOBJECT_TITLE = $row['ESOBJECT_TITLE'];
        $this -> ESOBJECT_REP_ID = $row['ESOBJECT_REP_ID'];
        $this -> ESOBJECT_OBJECT_ID = $row['ESOBJECT_OBJECT_ID'];
        $this -> ESOBJECT_OBJECT_VERSION = $row['ESOBJECT_OBJECT_VERSION'];
        $this -> ESOBJECT_VERSIONED_OBJECT_ID = $row['ESOBJECT_VERSIONED_OBJECT_ID'];
        $this -> ESOBJECT_MIMETYPE = $row['ESOBJECT_MIMETYPE'];
        $this -> ESOBJECT_TIMESTAMP = $row['ESOBJECT_TIMESTAMP'];
        $this -> ESOBJECT_PATH = $row['ESOBJECT_PATH'];
        $this -> ESOBJECT_LMS_ID = $row['ESOBJECT_LMS_ID'];
        $this -> ESOBJECT_COURSE_ID = $row['ESOBJECT_COURSE_ID'];
        $this -> ESOBJECT_RESOURCE_ID = $row['ESOBJECT_RESOURCE_ID'];
        $this -> ESOBJECT_RESOURCE_TYPE = $row['ESOBJECT_RESOURCE_TYPE'];
        $this -> ESOBJECT_RESOURCE_VERSION = $row['ESOBJECT_RESOURCE_VERSION'];
        $this -> ESOBJECT_PREVIEW_PATH = $row['ESOBJECT_PREVIEW_PATH'];
        $this -> ESOBJECT_FILE_PATH = $row['ESOBJECT_FILE_PATH'];
        $this -> ESOBJECT_ALF_TIMESTAMP = $row['ESOBJECT_ALF_TIMESTAMP'];
        $this -> ESOBJECT_ALF_FILENAME = $row['ESOBJECT_ALF_FILENAME'];

        $this -> ESModule -> setModuleID($this -> ESOBJECT_ESMODULE_ID);

        return $this;
    }

    /**
     *
     */
    final public function setCache() {
        return true;
    }

    /**
     * @param Node $p_node
     */
    final public function setAlfrescoNode(Node $p_node) {
        $this -> AlfrescoNode = $p_node;
    }

    /**
     * Get object's mime-type.
     *
     * @return string
     */
    final public function getMimeType() {
        return $this -> ESOBJECT_MIMETYPE;
    }

    /**
     * Get object's resource-type.
     *
     * @return string
     */
    final public function getResourceType() {
        return $this -> ESOBJECT_RESOURCE_TYPE;
    }

    /**
     * Get object's resource-version.
     *
     * @return string
     */
    final public function getResourceVersion() {
        return $this -> ESOBJECT_RESOURCE_VERSION;
    }

    /**
     *
     */
    final public function getTitle() {
        return $this -> ESOBJECT_TITLE;
    }

    /**
     * Get object's title.
     *
     * @return string
     */
    final public function getFilename() {
        return $this -> ESOBJECT_ALF_FILENAME;
    }

    /**
     *
     */
    final public function getPath() {
        return MC_ROOT_URI . $this -> ESModule -> getTmpFilepath() . '/' . $this -> getSubUri() . '/' . $this -> getObjectIdVersion();
    }

    /**
     *
     */
    final public function getFilePath()// deprecated
    {
        return CC_RENDER_PATH . DIRECTORY_SEPARATOR . $this -> ESModule -> getName() . DIRECTORY_SEPARATOR . $this -> getSubPath() . DIRECTORY_SEPARATOR . $this -> getObjectIdVersion();
    }

    /**
     *
     * @return string
     */
    final public function getEsobjectFilePath() {
        return $this -> ESOBJECT_FILE_PATH;
    }

    /**
     *
     */
    final public function setFilePath($path)// deprecated
    {
        $this -> ESOBJECT_FILE_PATH = $path;
    }

    /**
     *
     */
    final public function getSubUri() {
        return $this -> ESOBJECT_PATH;
    }

    /**
     *
     */
    final public function setSubUri($sub_uri) {
        $this -> ESOBJECT_PATH = $sub_uri;
    }

    /**
     *
     */
    final public function getSubPath() {
        return str_replace('/', DIRECTORY_SEPARATOR, $this -> ESOBJECT_PATH);
    }

    /**
     *
     */
    final public function getPreviewPath() {
        return $this -> ESOBJECT_PATH;
    }

    /**
     *
     */
    final public function getObjectID() {
        return $this -> ESOBJECT_OBJECT_ID;
    }

    /**
     *
     */
    final public function getModuleID() {
        return $this -> ESOBJECT_ESMODULE_ID;
    }

    /**
     *
     */
    final public function getObjectVersion() {
        return $this -> ESOBJECT_OBJECT_VERSION;
    }

    /**
     *
     */
    final public function getVersionedObjectId() {
        return $this -> ESOBJECT_VERSIONED_OBJECT_ID;
    }

    /**
     *
     */
    final public function getObjectIdVersion() {
        return $this -> ESOBJECT_OBJECT_ID . $this -> ESOBJECT_OBJECT_VERSION;
    }

    /**
     *
     */
    final public function getModule() {
        include_once (dirname(__FILE__) . '/ESModule.php');
        $obj_module = new ESModule($this -> getModuleID());
        return $obj_module;
    }

    /**
     *
     * @return string
     */
    public function getLicense() {
        return $this -> ESOBJECT_LICENSE;
    }

    /**
     *
     * @return string
     */
    public function getLicenseAuthor() {
        return $this -> ESOBJECT_LICENSE_AUTHOR;
    }

    /**
     * Set the appropriate module for this object.
     *
     * @return bool
     */
    public function setModule() {
        // runtime sanity
        if (empty($this -> AlfrescoNode)) {
            throw new Exception('No Alfresco-properties set.');
        }

        if ($this -> AlfrescoNode -> properties['{http://www.campuscontent.de/model/1.0}remoterepositorytype'] == 'YOUTUBE') {

            error_log('Property {http://www.campuscontent.de/model/1.0}remoterepositorytype equals "YOUTUBE", using module "url".');

            $this -> ESModule -> setName('url');
        }

        if ($this -> AlfrescoNode -> properties['{http://www.campuscontent.de/model/1.0}replicationsource'] == 'oai:dmglib.org') {

            error_log('Property {http://www.campuscontent.de/model/1.0}replicationsource equals "oai:dmglib.org", using module "url".');

            $this -> ESModule -> setName('url');
        }

        // load appropriate module
        if (!empty($this -> AlfrescoNode -> properties['{http://www.campuscontent.de/model/1.0}wwwurl'])) {
            error_log('Property {http://www.campuscontent.de/model/1.0}wwwurl found, using module "url".');

            $this -> ESModule -> setName('url');
        } else if ($this -> ESOBJECT_MIMETYPE == 'application/zip') {
            if (!$this -> ESModule -> setModuleByResource($this -> ESOBJECT_RESOURCE_TYPE, $this -> ESOBJECT_RESOURCE_VERSION)) {
                error_log('Could not set module by resource-type/-version, using default ("doc") module.');

                // return false;
                $this -> ESModule -> setName('doc');
            }
        } else {
            if (!$this -> ESModule -> setModuleByMimetype($this -> ESOBJECT_MIMETYPE)) {
                error_log('Could not select module by mime-type.');

                return false;
            }
        }

        $this -> ESModule -> loadModuleData();

        $this -> ESOBJECT_ESMODULE_ID = $this -> ESModule -> getModuleId();

        return true;
    }

    /**
     *
     */
    final public function setData($p_DataArray) {
        if (!empty($p_DataArray['ESOBJECT_ESAPPLICATION_ID'])) {
            $this -> ESOBJECT_ESAPPLICATION_ID = $p_DataArray['ESOBJECT_ESAPPLICATION_ID'];
        }

        if (!empty($p_DataArray['ESOBJECT_ESMODULE_ID'])) {
            $this -> ESOBJECT_ESMODULE_ID = $p_DataArray['ESOBJECT_ESMODULE_ID'];
        }

        if (!empty($p_DataArray['ESOBJECT_TITLE'])) {
            $this -> ESOBJECT_TITLE = $p_DataArray['ESOBJECT_TITLE'];
        }
        if (!empty($p_DataArray['ESOBJECT_REP_ID'])) {
            $this -> ESOBJECT_REP_ID = $p_DataArray['ESOBJECT_REP_ID'];
        }
        if (!empty($p_DataArray['ESOBJECT_MIMETYPE'])) {
            $this -> ESOBJECT_MIMETYPE = $p_DataArray['ESOBJECT_MIMETYPE'];
        }
        if (!empty($p_DataArray['ESOBJECT_TIMESTAMP'])) {
            $this -> ESOBJECT_TIMESTAMP = $p_DataArray['ESOBJECT_TIMESTAMP'];
        }
        if (!empty($p_DataArray['ESOBJECT_PATH'])) {
            $this -> ESOBJECT_PATH = $p_DataArray['ESOBJECT_PATH'];
        }
        if (!empty($p_DataArray['ESOBJECT_LMS_ID'])) {
            $this -> ESOBJECT_LMS_ID = $p_DataArray['ESOBJECT_LMS_ID'];
        }
        if (!empty($p_DataArray['ESOBJECT_COURSE_ID'])) {
            $this -> ESOBJECT_COURSE_ID = $p_DataArray['ESOBJECT_COURSE_ID'];
        }
        if (!empty($p_DataArray['ESOBJECT_RESOURCE_ID'])) {
            $this -> ESOBJECT_RESOURCE_ID = $p_DataArray['ESOBJECT_RESOURCE_ID'];
        }
        if (!empty($p_DataArray['ESOBJECT_RESOURCE_TYPE'])) {
            $this -> ESOBJECT_RESOURCE_TYPE = $p_DataArray['ESOBJECT_RESOURCE_TYPE'];
        }
        if (!empty($p_DataArray['ESOBJECT_RESOURCE_VERSION'])) {
            $this -> ESOBJECT_RESOURCE_VERSION = $p_DataArray['ESOBJECT_RESOURCE_VERSION'];
        }
        if (!empty($p_DataArray['ESOBJECT_PREVIEW_PATH'])) {
            $this -> ESOBJECT_PREVIEW_PATH = $p_DataArray['ESOBJECT_PREVIEW_PATH'];
        }
        if (!empty($p_DataArray['ESOBJECT_FILE_PATH'])) {
            $this -> ESOBJECT_FILE_PATH = $p_DataArray['ESOBJECT_FILE_PATH'];
        }
        if (!empty($p_DataArray['ESOBJECT_ALF_TIMESTAMP'])) {
            $this -> ESOBJECT_ALF_TIMESTAMP = $p_DataArray['ESOBJECT_ALF_TIMESTAMP'];
        }
        if (!empty($p_DataArray['ESOBJECT_ALF_FILENAME'])) {
            $this -> ESOBJECT_ALF_FILENAME = $p_DataArray['ESOBJECT_ALF_FILENAME'];
        }

        return true;
    }

    /**
     *
     * @param Node $p_node
     */
    final public function setDataByNode(Node $p_node) {
        $this -> ESOBJECT_ESAPPLICATION_ID = 1;
        $this -> ESOBJECT_RESOURCE_TYPE = '';
        $this -> ESOBJECT_RESOURCE_VERSION = '';

        $this -> AlfrescoNode = $p_node;

        if (!empty($p_node -> properties['{http://www.campuscontent.de/model/lom/1.0}title'])) {
            $this -> ESOBJECT_TITLE = $p_node -> properties['{http://www.campuscontent.de/model/lom/1.0}title'];
        }

        // not all objects have mime-types
        if (!empty($p_node -> properties['{http://www.campuscontent.de/model/lom/1.0}format'])) {
            $this -> ESOBJECT_MIMETYPE = $p_node -> properties['{http://www.campuscontent.de/model/lom/1.0}format'];
        }

        if (empty($p_node -> properties['{http://www.alfresco.org/model/content/1.0}modified'])) {
            throw new Exception('No timestamp found in properties.');
        }

        $this -> ESOBJECT_TIMESTAMP = $p_node -> properties['{http://www.alfresco.org/model/content/1.0}modified'];
        $this -> ESOBJECT_ALF_TIMESTAMP = $p_node -> properties['{http://www.alfresco.org/model/content/1.0}modified'];

        if (!empty($p_node -> properties['{http://www.alfresco.org/model/content/1.0}name'])) {
            $this -> ESOBJECT_ALF_FILENAME = $p_node -> properties['{http://www.alfresco.org/model/content/1.0}name'];
        }

        if (!empty($p_node -> properties['{http://www.campuscontent.de/model/1.0}ccressourcetype'])) {
            $this -> ESOBJECT_RESOURCE_TYPE = $p_node -> properties['{http://www.campuscontent.de/model/1.0}ccressourcetype'];

            if ($this -> ESOBJECT_RESOURCE_TYPE == 'imsqti') {
                $this -> ESOBJECT_MIMETYPE = 'application/zip';
            }
        }

        if (!empty($p_node -> properties['{http://www.campuscontent.de/model/1.0}ccressourceversion'])) {
            $this -> ESOBJECT_RESOURCE_VERSION = $p_node -> properties['{http://www.campuscontent.de/model/1.0}ccressourceversion'];
        }

        // license-handling
        $this -> ESOBJECT_LICENSE = null;
        if (!empty($p_node -> properties['{http://www.campuscontent.de/model/1.0}commonlicense_key'])) {
            if (!empty($p_node -> properties['{http://www.campuscontent.de/model/1.0}lifecyclecontributer_author'])) {
                $name = $this -> getFNfromVcard($p_node -> properties['{http://www.campuscontent.de/model/1.0}lifecyclecontributer_author']);
                if ($name) {
                    $this -> ESOBJECT_LICENSE_AUTHOR = $name;
                }
            }

            // convert to image + link??? irgendwie schon
            $license = $p_node -> properties['{http://www.campuscontent.de/model/1.0}commonlicense_key'];

            $factory = new ESRender_LicenseFactory_Implementation();
            $this -> ESOBJECT_LICENSE = $factory -> getLicense($license, $this -> ESOBJECT_LICENSE_AUTHOR);
        }

        return true;
    }

    final public function setData2Db() {
        $Sql = 'insert into ' . $this -> DB -> quoteIdentifier('ESOBJECT') . ' (' . $this -> DB -> quoteIdentifier('ESOBJECT_ESAPPLICATION_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_ESMODULE_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_TITLE') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_REP_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_OBJECT_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_OBJECT_VERSION') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_MIMETYPE') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_TIMESTAMP') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_LMS_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_COURSE_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_RESOURCE_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_RESOURCE_TYPE') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_RESOURCE_VERSION') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_PATH') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_PREVIEW_PATH') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_FILE_PATH') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_ALF_TIMESTAMP') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_ALF_FILENAME') . ') values (' . $this -> DB -> quote(intval($this -> ESOBJECT_ESAPPLICATION_ID), 'integer') . ', ' . $this -> DB -> quote(intval($this -> ESOBJECT_ESMODULE_ID), 'integer') . ', ' . $this -> DB -> quote($this -> ESOBJECT_TITLE, 'text') . ', ' . $this -> DB -> quote($this -> ESOBJECT_REP_ID, 'text') . ', ' . $this -> DB -> quote($this -> ESOBJECT_OBJECT_ID, 'text') . ', ' . $this -> DB -> quote($this -> ESOBJECT_OBJECT_VERSION, 'text') . ', ' . $this -> DB -> quote($this -> ESOBJECT_MIMETYPE, 'text') . ', ' . $this -> DB -> quote(date('Y-m-d H:i:s', strtotime($this -> ESOBJECT_TIMESTAMP)), 'text') . ', ' . $this -> DB -> quote($this -> ESOBJECT_LMS_ID, 'text') . ', ' . $this -> DB -> quote($this -> ESOBJECT_COURSE_ID, 'text') . ', ' . $this -> DB -> quote($this -> ESOBJECT_RESOURCE_ID, 'text') . ', ' . $this -> DB -> quote($this -> ESOBJECT_RESOURCE_TYPE, 'text') . ', ' . $this -> DB -> quote($this -> ESOBJECT_RESOURCE_VERSION, 'text') . ', ' . $this -> DB -> quote($this -> ESOBJECT_PATH, 'text') . ', ' . $this -> DB -> quote($this -> ESOBJECT_PREVIEW_PATH, 'text') . ', ' . $this -> DB -> quote($this -> ESOBJECT_FILE_PATH, 'text') . ', ' . $this -> DB -> quote(date('Y-m-d H:i:s', strtotime($this -> ESOBJECT_ALF_TIMESTAMP)), 'text') . ', ' . $this -> DB -> quote($this -> ESOBJECT_ALF_FILENAME, 'text') . ');';

        $Result = $this -> DB -> query($Sql);
        if (Plattform::isError($Result)) {
            error_log('Error storing object-data in DB.', E_USER_ERROR);
            return false;
        }

        $LastInsertId = $this -> DB -> lastInsertId();
        if (Plattform::isError($LastInsertId)) {
            error_log('Error reading auto-generated id.', E_USER_ERROR);
            return false;
        }

        $this -> ESOBJECT_ID = $LastInsertId;

        return true;
    }

    /**
     * Tell if current object has different timestamp than in repository,
     * which could mean the object isn't available in a new version, but
     * something changed nevertheless.
     *
     * @return bool
     */
    public function timestampsDiffer() {
        if (!$this -> AlfrescoNode) {
            throw new Exception('No alfresco content-node set.');
        }

        if ($this -> ESOBJECT_ALF_TIMESTAMP != $this -> AlfrescoNode -> properties['{http://www.alfresco.org/model/content/1.0}modified']) {
            return true;
        }

        return false;
    }

    public function addToConversionQueue($format, $dirSep, $filename, $outputFilename, $renderPath, $mimeType) {

        $Sql = 'insert into ' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION') . ' (' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION_OBJECT_ID') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION_FORMAT') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION_DIR_SEPERATOR') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION_FILENAME') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION_OUTPUT_FILENAME') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION_RENDER_PATH') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION_TIME') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION_STATUS') . ', ' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION_MIMETYPE') . ') values (' . $this -> DB -> quote($this -> ESOBJECT_ID, 'integer') . ', ' . $this -> DB -> quote($format, 'text') . ', ' . $this -> DB -> quote($dirSep, 'text') . ', ' . $this -> DB -> quote($filename, 'text') . ', ' . $this -> DB -> quote($outputFilename, 'text') . ', ' . $this -> DB -> quote($renderPath, 'text') . ', ' . $this -> DB -> quote(time(), 'integer') . ', ' . $this -> DB -> quote(self::CONVERSION_STATUS_WAIT, 'text') . ', ' . $this -> DB -> quote($mimeType, 'text') . ');';

        $Result = $this -> DB -> query($Sql);
        if (Plattform::isError($Result)) {
            error_log('Error storing conversion entry in DB.', E_USER_ERROR);
            return false;
        }
        return true;
    }

    public function setConversionStateProcessed($objId, $format) {
        $this -> setConversionState($objId, $format, self::CONVERSION_STATUS_PROCESSED);
    }

    public function setConversionStateProcessing($objId, $format) {
        $this -> setConversionState($objId, $format, self::CONVERSION_STATUS_PROCESSING);
    }

    protected function setConversionState($objId, $format, $state) {
        $Sql = 'update ' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION') . ' set ' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION_STATUS') . ' = ' . $this -> DB -> quote($state, 'text') . ' ';
        ' where ' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION_OBJECT_ID') . '= ' . $this -> DB -> quote($objId, 'integer') . ' and ' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION_FORMAT') . ' = ' . $this -> DB -> quote($format, 'text') . ';';
        $Result = $this -> DB -> query($Sql);
        if (Plattform::isError($Result)) {
            error_log('Error setting conversion status ' . $state . ' in DB.', E_USER_ERROR);
            return false;
        }
        return true;
    }

    public function inConversionQueue($format) {

        $Sql = 'select * from ' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION') . ' where ' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION_OBJECT_ID') . '= ' . $this -> DB -> quote($this -> ESOBJECT_ID, 'integer') . ' and ' . $this -> DB -> quoteIdentifier('ESOBJECT_CONVERSION_FORMAT') . ' = ' . $this -> DB -> quote($format, 'text');
        switch($this->DB->getPhptype()) {
            case 'mysql' :
                $Sql .= ' LIMIT 0, 1';
                break;
            case 'pgsql' :
                $Sql .= ' LIMIT 1 OFFSET 0';
                break;

            default :
                throw new Exception('Unhandled db-driver.');
        }

        $res = $this -> DB -> query($Sql);
        if (Plattform::isError($res)) {
            return false;
        }

        $row = $res -> fetchRow(DBMC_FETCHMODE_ASSOC);
        if (!$row) {
            return false;
        }
        return true;
    }

    final public function getPathfile() {
        return MC_ROOT_URI . 'modules/' . $this -> ESModule -> getName() . '/files/' . $this -> getSubUri_file() . '/' . $this -> getObjectIdVersion();
    }

    final public function getSubUri_file() {
        return $this -> ESOBJECT_FILE_PATH;
    }

}
