<?php
/**
 * This product Copyright 2010 metaVentis GmbH.  For detailed notice,
 * see the "NOTICE" file with this distribution.
 * 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */


/**
 * handles the current  session
 *
 * @author steffen gross / matthias hupfer

 */
class mc_Session
{
	protected $name;

	protected $id;

	/**
	 *
	 */
	public function __construct($p_name = null, $p_id = null)
	{
		if (empty($p_name))
		{
			$p_name = session_name();
		}

		if (empty($p_name))
		{
			mc_Debug::error($p_name, 'session name is not available');
			false;
		}

		if (empty($p_id))
		{
			$p_id = session_id();
		}

		if (empty($p_id))
		{
			mc_Debug::error($p_id, 'session id is not available');
			false;
		}

		$this->name = $p_name;
		$this->id   = $p_id;

		return true;
	} // end constructor



	/**
	 *
	 */
	final public function getName()
	{
		return $this->name;
	}

	/**
	 *
	 */
	final public function getId()
	{
		return $this->id;
	}



	/**
	 *
	 */
	final public static function start($p_sname, $p_sid = null)
	{
		if (empty($p_sid))
		{
			$p_sid = mc_Request::fetch($p_sname, 'string', null, '/^[0-9a-f]{32}$/', MC_REQUEST_FORM);
		}

		if (empty($p_sid))
		{
			die(mc_Debug::error(null, 'session id for name "'.$p_sname.'" has not been found or is empty'));
		}

		session_name($p_sname);
		session_id($p_sid);
		session_start();

		return true;
	} // end method start



	/**
	 *
	 */
	public static function init($p_user_ident)
	{
		return true;
	} // end method init



	/**
	 *
	 */
	public static function destroy()
	{
		$_SESSION = array();
		session_destroy();
		return true;
	} // end method destroy



	/**
	 *
	 */
	public static function setTemp($p_name, $p_value)
	{
		$_SESSION['TEMP'][$p_name] = $p_value;
		return true;
	} // end method setTemp

	/**
	 *
	 */
	public static function getTemp($p_name, $p_value)
	{
		if (!isset($_SESSION['TEMP'][$p_name]))
		{
			return(mc_Debug::error('unknown temporary parameter "'.$p_name.'"'));
		}
		return $_SESSION['TEMP'][$p_name];
	} // end method getTemp



	/**
	 * environment requests
	 */
	public static function getApp()            { return $_SESSION['APP_IDENT']; }
	public static function getVersion()        { return $_SESSION['MC_VERSION']; }
	public static function getAuthType()       { return $_SESSION['USER_AUTH']['AUTH_TYPE']; }
	public static function getBlockedCols()    { return $_SESSION['USER_AUTH']['BLOCKEDCOLS']; }
//	public static function getCOMPLEMENTFORM()    { return $_SESSION['USER_AUTH']['COMPLEMENTFORM']; } // ???
	public static function getCaller()         { return $_SESSION['CALLED_BY']; }
	public static function getCourseInlist()   { return $_SESSION['COURSE_INLIST']; }
	public static function getCourseRAInlist() { return $_SESSION['COURSE_INLIST_C_ADMIN']; }
	public static function getClosedAccess()   { return $_SESSION['COURSE_INLIST_CLOSED_ACCESS']; }
	public static function getMaxUploadSize()  { return $_SESSION['MAX_MB_UPLOAD_SIZE']; }
	public static function getCookieCheck()    { return (empty($_SESSION['SESSION_CHECK']) ? null : $_SESSION['SESSION_CHECK']); }
	public static function isRestricted()      { return (($_SESSION['RESTRICT'] == 'Y') ? true : false); }



	/**
	 * room requests
	 */
	public static function getRoom()     { return $_SESSION['COURSE_ID']; }
	public static function getTheme()    { return $_SESSION['COURSE_CONF']['COURSE_THEME']; }
	public static function getRoomNode() { return $_SESSION['VT_ROOM_NODE_ID']; }
	public static function getConf($p_name)
	{
		if (!isset($_SESSION['COURSE_CONF'][$p_name]))
		{
			return(mc_Debug::error('unknown room configuration "'.$p_name.'"'));
		}
		return $_SESSION['COURSE_CONF'][$p_name];
	}// end method getConf



	/**
	 * group requests
	 */
	public static function getRoomGroup()      { return $_SESSION['COURSE_GROUP']; }
	public static function getUserGroup()      { return $_SESSION['USER_PROFILE']['GROUP']; }
	public static function getAllUserGroups()  { return $_SESSION['GROUP_INLIST_ALL']; }
	public static function getRoomUserGroups() { return $_SESSION['GROUP_INLIST_COURSE']; }



	/**
	 * user requests
	 */
	public static function getUser()           { return $_SESSION['USER_IDENT']; }
	public static function getSelname()        { return $_SESSION['USER_SELNAME']; }
	public static function getChatname()       { return $_SESSION['CHAT']['NICKNAME']; }
	public static function getAddressId()      { return $_SESSION['USER_PROFILE']['SIMPLADR_ID']; }
	public static function getLang()           { return $_SESSION['USER_PROFILE']['LANGUAGE_ISO']; }
	public static function getLangId()         { return $_SESSION['USER_PROFILE']['LANGUAGE']; }
	public static function getStyle()          { return $_SESSION['USER_PROFILE']['STYLE']; }
	public static function getHomeNode()       { return $_SESSION['USER_PROFILE']['HOMENODE']; }
	public static function getOpenFMNodes()    { return $_SESSION['USER_PROFILE']['OPENNODES']; }
	public static function getOpenCBNodes()    { return $_SESSION['USER_PROFILE']['COPENNODES']; }
	public static function isAdmin()           { return $_SESSION['IS_ADMIN']; }
	public static function isLocalAdmin()      { return $_SESSION['IS_LOCAL_ROOM_ADMIN']; }
	public static function isTeacher()         { return $_SESSION['IS_TEACHER']; }
	public static function isLocalTeacher()    { return $_SESSION['IS_LOCAL_TEACHER']; }



	/**
	 * file manager requests
	 */
	public static function getHtmlCourseNode() { return $_SESSION['VT_HTML_COURSE_NODE_ID']; }
	public static function getVTRootNodes()    { return $_SESSION['VT_ROOT_NODES']; }



	/**
	 *
	 */
	public static function addSid($p_link)
	{
		$l_info = parse_url($p_link);
		if (empty($l_info['query']))
		{
			return $l_link.'?'.SID;
		}

		if (strpos(SID, $l_info['query']) === false)
		{
			return $l_link.'&'.SID;
		}

		return $l_link;
	} // end method addSid



	/**
	 *
	 */
	public static function getFormElement($p_name = null)
	{
		if (empty($p_name)) { $p_name = session_name(); }
		return '<input type="hidden" name="'.$p_name.'" value="'.session_id().'">';
	} // end method getFormElement



} // end class mc_Session

?>