<?php

/**
 * This product Copyright 2010 metaVentis GmbH.  For detailed notice,
 * see the "NOTICE" file with this distribution.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

// load plugin config
require_once(dirname(__FILE__).'/config.php');

include_once ('../../conf.inc.php');
include_once (MC_ROOT_PATH.'func/extern/videotoolkit/phpvideotoolkit.php5.php');
include_once (MC_ROOT_PATH.'func/classes.new/ESRender/Module/PhpVideoToolkit/Helper.php');


if ( ! file_exists(PHPVIDEOTOOLKIT_FFMPEG_BINARY) )
{
	error_log('Configured ffmpeg-binary '.PHPVIDEOTOOLKIT_FFMPEG_BINARY.' not found. Please edit modules/audio/config.php accordingly.');
}

if ( ! file_exists(PHPVIDEOTOOLKIT_FLVTOOLS_BINARY) )
{
	error_log('Configured flvtools-binary '.PHPVIDEOTOOLKIT_FLVTOOLS_BINARY.' not found. Please edit modules/audio/config.php accordingly.');
}

if ( ! file_exists(PHPVIDEOTOOLKIT_MENCODER_BINARY) )
{
	error_log('Configured mencoder-binary '.PHPVIDEOTOOLKIT_MENCODER_BINARY.' not found. Please edit modules/audio/config.php accordingly.');
}

/**
 * handles
 *
 * @author steffen gross / matthias hupfer
 * @version 1.0
 * @package core
 * @subpackage classes.new
 */
class mod_audio
extends ESRender_Module_PhpVideoToolkit_Abstract
{

	protected $filename;

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_PhpVideoToolkit_Abstract::getOutputFilename()
	 */
	protected function getOutputFilename($format = self::FORMAT_FLASH)
	{
		$filename = $this->getCacheFileName();
		switch( $format )
		{
			case self::FORMAT_FLASH:
				$filename .= '.mp3';
				break;
			case self::FORMAT_IPAD:
				$filename .= '.mp3';
				break;
			default:
				throw new Exception('Unhandled format.');
		}

		return $filename;
	}


	protected function prepareRenderData(
		EsApplication $RemoteApplication,
		array $requestData)
	{
		$data = parent::prepareRenderData($RemoteApplication, $requestData);

		$format = $this->formatByUserAgent();

		$object_url = dirname($this->_ESOBJECT->getPath())
			. '/' . basename($this->getOutputFilename($format))
			. '?' . session_name() . '=' . session_id();
		$data['audio_url'] = $object_url;

		$player_url_swf = MC_MODULES_URI . '/audio/audio-player/player.swf';
		$data['player_url_swf'] = $player_url_swf;

		$player_url_js = MC_MODULES_URI . '/audio/audio-player/audio-player.js';
		$data['player_url_js'] = $player_url_js;

		$player_id = 'player_' . md5($object_url . microtime(true));
		$data['player_id'] = $player_id;

		return $data;
	}

	/**
	 * Helper-method to allow re-using inline-templates in inline() and
	 * display().
	 *
	 * @param array $data
	 * @throws Exception
	 */
	protected function renderInlineTemplate(array $data)
	{
		$inline_template = '';
		switch( $this->formatByUserAgent() )
		{
			case self::FORMAT_FLASH:
				$inline_template = '/module/audio/inline';
				break;

			case self::FORMAT_IPAD:
				$inline_template = '/module/audio/inline.ipad';
				break;

			default:
				throw new Exception('Unhandled format.');
		}

		$Template = $this->getTemplate();
		return $Template->render($inline_template, $data);
	}

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_Base::display()
	 */
	final public function display(
		EsApplication $RemoteApplication,
		array $requestData)
	{
	    global $Locale, $ROOT_URI;
		$data = $this->prepareRenderData($RemoteApplication, $requestData);
		//$data['inline'] = $this->renderInlineTemplate($data);
        $data['ajax_url'] = $ROOT_URI . 'application/esmain/index.php?'.'app_id='
            .$requestData['app_id'].'&session='.$requestData['session']
            .'&rep_id='.$requestData['rep_id'].'&obj_id='.$requestData['object_id'].'&resource_id='
            .$requestData['resource_id'].'&course_id='.$requestData['course_id']
            .'&display=inline&language='.$Locale->getLanguageTwoLetters().'&u='.urlencode($requestData['user_name_encr']);
        
		$Template = $this->getTemplate();
		echo $Template->render('/module/audio/display', $data);

		return true;
	}

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_Base::inline()
	 */
	protected function inline(
			EsApplication $RemoteApplication,
			array $requestData)
	{
		$data = $this->prepareRenderData($RemoteApplication, $requestData);

		echo $this->renderInlineTemplate($data);

		return true;
	}
    
        /**
     * (non-PHPdoc)
     * @see ESRender_Module_Base::locked()
     * 
     */
    final public function locked(EsApplication $RemoteApplication, array $requestData) {
        $template = $this->getTemplate();
        $toolkitOutput = MC_ROOT_PATH . 'error/conversion/' . $this -> _ESOBJECT -> getObjectID() . '_' . $this->_ESOBJECT->getObjectVersion() . '.log';
        $progress = ESRender_Module_PhpVideoToolkit_Helper::getConversionProgress($toolkitOutput, $this->formatByUserAgent());
        if(empty($progress) || is_array($progress))
            $progress = '0';
        echo $template->render('/module/audio/lock', array('callback' => $requestData['callback'], 'progress' => $progress));
        return true;
    }


}
