<?php
/**
 * This product Copyright 2013 metaVentis GmbH.  For detailed notice,
 * see the "NOTICE" file with this distribution.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * 
 * 
 * 
 * 
 */

include_once ('../../conf.inc.php');

define('DOCTYPE_PDF', 'DOCTYPE_PDF');
define('DOCTYPE_ODF', 'DOCTYPE_ODF');
define('DOCTYPE_UNKNOWN', 'DOCTYPE_UNKNOWN');

/**
 * This module handles documents of type pdf and odf (the basic odf formats) assigned in db 
 *
 * @author steffen hippeli
 * @version 1.1
 * @package modules
 * @subpackage doc
 */
class mod_doc
extends ESRender_Module_ContentNode_Abstract {

    private $doctype;

    /**
     * Extension: set doctype
     */    
    public function __construct($Name, ESRender_Application_Interface $RenderApplication, ESObject $p_esobject, Logger $Logger, Phools_Template_Interface $Template) {
        parent::__construct($Name, $RenderApplication, $p_esobject, $Logger, $Template);
        $this -> setDoctype();
    }

    protected function renderTemplate(EsApplication $RemoteApplication, array $requestData, $TemplateName) {
        $Logger = $this -> getLogger();
        $template_data = parent::prepareRenderData($RemoteApplication, $requestData);
        $object_url = dirname($this -> _ESOBJECT -> getPath()) . '/' . basename($this -> getOutputFilename()) . '?' . session_name() . '=' . session_id();
        $template_data['path'] = $object_url;
        $Template = $this -> getTemplate();
        $rendered = $Template -> render($TemplateName, $template_data);
        return $rendered;
    }

    protected function getOutputFilename() {
        $Logger = $this -> getLogger();
        $filename = $this -> getCacheFileName();
        $filename = str_replace('\\', '/', $filename);
        return $filename;
    }

    final protected function display(EsApplication $RemoteApplication, array $requestData) {
        $Logger = $this -> getLogger();
        echo $this -> renderTemplate($RemoteApplication, $requestData, $this -> getThemeByDoctype());
        return true;
    }

    /**
     * Determine if current doctype should be rendered or downloaded depending on user agent capabilities
     */
    protected function userAgentSupported() {
        
        
        
        
        
        return false; // until this module is tested appropriate
        
        
        
        
        
        switch($this->getDoctype()) {
            case DOCTYPE_PDF :
                if ($this -> checkPdfUserAgents())
                    return true;
                else
                    return false;
                break;
            case DOCTYPE_ODF :
                return true;
                break;
            default :
                return false;
        }
    }

    /**
     * pdf.js is not supported by every browser
     * At this point we decide which browser should have a try
     */
    protected function checkPdfUserAgents() {
        $ua = $this -> fetchUserAgentInfos();
        switch($ua['platform']) {
            
            case 'windows':
                switch($ua['ub']) {
                    case 'Firefox':
                    case 'Chrome':
                    case 'Opera':
                    case 'MSIE11':
                        return true;
                    case 'MSIE':
                        if($ua['version'] == '10.0' || $ua['version'] == '9.0')
                            return true;
                        else
                            return false;
                    case 'Safari':                
                    default:
                        return false;
                }
            case 'mac':
                //untested
            case 'linux':
                //untested
            default:
                return false;   
        }
    }

    /**
     * Load theme according to current doctype
     */
    protected function getThemeByDoctype() {
        switch($this->getDoctype()) {
            case DOCTYPE_PDF :
                return '/module/doc/pdf/display';
                break;
            case DOCTYPE_ODF :
                return '/module/doc/odf/display';
                break;
            default :
                return '';
        }
    }

    /**
     * Set doctype  - accumulate some odf mimetypes
     */
    protected function setDoctype() {
        if (strpos($this -> _ESOBJECT -> getMimeType(), 'opendocument') !== false) {
            $this -> doctype = DOCTYPE_ODF;
        } else if (strpos($this -> _ESOBJECT -> getMimeType(), 'pdf') !== false) {
            $this -> doctype = DOCTYPE_PDF;
        } else {
            $this -> doctype = DOCTYPE_UNKNOWN;
        }
    }

    /**
     * Doctype getter
     */
    protected function getDoctype() {
        if (!$this -> doctype)
            $this -> setDoctype();
        return $this -> doctype;
    }

    /**
     * (non-PHPdoc)
     * @see ESRender_Module_Interface::process()
     * 
     * Set display mode download if user agent does not support rendering
     * 
     */
    public function process($p_kind, EsApplication $RemoteApplication, array $requestData) {
        $Logger = $this -> getLogger();
        
        $Logger -> debug('Calling Module::download() fix for docs');
        return $this -> download($RemoteApplication, $requestData);
        
        
        switch( strtolower($p_kind) ) {
            case ESRender_Application_Interface::DISPLAY_MODE_DOWNLOAD :
                $Logger -> debug('Calling Module::download()');
                return $this -> download($RemoteApplication, $requestData);
                break;

            case ESRender_Application_Interface::DISPLAY_MODE_INLINE :
                $Logger -> debug('Calling Module::inline()');
                return $this -> inline($RemoteApplication, $requestData);
                break;

            case ESRender_Application_Interface::DISPLAY_MODE_WINDOW :
                $Logger -> debug('Calling Module::display()');
                if ($this -> userAgentSupported())
                    return $this -> display($RemoteApplication, $requestData);
                else
                    return $this -> download($RemoteApplication, $requestData);
                break;
            default :
                throw new Exception('Unhandled display-kind "' . $p_kind . '".');
        }

        return true;
    }

    /**
     * Fetch some information about user agent
     */
    protected function fetchUserAgentInfos() {
        $u_agent = $_SERVER['HTTP_USER_AGENT'];
        $bname = 'Unknown';
        $platform = 'Unknown';
        $version = "";

        //First get the platform?
        if (preg_match('/linux/i', $u_agent)) {
            $platform = 'linux';
        } elseif (preg_match('/macintosh|mac os x/i', $u_agent)) {
            $platform = 'mac';
        } elseif (preg_match('/windows|win32/i', $u_agent)) {
            $platform = 'windows';
        }

        // Next get the name of the useragent yes seperately and for good reason
        if (preg_match('/MSIE/i', $u_agent) && !preg_match('/Opera/i', $u_agent)) {
           // $bname = 'Internet Explorer';
            $ub = "MSIE";
        } else if (preg_match('/Trident\/7.0/i', $u_agent)) {
           // $bname = 'Internet Explorer';
            $ub = "MSIE11";            
        } elseif (preg_match('/Firefox/i', $u_agent)) {
           // $bname = 'Mozilla Firefox';
            $ub = "Firefox";
        } elseif (preg_match('/Chrome/i', $u_agent)) {
           // $bname = 'Google Chrome';
            $ub = "Chrome";
        } elseif (preg_match('/Safari/i', $u_agent)) {
           // $bname = 'Apple Safari';
            $ub = "Safari";
        } elseif (preg_match('/Opera/i', $u_agent)) {
           // $bname = 'Opera';
            $ub = "Opera";
        } elseif (preg_match('/Netscape/i', $u_agent)) {
            //$bname = 'Netscape';
            $ub = "Netscape";
        }

        // finally get the correct version number
        $known = array('Version', $ub, 'other');
        $pattern = '#(?<browser>' . join('|', $known) . ')[/ ]+(?<version>[0-9.|a-zA-Z.]*)#';
        if (!preg_match_all($pattern, $u_agent, $matches)) {
            // we have no matching number just continue
        }

        // see how many we have
        $i = count($matches['browser']);
        if ($i != 1) {
            //we will have two since we are not using 'other' argument yet
            //see if version is before or after the name
            if (strripos($u_agent, "Version") < strripos($u_agent, $ub)) {
                $version = $matches['version'][0];
            } else {
                $version = $matches['version'][1];
            }
        } else {
            $version = $matches['version'][0];
        }

        if ($version == null || $version == "") {$version = "?";
        }

        return array('ub' => $ub, 'version' => $version, 'platform' => $platform);
    }

}
