<?php
/**
 * This product Copyright 2010 metaVentis GmbH.  For detailed notice,
 * see the "NOTICE" file with this distribution.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

require_once(dirname(__FILE__).'/../../conf.inc.php');
require_once(dirname(__FILE__).'/config.php');
if ( ! defined('IMAGEMAGICK_PATH') )
{
	throw new Exception('Missing config for "IMAGEMAGICK_PATH".');
}

require_once(MC_ROOT_PATH.'func/extern/phmagick/phMagick.php');


/**
 * handles
 *
 * @author steffen gross / matthias hupfer
 * @version 1.0
 * @package core
 * @subpackage classes.new
 */
class mod_picture
	extends ESRender_Module_ContentNode_Abstract
{

	/**
	 *
	 * @var int
	 */
	const IMAGE_WIDTH = 800;

	/**
	 *
	 * @var int
	 */
	const THUMBNAIL_WIDTH = 320;

	/**
	 * Detect if image is an animated GIF.
	 *
	 * @param string $filename
	 *
	 * @return bool
	 */
	protected function is_ani($filename)
	{
		$fh = @fopen($filename, 'rb');
		if ( ! $fh )
		{
			return false;
		}

		$count = 0;
		//an animated gif contains multiple "frames", with each frame having a
		//header made up of:
		// * a static 4-byte sequence (\x00\x21\xF9\x04)
		// * 4 variable bytes
		// * a static 2-byte sequence (\x00\x2C)

		// We read through the file til we reach the end of the file, or we've found
		// at least 2 frame headers
		while(!feof($fh) && $count < 2)
		{
			$chunk = fread($fh, 1024 * 100); //read 100kb at a time
			$count += preg_match_all('#\x00\x21\xF9\x04.{4}\x00\x2C#s', $chunk, $matches);
		}

		fclose($fh);

		return $count > 1;
	}

	/**
	 *
	 * @param string $SourceFile
	 * @param string $DestinationFile
	 * @param int $Width
	 * @param int $Height
	 *
	 * @return bool
	 */
	protected function resizeImage($SourceFile, $DestinationFile, $Width, $Height = 0)
	{
		$Logger = $this->getLogger();

		$phMagick = new phMagick($SourceFile, $DestinationFile);
		$phMagick->setImageMagickPath(IMAGEMAGICK_PATH);

		$phMagick->resize($Width, $Height);

		return true;
	}

	/**
	 *
	 * @return string
	 */
	protected function getImageFilename()
	{
		return $this->_ESOBJECT->getFilePath() . '.jpg';
	}

	/**
	 *
	 * @return string
	 */
	protected function getThumbnailFilename()
	{
		return $this->_ESOBJECT->getFilePath() . '_tn.jpg';
	}

	protected function renderTemplate(
		EsApplication $RemoteApplication,
		array $requestData,
		$TemplateName)
	{
		$Logger = $this->getLogger();

		$m_mimeType = $this->_ESOBJECT->getMimeType();
		$m_path = $this->_ESOBJECT->getPath();
		$m_name = $this->_ESOBJECT->getTitle();
		$f_path = $this->_ESOBJECT->getFilePath();

		// resize thumbnail to scale-up previously generated thumbnails
		$this->resizeImage($this->getImageFilename(), $this->getThumbnailFilename(), self::THUMBNAIL_WIDTH);
		$Logger->debug('Created thumbnail "'.$this->getThumbnailFilename().'" from image "'.$this->getImageFilename().'".');

		$imageUrl = $m_path . '.jpg?' . session_name() . '=' . session_id();
		$thumbnailUrl = $m_path . '_tn.jpg?' . session_name() . '=' . session_id();

		$template_data = parent::prepareRenderData($RemoteApplication, $requestData);

		$template_data['title'] = (empty($title) ? $this->_ESOBJECT->getTitle() : $title);
		$template_data['image_url'] = $imageUrl;
		$template_data['thumbnail_url'] = $thumbnailUrl;

		$Template = $this->getTemplate();
		$rendered = $Template->render($TemplateName, $template_data);

		return $rendered;
	}

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_ContentNode_Abstract::createInstance()
	 */
	final public function createInstance(
		EsApplication $RemoteApplication,
		array $requestData)
	{
		if ( ! parent::createInstance($RemoteApplication, $requestData) )
		{
			return false;
		}

		$Logger = $this->getLogger();

		$m_mimeType = $this->_ESOBJECT->getMimeType();
		$m_path = $this->_ESOBJECT->getPath();
		$m_name = $this->_ESOBJECT->getTitle();
		$f_path = $this->_ESOBJECT->getFilePath();

		$ObjectFilename = str_replace('\\', '/', $f_path);

		// extract a single frame from animated GIF
		if ( $this->is_ani($ObjectFilename) )
		{
			$phMagick = new phMagick($ObjectFilename);
			$phMagick->setImageMagickPath(IMAGEMAGICK_PATH);

			// use first image in sequence as input for resize
			$ObjectFilename .= '[0]';
			$phMagick->setDestination($ObjectFilename);
		}

		$this->resizeImage($ObjectFilename, $this->getImageFilename(), self::IMAGE_WIDTH);
		$Logger->debug('Resized picture "'.$ObjectFilename.'" to "'.$this->getImageFilename().'".');

		$this->resizeImage($this->getImageFilename(), $this->getThumbnailFilename(), self::THUMBNAIL_WIDTH);
		$Logger->debug('Created picture-thumbnail "'.$this->getThumbnailFilename().'".');

		return true;
	}

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_ContentNode_Abstract::display()
	 */
	final protected function display(
		EsApplication $RemoteApplication,
		array $requestData)
	{
		$Logger = $this->getLogger();

		echo $this->renderTemplate(
			$RemoteApplication,
			$requestData,
			'/module/picture/display');

		return true;
	}

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_ContentNode_Abstract::inline()
	 */
	protected function inline(
		EsApplication $RemoteApplication,
		array $requestData)
	{
		$Logger = $this->getLogger();

		echo $this->renderTemplate(
			$RemoteApplication,
			$requestData,
			'/module/picture/inline');

		return true;
	}

	/**
	 * (non-PHPdoc)
	 * @see ESRender_Module_Base::getTimesOfUsage()
	 */
	public function getTimesOfUsage()
	{
		return 20;
	}

}

