<?php

/**
 * Module to handle URL's.
 *
 *
 */

define("VIDEO_TOKEN_YOUTUBE", "youtube.com/watch?");
define("VIDEO_TOKEN_VIMEO", "vimeo.com");
define("VIDEO_TOKENS", serialize(array(VIDEO_TOKEN_YOUTUBE, VIDEO_TOKEN_VIMEO)));

class mod_url
extends ESRender_Module_NonContentNode_Abstract {

    /**
     * "Display" the url by presentung a intermediate page announcing the
     * redirect.
     *
     * (non-PHPdoc)
     * @see ESRender_Module_Base::display()
     */
    protected function display(EsApplication $RemoteApplication, array $requestData) {
        if (!$this -> validate()) {
            error_log('URL-property is empty.');
            return false;
        }

        if ($this -> detectVideo())
            $embedding = $this -> getVideoEmbedding();
        else
            $embedding = $this -> getLinkEmbedding();

        $Template = $this -> getTemplate();
        echo $Template -> render('/module/url/display', array('embedding' => $embedding, 'title' => $this->_ESOBJECT->getTitle()));

        return true;
    }

    protected function inline(EsApplication $RemoteApplication, array $requestData) {
        if (!$this -> validate()) {
            error_log('URL-property is empty.');
            return false;
        }

        if ($this -> detectVideo())
            $embedding = $this -> getVideoEmbedding();
        else
            $embedding = $this -> getLinkEmbedding();

        $Template = $this -> getTemplate();
          
        
        $factory = new ESRender_LicenseFactory_Implementation();
        $lic = $factory->getLicense(ESRender_License_Interface::CC_BY, 'Youtube');
        $license = $lic -> renderFooter($this -> getTemplate());

        echo $Template -> render('/module/url/inline', array('embedding' => $embedding, 'license' => $license));
        return true;
    }

    private function validate() {
        if (!$this -> getUrl() && !$this -> isYoutubeRemoteObject())
            return false;
        return true;
    }
    
    protected function isYoutubeRemoteObject() {
        if($this -> _ESOBJECT -> AlfrescoNode -> properties['{http://www.campuscontent.de/model/1.0}remoterepositorytype'] == 'YOUTUBE')
            return true;
        return false;
    }

    protected function getLinkEmbedding() {
        $htm =  '<script> if (typeof single != "undefined") location.href="'.htmlentities($this -> getUrl(), ENT_QUOTES, 'UTF-8').'";</script>';    
        $htm .= '<a href="' . htmlentities($this -> getUrl(), ENT_QUOTES, 'UTF-8') . '" target="_blank"><es:title xmlns:es="http://edu-sharing.net/object" >' . htmlspecialchars($this -> getUrl(), ENT_QUOTES, 'UTF-8') . '</es:title></a>';
        return $htm;         
    }

    protected function getVideoEmbedding() {

        
        $objId = $this -> _ESOBJECT -> getObjectID();

        if($this->isYoutubeRemoteObject()){
            $vidId = $this->_ESOBJECT->AlfrescoNode->properties['{http://www.campuscontent.de/model/1.0}remotenodeid'];
            return '<iframe id="' . $objId . '" width="400px" height="300px" src="//www.youtube.com/embed/' . $vidId . '?modestbranding=1" frameborder="0" allowfullscreen></iframe><p class="caption"><es:title></es:title></p>';
         }
        else if (strpos($this -> getUrl(), VIDEO_TOKEN_YOUTUBE) !== false) {
                    $parsedUrl = parse_url($this -> getUrl());
                    $paramsArr = explode('&', $parsedUrl['query']);
                    foreach ($paramsArr as $param) {
                        $item = explode('=', $param);
                        $params[$item[0]] = $item[1];
                    }
                    $vidId = $params['v'];
                    return '<iframe id="' . $objId . '" width="400px" height="300px" src="//www.youtube.com/embed/' . $vidId . '?modestbranding=1" frameborder="0" allowfullscreen></iframe><p class="caption"><es:title></es:title></p>';
                }
        else if (strpos($this -> getUrl(), VIDEO_TOKEN_VIMEO) !== false) {
            $urlArr = explode('/', $this -> getUrl());
            $vidId = end($urlArr);
            return '<iframe id="' . $objId . '" width="400px" height="300px" src="//player.vimeo.com/video/' . $vidId . '" frameborder="0" webkitallowfullscreen mozallowfullscreen allowfullscreen></iframe><p class="caption"><es:title></es:title></p>';
        }
        return '';
    }

    protected function getUrl() {
        if(!empty($this -> _ESOBJECT -> AlfrescoNode -> properties[$this -> getUrlProperty()]))
            return $this -> _ESOBJECT -> AlfrescoNode -> properties[$this -> getUrlProperty()];
        return false;
    }

    protected function detectVideo() {

        if($this -> isYoutubeRemoteObject())
            return true;
        
        $needles = unserialize(VIDEO_TOKENS);
        foreach ($needles as $needle) {
            if (strpos($this -> getUrl(), $needle) !== false)
                return true;
        }
        return false;
    }

    /**
     * The object's property containing the url.
     *
     * @var string
     */
    var $UrlProperty = '{http://www.campuscontent.de/model/1.0}wwwurl';

    /**
     * Set the name of the property which should contain the url of interest.
     *
     * @param string $UrlProperty
     *
     * @return mod_url
     */
    public function setUrlProperty($UrlProperty) {
        assert(is_string($UrlProperty));
        $this -> UrlProperty = $UrlProperty;
        return $this;
    }

    /**
     * Get the name of the property which should contain the url of interest.
     *
     * @return string
     */
    protected function getUrlProperty() {
        return $this -> UrlProperty;
    }

}
