<?php

/**
 * This product Copyright 2010 metaVentis GmbH.  For detailed notice,
 * see the "NOTICE" file with this distribution.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 */

// load plugin config
require_once(dirname(__FILE__).'/config.php');

include_once (dirname(__FILE__).'/../../conf.inc.php');
include_once (MC_ROOT_PATH.'func/extern/videotoolkit/phpvideotoolkit.php5.php');
include_once (MC_ROOT_PATH.'func/classes.new/ESRender/Module/PhpVideoToolkit/Helper.php');

if ( ! file_exists(PHPVIDEOTOOLKIT_FFMPEG_BINARY) )
{
    error_log('Configured ffmpeg-binary '.PHPVIDEOTOOLKIT_FFMPEG_BINARY.' not found. Please edit modules/audio/config.php accordingly.');
}

if ( ! file_exists(PHPVIDEOTOOLKIT_FLVTOOLS_BINARY) )
{
    error_log('Configured flvtools-binary '.PHPVIDEOTOOLKIT_FLVTOOLS_BINARY.' not found. Please edit modules/audio/config.php accordingly.');
}

if ( ! file_exists(PHPVIDEOTOOLKIT_MENCODER_BINARY) )
{
    error_log('Configured mencoder-binary '.PHPVIDEOTOOLKIT_MENCODER_BINARY.' not found. Please edit modules/audio/config.php accordingly.');
}

/**
 * handles
 *
 * @author steffen gross / matthias hupfer
 * @version 1.0
 * @package core
 * @subpackage classes.new
 */
class mod_video
extends ESRender_Module_PhpVideoToolkit_Abstract
{

    protected $filename;

    protected function prepareRenderData(
        EsApplication $RemoteApplication,
        array $requestData)
    {
        $format = $this->formatByUserAgent();

        $template_data = parent::prepareRenderData($RemoteApplication, $requestData);

        $object_url = dirname($this->_ESOBJECT->getPath())
            . '/' . basename($this->getOutputFilename($format))
            . '?' . session_name() . '=' . session_id();

        $player_url = MC_MODULES_URI . '/video/flowplayer/flowplayer-3.1.0.swf';

        $player_id = 'player_' . md5($object_url . microtime(true));

        $template_data['video_url'] = $object_url;
        $template_data['player_url'] = $player_url;
        $template_data['player_id'] = $player_id;

        return $template_data;
    }

    /**
     * (non-PHPdoc)
     * @see ESRender_Module_PhpVideoToolkit_Abstract::getOutputFilename()
     */
    protected function getOutputFilename($format = self::FORMAT_FLASH)
    {
        $Logger = $this->getLogger();

        $filename = $this->getCacheFileName();
        
        $filename = str_replace('\\','/', $filename);
    //  $filename = str_replace('/',DIRECTORY_SEPARATOR, $filename);
        
        switch( $format )
        {
            case self::FORMAT_FLASH:
                $filename .= '.flv';
                break;
            case self::FORMAT_IPAD:
                $filename .= '.ipad.mp4';
                break;
            default:
                throw new Exception('Unhandled format.');
        }

        return $filename;
    }

    /**
     * Helper-method to allow re-using inline-templates in inline() and
     * display().
     *
     * @param array $template_data
     * @throws Exception
     */
    protected function renderInlineTemplate(array $template_data)
    {
        $inline_template = '';
        switch( $this->formatByUserAgent() )
        {
            case self::FORMAT_FLASH:
                $inline_template = '/module/video/inline';
                break;

            case self::FORMAT_IPAD:
                $inline_template = '/module/video/inline.ipad';
                break;

            default:
                throw new Exception('Unhandled format.');
        }

        $Template = $this->getTemplate();
        return $Template->render($inline_template, $template_data);
    }

    /**
     * (non-PHPdoc)
     * @see ESRender_Module_Base::display()
     */
    final public function display(
        EsApplication $RemoteApplication,
        array $requestData)
    {
        global $Locale, $ROOT_URI;
        $template_data = $this->prepareRenderData($RemoteApplication, $requestData);

        //$template_data['inline'] = $this->renderInlineTemplate($template_data);

        //load resource asynchr. with display mode inline!
        $template_data['ajax_url'] = $ROOT_URI . 'application/esmain/index.php?'.'app_id='
            .$requestData['app_id'].'&session='.$requestData['session']
            .'&rep_id='.$requestData['rep_id'].'&obj_id='.$requestData['object_id'].'&resource_id='
            .$requestData['resource_id'].'&course_id='.$requestData['course_id']
            .'&display=inline&language='.$Locale->getLanguageTwoLetters().'&u='.urlencode($requestData['user_name_encr']);
        $Template = $this->getTemplate();
        echo $Template->render('/module/video/display', $template_data);

        return true;
    }

    /**
     * (non-PHPdoc)
     * @see ESRender_Module_Base::inline()
     */
    final public function inline(
        EsApplication $RemoteApplication,
        array $requestData)
    {
        $template_data = $this->prepareRenderData($RemoteApplication, $requestData);

        echo $this->renderInlineTemplate($template_data);

        return true;
    }
    
    /**
     * (non-PHPdoc)
     * @see ESRender_Module_Base::locked()
     * 
     */
    final public function locked(EsApplication $RemoteApplication, array $requestData) {
        $template = $this->getTemplate();
        $toolkitOutput = MC_ROOT_PATH . 'error/conversion/' . $this -> _ESOBJECT -> getObjectID() . '_' . $this->_ESOBJECT->getObjectVersion() . '.log';
        $progress = ESRender_Module_PhpVideoToolkit_Helper::getConversionProgress($toolkitOutput, $this-> formatByUserAgent());
        if(empty($progress) || is_array($progress))
            $progress = '0';
        echo $template->render('/module/video/lock', array('callback' => $requestData['callback'], 'progress' => $progress));
        return true;
    }

}

